@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1996-2018. All Rights Reserved.
+ * Copyright Ericsson AB 1996-2020. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -567,7 +567,7 @@ BIF_RETTYPE binary_to_list_1(BIF_ALIST_1)
 	if (size < L2B_B2L_MIN_EXEC_REDS*ERTS_B2L_BYTES_PER_REDUCTION) {
 	    if (reds_left <= L2B_B2L_RESCHED_REDS) {
 		/* Yield and do it with full context reds... */
-		ERTS_BIF_YIELD1(bif_export[BIF_binary_to_list_1],
+		ERTS_BIF_YIELD1(&bif_trap_export[BIF_binary_to_list_1],
 				BIF_P, BIF_ARG_1);
 	    }
 	    /* Allow a bit more reductions... */
@@ -621,7 +621,7 @@ BIF_RETTYPE binary_to_list_3(BIF_ALIST_3)
 	if (size < L2B_B2L_MIN_EXEC_REDS*ERTS_B2L_BYTES_PER_REDUCTION) {
 	    if (reds_left <= L2B_B2L_RESCHED_REDS) {
 		/* Yield and do it with full context reds... */
-		ERTS_BIF_YIELD3(bif_export[BIF_binary_to_list_3],
+		ERTS_BIF_YIELD3(&bif_trap_export[BIF_binary_to_list_3],
 				BIF_P, BIF_ARG_1, BIF_ARG_2, BIF_ARG_3);
 	    }
 	    /* Allow a bit more reductions... */
@@ -668,7 +668,7 @@ BIF_RETTYPE bitstring_to_list_1(BIF_ALIST_1)
 	if (size < L2B_B2L_MIN_EXEC_REDS*ERTS_B2L_BYTES_PER_REDUCTION) {
 	    if (reds_left <= L2B_B2L_RESCHED_REDS) {
 		/* Yield and do it with full context reds... */
-		ERTS_BIF_YIELD1(bif_export[BIF_bitstring_to_list_1],
+		ERTS_BIF_YIELD1(&bif_trap_export[BIF_bitstring_to_list_1],
 				BIF_P, BIF_ARG_1);
 	    }
 	    /* Allow a bit more reductions... */
@@ -1041,7 +1041,7 @@ HIPE_WRAPPER_BIF_DISABLE_GC(list_to_binary, 1)
 
 BIF_RETTYPE list_to_binary_1(BIF_ALIST_1)
 {
-    return erts_list_to_binary_bif(BIF_P, BIF_ARG_1, bif_export[BIF_list_to_binary_1]);
+    return erts_list_to_binary_bif(BIF_P, BIF_ARG_1, &bif_trap_export[BIF_list_to_binary_1]);
 }
 
 HIPE_WRAPPER_BIF_DISABLE_GC(iolist_to_binary, 1)
@@ -1054,7 +1054,7 @@ BIF_RETTYPE iolist_to_binary_1(BIF_ALIST_1)
         }
         BIF_ERROR(BIF_P, BADARG);
     }
-    return erts_list_to_binary_bif(BIF_P, BIF_ARG_1, bif_export[BIF_iolist_to_binary_1]);
+    return erts_list_to_binary_bif(BIF_P, BIF_ARG_1, &bif_trap_export[BIF_iolist_to_binary_1]);
 }
 
 static int bitstr_list_len(ErtsIOListState *);
@@ -1081,7 +1081,7 @@ BIF_RETTYPE list_to_bitstring_1(BIF_ALIST_1)
 	else {
 	    ErtsL2BState state = ERTS_L2B_STATE_INITER(BIF_P,
 						       BIF_ARG_1,
-						       bif_export[BIF_list_to_bitstring_1],
+						       &bif_trap_export[BIF_list_to_bitstring_1],
 						       bitstr_list_len,
 						       list_to_bitstr_buf_yielding);
 	    int orig_reds_left = ERTS_BIF_REDS_LEFT(BIF_P);
