@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1996-2018. All Rights Reserved.
+ * Copyright Ericsson AB 1996-2020. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -25,46 +25,58 @@
 #include "erl_node_tables.h"
 #include "zlib.h"
 
-#define DFLAG_PUBLISHED           0x01
-#define DFLAG_ATOM_CACHE          0x02
-#define DFLAG_EXTENDED_REFERENCES 0x04
-#define DFLAG_DIST_MONITOR        0x08
-#define DFLAG_FUN_TAGS            0x10
-#define DFLAG_DIST_MONITOR_NAME   0x20
-#define DFLAG_HIDDEN_ATOM_CACHE   0x40
-#define DFLAG_NEW_FUN_TAGS        0x80
-#define DFLAG_EXTENDED_PIDS_PORTS 0x100
-#define DFLAG_EXPORT_PTR_TAG      0x200
-#define DFLAG_BIT_BINARIES        0x400
-#define DFLAG_NEW_FLOATS          0x800
-#define DFLAG_UNICODE_IO          0x1000
-#define DFLAG_DIST_HDR_ATOM_CACHE 0x2000
-#define DFLAG_SMALL_ATOM_TAGS     0x4000
-#define DFLAG_INTERNAL_TAGS       0x8000   /* used by ETS 'compressed' option */
-#define DFLAG_UTF8_ATOMS          0x10000
-#define DFLAG_MAP_TAG             0x20000
-#define DFLAG_BIG_CREATION        0x40000
-#define DFLAG_SEND_SENDER         0x80000
-#define DFLAG_BIG_SEQTRACE_LABELS 0x100000
-#define DFLAG_NO_MAGIC            0x200000 /* internal for pending connection */
-#define DFLAG_EXIT_PAYLOAD        0x400000
-#define DFLAG_FRAGMENTS           0x800000
+#define DFLAG_PUBLISHED               ((Uint64)0x01)
+#define DFLAG_ATOM_CACHE              ((Uint64)0x02)
+#define DFLAG_EXTENDED_REFERENCES     ((Uint64)0x04)
+#define DFLAG_DIST_MONITOR            ((Uint64)0x08)
+#define DFLAG_FUN_TAGS                ((Uint64)0x10)
+#define DFLAG_DIST_MONITOR_NAME       ((Uint64)0x20)
+#define DFLAG_HIDDEN_ATOM_CACHE       ((Uint64)0x40)
+#define DFLAG_NEW_FUN_TAGS            ((Uint64)0x80)
+#define DFLAG_EXTENDED_PIDS_PORTS    ((Uint64)0x100)
+#define DFLAG_EXPORT_PTR_TAG         ((Uint64)0x200)
+#define DFLAG_BIT_BINARIES           ((Uint64)0x400)
+#define DFLAG_NEW_FLOATS             ((Uint64)0x800)
+#define DFLAG_UNICODE_IO            ((Uint64)0x1000)
+#define DFLAG_DIST_HDR_ATOM_CACHE   ((Uint64)0x2000)
+#define DFLAG_SMALL_ATOM_TAGS       ((Uint64)0x4000)
+#define DFLAG_ETS_COMPRESSED        ((Uint64)0x8000) /* internal */
+#define DFLAG_UTF8_ATOMS           ((Uint64)0x10000)
+#define DFLAG_MAP_TAG              ((Uint64)0x20000)
+#define DFLAG_BIG_CREATION         ((Uint64)0x40000)
+#define DFLAG_SEND_SENDER          ((Uint64)0x80000)
+#define DFLAG_BIG_SEQTRACE_LABELS ((Uint64)0x100000)
+#define DFLAG_PENDING_CONNECT     ((Uint64)0x200000) /* internal */
+#define DFLAG_EXIT_PAYLOAD        ((Uint64)0x400000)
+#define DFLAG_FRAGMENTS           ((Uint64)0x800000)
+#define DFLAG_HANDSHAKE_23       ((Uint64)0x1000000)
+#define DFLAG_RESERVED                   0xfe000000
+/*
+ * As the old handshake only support 32 flag bits, we reserve the remaining
+ * bits in the lower 32 for changes in the handshake protocol or potentially
+ * new capabilities that we also want to backport to OTP-22 or older.
+ */
+#define DFLAG_SPAWN            (((Uint64)0x1) << 32)
+#define DFLAG_NAME_ME          (((Uint64)0x2) << 32)
+
 
 /* Mandatory flags for distribution */
 #define DFLAG_DIST_MANDATORY (DFLAG_EXTENDED_REFERENCES         \
                               | DFLAG_EXTENDED_PIDS_PORTS       \
 			      | DFLAG_UTF8_ATOMS                \
-			      | DFLAG_NEW_FUN_TAGS)
+			      | DFLAG_NEW_FUN_TAGS              \
+                              | DFLAG_BIG_CREATION)
 
 /*
  * Additional optimistic flags when encoding toward pending connection.
- * If remote node (erl_interface) does not supporting these then we may need
+ * If remote node (erl_interface) does not support these then we may need
  * to transcode messages enqueued before connection setup was finished.
  */
 #define DFLAG_DIST_HOPEFULLY (DFLAG_EXPORT_PTR_TAG              \
                               | DFLAG_BIT_BINARIES              \
                               | DFLAG_DIST_MONITOR              \
-                              | DFLAG_DIST_MONITOR_NAME)
+                              | DFLAG_DIST_MONITOR_NAME         \
+                              | DFLAG_SPAWN)
 
 /* Our preferred set of flags. Used for connection setup handshake */
 #define DFLAG_DIST_DEFAULT (DFLAG_DIST_MANDATORY | DFLAG_DIST_HOPEFULLY \
@@ -75,11 +87,12 @@
                             | DFLAG_SMALL_ATOM_TAGS           \
                             | DFLAG_UTF8_ATOMS                \
                             | DFLAG_MAP_TAG                   \
-                            | DFLAG_BIG_CREATION              \
                             | DFLAG_SEND_SENDER               \
                             | DFLAG_BIG_SEQTRACE_LABELS       \
                             | DFLAG_EXIT_PAYLOAD              \
-                            | DFLAG_FRAGMENTS)
+                            | DFLAG_FRAGMENTS                 \
+                            | DFLAG_HANDSHAKE_23              \
+                            | DFLAG_SPAWN)
 
 /* Flags addable by local distr implementations */
 #define DFLAG_DIST_ADDABLE    DFLAG_DIST_DEFAULT
@@ -87,6 +100,7 @@
 /* Flags rejectable by local distr implementation */
 #define DFLAG_DIST_REJECTABLE (DFLAG_DIST_HDR_ATOM_CACHE         \
                                | DFLAG_HIDDEN_ATOM_CACHE         \
+                               | DFLAG_FRAGMENTS                 \
                                | DFLAG_ATOM_CACHE)
 
 /* Flags for all features needing strict order delivery */
@@ -130,9 +144,17 @@ enum dop {
     DOP_PAYLOAD_EXIT_TT        = 25,
     DOP_PAYLOAD_EXIT2          = 26,
     DOP_PAYLOAD_EXIT2_TT       = 27,
-    DOP_PAYLOAD_MONITOR_P_EXIT = 28
+    DOP_PAYLOAD_MONITOR_P_EXIT = 28,
+
+    DOP_SPAWN_REQUEST       = 29,
+    DOP_SPAWN_REQUEST_TT    = 30,
+    DOP_SPAWN_REPLY         = 31,
+    DOP_SPAWN_REPLY_TT      = 32
 };
 
+#define ERTS_DIST_SPAWN_FLAG_LINK       (1 << 0)
+#define ERTS_DIST_SPAWN_FLAG_MONITOR    (1 << 1)
+
 /* distribution trap functions */
 extern Export* dmonitor_node_trap;
 
@@ -144,7 +166,7 @@ typedef enum {
 
 /* Must be larger or equal to 16 */
 #ifdef DEBUG
-#define ERTS_DIST_FRAGMENT_SIZE 16
+#define ERTS_DIST_FRAGMENT_SIZE 1024
 #else
 /* This should be made configurable */
 #define ERTS_DIST_FRAGMENT_SIZE (64 * 1024)
@@ -175,6 +197,9 @@ extern int erts_is_alive;
 /* dist_ctrl_{g,s}et_option/2 */
 #define ERTS_DIST_CTRL_OPT_GET_SIZE     ((Uint32) (1 << 0))
 
+/* for emulator internal testing... */
+extern int erts_dflags_test_remove_hopefull_flags;
+
 #ifdef DEBUG
 #define ERTS_DBG_CHK_NO_DIST_LNK(D, R, L) \
     erts_dbg_chk_no_dist_proc_link((D), (R), (L))
@@ -193,23 +218,75 @@ extern int erts_is_alive;
 
 typedef enum { TTBSize, TTBEncode, TTBCompress } TTBState;
 typedef struct TTBSizeContext_ {
-    Uint flags;
+    Uint64 dflags;
     int level;
+    Sint vlen;
+    int iovec;
+    Uint fragment_size;
+    Uint last_result;
+    Uint extra_size;
     Uint result;
     Eterm obj;
     ErtsWStack wstack;
 } TTBSizeContext;
 
+#define ERTS_INIT_TTBSizeContext(Ctx, Flags)                    \
+    do {                                                        \
+        (Ctx)->wstack.wstart = NULL;                            \
+        (Ctx)->dflags = (Flags);                                 \
+        (Ctx)->level = 0;                                       \
+        (Ctx)->vlen = -1;                                       \
+        (Ctx)->fragment_size = ~((Uint) 0);                     \
+        (Ctx)->extra_size = 0;                                  \
+        (Ctx)->last_result = 0;                                 \
+    } while (0)
+
 typedef struct TTBEncodeContext_ {
-    Uint flags;
-    Uint hopefull_flags;
+    Uint64 dflags;
+    Uint64 hopefull_flags;
+    byte *hopefull_flagsp;
     int level;
     byte* ep;
     Eterm obj;
     ErtsWStack wstack;
     Binary *result_bin;
+    byte *cptr;
+    Sint vlen;
+    Uint size;
+    byte *payload_ixp;
+    byte *hopefull_ixp;
+    SysIOVec* iov;
+    ErlDrvBinary** binv;
+    Eterm *termv;
+    int iovec;
+    Uint fragment_size;
+    Sint frag_ix;
+    ErlIOVec *fragment_eiovs;
+#ifdef DEBUG
+    int debug_fragments;
+    int debug_vlen;
+#endif
 } TTBEncodeContext;
 
+#define ERTS_INIT_TTBEncodeContext(Ctx, Flags)                  \
+    do {                                                        \
+        (Ctx)->wstack.wstart = NULL;                            \
+        (Ctx)->dflags = (Flags);                                 \
+        (Ctx)->level = 0;                                       \
+        (Ctx)->vlen = 0;                                        \
+        (Ctx)->size = 0;                                        \
+        (Ctx)->termv = NULL;                                    \
+        (Ctx)->iov = NULL;                                      \
+        (Ctx)->binv = NULL;                                     \
+        (Ctx)->fragment_size = ~((Uint) 0);                     \
+        if ((Flags) & DFLAG_PENDING_CONNECT) {                  \
+            (Ctx)->hopefull_flags = 0;                          \
+            (Ctx)->hopefull_flagsp = NULL;                      \
+            (Ctx)->hopefull_ixp = NULL;                         \
+            (Ctx)->payload_ixp = NULL;                          \
+        }                                                       \
+    } while (0)
+
 typedef struct {
     Uint real_size;
     Uint dest_len;
@@ -246,7 +323,7 @@ typedef struct erts_dsig_send_context {
     Eterm ctl;
     Eterm msg;
     Eterm from;
-    Eterm ctl_heap[6];
+    Eterm ctl_heap[8]; /* 7-tuple (SPAWN_REQUEST_TT) */
     Eterm return_term;
 
     DistEntry *dep;
@@ -258,12 +335,13 @@ typedef struct erts_dsig_send_context {
     enum erts_dsig_send_phase phase;
     Sint reds;
 
-    Uint32 max_finalize_prepend;
     Uint data_size, dhdr_ext_size;
+    byte *dhdrp, *extp;
     ErtsAtomCacheMap *acmp;
     ErtsDistOutputBuf *obuf;
-    Uint fragments;
-    Uint32 flags;
+    Uint alloced_fragments, fragments;
+    Sint vlen;
+    Uint64 dflags;
     Process *c_p;
     union {
 	TTBSizeContext sc;
@@ -305,6 +383,7 @@ extern int erts_dsig_send_exit2(ErtsDSigSendContext *, Eterm, Eterm, Eterm);
 extern int erts_dsig_send_demonitor(ErtsDSigSendContext *, Eterm, Eterm, Eterm);
 extern int erts_dsig_send_monitor(ErtsDSigSendContext *, Eterm, Eterm, Eterm);
 extern int erts_dsig_send_m_exit(ErtsDSigSendContext *, Eterm, Eterm, Eterm, Eterm);
+extern int erts_dsig_send_spawn_reply(ErtsDSigSendContext *, Eterm, Eterm, Eterm, Eterm, Eterm);
 
 extern int erts_dsig_send(ErtsDSigSendContext *dsdp);
 extern int erts_dsend_context_dtor(Binary*);
@@ -333,4 +412,8 @@ extern int erts_dsig_prepare(ErtsDSigSendContext *,
 
 void erts_dist_print_procs_suspended_on_de(fmtfn_t to, void *to_arg);
 int erts_auto_connect(DistEntry* dep, Process *proc, ErtsProcLocks proc_locks);
+
+Uint erts_ttb_iov_size(int use_termv, Sint vlen, Uint fragments);
+void erts_ttb_iov_init(TTBEncodeContext *ctx, int use_termv, char *ptr,
+                       Sint vlen, Uint fragments, Uint fragments_size);
 #endif
