@@ -327,7 +327,7 @@ crc32_1(BIF_ALIST_1)
     res_sum = erts_make_integer(chksum,BIF_P);
     if (rest != NIL) {
 	BUMP_ALL_REDS(BIF_P);
-	BIF_TRAP2(bif_export[BIF_crc32_2], BIF_P, res_sum, rest);
+	BIF_TRAP2(&bif_trap_export[BIF_crc32_2], BIF_P, res_sum, rest);
     }
     BIF_RET(res_sum);
 }
@@ -354,7 +354,7 @@ crc32_2(BIF_ALIST_2)
     res_sum = erts_make_integer(chksum,BIF_P);
     if (rest != NIL) {
 	BUMP_ALL_REDS(BIF_P);
-	BIF_TRAP2(bif_export[BIF_crc32_2], BIF_P, res_sum, rest);
+	BIF_TRAP2(&bif_trap_export[BIF_crc32_2], BIF_P, res_sum, rest);
     }
     BIF_RET(res_sum);
 }
@@ -407,7 +407,7 @@ adler32_1(BIF_ALIST_1)
     res_sum = erts_make_integer(chksum,BIF_P);
     if (rest != NIL) {
 	BUMP_ALL_REDS(BIF_P);
-	BIF_TRAP2(bif_export[BIF_adler32_2], BIF_P, res_sum, rest);
+	BIF_TRAP2(&bif_trap_export[BIF_adler32_2], BIF_P, res_sum, rest);
     }
     BIF_RET(res_sum);
 }
@@ -434,7 +434,7 @@ adler32_2(BIF_ALIST_2)
     res_sum = erts_make_integer(chksum,BIF_P);
     if (rest != NIL) {
 	BUMP_ALL_REDS(BIF_P);
-	BIF_TRAP2(bif_export[BIF_adler32_2], BIF_P, res_sum, rest);
+	BIF_TRAP2(&bif_trap_export[BIF_adler32_2], BIF_P, res_sum, rest);
     }
     BIF_RET(res_sum);
 }
@@ -575,7 +575,7 @@ md5_update_2(BIF_ALIST_2)
     bin = new_binary(BIF_P, (byte *) &context, sizeof(MD5_CTX));
     if (rest != NIL) {
 	BUMP_ALL_REDS(BIF_P);
-	BIF_TRAP2(bif_export[BIF_md5_update_2], BIF_P, bin, rest);
+	BIF_TRAP2(&bif_trap_export[BIF_md5_update_2], BIF_P, bin, rest);
     }
     BUMP_REDS(BIF_P,res);
     BIF_RET(bin);
