@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1999-2018. All Rights Reserved.
+ * Copyright Ericsson AB 1999-2020. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -32,8 +32,7 @@
 #include "bif.h"
 #include "erl_binary.h"
 
-
-static Eterm keyfind(int Bif, Process* p, Eterm Key, Eterm Pos, Eterm List);
+static Eterm keyfind(Export* Bif, Process* p, Eterm Key, Eterm Pos, Eterm List);
 
 /* erlang:'++'/2
  *
@@ -308,12 +307,12 @@ static Eterm append(Export *bif_entry, BIF_ALIST_2) {
 Eterm
 ebif_plusplus_2(BIF_ALIST_2)
 {
-    return append(bif_export[BIF_ebif_plusplus_2], BIF_CALL_ARGS);
+    return append(&bif_trap_export[BIF_ebif_plusplus_2], BIF_CALL_ARGS);
 }
 
 BIF_RETTYPE append_2(BIF_ALIST_2)
 {
-    return append(bif_export[BIF_append_2], BIF_CALL_ARGS);
+    return append(&bif_trap_export[BIF_append_2], BIF_CALL_ARGS);
 }
 
 /* erlang:'--'/2
@@ -954,10 +953,8 @@ static int subtract_continue(Process *p, ErtsSubtractContext *context) {
         case SUBTRACT_STAGE_SET_FINISH: {
             return subtract_set_finish(p, context);
         }
-
-        default:
-            ERTS_ASSERT(!"unreachable");
     }
+    ERTS_INTERNAL_ERROR("unreachable");
 }
 
 static int subtract_start(Process *p, Eterm lhs, Eterm rhs,
@@ -1039,11 +1036,11 @@ static Eterm subtract(Export *bif_entry, BIF_ALIST_2) {
 }
 
 BIF_RETTYPE ebif_minusminus_2(BIF_ALIST_2) {
-    return subtract(bif_export[BIF_ebif_minusminus_2], BIF_CALL_ARGS);
+    return subtract(&bif_trap_export[BIF_ebif_minusminus_2], BIF_CALL_ARGS);
 }
 
 BIF_RETTYPE subtract_2(BIF_ALIST_2) {
-    return subtract(bif_export[BIF_subtract_2], BIF_CALL_ARGS);
+    return subtract(&bif_trap_export[BIF_subtract_2], BIF_CALL_ARGS);
 }
 
 
@@ -1068,7 +1065,7 @@ BIF_RETTYPE lists_member_2(BIF_ALIST_2)
     while (is_list(list)) {
 	if (--max_iter < 0) {
 	    BUMP_ALL_REDS(BIF_P);
-	    BIF_TRAP2(bif_export[BIF_lists_member_2], BIF_P, term, list);
+	    BIF_TRAP2(&bif_trap_export[BIF_lists_member_2], BIF_P, term, list);
 	}
 	item = CAR(list_val(list));
 	if ((item == term) || (non_immed_key && eq(item, term))) {
@@ -1130,7 +1127,7 @@ static BIF_RETTYPE lists_reverse_alloc(Process *c_p,
     }
 
     ASSERT(is_list(tail) && cells_left == 0);
-    BIF_TRAP2(bif_export[BIF_lists_reverse_2], c_p, list, tail);
+    BIF_TRAP2(&bif_trap_export[BIF_lists_reverse_2], c_p, list, tail);
 }
 
 static BIF_RETTYPE lists_reverse_onheap(Process *c_p,
@@ -1179,7 +1176,7 @@ static BIF_RETTYPE lists_reverse_onheap(Process *c_p,
         }
 
         BUMP_ALL_REDS(c_p);
-        BIF_TRAP2(bif_export[BIF_lists_reverse_2], c_p, list, tail);
+        BIF_TRAP2(&bif_trap_export[BIF_lists_reverse_2], c_p, list, tail);
     }
 
     BIF_ERROR(c_p, BADARG);
@@ -1209,7 +1206,7 @@ lists_keymember_3(BIF_ALIST_3)
 {
     Eterm res;
 
-    res = keyfind(BIF_lists_keymember_3, BIF_P,
+    res = keyfind(&bif_trap_export[BIF_lists_keymember_3], BIF_P,
 		  BIF_ARG_1, BIF_ARG_2, BIF_ARG_3);
     if (is_value(res) && is_tuple(res)) {
 	return am_true;
@@ -1223,7 +1220,7 @@ lists_keysearch_3(BIF_ALIST_3)
 {
     Eterm res;
     
-    res = keyfind(BIF_lists_keysearch_3, BIF_P,
+    res = keyfind(&bif_trap_export[BIF_lists_keysearch_3], BIF_P,
 		  BIF_ARG_1, BIF_ARG_2, BIF_ARG_3);
     if (is_non_value(res) || is_not_tuple(res)) {
 	return res;
@@ -1236,12 +1233,12 @@ lists_keysearch_3(BIF_ALIST_3)
 BIF_RETTYPE
 lists_keyfind_3(BIF_ALIST_3)
 {
-    return keyfind(BIF_lists_keyfind_3, BIF_P,
+    return keyfind(&bif_trap_export[BIF_lists_keyfind_3], BIF_P,
 		   BIF_ARG_1, BIF_ARG_2, BIF_ARG_3);
 }
 
 static Eterm
-keyfind(int Bif, Process* p, Eterm Key, Eterm Pos, Eterm List)
+keyfind(Export *Bif, Process* p, Eterm Key, Eterm Pos, Eterm List)
 {
     int max_iter = 10 * CONTEXT_REDS;
     Sint pos;
@@ -1257,7 +1254,7 @@ keyfind(int Bif, Process* p, Eterm Key, Eterm Pos, Eterm List)
 	while (is_list(List)) {
 	    if (--max_iter < 0) {
 		BUMP_ALL_REDS(p);
-		BIF_TRAP3(bif_export[Bif], p, Key, Pos, List);
+		BIF_TRAP3(Bif, p, Key, Pos, List);
 	    }
 	    term = CAR(list_val(List));
 	    List = CDR(list_val(List));
@@ -1282,7 +1279,7 @@ keyfind(int Bif, Process* p, Eterm Key, Eterm Pos, Eterm List)
 	while (is_list(List)) {
 	    if (--max_iter < 0) {
 		BUMP_ALL_REDS(p);
-		BIF_TRAP3(bif_export[Bif], p, Key, Pos, List);
+		BIF_TRAP3(Bif, p, Key, Pos, List);
 	    }
 	    term = CAR(list_val(List));
 	    List = CDR(list_val(List));
@@ -1300,7 +1297,7 @@ keyfind(int Bif, Process* p, Eterm Key, Eterm Pos, Eterm List)
 	while (is_list(List)) {
 	    if (--max_iter < 0) {
 		BUMP_ALL_REDS(p);
-		BIF_TRAP3(bif_export[Bif], p, Key, Pos, List);
+		BIF_TRAP3(Bif, p, Key, Pos, List);
 	    }
 	    term = CAR(list_val(List));
 	    List = CDR(list_val(List));
