@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB and Kjell Winblad 1998-2018. All Rights Reserved.
+ * Copyright Ericsson AB and Kjell Winblad 1998-2020. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -104,7 +104,8 @@ static int db_last_catree(Process *p, DbTable *tbl,
 static int db_prev_catree(Process *p, DbTable *tbl,
                           Eterm key,
                           Eterm *ret);
-static int db_put_catree(DbTable *tbl, Eterm obj, int key_clash_fail);
+static int db_put_catree(DbTable *tbl, Eterm obj, int key_clash_fail,
+                         SWord *consumed_reds_p);
 static int db_get_catree(Process *p, DbTable *tbl,
                          Eterm key,  Eterm *ret);
 static int db_member_catree(DbTable *tbl, Eterm key, Eterm *ret);
@@ -160,6 +161,10 @@ db_lookup_dbterm_catree(Process *, DbTable *, Eterm key, Eterm obj,
                         DbUpdateHandle*);
 static void db_finalize_dbterm_catree(int cret, DbUpdateHandle *);
 static int db_get_binary_info_catree(Process*, DbTable*, Eterm key, Eterm *ret);
+static int db_put_dbterm_catree(DbTable* tbl,
+                                void* obj,
+                                int key_clash_fail,
+                                SWord *consumed_reds_p);
 
 static void split_catree(DbTableCATree *tb,
                          DbTableCATreeNode* ERTS_RESTRICT base,
@@ -213,6 +218,12 @@ DbTableMethod db_catree =
     db_foreach_offheap_catree,
     db_lookup_dbterm_catree,
     db_finalize_dbterm_catree,
+    db_eterm_to_dbterm_tree_common,
+    db_dbterm_list_prepend_tree_common,
+    db_dbterm_list_remove_first_tree_common,
+    db_put_dbterm_catree,
+    db_free_dbterm_tree_common,
+    db_get_dbterm_key_tree_common,
     db_get_binary_info_catree,
     db_first_catree, /* raw_first same as first */
     db_next_catree   /* raw_next same as next */
@@ -1367,7 +1378,7 @@ static void split_catree(DbTableCATree *tb,
     }
 }
 
-/* @brief Free the entire catree and its sub-trees.
+/** @brief Free the entire catree and its sub-trees.
  *
  * @param reds Reductions to spend.
  * @return Reductions left. Negative value if not done.
@@ -1464,7 +1475,7 @@ static SWord db_free_table_continue_catree(DbTable *tbl, SWord reds)
     return reds;
 }
 
-/* @brief Free all objects of a base node, but keep the base node.
+/** @brief Free all objects of a base node, but keep the base node.
  *
  * @param reds Reductions to spend.
  * @return Reductions left. Negative value if not done.
@@ -1632,7 +1643,27 @@ static int db_prev_catree(Process *p, DbTable *tbl, Eterm key, Eterm *ret)
     return result;
 }
 
-static int db_put_catree(DbTable *tbl, Eterm obj, int key_clash_fail)
+static int db_put_dbterm_catree(DbTable* tbl,
+                                void* obj,
+                                int key_clash_fail,
+                                SWord *consumed_reds_p)
+{
+    TreeDbTerm *value_to_insert = obj;
+    DbTableCATree *tb = &tbl->catree;
+    Eterm key = GETKEY(tb, value_to_insert->dbterm.tpl);
+    FindBaseNode fbn;
+    DbTableCATreeNode* node = find_wlock_valid_base_node(tb, key, &fbn);
+    int result = db_put_dbterm_tree_common(&tb->common,
+                                           &node->u.base.root,
+                                           value_to_insert,
+                                           key_clash_fail,
+                                           NULL);
+    wunlock_adapt_base_node(tb, node, fbn.parent, fbn.current_level);
+    return result;
+}
+
+static int db_put_catree(DbTable *tbl, Eterm obj, int key_clash_fail,
+                         SWord *consumed_reds_p)
 {
     DbTableCATree *tb = &tbl->catree;
     Eterm key = GETKEY(&tb->common, tuple_val(obj));
@@ -1776,7 +1807,7 @@ TreeDbTerm** catree_find_prev_root(CATreeRootIterator *iter, Eterm* keyp)
     return catree_find_nextprev_root(iter, 0, keyp);
 }
 
-/* @brief Find root of tree where object with smallest key of all larger than
+/** @brief Find root of tree where object with smallest key of all larger than
  * partially bound key may reside. Can be used as a starting point for
  * a reverse iteration with pb_key.
  *
@@ -1829,7 +1860,7 @@ TreeDbTerm** catree_find_next_from_pb_key_root(Eterm pb_key,
     }
 }
 
-/* @brief Find root of tree where object with largest key of all smaller than
+/** @brief Find root of tree where object with largest key of all smaller than
  * partially bound key may reside. Can be used as a starting point for
  * a forward iteration with pb_key.
  *
