@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1997-2018. All Rights Reserved.
+ * Copyright Ericsson AB 1997-2020. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -429,6 +429,7 @@ erl_first_process_otp(char* mod_name, int argc, char** argv)
     args = CONS(hp, boot_mod, args);
 
     so.flags = erts_default_spo_flags;
+    so.opts = NIL;
     res = erl_spawn_system_process(&parent, am_erl_init, am_start, args, &so);
     ASSERT(is_internal_pid(res));
 
@@ -462,6 +463,7 @@ erl_system_process_otp(Eterm parent_pid, char* modname, int off_heap_msgq, int p
     so.priority       = prio;
     so.max_gen_gcs    = (Uint16) erts_atomic32_read_nob(&erts_max_gen_gcs);
     so.scheduler      = 0;
+    so.opts = NIL;
 
     res = erl_spawn_system_process(parent, mod, am_start, NIL, &so);
     ASSERT(is_internal_pid(res));
@@ -484,6 +486,7 @@ Eterm erts_internal_spawn_system_process_3(BIF_ALIST_3) {
     ASSERT(erts_list_length(args) >= 0);
 
     so.flags = erts_default_spo_flags;
+    so.opts = NIL;
     res = erl_spawn_system_process(BIF_P, mod, func, args, &so);
 
     if (is_non_value(res)) {
@@ -583,7 +586,7 @@ void erts_usage(void)
 		 ERTS_ASYNC_THREAD_MIN_STACK_SIZE,
 		 ERTS_ASYNC_THREAD_MAX_STACK_SIZE);
     erts_fprintf(stderr, "-A number      set number of threads in async thread pool,\n");
-    erts_fprintf(stderr, "               valid range is [0-%d]\n",
+    erts_fprintf(stderr, "               valid range is [1-%d]\n",
 		 ERTS_MAX_NO_OF_ASYNC_THREADS);
     erts_fprintf(stderr, "-B[c|d|i]      c to have Ctrl-c interrupt the Erlang shell,\n");
     erts_fprintf(stderr, "               d (or no extra option) to disable the break\n");
@@ -694,7 +697,7 @@ void erts_usage(void)
     erts_fprintf(stderr, "-SDPcpu p1:p2  specify dirty CPU schedulers (p1) and dirty CPU schedulers\n");
     erts_fprintf(stderr, "               online (p2) as percentages of logical processors configured\n");
     erts_fprintf(stderr, "               and logical processors available, respectively\n");
-    erts_fprintf(stderr, "-SDio n        set number of dirty I/O schedulers, valid range is [0-%d]\n",
+    erts_fprintf(stderr, "-SDio n        set number of dirty I/O schedulers, valid range is [1-%d]\n",
 		 ERTS_MAX_NO_OF_DIRTY_IO_SCHEDULERS);
     erts_fprintf(stderr, "-t size        set the maximum number of atoms the emulator can handle\n");
     erts_fprintf(stderr, "               valid range is [%d-%d]\n",
@@ -774,6 +777,7 @@ early_init(int *argc, char **argv) /*
     int ncpu;
     int ncpuonln;
     int ncpuavail;
+    int ncpuquota;
     int schdlrs;
     int schdlrs_onln;
     int schdlrs_percentage = 100;
@@ -811,7 +815,8 @@ early_init(int *argc, char **argv) /*
                                      &max_reader_groups,
 				     &ncpu,
 				     &ncpuonln,
-				     &ncpuavail);
+				     &ncpuavail,
+				     &ncpuquota);
 
     ignore_break = 0;
     replace_intr = 0;
@@ -838,9 +843,18 @@ early_init(int *argc, char **argv) /*
      * can initialize the allocators.
      */
     no_schedulers = (Uint) (ncpu > 0 ? ncpu : 1);
-    no_schedulers_online = (ncpuavail > 0
-			    ? ncpuavail
-			    : (ncpuonln > 0 ? ncpuonln : no_schedulers));
+
+    if (ncpuavail > 0) {
+        if (ncpuquota > 0) {
+            no_schedulers_online = MIN(ncpuquota, ncpuavail);
+        } else {
+            no_schedulers_online = ncpuavail;
+        }
+    } else if (ncpuonln > 0) {
+        no_schedulers_online = ncpuonln;
+    } else {
+        no_schedulers_online = no_schedulers;
+    }
 
     schdlrs = no_schedulers;
     schdlrs_onln = no_schedulers_online;
@@ -1054,7 +1068,7 @@ early_init(int *argc, char **argv) /*
 			} else if (sys_strncmp(type, "io", 2) == 0) {
 			    arg = get_arg(argv[i]+5, argv[i+1], &i);
 			    dirty_io_scheds = atoi(arg);
-			    if (dirty_io_scheds < 0 ||
+			    if (dirty_io_scheds < 1 ||
 				dirty_io_scheds > ERTS_MAX_NO_OF_DIRTY_IO_SCHEDULERS) {
 				erts_fprintf(stderr,
 					     "bad number of dirty I/O schedulers %s\n",
@@ -1129,6 +1143,9 @@ early_init(int *argc, char **argv) /*
 	    i++;
 	}
 
+        if (erts_async_max_threads < 1)
+            erts_async_max_threads = 1;
+
 	/* apply any scheduler percentages */
 	if (schdlrs_percentage != 100 || schdlrs_onln_percentage != 100) {
 	    schdlrs = schdlrs * schdlrs_percentage / 100;
