@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2014-2018. All Rights Reserved.
+ * Copyright Ericsson AB 2014-2020. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -973,7 +973,13 @@ BIF_RETTYPE maps_keys_1(BIF_ALIST_1) {
 HIPE_WRAPPER_BIF_DISABLE_GC(maps_merge, 2)
 
 BIF_RETTYPE maps_merge_2(BIF_ALIST_2) {
-    if (is_flatmap(BIF_ARG_1)) {
+    if (BIF_ARG_1 == BIF_ARG_2) {
+	/* Merging upon itself always returns itself */
+	if (is_map(BIF_ARG_1)) {
+	    return BIF_ARG_1;
+	}
+	BIF_P->fvalue = BIF_ARG_1;
+    } else if (is_flatmap(BIF_ARG_1)) {
 	if (is_flatmap(BIF_ARG_2)) {
 	    BIF_RET(flatmap_merge(BIF_P, BIF_ARG_1, BIF_ARG_2));
 	} else if (is_hashmap(BIF_ARG_2)) {
@@ -1008,6 +1014,9 @@ static Eterm flatmap_merge(Process *p, Eterm nodeA, Eterm nodeB) {
     n1   = flatmap_get_size(mp1);
     n2   = flatmap_get_size(mp2);
 
+    if (n1 == 0) return nodeB;
+    if (n2 == 0) return nodeA;
+
     need = MAP_HEADER_FLATMAP_SZ + 1 + 2 * (n1 + n2);
 
     hp     = HAlloc(p, need);
@@ -1127,6 +1136,7 @@ static Eterm map_merge_mixed(Process *p, Eterm flat, Eterm tree, int swap_args)
 
     mp = (flatmap_t*)flatmap_val(flat);
     n  = flatmap_get_size(mp);
+    if (n == 0) return tree;
 
     ks = flatmap_get_keys(mp);
     vs = flatmap_get_values(mp);
@@ -2061,10 +2071,7 @@ Eterm erts_hashmap_insert(Process *p, Uint32 hx, Eterm key, Eterm value,
 	hp  = HAlloc(p, size);
 	res = erts_hashmap_insert_up(hp, key, value, &upsz, &stack);
     }
-
     DESTROY_ESTACK(stack);
-    ERTS_VERIFY_UNUSED_TEMP_ALLOC(p);
-    ERTS_HOLE_CHECK(p);
 
     return res;
 }
@@ -2602,8 +2609,6 @@ unroll:
     HRelease(p, hp_end, hp);
 not_found:
     DESTROY_ESTACK(stack);
-    ERTS_VERIFY_UNUSED_TEMP_ALLOC(p);
-    ERTS_HOLE_CHECK(p);
     UnUseTmpHeapNoproc(2);
     return res;
 }
