@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1997-2018. All Rights Reserved.
+ * Copyright Ericsson AB 1997-2020. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -334,9 +334,6 @@ erts_queue_dist_message(Process *rcvr,
     else {
 	LINK_MESSAGE(rcvr, mp);
 
-        if (rcvr_locks & ERTS_PROC_LOCK_MAIN)
-            erts_proc_sig_fetch(rcvr);
-
 	if (!(rcvr_locks & ERTS_PROC_LOCK_MSGQ))
 	    erts_proc_unlock(rcvr, ERTS_PROC_LOCK_MSGQ);
 
@@ -401,9 +398,6 @@ queue_messages(Process* receiver,
         erts_enqueue_signals(receiver, first, last, NULL, len, state);
     }
 
-    if (receiver_locks & ERTS_PROC_LOCK_MAIN)
-        erts_proc_sig_fetch(receiver);
-
     if (locked_msgq) {
 	erts_proc_unlock(receiver, ERTS_PROC_LOCK_MSGQ);
     }
@@ -455,6 +449,24 @@ erts_queue_message(Process* receiver, ErtsProcLocks receiver_locks,
     queue_messages(receiver, receiver_locks, mp, &mp->next, 1);
 }
 
+/**
+ *
+ * @brief Send one message from *NOT* a local process.
+ *
+ * But with a token!
+ */
+void
+erts_queue_message_token(Process* receiver, ErtsProcLocks receiver_locks,
+                         ErtsMessage* mp, Eterm msg, Eterm from, Eterm token)
+{
+    ASSERT(is_not_internal_pid(from));
+    ERL_MESSAGE_TERM(mp) = msg;
+    ERL_MESSAGE_FROM(mp) = from;
+    ERL_MESSAGE_TOKEN(mp) = token;
+    queue_messages(receiver, receiver_locks, mp, &mp->next, 1);
+}
+
+
 /**
  * @brief Send one message from a local process.
  *
@@ -641,12 +653,16 @@ erts_send_message(Process* sender,
     Eterm utag = NIL;
 #endif
     erts_aint32_t receiver_state;
+#ifdef ERTS_ENABLE_LOCK_CHECK
+    int have_receiver_main_lock = 0;
+#endif
 #ifdef SHCOPY_SEND
     erts_shcopy_t info;
 #else
     erts_literal_area_t litarea;
     INITIALIZE_LITERAL_PURGE_AREA(litarea);
 #endif
+    
 
 #ifdef USE_VM_PROBES
     *sender_name = *receiver_name = '\0';
@@ -674,7 +690,7 @@ erts_send_message(Process* sender,
          * Make sure we don't use the heap between those instances.
          */
         if (have_seqtrace(stoken)) {
-	    seq_trace_update_send(sender);
+	    seq_trace_update_serial(sender);
 	    seq_trace_output(stoken, message, SEQ_TRACE_SEND,
 			     receiver->common.id, sender);
 
@@ -704,6 +720,13 @@ erts_send_message(Process* sender,
                                             + seq_trace_size),
                                            &hp,
                                            &ohp);
+#ifdef ERTS_ENABLE_LOCK_CHECK
+        if ((*receiver_locks) & ERTS_PROC_LOCK_MAIN) {
+            have_receiver_main_lock = 1;
+            erts_proc_lc_require_lock(receiver, ERTS_PROC_LOCK_MAIN,
+                                      __FILE__, __LINE__);
+        }
+#endif
 
 #ifdef SHCOPY_SEND
 	if (is_not_immed(message))
@@ -741,6 +764,12 @@ erts_send_message(Process* sender,
 	if (receiver == sender && !(receiver_state & ERTS_PSFLG_OFF_HEAP_MSGQ)) {
 	    mp = erts_alloc_message(0, NULL);
 	    msize = 0;
+#ifdef ERTS_ENABLE_LOCK_CHECK
+            ASSERT((*receiver_locks) & ERTS_PROC_LOCK_MAIN);
+            have_receiver_main_lock = 1;
+            erts_proc_lc_require_lock(receiver, ERTS_PROC_LOCK_MAIN,
+                                      __FILE__, __LINE__);
+#endif
 	}
 	else {
 #ifdef SHCOPY_SEND
@@ -755,6 +784,13 @@ erts_send_message(Process* sender,
 					       msize,
 					       &hp,
 					       &ohp);
+#ifdef ERTS_ENABLE_LOCK_CHECK
+            if ((*receiver_locks) & ERTS_PROC_LOCK_MAIN) {
+                have_receiver_main_lock = 1;
+                erts_proc_lc_require_lock(receiver, ERTS_PROC_LOCK_MAIN,
+                                          __FILE__, __LINE__);
+            }
+#endif
 #ifdef SHCOPY_SEND
             if (is_not_immed(message))
                 message = copy_shared_perform(message, msize, &info, &hp, ohp);
@@ -777,6 +813,11 @@ erts_send_message(Process* sender,
 
     erts_queue_proc_message(sender, receiver, *receiver_locks, mp, message);
 
+#ifdef ERTS_ENABLE_LOCK_CHECK
+    if (have_receiver_main_lock)
+        erts_proc_lc_unrequire_lock(receiver, ERTS_PROC_LOCK_MAIN);
+#endif
+
     if (msize > ERTS_MSG_COPY_WORDS_PER_REDUCTION) {
         Uint reds = msize / ERTS_MSG_COPY_WORDS_PER_REDUCTION;
         if (reds > CONTEXT_REDS)
