@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1996-2018. All Rights Reserved.
+ * Copyright Ericsson AB 1996-2020. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -55,6 +55,7 @@
 
 #define WANT_NONBLOCKING    /* must define this to pull in defs from sys.h */
 #include "sys.h"
+#include "erl_osenv.h"
 
 #include "erl_threads.h"
 
@@ -87,7 +88,7 @@ static Eterm forker_port;
 /* Used by the fd driver iff the fd could not be set to non-blocking */
 typedef struct ErtsSysBlocking_ {
     ErlDrvPDL pdl;
-    int res;
+    ErlDrvSSizeT res;
     int err;
     unsigned int pkey;
 } ErtsSysBlocking;
@@ -112,6 +113,9 @@ typedef struct driver_data {
     int status;
     int terminating;
     ErtsSysBlocking *blocking;
+    int busy;
+    ErlDrvSizeT high_watermark;
+    ErlDrvSizeT low_watermark;
 } ErtsSysDriverData;
 
 #define DIR_SEPARATOR_CHAR    '/'
@@ -169,7 +173,7 @@ typedef struct driver_data {
 void
 erl_sys_late_init(void)
 {
-    SysDriverOpts opts;
+    SysDriverOpts opts = {0};
     Port *port;
 
     sys_signal(SIGPIPE, SIG_IGN); /* Ignore - we'll handle the write failure */
@@ -371,7 +375,8 @@ create_driver_data(ErlDrvPort port_num,
                    int read_write,
                    int exit_status,
                    int pid,
-                   int is_blocking)
+                   int is_blocking,
+                   SysDriverOpts* opts)
 {
     Port *prt;
     ErtsSysDriverData *driver_data;
@@ -430,6 +435,10 @@ create_driver_data(ErlDrvPort port_num,
         driver_data->ofd = NULL;
     }
 
+    driver_data->busy = 0;
+    driver_data->high_watermark = opts->high_watermark;
+    driver_data->low_watermark = opts->low_watermark;
+    
     return driver_data;
 }
 
@@ -719,7 +728,7 @@ static ErlDrvData spawn_start(ErlDrvPort port_num, char* name,
 
     dd = create_driver_data(port_num, ifd[0], ofd[1], opts->packet_bytes,
                              DO_WRITE | DO_READ, opts->exit_status,
-                             0, 0);
+                            0, 0, opts);
 
     {
         /* send ofd[0] + ifd[1] + stderrfd to forker port */
@@ -999,7 +1008,7 @@ static ErlDrvData fd_start(ErlDrvPort port_num, char* name,
     return (ErlDrvData)create_driver_data(port_num, opts->ifd, opts->ofd,
                                           opts->packet_bytes,
                                           opts->read_write, 0, -1,
-                                          !non_blocking);
+                                          !non_blocking, opts);
 }
 
 static void clear_fd_data(ErtsSysFdData *fdd)
@@ -1075,7 +1084,8 @@ static ErlDrvData vanilla_start(ErlDrvPort port_num, char* name,
 
     res = (ErlDrvData)(long)create_driver_data(port_num, fd, fd,
                                                opts->packet_bytes,
-                                               opts->read_write, 0, -1, 0);
+                                               opts->read_write, 0, -1, 0,
+                                               opts);
     return res;
 }
 
@@ -1108,10 +1118,10 @@ static void outputv(ErlDrvData e, ErlIOVec* ev)
     int pb = dd->packet_bytes;
     int ofd = dd->ofd ? dd->ofd->fd : -1;
     ssize_t n;
-    ErlDrvSizeT sz;
     char lb[4];
     char* lbp;
     ErlDrvSizeT len = ev->size;
+    ErlDrvSizeT qsz;
 
     /* (len > ((unsigned long)-1 >> (4-pb)*8)) */
     /*    if (pb >= 0 && (len & (((ErlDrvSizeT)1 << (pb*8))) - 1) != len) {*/
@@ -1130,14 +1140,20 @@ static void outputv(ErlDrvData e, ErlIOVec* ev)
     if (dd->blocking)
         driver_pdl_lock(dd->blocking->pdl);
 
-    if ((sz = driver_sizeq(ix)) > 0) {
-	driver_enqv(ix, ev, 0);
-
+    qsz = driver_sizeq(ix);
+    if (qsz) {
+        if (qsz == (ErlDrvSizeT) -1) {
+            if (dd->blocking)
+                driver_pdl_unlock(dd->blocking->pdl);
+            driver_failure_posix(ix, EINVAL);
+            return;
+        }
+        driver_enqv(ix, ev, 0);
+        qsz += ev->size;
+        if (!dd->busy && qsz >= dd->high_watermark)
+            set_busy_port(ix, (dd->busy = !0));
         if (dd->blocking)
             driver_pdl_unlock(dd->blocking->pdl);
-
-	if (sz + ev->size >= (1 << 13))
-	    set_busy_port(ix, 1);
     }
     else if (!dd->blocking) {
         /* We try to write directly if the fd in non-blocking */
@@ -1154,11 +1170,17 @@ static void outputv(ErlDrvData e, ErlIOVec* ev)
 	    n = 0;
 	}
 	driver_enqv(ix, ev, n);  /* n is the skip value */
+        qsz = ev->size - n;
+        if (!dd->busy && qsz >= dd->high_watermark)
+            set_busy_port(ix, (dd->busy = !0));
 	driver_select(ix, ofd, ERL_DRV_WRITE|ERL_DRV_USE, 1);
     }
     else {
         if (ev->size != 0) {
             driver_enqv(ix, ev, 0);
+            qsz = ev->size;
+            if (!dd->busy && qsz >= dd->high_watermark)
+                set_busy_port(ix, (dd->busy = !0));
             driver_pdl_unlock(dd->blocking->pdl);
             driver_async(ix, &dd->blocking->pkey,
                          fd_async, dd, NULL);
@@ -1166,6 +1188,7 @@ static void outputv(ErlDrvData e, ErlIOVec* ev)
             driver_pdl_unlock(dd->blocking->pdl);
         }
     }
+
     /* return 0;*/
 }
 
@@ -1177,7 +1200,7 @@ static void output(ErlDrvData e, char* buf, ErlDrvSizeT len)
     int pb = dd->packet_bytes;
     int ofd = dd->ofd ? dd->ofd->fd : -1;
     ssize_t n;
-    ErlDrvSizeT sz;
+    ErlDrvSizeT qsz;
     char lb[4];
     char* lbp;
     struct iovec iv[2];
@@ -1192,11 +1215,15 @@ static void output(ErlDrvData e, char* buf, ErlDrvSizeT len)
     put_int32(len, lb);
     lbp = lb + (4-pb);
 
-    if ((sz = driver_sizeq(ix)) > 0) {
+    qsz = driver_sizeq(ix);
+    if (qsz) {
+        if (qsz == (ErlDrvSizeT) -1) {
+            driver_failure_posix(ix, EINVAL);
+            return;
+        }
 	driver_enq(ix, lbp, pb);
 	driver_enq(ix, buf, len);
-	if (sz + len + pb >= (1 << 13))
-	    set_busy_port(ix, 1);
+        qsz += len + pb;
     }
     else {
 	iv[0].iov_base = lbp;
@@ -1213,6 +1240,7 @@ static void output(ErlDrvData e, char* buf, ErlDrvSizeT len)
 	    }
 	    n = 0;
 	}
+        qsz = pb + len - n;
 	if (n < pb) {
 	    driver_enq(ix, lbp+n, pb-n);
 	    driver_enq(ix, buf, len);
@@ -1223,6 +1251,10 @@ static void output(ErlDrvData e, char* buf, ErlDrvSizeT len)
 	}
 	driver_select(ix, ofd, ERL_DRV_WRITE|ERL_DRV_USE, 1);
     }
+
+    if (!dd->busy && qsz >= dd->high_watermark)
+        set_busy_port(ix, (dd->busy = !0));
+
     return; /* 0; */
 }
 
@@ -1479,6 +1511,8 @@ static void ready_output(ErlDrvData e, ErlDrvEvent ready_fd)
     int vsize;
 
     if ((iv = (struct iovec*) driver_peekq(ix, &vsize)) == NULL) {
+        if (dd->busy)
+            set_busy_port(ix, (dd->busy = 0));
 	driver_select(ix, ready_fd, ERL_DRV_WRITE, 0);
         if (dd->pid > 0 && dd->ofd->fd < 0) {
             /* The port was opened with 'in' option, which means we
@@ -1494,8 +1528,13 @@ static void ready_output(ErlDrvData e, ErlDrvEvent ready_fd)
     }
     vsize = vsize > MAX_VSIZE ? MAX_VSIZE : vsize;
     if ((n = writev(ready_fd, iv, vsize)) > 0) {
-	if (driver_deq(ix, n) == 0)
-	    set_busy_port(ix, 0);
+        ErlDrvSizeT qsz = driver_deq(ix, n);
+        if (qsz == (ErlDrvSizeT) -1) {
+            driver_failure_posix(ix, EINVAL);
+            return;
+        }
+        if (dd->busy && qsz < dd->low_watermark)
+            set_busy_port(ix, (dd->busy = 0));
     }
     else if (n < 0) {
 	if (errno == ERRNO_BLOCK || errno == EINTR)
@@ -1519,7 +1558,7 @@ static void stop_select(ErlDrvEvent fd, void* _)
 static void
 fd_async(void *async_data)
 {
-    int res;
+    ErlDrvSSizeT res;
     ErtsSysDriverData *dd = (ErtsSysDriverData *)async_data;
     SysIOVec      *iov0;
     SysIOVec      *iov;
@@ -1544,7 +1583,6 @@ fd_async(void *async_data)
         } while (res < 0 && errno == EINTR);
         if (res < 0)
             err = errno;
-        err = errno;
 
         erts_free(ERTS_ALC_T_SYS_WRITE_BUF, iov);
     }
@@ -1560,10 +1598,18 @@ void fd_ready_async(ErlDrvData drv_data,
     ASSERT(dd->blocking);
 
     if (dd->blocking->res > 0) {
+        ErlDrvSizeT qsz;
         driver_pdl_lock(dd->blocking->pdl);
-        if (driver_deq(port_num, dd->blocking->res) == 0) {
+        qsz = driver_deq(port_num, dd->blocking->res);
+        if (qsz == (ErlDrvSizeT) -1) {
             driver_pdl_unlock(dd->blocking->pdl);
-            set_busy_port(port_num, 0);
+            driver_failure_posix(port_num, EINVAL);
+            return;
+        }
+        if (dd->busy && qsz < dd->low_watermark)
+            set_busy_port(port_num, (dd->busy = 0));
+        driver_pdl_unlock(dd->blocking->pdl);
+        if (qsz == 0) {
             if (dd->terminating) {
                 /* The port is has been ordered to terminate
                    from either fd_flush or port_inp_failure */
@@ -1576,14 +1622,12 @@ void fd_ready_async(ErlDrvData drv_data,
                 return; /* -1; */
             }
         } else {
-            driver_pdl_unlock(dd->blocking->pdl);
             /* still data left to write in queue */
             driver_async(port_num, &dd->blocking->pkey, fd_async, dd, NULL);
             return /* 0; */;
         }
     } else if (dd->blocking->res < 0) {
         if (dd->blocking->err == ERRNO_BLOCK) {
-            set_busy_port(port_num, 1);
             /* still data left to write in queue */
             driver_async(port_num, &dd->blocking->pkey, fd_async, dd, NULL);
         } else
