@@ -301,8 +301,8 @@ http(Config) when is_list(Config) ->
                    StrA = list_to_atom(Str),
                    StrB = list_to_binary(Str),
                    Bin = <<StrB/binary,": ",ValB/binary,"\r\n",Rest/binary>>,
-                   {ok, {http_header,N,StrA,undefined,Val}, Rest} = decode_pkt(httph,Bin),
-                   {ok, {http_header,N,StrA,undefined,ValB}, Rest} = decode_pkt(httph_bin,Bin),
+                   {ok, {http_header,N,StrA,Str,Val}, Rest} = decode_pkt(httph,Bin),
+                   {ok, {http_header,N,StrA,StrB,ValB}, Rest} = decode_pkt(httph_bin,Bin),
                    N + 1
            end,
     lists:foldl(HdrF, 1, http_hdr_strings()),
@@ -330,6 +330,15 @@ http(Config) when is_list(Config) ->
     %% Response with empty phrase
     {ok,{http_response,{1,1},200,[]},<<>>} = decode_pkt(http, <<"HTTP/1.1 200\r\n">>, []),
     {ok,{http_response,{1,1},200,<<>>},<<>>} = decode_pkt(http_bin, <<"HTTP/1.1 200\r\n">>, []),
+
+
+    %% Test error cases
+    {ok,{http_error,"Host\t: localhost:8000\r\n"},<<"a">>} =
+        decode_pkt(httph, <<"Host\t: localhost:8000\r\na">>, []),
+    {ok,{http_error,"Host : localhost:8000\r\n"},<<"a">>} =
+        decode_pkt(httph, <<"Host : localhost:8000\r\na">>, []),
+    {ok,{http_error," : localhost:8000\r\n"},<<"a">>} =
+        decode_pkt(httph, <<" : localhost:8000\r\na">>, []),
     ok.
 
 http_with_bin(http) ->
@@ -364,31 +373,40 @@ http_request(Msg) ->
             {http_request, 'POST', {abs_path,  "/invalid/url"  }, {1,1}},
             {http_request, 'POST', {abs_path,<<"/invalid/url">>}, {1,1}}},
            {"Connection: close\r\n",
-            {http_header,2,'Connection',undefined,  "close"},
-            {http_header,2,'Connection',undefined,<<"close">>}},	 
-           {"Host\t : localhost:8000\r\n", % white space before :
-            {http_header,14,'Host',undefined,  "localhost:8000"},
-            {http_header,14,'Host',undefined,<<"localhost:8000">>}},
+            {http_header,2,'Connection',  "Connection"  ,  "close"},
+            {http_header,2,'Connection',<<"Connection">>,<<"close">>}},
            {"User-Agent: perl post\r\n",
-            {http_header,24,'User-Agent',undefined,  "perl post"},
-            {http_header,24,'User-Agent',undefined,<<"perl post">>}},
+            {http_header,24,'User-Agent',  "User-Agent"  ,  "perl post"},
+            {http_header,24,'User-Agent',<<"User-Agent">>,<<"perl post">>}},
            {"Content-Length: 4\r\n",
-            {http_header,38,'Content-Length',undefined,  "4"},
-            {http_header,38,'Content-Length',undefined,<<"4">>}},
+            {http_header,38,'Content-Length',  "Content-Length"  ,  "4"},
+            {http_header,38,'Content-Length',<<"Content-Length">>,<<"4">>}},
            {"Content-Type: text/xml; charset=utf-8\r\n",
-            {http_header,42,'Content-Type',undefined,  "text/xml; charset=utf-8"},
-            {http_header,42,'Content-Type',undefined,<<"text/xml; charset=utf-8">>}},
+            {http_header,42,'Content-Type',  "Content-Type"  ,  "text/xml; charset=utf-8"},
+            {http_header,42,'Content-Type',<<"Content-Type">>,<<"text/xml; charset=utf-8">>}},
            {"Other-Field: with some text\r\n",
-            {http_header,0,  "Other-Field"  ,undefined,  "with some text"},
-            {http_header,0,<<"Other-Field">>,undefined,<<"with some text">>}},
+            {http_header,0,  "Other-Field"  ,  "Other-Field"  ,  "with some text"},
+            {http_header,0,<<"Other-Field">>,<<"Other-Field">>,<<"with some text">>}},
+           {"Content--Type: text/xml; charset=utf-8\r\n",
+            {http_header,0,  "Content--type"  ,  "Content--Type"  ,  "text/xml; charset=utf-8"},
+            {http_header,0,<<"Content--type">>,<<"Content--Type">>,<<"text/xml; charset=utf-8">>}},
+           {"Content---Type: text/xml; charset=utf-8\r\n",
+            {http_header,0,  "Content---Type"  ,  "Content---Type"  ,  "text/xml; charset=utf-8"},
+            {http_header,0,<<"Content---Type">>,<<"Content---Type">>,<<"text/xml; charset=utf-8">>}},
+           {"CONTENT-type: text/xml; charset=utf-8\r\n",
+            {http_header,42,'Content-Type',  "CONTENT-type"  ,  "text/xml; charset=utf-8"},
+            {http_header,42,'Content-Type',<<"CONTENT-type">>,<<"text/xml; charset=utf-8">>}},
+           {"OTHER-field: with some text\r\n",
+            {http_header,0,  "Other-Field"  ,  "OTHER-field"  ,  "with some text"},
+            {http_header,0,<<"Other-Field">>,<<"OTHER-field">>,<<"with some text">>}},
            {"Make-sure-a-LONG-HEaDer-fIeLd-is-fORMATTED-NicelY: with some text\r\n",
-            {http_header,0,  "Make-Sure-A-Long-Header-Field-Is-Formatted-Nicely"  ,undefined,  "with some text"},
-            {http_header,0,<<"Make-Sure-A-Long-Header-Field-Is-Formatted-Nicely">>,undefined,<<"with some text">>}},
+            {http_header,0,  "Make-Sure-A-Long-Header-Field-Is-Formatted-Nicely"  ,  "Make-sure-a-LONG-HEaDer-fIeLd-is-fORMATTED-NicelY"  ,  "with some text"},
+            {http_header,0,<<"Make-Sure-A-Long-Header-Field-Is-Formatted-Nicely">>,<<"Make-sure-a-LONG-HEaDer-fIeLd-is-fORMATTED-NicelY">>,<<"with some text">>}},
            {"Multi-Line: Once upon a time in a land far far away,\r\n"
             " there lived a princess imprisoned in the highest tower\r\n"
             " of the most haunted castle.\r\n",
-            {http_header,0,  "Multi-Line"  ,undefined,  "Once upon a time in a land far far away,\r\n there lived a princess imprisoned in the highest tower\r\n of the most haunted castle."},
-            {http_header,0,<<"Multi-Line">>,undefined,<<"Once upon a time in a land far far away,\r\n there lived a princess imprisoned in the highest tower\r\n of the most haunted castle.">>}},
+            {http_header,0,  "Multi-Line"  ,  "Multi-Line"  ,  "Once upon a time in a land far far away,\r\n there lived a princess imprisoned in the highest tower\r\n of the most haunted castle."},
+            {http_header,0,<<"Multi-Line">>,<<"Multi-Line">>,<<"Once upon a time in a land far far away,\r\n there lived a princess imprisoned in the highest tower\r\n of the most haunted castle.">>}},
            {"\r\n",
             http_eoh,
             http_eoh}],
@@ -404,17 +422,17 @@ http_response(Msg) ->
             {http_response, {1,0}, 404,   "Object Not Found"},
             {http_response, {1,0}, 404, <<"Object Not Found">>}},
            {"Server: inets/4.7.16\r\n",
-            {http_header, 30, 'Server', undefined,   "inets/4.7.16"},
-            {http_header, 30, 'Server', undefined, <<"inets/4.7.16">>}},
+            {http_header, 30, 'Server',   "Server"  ,   "inets/4.7.16"},
+            {http_header, 30, 'Server', <<"Server">>, <<"inets/4.7.16">>}},
            {"Date: Fri, 04 Jul 2008 17:16:22 GMT\r\n",
-            {http_header, 3, 'Date', undefined,   "Fri, 04 Jul 2008 17:16:22 GMT"},
-            {http_header, 3, 'Date', undefined, <<"Fri, 04 Jul 2008 17:16:22 GMT">>}},
+            {http_header, 3, 'Date',   "Date"  ,   "Fri, 04 Jul 2008 17:16:22 GMT"},
+            {http_header, 3, 'Date', <<"Date">>, <<"Fri, 04 Jul 2008 17:16:22 GMT">>}},
            {"Content-Type: text/html\r\n",
-            {http_header, 42, 'Content-Type', undefined,   "text/html"},
-            {http_header, 42, 'Content-Type', undefined, <<"text/html">>}},
+            {http_header, 42, 'Content-Type',   "Content-Type"  ,   "text/html"},
+            {http_header, 42, 'Content-Type', <<"Content-Type">>, <<"text/html">>}},
            {"Content-Length: 207\r\n",
-            {http_header, 38, 'Content-Length', undefined,   "207"},
-            {http_header, 38, 'Content-Length', undefined, <<"207">>}},
+            {http_header, 38, 'Content-Length',   "Content-Length"  ,   "207"},
+            {http_header, 38, 'Content-Length', <<"Content-Length">>, <<"207">>}},
            {"\r\n",
             http_eoh,
             http_eoh}],
@@ -542,7 +560,7 @@ otp_8536_do(N) ->
     Bin = <<Hdr/binary, ": ", Data/binary, "\r\n\r\n">>,
 
     io:format("Bin='~p'\n",[Bin]),
-    {ok,{http_header,0,Hdr2,undefined,Data2},<<"\r\n">>} = decode_pkt(httph_bin, Bin,  []),
+    {ok,{http_header,0,Hdr2,Hdr2,Data2},<<"\r\n">>} = decode_pkt(httph_bin, Bin,  []),
 
     %% Do something to trash the C-stack, how about another decode_packet:
     decode_pkt(httph_bin,<<Letters/binary, ": ", Data/binary, "\r\n\r\n">>, []),
