@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2011-2016. All Rights Reserved.
+%% Copyright Ericsson AB 2011-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -34,7 +34,8 @@
 %% Exported end user tests
 
 -export([seq/3, seq_r/3]).
--export([loaded/1, a_function/1, a_called_function/1, dec/1, nif_dec/1, dead_tracer/1]).
+-export([loaded/1, a_function/1, a_called_function/1, dec/1, nif_dec/1, dead_tracer/1,
+        return_stop/1,catch_crash/1]).
 
 -define(US_ERROR, 10000).
 -define(R_ERROR, 0.8).
@@ -64,6 +65,7 @@
 -export([all/0, suite/0,
 	 init_per_testcase/2, end_per_testcase/2, not_run/1]).
 -export([basic/1, on_and_off/1, info/1,
+         disable_ongoing/1,
 	 pause_and_restart/1, scheduling/1, called_function/1, combo/1, 
 	 bif/1, nif/1]).
 
@@ -88,7 +90,9 @@ all() ->
 	true -> [not_run];
 	false ->
 	    [basic, on_and_off, info, pause_and_restart, scheduling,
-	     combo, bif, nif, called_function, dead_tracer]
+             disable_ongoing,
+	     combo, bif, nif, called_function, dead_tracer, return_stop,
+             catch_crash]
     end.
 
 not_run(Config) when is_list(Config) ->
@@ -117,6 +121,56 @@ basic(Config) when is_list(Config) ->
     Pid ! quit,
     ok.
 
+%% Tests disable ongoing call_time traced function
+disable_ongoing(Config) when is_list(Config) ->
+    P = erlang:trace_pattern({'_','_','_'}, false, [call_time]),
+    %%
+    1 = erlang:trace_pattern({?MODULE,disong_a, 1}, true, [call_time]),
+    1 = erlang:trace_pattern({?MODULE,disong_b, 1}, true, [call_time]),
+    1 = erlang:trace_pattern({?MODULE,disong_c, 1}, true, [call_time]),
+    1 = erlang:trace_pattern({?MODULE,disong_d, 0}, true, [call_time]),
+
+    Pid = setup(),
+
+    Self = self(),
+    Pid  ! {self(), execute, fun() -> disong_a(Self) end},
+
+    c_ready = receive M1 -> M1 end,
+
+    1 = erlang:trace_pattern({?MODULE,disong_b,1}, false, [call_time]),
+
+    Pid ! go_on,
+
+    {Pid, answer, _} = receive M2 -> M2
+                       after 1000 -> timeout end,
+
+    P = erlang:trace_pattern({'_','_','_'}, false, [call_time]),
+    Pid ! quit,
+    ok.
+
+disong_a(Pid) ->
+    ForceStack = id(a),
+    disong_b(Pid),
+    disong_d(),
+    ForceStack.
+
+disong_b(Pid) ->
+    ForceStack = id(b),
+    disong_c(Pid),
+    ForceStack.
+
+disong_c(Pid) ->
+    Pid ! c_ready,
+    receive
+        go_on -> ok
+    end.
+
+disong_d() ->
+    ok.
+
+id(I) ->
+    I.
+
 %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 
@@ -254,8 +308,7 @@ combo(Config) when is_list(Config) ->
     2 = erlang:trace_pattern({erlang, term_to_binary, '_'}, [], [local]),
     2 = erlang:trace_pattern({erlang, term_to_binary, '_'}, true, [call_time]),
     2 = erlang:trace_pattern({erlang, term_to_binary, '_'}, MetaMs, [{meta,MetaTracer}]),
-    %% not implemented
-    %2 = erlang:trace_pattern({erlang, term_to_binary, '_'}, true, [call_count]),
+    2 = erlang:trace_pattern({erlang, term_to_binary, '_'}, true, [call_count]),
 
     1 = erlang:trace(Self, true, [{tracer,LocalTracer} | Flags]),
     %%
@@ -284,9 +337,7 @@ combo(Config) when is_list(Config) ->
     {value,{match_spec,[]}}    = lists:keysearch(match_spec, 1, TraceInfoBif),
     {value,{meta, MetaTracer}}  = lists:keysearch(meta, 1, TraceInfoBif),
     {value,{meta_match_spec,MetaMs}} = lists:keysearch(meta_match_spec, 1, TraceInfoBif),
-    %% not implemented
-    {value,{call_count,false}} = lists:keysearch(call_count, 1, TraceInfoBif),
-    %{value,{call_count,0}} = lists:keysearch(call_count, 1, TraceInfoBif),
+    {value,{call_count,0}} = lists:keysearch(call_count, 1, TraceInfoBif),
     {value,{call_time,[]}} = lists:keysearch(call_time, 1, TraceInfoBif),
 
     %%
@@ -355,8 +406,12 @@ bif(Config) when is_list(Config) ->
     %%
     2 = erlang:trace_pattern({erlang, binary_to_term, '_'}, true, [call_time]),
     2 = erlang:trace_pattern({erlang, term_to_binary, '_'}, true, [call_time]),
+    1 = erlang:trace_pattern({?MODULE, with_bif, 1}, true, [call_time]),
     Pid = setup(),
-    {L, T1} = execute(Pid, fun() -> with_bif(M) end),
+    {L, Tot1} = execute(Pid, fun() -> with_bif(M) end),
+
+    {call_time,[{Pid,_,S,Us}]} = erlang:trace_info({?MODULE,with_bif,1}, call_time),
+    T1 = Tot1 - (S*1000000 + Us),
 
     ok = check_trace_info({erlang, binary_to_term, 1}, [{Pid, M - 1, 0, 0}], T1/2),
     ok = check_trace_info({erlang, term_to_binary, 1}, [{Pid, M - 1, 0, 0}], T1/2),
@@ -365,9 +420,9 @@ bif(Config) when is_list(Config) ->
 
     2 = erlang:trace_pattern({erlang, term_to_binary, '_'}, false, [call_time]),
 
-    {L, T2} = execute(Pid, fun() -> with_bif(M) end),
+    {L, _T2} = execute(Pid, fun() -> with_bif(M) end),
 
-    ok = check_trace_info({erlang, binary_to_term, 1}, [{Pid, M*2 - 2, 0, 0}], T1/2 + T2),
+    ok = check_trace_info({erlang, binary_to_term, 1}, [{Pid, M*2 - 2, 0, 0}], T1),
     ok = check_trace_info({erlang, term_to_binary, 1}, false, none),
 
     %%
@@ -386,12 +441,14 @@ nif(Config) when is_list(Config) ->
     1 = erlang:trace_pattern({?MODULE, nif_dec,  '_'}, true, [call_time]),
     1 = erlang:trace_pattern({?MODULE, with_nif, '_'}, true, [call_time]),
     Pid = setup(),
-    {_, T1} = execute(Pid, fun() -> with_nif(M) end),
+    {_, Tot1} = execute(Pid, fun() -> with_nif(M) end),
+
+    {call_time,[{Pid,_,S,Us}]} = erlang:trace_info({?MODULE,with_nif,1}, call_time),
+    T1 = Tot1 - (S*1000000 + Us),
 
     % the nif is called M - 1 times, the last time the function with 'with_nif'
     % returns ok and does not call the nif.
-    ok = check_trace_info({?MODULE, nif_dec,  1}, [{Pid, M-1, 0, 0}], T1/5*4),
-    ok = check_trace_info({?MODULE, with_nif, 1}, [{Pid, M, 0, 0}], T1/5),
+    ok = check_trace_info({?MODULE, nif_dec,  1}, [{Pid, M-1, 0, 0}], T1),
 
     %%
     P = erlang:trace_pattern({'_','_','_'}, false, [call_time]),
@@ -429,6 +486,8 @@ called_function(Config) when is_list(Config) ->
 %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 dead_tracer(Config) when is_list(Config) ->
+    TracedMFAs = dead_tracer_mfas(),
+
     Self = self(),
     FirstTracer = tracer(),
     StartTracing = fun() -> turn_on_tracing(Self) end,
@@ -443,14 +502,14 @@ dead_tracer(Config) when is_list(Config) ->
     erlang:yield(),
 
     %% Collect and check that we only get call_time info for the current process.
-    Info1 = collect_all_info(),
+    Info1 = collect_all_info(TracedMFAs),
     [] = other_than_self(Info1),
     io:format("~p\n", [Info1]),
 
     %% Note that we have not turned off tracing for the current process,
     %% but that the tracer has terminated. No more call_time information should be recorded.
     [1,2,3] = seq(1, 3, fun(I) -> I + 1 end),
-    [] = collect_all_info(),
+    [] = collect_all_info(TracedMFAs),
 
     %% When we start a second tracer process, that tracer process must
     %% not inherit the tracing flags and the dead tracer (even though
@@ -459,7 +518,7 @@ dead_tracer(Config) when is_list(Config) ->
     tell_tracer(SecondTracer, StartTracing),
     Seq20 = lists:seq(1, 20),
     Seq20 = seq(1, 20, fun(I) -> I + 1 end),
-    Info2 = collect_all_info(),
+    Info2 = collect_all_info(TracedMFAs),
     io:format("~p\n", [Info2]),
     [] = other_than_self(Info2),
     SecondTracer ! quit,
@@ -495,9 +554,21 @@ turn_on_tracing(Pid) ->
     _ = now(),
     ok.
 
-collect_all_info() ->
-    collect_all_info([{?MODULE,F,A} || {F,A} <- module_info(functions)] ++
-                     erlang:system_info(snifs)).
+%% We want to trace functions local to this module as well as all BIFs, and for
+%% the latter we need to ensure that their modules are loaded.
+dead_tracer_mfas() ->
+    Modules = [M || {M,_F,_A} <- erlang:system_info(snifs)],
+    Whitelist0 = gb_sets:from_list(Modules),
+    Whitelist = case code:ensure_modules_loaded(Modules) of
+                  {error, Reasons} ->
+                      Blacklist = gb_sets:from_list([M || {M, _} <- Reasons]),
+                      gb_sets:subtract(Whitelist0, Blacklist);
+                  ok ->
+                      Whitelist0
+              end,
+    EligibleSNIFs = [MFA || {M,_F,_A}=MFA <- erlang:system_info(snifs),
+                            gb_sets:is_element(M, Whitelist)],
+    [{?MODULE,F,A} || {F,A} <- module_info(functions)] ++ EligibleSNIFs.
 
 collect_all_info([MFA|T]) ->
     CallTime = erlang:trace_info(MFA, call_time),
@@ -512,9 +583,77 @@ collect_all_info([MFA|T]) ->
     end;
 collect_all_info([]) -> [].
 
-%%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%%% The Tests
-%%%
+
+%% OTP-16111: Verify call_time does not increase after traced function returns.
+return_stop(_Config) ->
+    P = erlang:trace_pattern({'_','_','_'}, false, [call_time]),
+    %%
+    1 = erlang:trace_pattern({?MODULE,aaa,  '_'}, true, [call_time]),
+    1 = erlang:trace_pattern({?MODULE,bbb,  '_'}, true, [call_time]),
+    Pid = setup(),
+    {aaa, T1} = execute(Pid, fun() -> aaa()  end),
+    {call_time, [{Pid, 1, 0, US1}]} = erlang:trace_info({?MODULE,aaa,0}, call_time),
+    io:format("T1=~p us, US1=~p us\n", [T1, US1]),
+    true = (US1 =< T1),
+
+    {call_time, [{Pid, 1, 0, US1}]} = erlang:trace_info({?MODULE,aaa,0}, call_time),
+
+    execute(Pid, fun() -> loaded(1000000)  end),
+    {call_time, [{Pid, 1, 0, US1}]} = erlang:trace_info({?MODULE,aaa,0}, call_time),
+
+    {bbb,  _} = execute(Pid, fun() -> bbb()  end),
+    {call_time, [{Pid, 1, 0, US1}]} = erlang:trace_info({?MODULE,aaa,0}, call_time),
+
+    1 = erlang:trace_pattern({?MODULE,spinner,  1}, true, [call_time]),
+    1 = erlang:trace_pattern({?MODULE,quicky,  0}, true, [call_time]),
+    {spinner,  T2} = execute(Pid, fun() -> spinner(1000000)  end),
+    {call_time, [{Pid, 1, SpS, SpUS}]} = erlang:trace_info({?MODULE,spinner,1}, call_time),
+    Spinner = SpS*1000000 + SpUS,
+    {call_time, [{Pid, 1, 0, Quicky}]} = erlang:trace_info({?MODULE,quicky,0}, call_time),
+    io:format("T2=~p us, Spinner=~p us, Quicky=~p us\n", [T2, Spinner, Quicky]),
+
+    %% Before fix: quicky() got attributed the call_time of its caller spinner().
+    true = (Quicky =< Spinner),
+    true = (Spinner =< T2),
+
+    %%
+    P = erlang:trace_pattern({'_','_','_'}, false, [call_time]),
+    Pid ! quit,
+    ok.
+
+
+aaa() ->
+    aaa.
+bbb() ->
+   bbb.
+
+spinner(N) ->
+    quicky(),
+    loaded(N),
+    spinner.
+
+quicky() ->
+    done.
+
+%% OTP-16994: next_catch returned a bogus stack pointer when call_time tracing
+%% was enabled, crashing the emulator.
+catch_crash(_Config) ->
+    Fun = id(fun() -> catch_crash_1() end),
+
+    _ = erlang:trace_pattern({?MODULE,'_','_'}, true, [call_time]),
+    _ = erlang:trace(self(), true, [call]),
+
+    Res = (catch Fun()),
+
+    _ = erlang:trace_pattern({'_','_','_'}, false, [call_time]),
+    _ = erlang:trace(self(), false, [call]),
+
+    id(Res),
+
+    ok.
+
+catch_crash_1() ->
+    error(crash).
 
 %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -567,21 +706,29 @@ seq_r(Start, Stop, Succ, R) ->
     seq_r(Succ(Start), Stop, Succ, [Start | R]).
 
 % Check call time tracing data and print mismatches
-check_trace_info(Mfa, [{Pid, C,_,_}] = Expect, Time) ->
-    case erlang:trace_info(Mfa, call_time) of
-        % Time tests are somewhat problematic. We want to know if Time (EXPECTED_TIME) and S*1000000 + Us (ACTUAL_TIME)
-        % is the same.
-        % If the ratio EXPECTED_TIME/ACTUAL_TIME is ~ 1 or if EXPECTED_TIME - ACTUAL_TIME is near zero, the test is ok.
-        {call_time,[{Pid,C,S,Us}]} when S >= 0, Us >= 0,  abs(1 - Time/(S*1000000 + Us)) < ?R_ERROR; abs(Time - S*1000000 - Us) < ?US_ERROR ->
+check_trace_info(Mfa, [{Pid, ExpectedC,_,_}] = Expect, Time) ->
+    {call_time,[{Pid,C,S,Us}]} = erlang:trace_info(Mfa, call_time),
+    {Mod, Name, Arity} = Mfa,
+    IsBuiltin = erlang:is_builtin(Mod, Name, Arity),
+    if
+        %% Call count on BIFs may exceed number of calls as they often trap to
+        %% themselves.
+        IsBuiltin, C >= ExpectedC, S >= 0, Us >= 0,
+          abs(1 - Time/(S*1000000 + Us)) < ?R_ERROR;
+          abs(Time - S*1000000 - Us) < ?US_ERROR ->
             ok;
-        {call_time,[{Pid,C,S,Us}]} ->
+        not IsBuiltin, C =:= ExpectedC, S >= 0, Us >= 0,
+          abs(1 - Time/(S*1000000 + Us)) < ?R_ERROR;
+          abs(Time - S*1000000 - Us) < ?US_ERROR ->
+            ok;
+        true ->
             Sum = S*1000000 + Us,
-            io:format("Expected ~p -> {call_time, ~p (Time ~p us)}~n - got ~w s. ~w us. = ~w us. - ~w -> delta ~w (ratio ~.2f, should be 1.0)~n",
-                      [Mfa, Expect, Time, S, Us, Sum, Time, Sum - Time, Time/Sum]),
-            time_error;
-        Other ->
-            io:format("Expected ~p -> {call_time, ~p (Time ~p us)}~n - got ~p~n", [ Mfa, Expect, Time, Other]),
-            time_count_error
+            io:format("Expected ~p -> {call_time, ~p (Time ~p us)}~n - got ~w "
+                      "s. ~w us. = ~w us. - ~w -> delta ~w (ratio ~.2f, "
+                      "should be 1.0)~n",
+                      [Mfa, Expect, Time,
+                       S, Us, Sum, Time, Sum - Time, Time/Sum]),
+            time_error
     end;
 check_trace_info(Mfa, Expect, _) ->
     case erlang:trace_info(Mfa, call_time) of
@@ -646,7 +793,8 @@ setup() ->
     setup([]).
 
 setup(Opts) ->
-    Pid = spawn_link(fun() -> loop() end),
+    Pid = spawn_opt(fun() -> loop() end,
+                   [link, {max_heap_size, 10000}]),
     1 = erlang:trace(Pid, true, [call|Opts]),
     Pid.
 
@@ -670,9 +818,12 @@ loop() ->
         quit ->
             ok;
         {Pid, execute, Fun } when is_function(Fun) ->
+            %% Make sure we always run with the same amount of reductions.
+            erlang:yield(),
             Pid ! {self(), answer, erlang:apply(Fun, [])},
             loop();
         {Pid, execute, {M, F, A}} ->
+            erlang:yield(),
             Pid ! {self(), answer, erlang:apply(M, F, A)},
             loop()
     end.
