@@ -2,7 +2,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1998-2016. All Rights Reserved.
+ * Copyright Ericsson AB 1998-2020. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -38,7 +38,7 @@
  *  register their names with this server.
  *
  *  To be accessible to all Erlang nodes this server listens to a well
- *  known port EPMD_PORT_NO (curently port 4369) where requests
+ *  known port EPMD_PORT_NO (currently port 4369) where requests
  *  for connections can be sent.
  *
  *  To keep track of when registered Erlang nodes are terminated this
@@ -59,7 +59,7 @@
 
 /* We use separate data structures for node names and connections
    so that a request will not use a slot with a name that we
-   want to resuse later incrementing the "creation" */
+   want to reuse later incrementing the "creation" */
 
 
 /* forward declarations */
@@ -198,10 +198,46 @@ static EPMD_INLINE void select_fd_set(EpmdVars* g, int fd)
     }
 }
 
+#ifdef HAVE_SOCKLEN_T
+static const char *epmd_ntop(struct sockaddr_storage *sa, char *buff, socklen_t len) {
+#else
+static const char *epmd_ntop(struct sockaddr_storage *sa, char *buff, size_t len) {
+#endif
+    /* Save errno so that it is not changed by inet_ntop */
+    int myerrno = errno;
+    const char *res;
+#if !defined(EPMD6)
+    if (sa->ss_family == AF_INET6) {
+        struct sockaddr_in6 *addr = (struct sockaddr_in6 *)sa;
+        res = inet_ntop(
+            addr->sin6_family,
+            (void*)&addr->sin6_addr,
+            buff, len);
+    } else {
+        struct sockaddr_in *addr = (struct sockaddr_in *)sa;
+        res = inet_ntop(
+            addr->sin_family, &addr->sin_addr.s_addr,
+            buff, len);
+    }
+#else
+    struct sockaddr_in *addr = (struct sockaddr_in *)sa;
+    res = inet_ntoa(addr->sin_addr);
+    erts_snprintf(buff,len,"%s", res);
+    res = buff;
+#endif
+    errno = myerrno;
+    return res;
+}
+
 void run(EpmdVars *g)
 {
   struct EPMD_SOCKADDR_IN iserv_addr[MAX_LISTEN_SOCKETS];
   int listensock[MAX_LISTEN_SOCKETS];
+#if defined(EPMD6)
+  char socknamebuf[INET6_ADDRSTRLEN];
+#else
+  char socknamebuf[INET_ADDRSTRLEN];
+#endif
   int num_sockets = 0;
   int i;
   int opt;
@@ -410,32 +446,37 @@ void run(EpmdVars *g)
 	 in accept() waiting for the next request. */
 #if (defined(__WIN32__) || defined(NO_FCNTL))
       opt = 1;
-      /* Gives warning in VxWorks */
       if (ioctl(listensock[i], FIONBIO, &opt) != 0)
 #else
       opt = fcntl(listensock[i], F_GETFL, 0);
       if (fcntl(listensock[i], F_SETFL, opt | O_NONBLOCK) == -1)
-#endif /* __WIN32__ || VXWORKS */
-	dbg_perror(g,"failed to set non-blocking mode of listening socket %d",
-		   listensock[i]);
+#endif /* __WIN32__ */
+	dbg_perror(g,"failed to set non-blocking mode of listening socket %d on ipaddr %s",
+		   listensock[i], epmd_ntop(&iserv_addr[num_sockets],
+                                            socknamebuf, sizeof(socknamebuf)));
 
-      if (bind(listensock[i], (struct sockaddr*) &iserv_addr[i], salen) < 0)
+      if (bind(listensock[i], sa, salen) < 0)
 	{
 	  if (errno == EADDRINUSE)
 	    {
-	      dbg_tty_printf(g,1,"there is already a epmd running at port %d",
-			     g->port);
+	      dbg_tty_printf(g,1,"there is already a epmd running at port %d on ipaddr %s",
+			     g->port, epmd_ntop(&iserv_addr[num_sockets],
+                                                socknamebuf, sizeof(socknamebuf)));
 	      epmd_cleanup_exit(g,0);
 	    }
 	  else
 	    {
-	      dbg_perror(g,"failed to bind socket");
-	      epmd_cleanup_exit(g,1);
+              dbg_perror(g,"failed to bind on ipaddr %s",
+                         epmd_ntop(&iserv_addr[num_sockets],
+                                   socknamebuf, sizeof(socknamebuf)));
+              epmd_cleanup_exit(g,1);
 	    }
 	}
 
       if(listen(listensock[i], SOMAXCONN) < 0) {
-          dbg_perror(g,"failed to listen on socket");
+          dbg_perror(g,"failed to listen on ipaddr %s",
+                     epmd_ntop(&iserv_addr[num_sockets],
+                               socknamebuf, sizeof(socknamebuf)));
           epmd_cleanup_exit(g,1);
       }
       select_fd_set(g, listensock[i]);
@@ -525,7 +566,7 @@ void run(EpmdVars *g)
 
 /*
  *  This routine read as much of the packet as possible and
- *  if completed calls "do_request()" to fullfill the request.
+ *  if completed calls "do_request()" to fulfill the request.
  *
  */
 
@@ -665,6 +706,21 @@ static int do_accept(EpmdVars *g,int listensock)
     return conn_open(g,msgsock);
 }
 
+static void bump_creation(Node* node)
+{
+    if (++node->cr_counter < 4)
+        node->cr_counter = 4;
+}
+static unsigned int get_creation(Node* node)
+{
+    if (node->highvsn >= 6) {
+        return node->cr_counter;  /* 4..(2^32-1)*/
+    }
+    else {
+        return node->cr_counter % 3 + 1;   /* 1..3 */
+    }
+}
+
 /* buf is actually one byte larger than bsize,
    giving place for null termination */
 static void do_request(g, fd, s, buf, bsize)
@@ -706,8 +762,10 @@ static void do_request(g, fd, s, buf, bsize)
 	unsigned char protocol;
 	unsigned short highvsn;
 	unsigned short lowvsn;
+        unsigned int creation;
 	int namelen;
 	int extralen;
+        int replylen;
 	char *name; 
 	char *extra;
 	eport = get_int16(&buf[1]);
@@ -737,17 +795,22 @@ static void do_request(g, fd, s, buf, bsize)
 
 	extra = &buf[11+namelen+2];
 	extra[extralen]='\000';
-	wbuf[0] = EPMD_ALIVE2_RESP;
-	if ((node = node_reg2(g, namelen, name, fd, eport, nodetype, protocol,
-			      highvsn, lowvsn, extralen, extra)) == NULL) {
-	    wbuf[1] = 1; /* error */
-	    put_int16(99, wbuf+2);
-	} else {
-	    wbuf[1] = 0; /* ok */
-	    put_int16(node->creation, wbuf+2);
-	}
+        node = node_reg2(g, namelen, name, fd, eport, nodetype, protocol,
+                         highvsn, lowvsn, extralen, extra);
+        creation = node ? get_creation(node) :  99;
+        wbuf[1] = node ? 0 : 1; /* ok | error */
+        if (highvsn >= 6) {
+            wbuf[0] = EPMD_ALIVE2_X_RESP;
+            put_int32(creation, wbuf+2);
+            replylen = 6;
+        }
+        else {
+            wbuf[0] = EPMD_ALIVE2_RESP;
+            put_int16(creation, wbuf+2);
+            replylen = 4;
+        }
   
-	if (!reply(g, fd, wbuf, 4))
+	if (!reply(g, fd, wbuf, replylen))
 	  {
             node_unreg(g, name);
 	    dbg_tty_printf(g,1,"** failed to send ALIVE2_RESP for \"%s\"",
@@ -1033,18 +1096,9 @@ static int conn_open(EpmdVars *g,int fd)
   int i;
   Connection *s;
 
-#ifdef VXWORKS
-  /*
-   * Since file descriptors are global on VxWorks, we might get an fd that
-   * does not fit in the FD_SET.
-   *
-   * Note: This test would be harmless on Unix, but would fail on Windows
-   * because socket are numbered differently and FD_SETs are implemented
-   * differently.
-   */
+#if !defined(__WIN32__)
   if (fd >= FD_SETSIZE) {
-      dbg_tty_printf(g,0,"file descriptor %d: too high for FD_SETSIZE=%d",
-		     fd,FD_SETSIZE);
+      dbg_tty_printf(g,0,"fd does not fit in fd_set fd=%d, FD_SETSIZE=%d",fd, FD_SETSIZE);
       close(fd);
       return EPMD_FALSE;
   }
@@ -1200,8 +1254,8 @@ static int node_unreg(EpmdVars *g,char *name)
   for (; node; prev = &node->next, node = node->next)
     if (is_same_str(node->symname, name))
       {
-	dbg_tty_printf(g,1,"unregistering '%s:%d', port %d",
-		       node->symname, node->creation, node->port);
+	dbg_tty_printf(g,1,"unregistering '%s:%u', port %d",
+		       node->symname, get_creation(node), node->port);
 
 	*prev = node->next;	/* Link out from "reg" list */
 
@@ -1235,8 +1289,8 @@ static int node_unreg_sock(EpmdVars *g,int fd)
   for (; node; prev = &node->next, node = node->next)
     if (node->fd == fd)
       {
-	dbg_tty_printf(g,1,"unregistering '%s:%d', port %d",
-		       node->symname, node->creation, node->port);
+	dbg_tty_printf(g,1,"unregistering '%s:%u', port %d",
+		       node->symname, get_creation(node), node->port);
 
 	*prev = node->next;	/* Link out from "reg" list */
 
@@ -1264,19 +1318,8 @@ static int node_unreg_sock(EpmdVars *g,int fd)
 }
 
 /*
- *  Finding a node slot and a (name,creation) name is a bit tricky.
- *  We try in order
- *
- *  1. If the name was used before and we can reuse that slot but use
- *     a new "creation" digit in the range 1..3.
- *
- *  2. We try to find a new unused slot.
- *
- *  3. We try to use an used slot this isn't used any longer.
- *     FIXME: The criteria for *what* slot to steal should be improved.
- *     Perhaps use the oldest or something.
+ * Register a new node
  */
-
 static Node *node_reg2(EpmdVars *g,
 		       int namelen,
 		       char* name,
@@ -1346,7 +1389,7 @@ static Node *node_reg2(EpmdVars *g,
       }
 
   /* Try to find the name in the used queue so that we
-     can change "creation" number 1..3 */
+     can change "creation" number */
 
   prev = NULL;
 
@@ -1375,9 +1418,8 @@ static Node *node_reg2(EpmdVars *g,
 
 	g->nodes.unreg_count--;
 
-	/* When reusing we change the "creation" number 1..3 */
-
-	node->creation = node->creation % 3 + 1;
+	/* When reusing we change the "creation" number */
+	bump_creation(node);
 
 	break;
       }
@@ -1404,7 +1446,8 @@ static Node *node_reg2(EpmdVars *g,
 	      exit(1);
 	    }
 
-	  node->creation = (current_time(g) % 3) + 1; /* "random" 1-3 */
+	  node->cr_counter = current_time(g); /* "random" */
+          bump_creation(node);
 	}
     }
 
@@ -1423,11 +1466,11 @@ static Node *node_reg2(EpmdVars *g,
   select_fd_set(g, fd);
 
   if (highvsn == 0) {
-    dbg_tty_printf(g,1,"registering '%s:%d', port %d",
-		   node->symname, node->creation, node->port);
+    dbg_tty_printf(g,1,"registering '%s:%u', port %d",
+		   node->symname, get_creation(node), node->port);
   } else {
-    dbg_tty_printf(g,1,"registering '%s:%d', port %d",
-		   node->symname, node->creation, node->port);
+    dbg_tty_printf(g,1,"registering '%s:%u', port %d",
+		   node->symname, get_creation(node), node->port);
     dbg_tty_printf(g,1,"type %d proto %d highvsn %d lowvsn %d",
 		   nodetype, protocol, highvsn, lowvsn);
   }      
@@ -1561,8 +1604,8 @@ static void print_names(EpmdVars *g)
 
   for (node = g->nodes.reg; node; node = node->next)
     {
-      fprintf(stderr,"*****     active name     \"%s#%d\" at port %d, fd = %d\r\n",
-	      node->symname, node->creation, node->port, node->fd);
+      fprintf(stderr,"*****     active name     \"%s#%u\" at port %d, fd = %d\r\n",
+	      node->symname, get_creation(node), node->port, node->fd);
       count ++;
     }
 
@@ -1572,8 +1615,8 @@ static void print_names(EpmdVars *g)
 
   for (node = g->nodes.unreg; node; node = node->next)
     {
-      fprintf(stderr,"*****     old/unused name \"%s#%d\"\r\n",
-	      node->symname, node->creation);
+      fprintf(stderr,"*****     old/unused name \"%s#%u\"\r\n",
+	      node->symname, get_creation(node));
       count ++;
     }
 
