@@ -22,11 +22,6 @@
  * Most of this only used when beam is run as a separate process.
  */
 
-#pragma comment(linker,"/manifestdependency:\"type='win32' "\
-		"name='Microsoft.Windows.Common-Controls' "\
-		"version='6.0.0.0' processorArchitecture='*' "\
-		"publicKeyToken='6595b64144ccf1df' language='*'\"")
-
 #include <windows.h>
 #include <winuser.h>
 #include <wincon.h>
