@@ -106,7 +106,7 @@ static unsigned __stdcall thr_wrapper(LPVOID vtwd)
 
     ethr_set_stacklimit__(&c, twd->stacksize);
 
-    result = (ethr_sint32_t) ethr_make_ts_event__(&tsep);
+    result = (ethr_sint32_t) ethr_make_ts_event__(&tsep, 0);
 
     if (result == 0) {
 	tsep->iflgs |= ETHR_TS_EV_ETHREAD;
@@ -606,6 +606,30 @@ ethr_tsd_get(ethr_tsd_key key)
  * Thread specific events
  */
 
+ethr_ts_event *
+ethr_lookup_ts_event__(int busy_dup)
+{
+    return ethr_lookup_ts_event____(busy_dup);
+}
+
+ethr_ts_event *
+ethr_peek_ts_event(void)
+{
+    return ethr_peek_ts_event__();
+}
+
+void
+ethr_unpeek_ts_event(ethr_ts_event *tsep)
+{
+    ethr_unpeek_ts_event__(tsep);
+}
+
+ethr_ts_event *
+ethr_use_ts_event(ethr_ts_event *tsep)
+{
+    return ethr_use_ts_event__(tsep);
+}
+
 ethr_ts_event *
 ethr_get_ts_event(void)
 {
@@ -618,10 +642,3 @@ ethr_leave_ts_event(ethr_ts_event *tsep)
     ethr_leave_ts_event__(tsep);
 }
 
-ethr_ts_event *
-ethr_create_ts_event__(void)
-{
-    ethr_ts_event *tsep;
-    ethr_make_ts_event__(&tsep);
-    return tsep;
-}
