@@ -41,8 +41,8 @@ suite() ->
 %%--------------------------------------------------------------------
 init_per_suite(Config) ->
     application:start(sasl),
-    Gen = spawn(fun() -> gen() end),
-    [{gen,Gen}|Config].
+    do_log(?FUNCTION_NAME),
+    Config.
 
 %%--------------------------------------------------------------------
 %% @spec end_per_suite(Config0) -> void() | {save_config,Config1}
@@ -50,9 +50,7 @@ init_per_suite(Config) ->
 %% @end
 %%--------------------------------------------------------------------
 end_per_suite(Config) ->
-    Gen = proplists:get_value(gen, Config),
-    exit(Gen, kill),
-    ct:sleep(100),
+    do_log(?FUNCTION_NAME),
     application:stop(sasl),
     ok.
 
@@ -92,8 +90,7 @@ end_per_testcase(_TestCase, _Config) ->
 %% @end
 %%--------------------------------------------------------------------
 groups() ->
-    [{g1,[parallel,{repeat,10}],[tc1,tc2,tc3]},
-     {g2,[{repeat,10}],[tc1,tc2,tc3]}].
+    [{g1,[{repeat,10}],[tc1,tc2,tc3]}].
 
 %%--------------------------------------------------------------------
 %% @spec all() -> GroupsAndTestCases | {skip,Reason}
@@ -104,26 +101,24 @@ groups() ->
 %% @end
 %%--------------------------------------------------------------------
 all() -> 
-    [{group,g1},{group,g2}].
+    [{group,g1}].
 
 tc1(_) ->
-    ct:sleep(100),
+    do_log(?FUNCTION_NAME),
     ok.
 tc2(_) ->
-    ct:sleep(100),
+    do_log(?FUNCTION_NAME),
     ok.
 tc3(_) ->
-    ct:sleep(100),
+    do_log(?FUNCTION_NAME),
     ok.
 
 %%%-----------------------------------------------------------------
 
-gen() ->
-    gen_loop(1).
 
-gen_loop(N) ->
-    ct:log("Logger iteration: ~p", [N]),
-    error_logger:error_report(N),
-    error_logger:info_report(N),
-    ct:sleep(150),
-    gen_loop(N+1).
+do_log(What) ->
+    ct:log("Logger ~p", [What]),
+    error_logger:error_report(What),
+    error_logger:info_report(What),
+    logger:notice("~p",[What]),
+    timer:sleep(100).
