@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2012-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2012-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -47,16 +47,31 @@ function({function,Name,Arity,CLabel,Is0}, NoGetHdTl) ->
 
 undo_renames([{call_ext,2,send}|Is]) ->
     [send|undo_renames(Is)];
+
 undo_renames([{apply,A},{deallocate,N},return|Is]) ->
     [{apply_last,A,N}|undo_renames(Is)];
+
+undo_renames([{call,A,F},{'%',{var_info,{x,0},_}},{deallocate,N},return|Is]) ->
+    %% We've removed a redundant move of a literal to {x,0}.
+    [{call_last,A,F,N} | undo_renames(Is)];
 undo_renames([{call,A,F},{deallocate,N},return|Is]) ->
-    [{call_last,A,F,N}|undo_renames(Is)];
+    [{call_last,A,F,N} | undo_renames(Is)];
+
+undo_renames([{call_ext,A,F},{'%',{var_info,{x,0},_}},{deallocate,N},return|Is]) ->
+    [{call_ext_last,A,F,N} | undo_renames(Is)];
 undo_renames([{call_ext,A,F},{deallocate,N},return|Is]) ->
-    [{call_ext_last,A,F,N}|undo_renames(Is)];
+    [{call_ext_last,A,F,N} | undo_renames(Is)];
+
+undo_renames([{call,A,F},{'%',{var_info,{x,0},_}},return|Is]) ->
+    [{call_only,A,F} | undo_renames(Is)];
 undo_renames([{call,A,F},return|Is]) ->
     [{call_only,A,F}|undo_renames(Is)];
+
+undo_renames([{call_ext,A,F},{'%',{var_info,{x,0},_}},return|Is]) ->
+    [{call_ext_only,A,F} | undo_renames(Is)];
 undo_renames([{call_ext,A,F},return|Is]) ->
     [{call_ext_only,A,F}|undo_renames(Is)];
+
 undo_renames([{bif,raise,_,_,_}=I|Is0]) ->
     %% A minor optimization. Done here because:
     %%  (1) beam_jump may move or share 'raise' instructions, and that
@@ -75,7 +90,8 @@ undo_renames([{bs_put,_,{bs_put_binary,1,_},
                [{atom,all},{literal,<<>>}]}|Is]) ->
     undo_renames(Is);
 undo_renames([{bs_put,Fail,{bs_put_binary,1,_Flags},
-               [{atom,all},{literal,BinString}]}|Is0]) ->
+               [{atom,all},{literal,BinString}]}|Is0])
+  when is_bitstring(BinString)->
     Bits = bit_size(BinString),
     Bytes = Bits div 8,
     case Bits rem 8 of
@@ -118,13 +134,6 @@ undo_rename({bs_init,F,{I,Extra,U,Flags},Live,[Sz,Src],Dst}) ->
     {I,F,Sz,Extra,Live,U,Src,Flags,Dst};
 undo_rename({bs_init,_,bs_init_writable=I,_,_,_}) ->
     I;
-undo_rename({test,bs_match_string=Op,F,[Ctx,Bin0]}) ->
-    Bits = bit_size(Bin0),
-    Bin = case Bits rem 8 of
-	      0 -> Bin0;
-	      Rem -> <<Bin0/bitstring,0:(8-Rem)>>
-	  end,
-    {test,Op,F,[Ctx,Bits,{string,Bin}]};
 undo_rename({put_map,Fail,assoc,S,D,R,L}) ->
     {put_map_assoc,Fail,S,D,R,L};
 undo_rename({put_map,Fail,exact,S,D,R,L}) ->
