@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2005-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2005-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -24,7 +24,7 @@
 -export([all/0, suite/0,groups/0,init_per_suite/1, end_per_suite/1, 
 	 init_per_group/2,end_per_group/2,
 	 init_per_testcase/2,end_per_testcase/2,
-         verify_highest_opcode/1,
+         verify_highest_opcode/1, expand_and_squeeze/1,
 	 size_shadow/1,int_float/1,otp_5269/1,null_fields/1,wiger/1,
 	 bin_tail/1,save_restore/1,
 	 partitioned_bs_match/1,function_clause/1,
@@ -45,7 +45,10 @@
          expression_before_match/1,erl_689/1,restore_on_call/1,
          restore_after_catch/1,matches_on_parameter/1,big_positions/1,
          matching_meets_apply/1,bs_start_match2_defs/1,
-         exceptions_after_match_failure/1, bad_phi_paths/1]).
+         exceptions_after_match_failure/1,
+         bad_phi_paths/1,many_clauses/1,
+         combine_empty_segments/1,hangs_forever/1,
+         bs_saved_position_units/1,empty_get_binary/1]).
 
 -export([coverage_id/1,coverage_external_ignore/2]).
 
@@ -61,10 +64,10 @@ all() ->
     [{group,p}].
 
 groups() -> 
-    [{p,[],
+    [{p,test_lib:parallel(),
       [verify_highest_opcode,
        size_shadow,int_float,otp_5269,null_fields,wiger,
-       bin_tail,save_restore,
+       bin_tail,save_restore,expand_and_squeeze,
        partitioned_bs_match,function_clause,unit,
        shared_sub_bins,bin_and_float,dec_subidentifiers,
        skip_optional_tag,decode_integer,wfbm,degenerated_match,bs_sum,
@@ -82,8 +85,9 @@ groups() ->
        expression_before_match,erl_689,restore_on_call,
        matches_on_parameter,big_positions,
        matching_meets_apply,bs_start_match2_defs,
-       exceptions_after_match_failure,bad_phi_paths]}].
-
+       exceptions_after_match_failure,bad_phi_paths,
+       many_clauses,combine_empty_segments,hangs_forever,
+       bs_saved_position_units,empty_get_binary]}].
 
 init_per_suite(Config) ->
     test_lib:recompile(?MODULE),
@@ -137,22 +141,59 @@ size_shadow(Config) when is_list(Config) ->
 
 size_shadow_1() ->
     L = 8,
-    F = fun(<<L:L,B:L>>) -> B end,
-    F(<<16:8, 7:16>>).
+    Fs = [fun(<<L:L,B:L>>) -> B end,
+          fun(A) ->
+                  (fun([<<L:L,B:L>>]) -> B end)([A])
+          end,
+          fun(A) ->
+                  (fun([<<L:L,B:L>>,<<L:L,B:L>>]) -> B end)([A,A])
+          end,
+          fun(A) ->
+                  <<Size:L,_/bits>> = A,
+                  Inner = fun([L], {#{key1 := <<L:L,B:L>>,
+                                    key2 := <<L:L,B:L>>}, L}) -> B end,
+                  Inner([Size], {#{key1 => A,key2 => A},Size})
+          end],
+    size_shadow_apply(Fs, <<16:8, 7:16>>).
 
 size_shadow_2(L) ->
-    F = fun(<<L:L,B:L>>) -> B end,
-    F(<<16:8, 7:16>>).
+    Fs = [fun(<<L:L,B:L>>) -> B end,
+          fun(A) ->
+                  (fun([<<L:L,B:L>>]) -> B end)([A])
+          end,
+          fun(A) ->
+                  (fun({<<L:L,B:L>>,<<L:L,B:L>>}) -> B end)({A,A})
+          end],
+    size_shadow_apply(Fs, <<16:8, 7:16>>).
 
 size_shadow_3() ->
     L = 8,
-    F = fun(<<L:L,B:L,L:L>>) -> B end,
-    F(<<16:8, 7:16,16:16>>).
+    Fs = [fun(<<L:L,B:L,L:L>>) -> B end,
+          fun(A) ->
+                  (fun({tag,[<<L:L,B:L,L:L>>]}) -> B end)({tag,[A]})
+          end,
+          fun(A) ->
+                  (fun({tag,<<L:L,B:L,L:L>>,<<L:L,B:L,L:L>>}) -> B end)({tag,A,A})
+          end],
+    size_shadow_apply(Fs, <<16:8, 7:16,16:16>>).
 
 size_shadow_4(L) ->
-    F = fun(<<L:L,B:L,L:L>>) -> B;
-	   (_) -> no end,
-    F(<<16:8, 7:16,15:16>>).
+    Fs = [fun(<<L:L,B:L,L:L>>) -> B;
+             (_) -> no
+          end,
+          fun(A) ->
+                  Inner = fun([<<L:L,B:L,L:L>>]) -> B;
+                             (_) -> no
+                          end,
+                  Inner([A])
+          end,
+          fun(A) ->
+                  Inner = fun({<<L:L,B:L,L:L>>,<<L:L,B:L,L:L>>}) -> B;
+                             (_) -> no
+                          end,
+                  Inner({A,A})
+          end],
+    size_shadow_apply(Fs, <<16:8, 7:16,15:16>>).
 
 size_shadow_5(X, Y) ->
     fun (<< A:Y >>, Y, B) -> fum(A, X, Y, B) end.
@@ -166,6 +207,14 @@ fum(A, B, C, D) ->
 size_shadow_7({int,N}, <<N:16,B:N/binary,T/binary>>) ->
     {B,T}.
 
+size_shadow_apply([F|Fs], Arg) when is_function(F, 1) ->
+    size_shadow_apply(Fs, Arg, F(Arg)).
+
+size_shadow_apply([F|Fs], Arg, Res) when is_function(F, 1) ->
+    Res = F(Arg),
+    size_shadow_apply(Fs, Arg, Res);
+size_shadow_apply([], _, Res) ->
+    Res.
 
 int_float(Config) when is_list(Config) ->
     %% OTP-5323
@@ -502,6 +551,9 @@ unit(Config) when is_list(Config) ->
     {'EXIT',_} = (catch unit_opt_2(<<1:32,33:7>>)),
     {'EXIT',_} = (catch unit_opt_2(<<2:32,55:7>>)),
 
+    <<0:64>> = unit_opt_3(<<1:128>>),
+    <<1:64>> = unit_opt_3(<<1:64>>),
+
     ok.
 
 peek1(<<B:8,_/bitstring>>) -> B.
@@ -533,6 +585,13 @@ unit_opt_2(<<St:32,KO/binary>> = Bin0) ->
           end,
     id(Bin).
 
+unit_opt_3(A) when is_binary(A) ->
+    %% There should be no test_unit instruction after the first segment, since
+    %% we already know A is a binary and its tail will still be a binary after
+    %% matching 8 bytes from it.
+    <<Bin:8/binary, _/binary>> = A,
+    Bin.
+
 shared_sub_bins(Config) when is_list(Config) ->
     {15,[<<>>,<<5>>,<<4,5>>,<<3,4,5>>,<<2,3,4,5>>]} = sum(<<1,2,3,4,5>>, [], 0),
     ok.
@@ -774,6 +833,8 @@ coverage(Config) when is_list(Config) ->
     {10,<<"-">>,""} = coverage_trim_2(<<"-">>, 10, []),
     {8,<<"-">>,"aa"} = coverage_trim_2(<<"aa-">>, 10, []),
 
+    {<<"abc">>,<<"tag">>} = coverage_trim_3([<<"abc","tag">>], 3),
+
     ok.
 
 coverage_fold(Fun, Acc, <<H,T/binary>>) ->
@@ -896,6 +957,12 @@ coverage_trim_2(<<C/utf8,R/binary>> = Bin, I, L) ->
             {I,Bin,lists:reverse(L)}
     end.
 
+coverage_trim_3(CipherTextFragment, TagLen) ->
+    CipherLen = iolist_size(CipherTextFragment) - TagLen,
+    <<CipherText:CipherLen/bytes, CipherTag:TagLen/bytes>> =
+        iolist_to_binary(CipherTextFragment),
+    {CipherText, CipherTag}.
+
 printable_char($a) -> true;
 printable_char(_) -> false.
 
@@ -1250,17 +1317,114 @@ zero_width(Config) when is_list(Config) ->
 	<<256:8>> -> ct:fail(should_not_match);
 	_ -> ok
     end,
+
+    %% Would crash in the segment squeezing functions in v3_kernel.
+    F = fun (<<42>>) -> star;
+            (<<V:0>>) -> V;
+            (_) -> no_match
+        end,
+    star = F(<<42>>),
+    0 = F(<<>>),
+    no_match = F(<<1>>),
+    no_match = F(whatever),
+
     ok.
 
 
 %% OTP_7650: A invalid size for binary segments could crash the compiler.
 bad_size(Config) when is_list(Config) ->
     Tuple = {a,b,c},
-    {'EXIT',{{badmatch,<<>>},_}} = (catch <<32:Tuple>> = id(<<>>)),
     Binary = <<1,2,3>>,
+    Atom = an_atom,
+    NaN = <<(-1):32>>,
+
+    {'EXIT',{{badmatch,<<>>},_}} = (catch <<32:Tuple>> = id(<<>>)),
     {'EXIT',{{badmatch,<<>>},_}} = (catch <<32:Binary>> = id(<<>>)),
+    {'EXIT',{{badmatch,<<>>},_}} = (catch <<32:Atom>> = id(<<>>)),
+    {'EXIT',{{badmatch,<<>>},_}} = (catch <<32:3.14>> = id(<<>>)),
+    {'EXIT',{{badmatch,<<>>},_}} = (catch <<32:"ZJV">> = id(<<>>)),
+    {'EXIT',{{badmatch,<<>>},_}} = (catch <<32:(-1)>> = id(<<>>)),
+
+    {'EXIT',{{badmatch,<<>>},_}} = (catch <<42.0:Tuple/float>> = id(<<>>)),
+    {'EXIT',{{badmatch,<<>>},_}} = (catch <<42.0:Binary/float>> = id(<<>>)),
+    {'EXIT',{{badmatch,<<>>},_}} = (catch <<42.0:Atom/float>> = id(<<>>)),
+    {'EXIT',{{badmatch,<<>>},_}} = (catch <<42.0:2.5/float>> = id(<<>>)),
+    {'EXIT',{{badmatch,<<>>},_}} = (catch <<42.0:1/float>> = id(<<>>)),
+    {'EXIT',{{badmatch,NaN},_}} = (catch <<42.0:32/float>> = id(NaN)),
+
+    %% Matched out value is ignored.
+    {'EXIT',{{badmatch,<<>>},_}} = (catch <<_:Binary>> = id(<<>>)),
+    {'EXIT',{{badmatch,<<>>},_}} = (catch <<_:Tuple>> = id(<<>>)),
+    {'EXIT',{{badmatch,<<>>},_}} = (catch <<_:Atom>> = id(<<>>)),
+    {'EXIT',{{badmatch,<<>>},_}} = (catch <<_:2.5>> = id(<<>>)),
+    {'EXIT',{{badmatch,<<1:1>>},_}} = (catch <<_:1/float>> = id(<<1:1>>)),
+    {'EXIT',{{badmatch,NaN},_}} = (catch <<_:32/float>> = id(NaN)),
+
+    no_match = bad_all_size(<<>>),
+    no_match = bad_all_size(<<1,2,3>>),
+
+    true = bad_size_1(<<0>>),
+    error = bad_size_1(<<0,1>>),
+
     ok.
 
+bad_all_size(Bin) ->
+    Res = bad_all_size_1(Bin),
+    Res = bad_all_size_2(Bin),
+    Res = bad_all_size_3(Bin),
+    Res = bad_all_size_4(Bin),
+    Res = bad_all_size_5(Bin),
+    Res = bad_all_size_6(Bin),
+    Res.
+
+bad_all_size_1(Bin) ->
+    case Bin of
+        <<B:all/binary>> -> B;
+        _ -> no_match
+    end.
+
+bad_all_size_2(Bin) ->
+    case Bin of
+        <<_:all/binary>> -> ok;
+        _ -> no_match
+    end.
+
+bad_all_size_3(Bin) ->
+    All = all,
+    case Bin of
+        <<B:All/binary>> -> B;
+        _ -> no_match
+    end.
+
+bad_all_size_4(Bin) ->
+    All = all,
+    case Bin of
+        <<_:All/binary>> -> ok;
+        _ -> no_match
+    end.
+
+bad_all_size_5(Bin) ->
+    All = case 0 of
+              0 -> all
+          end,
+    case Bin of
+        <<B:All/binary>> -> B;
+        _ -> no_match
+    end.
+
+bad_all_size_6(Bin) ->
+    All = case 0 of
+              0 -> all
+          end,
+    case Bin of
+        <<_:All/binary>> -> ok;
+        _ -> no_match
+    end.
+
+bad_size_1(<<0>>) -> true;
+bad_size_1(<<0:[]>>) -> false;
+bad_size_1(_) -> error.
+
 haystack(Config) when is_list(Config) ->
     <<0:10/unit:8>> = haystack_1(<<0:10/unit:8>>),
     [<<0:10/unit:8>>,
@@ -1324,23 +1488,182 @@ matched_out_size(Config) when is_list(Config) ->
     {<<1,2,3,7>>,19,42} = mos_bin(<<4,1,2,3,7,19,4,42>>),
     <<1,2,3,7>> = mos_bin(<<4,1,2,3,7,"abcdefghij">>),
 
+    false = mos_verify_sig(not_a_binary),
+    false = mos_verify_sig(<<>>),
+    false = mos_verify_sig(<<42:32>>),
+    <<"123456789">> = mos_verify_sig(<<77:32,0:77/unit:8,9:32,"123456789">>),
+
     ok.
 
-mos_int(<<L,I:L,X:32>>) ->
+mos_int(B) ->
+    Res = mos_int_plain(B),
+    Res = mos_int_list([B]),
+    Res = mos_int_tuple({a,[B],z}),
+
+    Res = mos_int_mixed([B]),
+    Res = mos_int_mixed({a,[B],z}),
+    42 = mos_int_mixed({30,12}),
+    no_match = mos_int_mixed([B,B,B]),
+
+    Res = mos_int_pats1({tag,[B]}, {0,1,2,3,4,5,6,7,8,9}),
+    Res = mos_int_pats2({tag,[B]}, {a,a,a,a,a,a,a,a,a,a}, [z]),
+    {I,X} = Res,
+    Res = mos_int_pats3({tag,[B]}, [I,{X,B,X},I]),
+    Res = mos_int_map(#{key => [B]}),
+    Key = {my,key},
+    Res = mos_int_map(Key, #{Key => [B]}),
+    {I,X,B} = mos_int_alias([[B]]),
+    Res = {I,X},
+    Res = mos_int_try([B]),
+    Res = mos_int_receive(B),
+    Res = mos_int_fun([B]),
+    Res = mos_int_exported(B),
+    Res = mos_int_utf(B),
+    Res.
+
+mos_int_plain(<<L,I:L,X:32>>) ->
     {I,X};
-mos_int(<<L,I:L,X:64>>) ->
+mos_int_plain(<<L,I:L,X:64>>) ->
     {I,X}.
 
-mos_bin(<<L,Bin:L/binary,X:8,L>>) ->
+mos_int_list([<<L,I:L,X:32>>]) ->
+    {I,X};
+mos_int_list([<<L,I:L,X:64>>]) ->
+    {I,X}.
+
+mos_int_tuple({a,[<<L,I:L,X:32>>],z}) ->
+    {I,X};
+mos_int_tuple({a,[<<L,I:L,X:64>>],z}) ->
+    {I,X}.
+
+mos_int_mixed({a,[<<L,I:L,X:32>>],z}) ->
+    {I,X};
+mos_int_mixed({a,[<<L,I:L,X:64>>],z}) ->
+    {I,X};
+mos_int_mixed([<<L,I:L,X:32>>]) ->
+    {I,X};
+mos_int_mixed([<<L,I:L,X:64>>]) ->
+    {I,X};
+mos_int_mixed({A,B}) when is_integer(A), is_integer(B) ->
+    A + B;
+mos_int_mixed(_) ->
+    no_match.
+
+mos_int_pats1({tag,[<<L,I:L,X:32>>]}, {_,_,_,_,_,_,_,_,_,_}) ->
+    {I,X};
+mos_int_pats1({tag,[<<L,I:L,X:64>>]}, {_,_,_,_,_,_,_,_,_,_}) ->
+    {I,X}.
+
+mos_int_pats2({tag,[<<L,I:L,X:32>>]}, {S,S,S,S,S,S,S,S,S,S}, [_|_]) ->
+    {I,X};
+mos_int_pats2({tag,[<<L,I:L,X:64>>]}, {S,S,S,S,S,S,S,S,S,S}, [_|_]) ->
+    {I,X}.
+
+mos_int_pats3({tag,[<<L,I:L,X:32>>]}, [I,{X,<<L,I:L,X:32>>,X},I]) ->
+    {I,X};
+mos_int_pats3({tag,[<<L,I:L,X:64>>]}, [I,{X,<<L,I:L,X:64>>,X},I]) ->
+    {I,X}.
+
+mos_int_map(#{key := [<<L,I:L,X:32>>]}) ->
+    {I,X};
+mos_int_map(#{key := [<<L,I:L,X:64>>]}) ->
+    {I,X}.
+
+mos_int_map(Key, Map) ->
+    case Map of
+        #{Key := [<<L,I:L,X:32>>]} -> {I,X};
+        #{Key := [<<L,I:L,X:64>>]} -> {I,X}
+    end.
+
+mos_int_alias([[<<L,I:L,X:32>> = B]]) ->
+    {I,X,B};
+mos_int_alias([[<<L,I:L,X:64>> = B]]) ->
+    {I,X,B}.
+
+mos_int_try(B) ->
+    try id(B) of
+        [<<L,I:L,X:32>>] -> {I,X};
+        [<<L,I:L,X:64>>] -> {I,X}
+    after
+        ok
+    end.
+
+mos_int_receive(Msg) ->
+    Res = (fun() ->
+                  self() ! Msg,
+                  receive
+                      <<L,I:L,X:32>> -> {I,X};
+                      <<L,I:L,X:64>> -> {I,X}
+                  end
+           end)(),
+    self() ! Msg,
+    Res = receive
+              <<L,I:L,X:32>> -> {I,X};
+              <<L,I:L,X:64>> -> {I,X}
+          end,
+    self() ! {tag,[Msg]},
+    Res = receive
+              {tag,[<<L,I:L,X:32>>]} -> {I,X};
+              {tag,[<<L,I:L,X:64>>]} -> {I,X}
+          end,
+    Res.
+
+mos_int_fun(B) ->
+    L = ignore_me,
+    F = fun ([<<L,I:L,X:32>>]) -> {I,X};
+            ([<<L,I:L,X:64>>]) -> {I,X}
+        end,
+    F(B).
+
+mos_int_exported(B) ->
+    case B of
+        <<L,I:L,X:32>> -> ok;
+        <<L,I:L,X:64>> -> ok
+    end,
+    {I,X}.
+
+mos_int_utf(B0) ->
+    B = id(<<B0/bits,777/utf8,7777/utf16,9999/utf32>>),
+    case B of
+        <<L,I:L,X:32,777/utf8,7777/utf16,9999/utf32>> -> {I,X};
+        <<L,I:L,X:64,777/utf8,7777/utf16,9999/utf32>> -> {I,X}
+    end.
+
+mos_bin(B) ->
+    Res = mos_bin_plain(B),
+    Res = mos_bin_tuple({outer,{inner,B}}),
+    Res.
+
+mos_bin_plain(<<L,Bin:L/binary,X:8,L>>) ->
     L = byte_size(Bin),
     {Bin,X};
-mos_bin(<<L,Bin:L/binary,X:8,L,Y:8>>) ->
+mos_bin_plain(<<L,Bin:L/binary,X:8,L,Y:8>>) ->
     L = byte_size(Bin),
     {Bin,X,Y};
-mos_bin(<<L,Bin:L/binary,"abcdefghij">>) ->
+mos_bin_plain(<<L,Bin:L/binary,"abcdefghij">>) ->
     L = byte_size(Bin),
     Bin.
 
+mos_bin_tuple({outer,{inner,<<L,Bin:L/binary,X:8,L>>}}) ->
+    L = byte_size(Bin),
+    {Bin,X};
+mos_bin_tuple({outer,{inner,<<L,Bin:L/binary,X:8,L,Y:8>>}}) ->
+    L = byte_size(Bin),
+    {Bin,X,Y};
+mos_bin_tuple({outer,{inner,<<L,Bin:L/binary,"abcdefghij">>}}) ->
+    L = byte_size(Bin),
+    Bin.
+
+mos_verify_sig(AlgSig) ->
+    try
+        <<AlgLen:32, _Alg:AlgLen/binary,
+          SigLen:32, Sig:SigLen/binary>> = AlgSig,
+        Sig
+    catch
+        _:_ ->
+            false
+    end.
+
 follow_fail_branch(_) ->
     42 = ffb_1(<<0,1>>, <<0>>),
     8 = ffb_1(<<0,1>>, [a]),
@@ -1526,6 +1849,7 @@ bad_literals(_Config) ->
     Mod:f(),
 
     {'EXIT',<<42>>} = (catch bad_literals_1()),
+    no_match = bad_literals_2(<<"abc">>),
 
     Sz = id(8),
     {'EXIT',{{badmatch,_},_}} = (catch <<-1:Sz>> = <<-1>>),
@@ -1541,6 +1865,13 @@ bad_literals_1() ->
 	error -> error
     end.
 
+bad_literals_2(<<atom:16>>) ->
+    fail;
+bad_literals_2(<<2.5:16>>) ->
+    fail;
+bad_literals_2(_) ->
+    no_match.
+
 signed_lit_match(V, Sz) ->
     case <<V:Sz>> of
 	<<V:Sz/signed>> ->
@@ -2033,4 +2364,243 @@ bad_phi_paths_1(Arg) ->
         end,
     id(B).
 
+combine_empty_segments(_Config) ->
+    <<0,1,2,3>> = combine_empty_segments_1(<<0,1,2,3>>),
+    ok.
+
+combine_empty_segments_1(A) ->
+    <<B/bits>> = A,
+    <<C/bits>> = B,
+    <<D/bits>> = C,
+    D.
+
+%% This never finishes compiling under +no_copt.
+hangs_forever(Config) ->
+    true = is_function(id(fun() -> hangs_forever_1(Config) end)),
+    ok.
+
+hangs_forever_1(V0) ->
+    case hangs_forever_1(V0) of
+        <<A:1>> -> A
+    end.
+
+
+%% ERL-1340: the unit of previously saved match positions wasn't updated.
+bs_saved_position_units(Config) when is_list(Config) ->
+    [<<0,1,2,3,4>>, <<5,6,7,8,9>>] = bspu_1(id(<<0,1,2,3,4,5,6,7,8,9>>)),
+    [] = bspu_1(id(<<>>)),
+
+    ok.
+
+bspu_1(<<Bin/binary>> = Bin) ->
+    [Chunk || <<Chunk:5/binary>> <= Bin].
+
+
+empty_get_binary(Config) when is_list(Config) ->
+    {<<>>, <<1,2,3,4:4>>} = egb_1(<<1,2,3,4:4>>),
+    {<<>>, <<1,2,3>>} = egb_1(<<1,2,3>>),
+    {<<>>, <<>>} = egb_1(<<>>),
+
+    <<0,1,0,2,0,3>> = egb_2(id(<<1,2,3>>)),
+    <<>> = egb_2(id(<<>>)),
+
+    ok.
+
+egb_1(Bytes) ->
+    {Term, Bytes} = begin
+                        <<V2@V0:0/binary-unit:8,V2@Buf1/bitstring>> = Bytes,
+                        V2@Conv2 = binary:copy(V2@V0),
+                        {V2@Conv2, V2@Buf1}
+                    end,
+    {Term, Bytes}.
+
+egb_2(Bin) ->
+    <<
+      <<K,N>> || <<K:0,N>> <= Bin
+    >>.
+
 id(I) -> I.
+
+expand_and_squeeze(Config) when is_list(Config) ->
+    %% UTF8 literals are expanded and then squeezed into integer16
+    [
+	{test,bs_get_integer2,_,_,[_,{integer,16}|_],_}
+	| _
+    ] = binary_match_to_asm([
+	?Q("<<$á/utf8,_/binary>>"),
+	?Q("<<$é/utf8,_/binary>>")
+    ]),
+
+    %% Sized integers are expanded and then squeezed into integer16
+    [
+	{test,bs_get_integer2,_,_,[_,{integer,16}|_],_}
+	| _
+    ] = binary_match_to_asm([
+	?Q("<<0:32,_/binary>>"),
+	?Q("<<\"bbbb\",_/binary>>")
+    ]),
+
+    %% Groups of 8 bits are squeezed into integer16
+    [
+	{test,bs_get_integer2,_,_,[_,{integer,16}|_],_}
+	| _
+    ] = binary_match_to_asm([
+	?Q("<<\"aaaa\",_/binary>>"),
+	?Q("<<\"bbbb\",_/binary>>")
+    ]),
+
+    %% Groups of 8 bits with empty binary are also squeezed
+    [
+	{test,bs_get_integer2,_,_,[_,{integer,16}|_],_}
+	| _
+    ] = binary_match_to_asm([
+	?Q("<<\"aaaa\",_/binary>>"),
+	?Q("<<\"bbbb\",_/binary>>"),
+	?Q("<<>>")
+    ]),
+
+    %% Groups of 8 bits with float lookup are not squeezed
+    [
+	{test,bs_get_integer2,_,_,[_,{integer,8}|_],_}
+	| _
+    ] = binary_match_to_asm([
+	?Q("<<\"aaaa\",_/binary>>"),
+	?Q("<<\"bbbb\",_/binary>>"),
+	?Q("<<_/float>>")
+    ]),
+
+    %% Groups of diverse bits go with minimum possible
+    [
+	{test,bs_get_integer2,_,_,[_,{integer,8}|_],_}
+	| _
+    ] = binary_match_to_asm([
+	?Q("<<\"aa\",_/binary>>"),
+	?Q("<<\"bb\",_/binary>>"),
+	?Q("<<\"c\",_/binary>>")
+    ]),
+
+    %% Groups of diverse bits go with minimum possible but are recursive...
+    [
+	{test,bs_get_integer2,_,_,[_,{integer,8}|_],_}
+	| RestDiverse
+    ] = binary_match_to_asm([
+	?Q("<<\"aaa\",_/binary>>"),
+	?Q("<<\"abb\",_/binary>>"),
+	?Q("<<\"c\",_/binary>>")
+    ]),
+
+    %% so we still perform a 16 bits lookup for the remaining
+    true = lists:any(fun({test,bs_get_integer2,_,_,[_,{integer,16}|_],_}) -> true;
+			(_) -> false end, RestDiverse),
+
+    %% Large match is kept as is if there is a sized match later
+    [
+	{test,bs_get_integer2,_,_,[_,{integer,64}|_],_}
+	| _
+    ] = binary_match_to_asm([
+	?Q("<<255,255,255,255,255,255,255,255>>"),
+	?Q("<<_:64>>")
+    ]),
+
+    %% Large match is kept as is with large matches before and after
+    [
+	{test,bs_get_integer2,_,_,[_,{integer,32}|_],_}
+	| _
+    ] = binary_match_to_asm([
+	?Q("<<A:32,_:A>>"),
+	?Q("<<0:32>>"),
+	?Q("<<_:32>>")
+    ]),
+
+    %% Large match is kept as is with large matches before and after
+    [
+	{test,bs_get_integer2,_,_,[_,{integer,32}|_],_}
+	| _
+    ] = binary_match_to_asm([
+	?Q("<<A:32,_:A>>"),
+	?Q("<<0,0,0,0>>"),
+	?Q("<<_:32>>")
+    ]),
+
+    %% Large match is kept as is with smaller but still large matches before and after
+    [
+	{test,bs_get_integer2,_,_,[_,{integer,32}|_],_}
+	| _
+    ] = binary_match_to_asm([
+	?Q("<<A:32, _:A>>"),
+	?Q("<<0:64>>"),
+	?Q("<<_:32>>")
+    ]),
+
+    %% There is no squeezing for groups with more than 16 matches
+    [
+	{test,bs_get_integer2,_,_,[_,{integer,8}|_],_}
+	| _
+    ] = binary_match_to_asm([
+	?Q("<<\"aa\", _/binary>>"),
+	?Q("<<\"bb\", _/binary>>"),
+	?Q("<<\"cc\", _/binary>>"),
+	?Q("<<\"dd\", _/binary>>"),
+	?Q("<<\"ee\", _/binary>>"),
+	?Q("<<\"ff\", _/binary>>"),
+	?Q("<<\"gg\", _/binary>>"),
+	?Q("<<\"hh\", _/binary>>"),
+	?Q("<<\"ii\", _/binary>>"),
+	?Q("<<\"jj\", _/binary>>"),
+	?Q("<<\"kk\", _/binary>>"),
+	?Q("<<\"ll\", _/binary>>"),
+	?Q("<<\"mm\", _/binary>>"),
+	?Q("<<\"nn\", _/binary>>"),
+	?Q("<<\"oo\", _/binary>>"),
+	?Q("<<\"pp\", _/binary>>")
+    ]),
+
+    ok.
+
+binary_match_to_asm(Matches) ->
+    Clauses = [
+	begin
+	    Ann = element(2, Match),
+	    {clause,Ann,[Match],[],[{integer,Ann,Return}]}
+	end || {Match,Return} <- lists:zip(Matches, lists:seq(1, length(Matches)))
+    ],
+
+    Module = [
+	{attribute,erl_anno:new(1),module,match_to_asm},
+	{attribute,erl_anno:new(2),export,[{example,1}]},
+	{function,erl_anno:new(3),example,1,Clauses}
+    ],
+
+    {ok,match_to_asm,{match_to_asm,_Exports,_Attrs,Funs,_},_} =
+	compile:forms(Module, [return, to_asm]),
+
+    [{function,example,1,2,AllInstructions}|_] = Funs,
+    [{label,_},{line,_},{func_info,_,_,_},{label,_},{'%',_},
+     {test,bs_start_match3,_,_,_,_},{bs_get_position,_,_,_}|Instructions] = AllInstructions,
+    Instructions.
+
+many_clauses(_Config) ->
+    Mod = list_to_atom(?MODULE_STRING ++ "_" ++
+			   atom_to_list(?FUNCTION_NAME)),
+    Seq = lists:seq(1, 200),
+    S = [one_clause(I) || I <- Seq],
+    Code = ?Q(["-module('@Mod@').\n"
+	       "-export([f/1]).\n"
+	       "f(Bin) ->\n"
+	       "case Bin of\n"
+               "  dummy -> _@_@S\n"
+               "end.\n"]),
+    %% merl:print(Code),
+    Opts = test_lib:opt_opts(?MODULE),
+    {ok,_} = merl:compile_and_load(Code, Opts),
+    _ = [begin
+             H = erlang:phash2(I),
+             Sz = 16,
+             <<Res0:Sz>> = <<H:Sz>>,
+             Res = I + Res0,
+             Res = Mod:f({I,<<Sz:8,H:Sz>>})
+         end || I <- Seq],
+    ok.
+
+one_clause(I) ->
+    ?Q(<<"{_@I@,<<L:8,Val:L>>} -> _@I@ + Val">>).
