@@ -2300,8 +2300,9 @@ beam_bool_SUITE(_Config) ->
     in_catch(),
     recv_semi(),
     andalso_repeated_var(),
-    erl1253(),
     erl1246(),
+    erl1253(),
+    erl1384(),
     ok.
 
 before_and_inside_if() ->
@@ -2763,6 +2764,18 @@ erl1253_andalso_true_3(X, Y, Z) ->
         true -> error
     end.
 
+erl1384() ->
+    gurka = erl1384_1(id(a)),
+    gaffel = erl1384_1(id(b)),
+    ok.
+
+erl1384_1(V) ->
+    case {id(false), V =/= a} of
+        {true, true} -> not_reachable;
+        {_, false} -> gurka;
+        _ -> gaffel
+    end.
+
 %%%
 %%% End of beam_bool_SUITE tests.
 %%%
