@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2006-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2006-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -164,6 +164,11 @@ md5_1(Beam) ->
 %% Cover some code that handles internal errors.
 
 silly_coverage(Config) when is_list(Config) ->
+    %% v3_core
+    BadAbstr = [{attribute,0,module,bad_module},
+                {function,0,foo,2,[bad_clauses]}],
+    expect_error(fun() -> v3_core:module(BadAbstr, []) end),
+
     %% sys_core_fold, sys_core_alias, sys_core_bsm, v3_kernel
     BadCoreErlang = {c_module,[],
 		     name,[],[],
@@ -184,6 +189,7 @@ silly_coverage(Config) when is_list(Config) ->
     expect_error(fun() -> beam_kernel_to_ssa:module(BadKernel, []) end),
 
     %% beam_ssa_lint
+    %% beam_ssa_bool
     %% beam_ssa_recv
     %% beam_ssa_share
     %% beam_ssa_pre_codegen
@@ -191,6 +197,7 @@ silly_coverage(Config) when is_list(Config) ->
     BadSSA = {b_module,#{},a,b,c,
               [{b_function,#{func_info=>{mod,foo,0}},args,bad_blocks,0}]},
     expect_error(fun() -> beam_ssa_lint:module(BadSSA, []) end),
+    expect_error(fun() -> beam_ssa_bool:module(BadSSA, []) end),
     expect_error(fun() -> beam_ssa_recv:module(BadSSA, []) end),
     expect_error(fun() -> beam_ssa_share:module(BadSSA, []) end),
     expect_error(fun() -> beam_ssa_pre_codegen:module(BadSSA, []) end),
@@ -198,7 +205,7 @@ silly_coverage(Config) when is_list(Config) ->
 
     %% beam_ssa_opt
     BadSSABlocks = #{0 => {b_blk,#{},[bad_code],{b_ret,#{},arg}}},
-    BadSSAOpt = {b_module,#{},a,[],c,
+    BadSSAOpt = {b_module,#{},a,[],[],
                  [{b_function,#{func_info=>{mod,foo,0}},[],
                    BadSSABlocks,0}]},
     expect_error(fun() -> beam_ssa_opt:module(BadSSAOpt, []) end),
@@ -230,15 +237,6 @@ silly_coverage(Config) when is_list(Config) ->
 		     {label,2}|non_proper_list]}],99},
     expect_error(fun() -> beam_block:module(BlockInput, []) end),
 
-    %% beam_except
-    ExceptInput = {?MODULE,[{foo,0}],[],
-		   [{function,foo,0,2,
-		     [{label,1},
-		      {line,loc},
-		      {func_info,{atom,?MODULE},{atom,foo},0},
-		      {label,2}|non_proper_list]}],99},
-    expect_error(fun() -> beam_except:module(ExceptInput, []) end),
-
     %% beam_jump
     JumpInput = BlockInput,
     expect_error(fun() -> beam_jump:module(JumpInput, []) end),
@@ -280,19 +278,39 @@ silly_coverage(Config) when is_list(Config) ->
 		      [{label,1},
 		       {func_info,{atom,?MODULE},{atom,foo},0},
 		       {label,2}|non_proper_list]}],99},
-    expect_error(fun() -> beam_validator:module(BeamValInput, []) end),
+    expect_error(fun() -> beam_validator:validate(BeamValInput, strong) end),
 
     ok.
 
 bad_ssa_lint_input() ->
     {b_module,#{},t,
-     [{foobar,1},{module_info,0},{module_info,1}],
+     [{a,1},{b,1},{c,1},{module_info,0},{module_info,1}],
      [],
      [{b_function,
-       #{func_info => {t,foobar,1},location => {"t.erl",4}},
+       #{func_info => {t,a,1},location => {"t.erl",4}},
        [{b_var,0}],
        #{0 => {b_blk,#{},[],{b_ret,#{},{b_var,'@undefined_var'}}}},
        3},
+      {b_function,
+       #{func_info => {t,b,1},location => {"t.erl",5}},
+       [{b_var,0}],
+       #{0 =>
+             {b_blk,#{},
+              [{b_set,#{},{b_var,'@first_var'},first_op,[]},
+               {b_set,#{},{b_var,'@second_var'},second_op,[]},
+               {b_set,#{},{b_var,'@ret'},succeeded,[{b_var,'@first_var'}]}],
+              {b_ret,#{},{b_var,'@ret'}}}},
+       3},
+      {b_function,
+       #{func_info => {t,c,1},location => {"t.erl",6}},
+       [{b_var,0}],
+       #{0 =>
+             {b_blk,#{},
+              [{b_set,#{},{b_var,'@first_var'},first_op,[]},
+               {b_set,#{},{b_var,'@ret'},succeeded,[{b_var,'@first_var'}]},
+               {b_set,#{},{b_var,'@second_var'},second_op,[]}],
+              {b_ret,#{},{b_var,'@ret'}}}},
+       3},
       {b_function,
        #{func_info => {t,module_info,0}},
        [],
