@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2003-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2003-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -589,6 +589,7 @@ nested_access(Config) when is_list(Config) ->
     ok.
 
 -record(rr, {a,b,c}).
+-record(fileheader, {read_md5,md5,eof,trailer}).
 
 coverage(Config) when is_list(Config) ->
     %% There should only remain one record test in the code below.
@@ -600,8 +601,23 @@ coverage(Config) when is_list(Config) ->
 	    ok
     end,
     #rr{a=1,b=2,c=42} = id(R),			%Test for correctness.
+
+    %% Cover beam_ssa_opt:ssa_opt_element/1 and friends.
+    error1 = check_file_header(#fileheader{read_md5=1,md5=2}),
+    error2 = check_file_header(#fileheader{trailer=true,eof=false}),
+    error3 = check_file_header(#fileheader{}),
+
     ok.
 
+check_file_header(FH) ->
+    if
+        FH#fileheader.read_md5 =/= FH#fileheader.md5 ->
+            error1;
+        FH#fileheader.trailer =/= FH#fileheader.eof ->
+            error2;
+        true ->
+            error3
+    end.
 
 -record(default_fun, {a = fun(X) -> X*X end}).
 
@@ -609,8 +625,9 @@ coverage(Config) when is_list(Config) ->
 -record(gb_nil, {}).
 -record(gb_foo, {hello=1}).
 -record(gb_bar, {hello=2,there=3}).
+-record(gb_rh, {mod,mid}).
 
-%% Taken from compilation_SUITE.
+%% Taken from compilation_SUITE and other places.
 grab_bag(_Config) ->
     T1 = fun() ->
 		 X = #foo{},
@@ -654,6 +671,23 @@ grab_bag(_Config) ->
 	 end,
     T4(),
 
+    %% Used to crash beam_ssa_bool during its development.
+    T5 = fun(RH) ->
+                 if
+                     is_record(RH, gb_rh) andalso
+                     is_atom(RH#gb_rh.mod) andalso
+                     RH#gb_rh.mid /= 42 -> ok;
+                     true -> error
+                 end
+         end,
+    ok = T5(#gb_rh{}),
+    ok = T5(#gb_rh{mod=atom,mid=0}),
+    error = T5(#gb_rh{mod=100,mid=0}),
+    error = T5(#gb_rh{mod=atom,mid=42}),
+    error = T5(#gb_nil{}),
+    error = T5(#gb_bar{}),
+    error = T5(atom),
+
     ok.
 
 %% ERIERL-436; the following code used to be very slow to compile.
