@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2003-2016. All Rights Reserved.
+%% Copyright Ericsson AB 2003-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -21,7 +21,8 @@
 
 -include_lib("common_test/include/ct.hrl").
 -compile({no_auto_import,[binary_part/2]}).
--export([id/1,recompile/1,parallel/0,uniq/0,opt_opts/1,get_data_dir/1,
+-export([id/1,recompile/1,recompile_core/1,parallel/0,
+         uniq/0,opt_opts/1,get_data_dir/1,
          is_cloned_mod/1,smoke_disasm/1,p_run/2,
          highest_opcode/1]).
 
@@ -45,6 +46,21 @@ recompile(Mod) when is_atom(Mod) ->
     %% Smoke-test of beam disassembler.
     smoke_disasm(Mod).
 
+recompile_core(Mod) when is_atom(Mod) ->
+    case whereis(cover_server) of
+        undefined -> ok;
+        _ ->
+            %% Re-compile the test suite if the cover server is running.
+            Beam = code:which(Mod),
+            Src = filename:rootname(Beam, ".beam"),
+            Opts = [bin_opt_info|opt_opts(Mod)],
+            io:format("Recompiling ~p (~p)\n", [Mod,Opts]),
+            c:c(Src, [from_core,{outdir,filename:dirname(Src)}|Opts])
+    end,
+
+    %% Smoke-test of beam disassembler.
+    smoke_disasm(Mod).
+
 smoke_disasm(Mod) when is_atom(Mod) ->
     smoke_disasm(code:which(Mod));
 smoke_disasm(File) when is_list(File) ->
@@ -69,6 +85,7 @@ opt_opts(Mod) ->
     {options,Opts} = lists:keyfind(options, 1, Comp),
     lists:filter(fun
                      (debug_info) -> true;
+                     (dialyzer) -> true;
                      (inline) -> true;
                      (no_bsm3) -> true;
                      (no_bsm_opt) -> true;
@@ -79,9 +96,11 @@ opt_opts(Mod) ->
                      (no_put_tuple2) -> true;
                      (no_recv_opt) -> true;
                      (no_share_opt) -> true;
+                     (no_shared_fun_wrappers) -> true;
                      (no_ssa_float) -> true;
                      (no_ssa_opt) -> true;
                      (no_stack_trimming) -> true;
+                     (no_swap) -> true;
                      (no_type_opt) -> true;
                      (_) -> false
                 end, Opts).
@@ -91,15 +110,20 @@ opt_opts(Mod) ->
 %% This function retrieves the path to the original data directory.
 
 get_data_dir(Config) ->
-    Data0 = proplists:get_value(data_dir, Config),
+    Data = proplists:get_value(data_dir, Config),
     Opts = [{return,list}],
-    Data1 = re:replace(Data0, "_no_opt_SUITE", "_SUITE", Opts),
-    Data2 = re:replace(Data1, "_post_opt_SUITE", "_SUITE", Opts),
-    Data3 = re:replace(Data2, "_inline_SUITE", "_SUITE", Opts),
-    Data4 = re:replace(Data3, "_r21_SUITE", "_SUITE", Opts),
-    Data5 = re:replace(Data4, "_no_module_opt_SUITE", "_SUITE", Opts),
-    Data = re:replace(Data5, "_no_type_opt_SUITE", "_SUITE", Opts),
-    re:replace(Data, "_no_ssa_opt_SUITE", "_SUITE", Opts).
+    Suffixes = ["_no_opt_SUITE",
+                "_no_copt_SUITE",
+                "_post_opt_SUITE",
+                "_inline_SUITE",
+                "_r21_SUITE",
+                "_no_module_opt_SUITE",
+                "_no_type_opt_SUITE",
+                "_no_ssa_opt_SUITE"],
+    lists:foldl(fun(Suffix, Acc) ->
+                        Opts = [{return,list}],
+                        re:replace(Acc, Suffix, "_SUITE", Opts)
+                end, Data, Suffixes).
 
 is_cloned_mod(Mod) ->
     is_cloned_mod_1(atom_to_list(Mod)).
@@ -107,6 +131,7 @@ is_cloned_mod(Mod) ->
 %% Test whether Mod is a cloned module.
 
 is_cloned_mod_1("_no_opt_SUITE") -> true;
+is_cloned_mod_1("_no_copt_SUITE") -> true;
 is_cloned_mod_1("_no_ssa_opt_SUITE") -> true;
 is_cloned_mod_1("_post_opt_SUITE") -> true;
 is_cloned_mod_1("_inline_SUITE") -> true;
