@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2010-2018. All Rights Reserved.
+ * Copyright Ericsson AB 2010-2020. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,6 +24,9 @@ ERL_NIF_TERM atom_true;
 ERL_NIF_TERM atom_false;
 ERL_NIF_TERM atom_sha;
 ERL_NIF_TERM atom_error;
+ERL_NIF_TERM atom_pkcs_padding;
+ERL_NIF_TERM atom_zero;
+ERL_NIF_TERM atom_random;
 ERL_NIF_TERM atom_rsa_pkcs1_padding;
 ERL_NIF_TERM atom_rsa_pkcs1_oaep_padding;
 ERL_NIF_TERM atom_rsa_no_padding;
@@ -52,6 +55,9 @@ ERL_NIF_TERM atom_block_size;
 ERL_NIF_TERM atom_key_length;
 ERL_NIF_TERM atom_iv_length;
 ERL_NIF_TERM atom_mode;
+ERL_NIF_TERM atom_encrypt;
+ERL_NIF_TERM atom_padding_size;
+ERL_NIF_TERM atom_padding_type;
 ERL_NIF_TERM atom_ecb_mode;
 ERL_NIF_TERM atom_cbc_mode;
 ERL_NIF_TERM atom_cfb_mode;
@@ -86,9 +92,11 @@ ERL_NIF_TERM atom_rsa;
 ERL_NIF_TERM atom_dss;
 ERL_NIF_TERM atom_ecdsa;
 
-#ifdef HAVE_ED_CURVE_DH
+#ifdef HAVE_EDDH
 ERL_NIF_TERM atom_x25519;
 ERL_NIF_TERM atom_x448;
+ERL_NIF_TERM atom_ed25519;
+ERL_NIF_TERM atom_ed448;
 #endif
 
 ERL_NIF_TERM atom_eddsa;
@@ -154,6 +162,9 @@ int init_atoms(ErlNifEnv *env, const ERL_NIF_TERM fips_mode, const ERL_NIF_TERM
 
     atom_sha = enif_make_atom(env,"sha");
     atom_error = enif_make_atom(env,"error");
+    atom_pkcs_padding = enif_make_atom(env,"pkcs_padding");
+    atom_zero = enif_make_atom(env,"zero");
+    atom_random = enif_make_atom(env,"random");
     atom_rsa_pkcs1_padding = enif_make_atom(env,"rsa_pkcs1_padding");
     atom_rsa_pkcs1_oaep_padding = enif_make_atom(env,"rsa_pkcs1_oaep_padding");
     atom_rsa_no_padding = enif_make_atom(env,"rsa_no_padding");
@@ -176,6 +187,9 @@ int init_atoms(ErlNifEnv *env, const ERL_NIF_TERM fips_mode, const ERL_NIF_TERM
     atom_key_length = enif_make_atom(env,"key_length");
     atom_iv_length = enif_make_atom(env,"iv_length");
     atom_mode = enif_make_atom(env,"mode");
+    atom_encrypt = enif_make_atom(env, "encrypt");
+    atom_padding_size = enif_make_atom(env, "padding_size");
+    atom_padding_type = enif_make_atom(env, "padding_type");
     atom_ecb_mode = enif_make_atom(env,"ecb_mode");
     atom_cbc_mode = enif_make_atom(env,"cbc_mode");
     atom_cfb_mode = enif_make_atom(env,"cfb_mode");
@@ -216,9 +230,11 @@ int init_atoms(ErlNifEnv *env, const ERL_NIF_TERM fips_mode, const ERL_NIF_TERM
     atom_dss = enif_make_atom(env,"dss");
     atom_ecdsa = enif_make_atom(env,"ecdsa");
 
-#ifdef HAVE_ED_CURVE_DH
+#ifdef HAVE_EDDH
     atom_x25519 = enif_make_atom(env,"x25519");
     atom_x448 = enif_make_atom(env,"x448");
+    atom_ed25519 = enif_make_atom(env,"ed25519");
+    atom_ed448 = enif_make_atom(env,"ed448");
 #endif
     atom_eddsa = enif_make_atom(env,"eddsa");
 #ifdef HAVE_EDDSA
