@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2010-2018. All Rights Reserved.
+ * Copyright Ericsson AB 2010-2020. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -61,7 +61,7 @@ static int library_initialized = 0;
 static ErlNifFunc nif_funcs[] = {
     {"info_lib", 0, info_lib, 0},
     {"info_fips", 0, info_fips, 0},
-    {"enable_fips_mode", 1, enable_fips_mode, 0},
+    {"enable_fips_mode_nif", 1, enable_fips_mode_nif, 0},
     {"hash_algorithms", 0, hash_algorithms, 0},
     {"pubkey_algorithms", 0, pubkey_algorithms, 0},
     {"cipher_algorithms", 0, cipher_algorithms, 0},
@@ -79,10 +79,12 @@ static ErlNifFunc nif_funcs[] = {
     {"mac_final_nif", 1, mac_final_nif, 0},
     {"cipher_info_nif", 1, cipher_info_nif, 0},
     {"aes_ige_crypt_nif", 4, aes_ige_crypt_nif, 0},
-    {"ng_crypto_init_nif", 4, ng_crypto_init_nif, 0},
+    {"ng_crypto_init_nif", 5, ng_crypto_init_nif, 0},
     {"ng_crypto_update_nif", 2, ng_crypto_update_nif, 0},
     {"ng_crypto_update_nif", 3, ng_crypto_update_nif, 0},
-    {"ng_crypto_one_time_nif", 5, ng_crypto_one_time_nif, 0},
+    {"ng_crypto_final_nif", 1, ng_crypto_final_nif, 0},
+    {"ng_crypto_get_data_nif", 1, ng_crypto_get_data_nif, 0},
+    {"ng_crypto_one_time_nif", 6, ng_crypto_one_time_nif, 0},
     {"strong_rand_bytes_nif", 1, strong_rand_bytes_nif, 0},
     {"strong_rand_range_nif", 1, strong_rand_range_nif, 0},
     {"rand_uniform_nif", 2, rand_uniform_nif, 0},
@@ -95,7 +97,7 @@ static ErlNifFunc nif_funcs[] = {
     {"dh_generate_key_nif", 4, dh_generate_key_nif, 0},
     {"dh_compute_key_nif", 3, dh_compute_key_nif, 0},
     {"evp_compute_key_nif", 3, evp_compute_key_nif, 0},
-    {"evp_generate_key_nif", 1, evp_generate_key_nif, 0},
+    {"evp_generate_key_nif", 2, evp_generate_key_nif, 0},
     {"privkey_to_pubkey_nif", 2, privkey_to_pubkey_nif, 0},
     {"srp_value_B_nif", 5, srp_value_B_nif, 0},
     {"srp_user_secret_nif", 7, srp_user_secret_nif, 0},
@@ -130,7 +132,11 @@ ERL_NIF_INIT(crypto,nif_funcs,load,NULL,upgrade,unload)
 
 static int verify_lib_version(void)
 {
+#if OPENSSL_VERSION_NUMBER < PACKED_OPENSSL_VERSION_PLAIN(1,1,0)
     const unsigned long libv = SSLeay();
+#else
+    const unsigned long libv = OpenSSL_version_num();
+#endif
     const unsigned long hdrv = OPENSSL_VERSION_NUMBER;
 
 #   define MAJOR_VER(V) ((unsigned long)(V) >> (7*4))
@@ -145,8 +151,10 @@ static int verify_lib_version(void)
 
 static int initialize(ErlNifEnv* env, ERL_NIF_TERM load_info)
 {
+#if OPENSSL_VERSION_NUMBER < PACKED_OPENSSL_VERSION_PLAIN(1,1,0)
 #ifdef OPENSSL_THREADS
     ErlNifSysInfo sys_info;
+#endif
 #endif
     get_crypto_callbacks_t* funcp;
     struct crypto_callbacks* ccb;
@@ -217,12 +225,14 @@ static int initialize(ErlNifEnv* env, ERL_NIF_TERM load_info)
     funcp = &get_crypto_callbacks;
 #endif
 
+#if OPENSSL_VERSION_NUMBER < PACKED_OPENSSL_VERSION_PLAIN(1,1,0)
 #ifdef OPENSSL_THREADS
     enif_system_info(&sys_info, sizeof(sys_info));
     if (sys_info.scheduler_threads > 1) {
 	nlocks = CRYPTO_num_locks();
     }
     /* else no need for locks */
+#endif
 #endif
 
     ccb = (*funcp)(nlocks);
@@ -237,8 +247,10 @@ static int initialize(ErlNifEnv* env, ERL_NIF_TERM load_info)
         return __LINE__;
 #endif
 
+#if OPENSSL_VERSION_NUMBER < PACKED_OPENSSL_VERSION_PLAIN(1,1,0)
 #ifdef OPENSSL_THREADS
     if (nlocks > 0) {
+	CRYPTO_set_add_lock_callback(ccb->add_lock_function);
 	CRYPTO_set_locking_callback(ccb->locking_function);
 	CRYPTO_set_id_callback(ccb->id_function);
 	CRYPTO_set_dynlock_create_callback(ccb->dyn_create_function);
@@ -246,6 +258,7 @@ static int initialize(ErlNifEnv* env, ERL_NIF_TERM load_info)
 	CRYPTO_set_dynlock_destroy_callback(ccb->dyn_destroy_function);
     }
 #endif /* OPENSSL_THREADS */
+#endif
 
     init_digest_types(env);
     init_mac_types(env);
