@@ -29,7 +29,7 @@ ERL_NIF_TERM info_fips(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[])
 #endif
 }
 
-ERL_NIF_TERM enable_fips_mode(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[])
+ERL_NIF_TERM enable_fips_mode_nif(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[])
 {/* (Boolean) */
     if (argv[0] == atom_true) {
 #ifdef FIPS_SUPPORT
