@@ -21,7 +21,7 @@
 #ifndef E_HMAC_H__
 #define E_HMAC_H__ 1
 
-#ifndef HAS_EVP_PKEY_CTX
+#if !defined(HAS_EVP_PKEY_CTX) || DISABLE_EVP_HMAC
 
 #include "common.h"
 
