@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2010-2017. All Rights Reserved.
+%% Copyright Ericsson AB 2010-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -92,9 +92,9 @@
 
 -type connect_option() :: {raddr, inet:ip_address()}
                         | {rport, pos_integer()}
-                        | {ssl_options, true | [ssl:connect_option()]}
+                        | {ssl_options, true | [ssl:tls_client_option()]}
                         | option()
-                        | ssl:connect_option()
+                        | ssl:tls_client_option()
                         | gen_tcp:connect_option().
 
 -type match() :: inet:ip_address()
@@ -102,9 +102,9 @@
                | [match()].
 
 -type listen_option() :: {accept, match()}
-                       | {ssl_options, true | [ssl:listen_option()]}
+                       | {ssl_options, true | [ssl:tls_server_option()]}
                        | option()
-                       | ssl:listen_option()
+                       | ssl:tls_server_option()
                        | gen_tcp:listen_option().
 
 -type option() :: {port, non_neg_integer()}
@@ -569,7 +569,11 @@ m({'DOWN', M, process, P, _} = T, #monitor{parent = MRef,
 
 %% l/2
 %%
-%% Transition listener state.
+%% Transition listener state. Or not anymore since any message causes
+%% the process to exit.
+
+-spec l(tuple(), #listener{})
+   -> no_return().
 
 %% Service process has died.
 l({'DOWN', _, process, Pid, _} = T, #listener{service = Pid,
@@ -716,7 +720,7 @@ tls_handshake(_, false, S) ->
 tls(connect, Sock, Opts) ->
     ssl:connect(Sock, Opts);
 tls(accept, Sock, Opts) ->
-    ssl:ssl_accept(Sock, Opts).
+    ssl:handshake(Sock, Opts).  %% assume no handshake option
 
 %% recv/2
 %%
@@ -839,7 +843,7 @@ start_fragment_timer(#transport{timeout = Tmo} = S) ->
 accept(ssl, LSock) ->
     case ssl:transport_accept(LSock) of
         {ok, Sock} ->
-            {ssl:ssl_accept(Sock), Sock};
+            ssl:handshake(Sock);
         {error, _} = No ->
             No
     end;
