@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2010-2019. All Rights Reserved.
+%% Copyright Ericsson AB 2010-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -128,7 +128,7 @@
 %% ===========================================================================
 
 %% Fraction of shuffle/parallel groups to randomly skip.
--define(SKIP, 0.25).
+-define(SKIP, 0.90).
 
 %% Positive number of testcases from which to select (randomly) from
 %% tc(), the list of testcases to run, or [] to run all. The random
@@ -305,7 +305,8 @@ names() ->
                              S  <- ?STRING_DECODES,
                              ST <- ?CALLBACKS,
                              SS <- ?SENDERS,
-                             CS <- ?SENDERS].
+                             CS <- ?SENDERS,
+                             ?SKIP =< rand:uniform()].
 
 names(Names, []) ->
     [N || N <- Names,
@@ -336,14 +337,9 @@ init_per_group(_) ->
 init_per_group(Name, Config)
   when Name == shuffle;
        Name == parallel ->
-    case rand:uniform() < ?SKIP of
-        true ->
-            {skip, random};
-        false ->
-            start_services(Config),
-            add_transports(Config),
-            replace({sleep, Name == parallel}, Config)
-    end;
+    start_services(Config),
+    add_transports(Config),
+    replace({sleep, Name == parallel}, Config);
 
 init_per_group(sctp = Name, Config) ->
     {_, Sctp} = lists:keyfind(Name, 1, Config),
