@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  * 
- * Copyright Ericsson AB 2001-2018. All Rights Reserved.
+ * Copyright Ericsson AB 2001-2020. All Rights Reserved.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -27,20 +27,21 @@
 
 #include <stdio.h>
 #include <string.h>
-#ifdef VXWORKS
-#include "reclaim.h"
-#endif
 
 #include "ei_runner.h"
+#include "my_ussi.h"
 
 static void cmd_ei_connect_init(char* buf, int len);
 static void cmd_ei_connect(char* buf, int len);
+static void cmd_ei_connect_host_port(char* buf, int len);
 static void cmd_ei_send(char* buf, int len);
 static void cmd_ei_format_pid(char* buf, int len);
 static void cmd_ei_send_funs(char* buf, int len);
 static void cmd_ei_reg_send(char* buf, int len);
 static void cmd_ei_rpc(char* buf, int len);
 static void cmd_ei_set_get_tracelevel(char* buf, int len);
+static void cmd_ei_make_refs(char* buf, int len);
+static void cmd_ei_make_pids(char* buf, int len);
 
 static void send_errno_result(int value);
 
@@ -52,14 +53,17 @@ static struct {
     int num_args;		/* Number of arguments. */
     void (*func)(char* buf, int len);
 } commands[] = {
-    "ei_connect_init",       3, cmd_ei_connect_init,
+    "ei_connect_init",       4, cmd_ei_connect_init,
     "ei_connect", 	     1, cmd_ei_connect,
+    "ei_connect_host_port",  2, cmd_ei_connect_host_port,
     "ei_send",  	     3, cmd_ei_send,
     "ei_send_funs",  	     3, cmd_ei_send_funs,
     "ei_reg_send", 	     3, cmd_ei_reg_send,
     "ei_rpc",  		     4, cmd_ei_rpc,
     "ei_set_get_tracelevel", 1, cmd_ei_set_get_tracelevel,
     "ei_format_pid",         2, cmd_ei_format_pid,
+    "ei_make_refs",          2, cmd_ei_make_refs,
+    "ei_make_pids",          2, cmd_ei_make_pids,
 };
 
 
@@ -107,9 +111,11 @@ TESTCASE(interpret)
 static void cmd_ei_connect_init(char* buf, int len)
 {
     int index = 0, r = 0;
-    long l;
+    long l, creation;
     char b[100];
     char cookie[MAXATOMLEN], * cp = cookie;
+    char socket_impl[10];
+    int use_ussi;
     ei_x_buff res;
     if (ei_decode_long(buf, &index, &l) < 0)
 	fail("expected int");
@@ -118,7 +124,23 @@ static void cmd_ei_connect_init(char* buf, int len)
 	fail("expected atom (cookie)");
     if (cookie[0] == '\0')
 	cp = NULL;
-    r = ei_connect_init(&ec, b, cp, 0);
+    if (ei_decode_long(buf, &index, &creation) < 0)
+	fail("expected int (creation)");
+    if (ei_decode_atom_as(buf, &index, socket_impl,
+                          sizeof(socket_impl), ERLANG_ASCII, NULL, NULL) < 0)
+	fail("expected atom (socket_impl)");
+    if (strcmp(socket_impl, "default") == 0)
+        use_ussi = 0;
+    else if (strcmp(socket_impl, "ussi") == 0)
+        use_ussi = 1;
+    else
+	fail1("expected atom 'default' or 'ussi', got '%s'", socket_impl);
+
+    if (use_ussi)
+        r = ei_connect_init_ussi(&ec, b, cp, (short)creation,
+                                 &my_ussi, sizeof(my_ussi), NULL);
+    else
+        r = ei_connect_init(&ec, b, cp, (short)creation);
     ei_x_new_with_version(&res);
     ei_x_encode_long(&res, r);
     send_bin_term(&res);
@@ -133,11 +155,20 @@ static void cmd_ei_connect(char* buf, int len)
     if (ei_decode_atom(buf, &index, node) < 0)
 	fail("expected atom");
     i=ei_connect(&ec, node);
-#ifdef VXWORKS
-    if(i >= 0) {
-	save_fd(i);
-    }
-#endif
+    send_errno_result(i);
+}
+
+static void cmd_ei_connect_host_port(char* buf, int len)
+{
+    int index = 0;
+    char hostname[256];
+    int i;
+    long port;
+    if (ei_decode_atom(buf, &index, hostname) < 0)
+	fail("expected atom");
+    if (ei_decode_long(buf, &index, &port) < 0)
+	fail("expected int");
+    i = ei_connect_host_port(&ec, hostname, (int)port);
     send_errno_result(i);
 }
 
@@ -183,6 +214,80 @@ static void cmd_ei_send(char* buf, int len)
     ei_x_free(&x);
 }
 
+static void cmd_ei_make_refs(char* buf, int len)
+{
+    int index = 0;
+    long fd;
+    erlang_pid pid;
+    ei_x_buff x;
+    int i;
+    int nref = 1000;
+
+    if (ei_decode_long(buf, &index, &fd) < 0)
+	fail("expected long");
+    if (ei_decode_pid(buf, &index, &pid) < 0)
+	fail("expected pid (node)");
+    if (ei_x_new_with_version(&x) < 0)
+	fail("ei_x_new_with_version");
+    if (ei_x_encode_tuple_header(&x, 2) < 0)
+        fail("ei_x_encode_tuple_header() failed");
+    if (ei_x_encode_atom(&x, ei_thisnodename(&ec)) < 0)
+        fail("ei_x_encode_atom() failed");
+    if (ei_x_encode_list_header(&x, nref) < 0)
+        fail("ei_x_encode_list_header() failed");
+    for (i = 0; i < nref; i++) {
+        erlang_ref ref;
+        if (ei_make_ref(&ec, &ref))
+            fail("ei_make_ref() failed");
+        if (ei_x_encode_ref(&x, &ref))
+            fail("ei_x_encode_ref() failed");
+    }
+    if (ei_x_encode_empty_list(&x) < 0)
+        fail("ei_x_encode_empty_list() failed");
+    send_errno_result(ei_send(fd, &pid, x.buff, x.index));
+    ei_x_free(&x);
+}
+
+static void cmd_ei_make_pids(char* buf, int len)
+{
+    int index = 0;
+    long fd;
+    erlang_pid from_pid;
+    erlang_pid *self;
+    ei_x_buff x;
+    int i;
+    int npid = 1000;
+
+    if (ei_decode_long(buf, &index, &fd) < 0)
+	fail("expected long");
+    if (ei_decode_pid(buf, &index, &from_pid) < 0)
+	fail("expected pid (node)");
+    if (ei_x_new_with_version(&x) < 0)
+	fail("ei_x_new_with_version");
+    if (ei_x_encode_tuple_header(&x, 2) < 0)
+        fail("ei_x_encode_tuple_header() failed");
+    if (ei_x_encode_atom(&x, ei_thisnodename(&ec)) < 0)
+        fail("ei_x_encode_atom() failed");
+    if (ei_x_encode_list_header(&x, 1+npid) < 0)
+        fail("ei_x_encode_list_header() failed");
+    self = ei_self(&ec);
+    if (!self)
+        fail("ei_self() failed");
+    if (ei_x_encode_pid(&x, self))
+        fail("ei_x_encode_pid() failed");
+    for (i = 0; i < npid; i++) {
+        erlang_pid pid;
+        if (ei_make_pid(&ec, &pid))
+            fail("ei_make_pid() failed");
+        if (ei_x_encode_pid(&x, &pid))
+            fail("ei_x_encode_pid() failed");
+    }
+    if (ei_x_encode_empty_list(&x) < 0)
+        fail("ei_x_encode_empty_list() failed");
+    send_errno_result(ei_send(fd, &from_pid, x.buff, x.index));
+    ei_x_free(&x);
+}
+
 static void cmd_ei_format_pid(char* buf, int len)
 {
     int index = 0;
@@ -225,7 +330,7 @@ static void cmd_ei_send_funs(char* buf, int len)
 	fail("expected Fun1");
     if (ei_decode_fun(buf, &index, &fun2) < 0)
 	fail("expected Fun2");
-    if (ei_decode_bitstring(buf, &index, &bitstring, &bitoffs, &bits) < 0)
+    if (ei_decode_bitstring(buf, &index, (const char**)&bitstring, &bitoffs, &bits) < 0)
 	fail("expected bitstring");
     if (ei_x_new_with_version(&x) < 0)
 	fail("ei_x_new_with_version");
