@@ -557,21 +557,32 @@ trans_fun([{move,Src,Dst}|Instructions], Env) ->
   Dst1 = mk_var(Dst),
   Src1 = trans_arg(Src),
   [hipe_icode:mk_move(Dst1,Src1) | trans_fun(Instructions,Env)];
-%%
-%% try/catch -- THESE ARE KNOWN TO MISCOMPILE, SEE OTP-15949
-%%
-trans_fun([{'catch'=Name,_,_}|_], _Env) ->
-  nyi(Name);
-trans_fun([{catch_end=Name,_}|_], _Env) ->
-  nyi(Name);
-trans_fun([{'try'=Name,_,_}|_], _Env) ->
-  nyi(Name);
-trans_fun([{try_end=Name,_}|_], _Env) ->
-  nyi(Name);
-trans_fun([{try_case=Name,_}|_], _Env) ->
-  nyi(Name);
-trans_fun([{try_case_end=Name,_}|_], _Env) ->
-  nyi(Name);
+%%--- catch --- ITS PROCESSING IS POSTPONED
+trans_fun([{'catch',N,{_,EndLabel}}|Instructions], Env) ->
+  NewContLbl = mk_label(new),
+  [{'catch',N,EndLabel},NewContLbl | trans_fun(Instructions,Env)];
+%%--- catch_end --- ITS PROCESSING IS POSTPONED
+trans_fun([{catch_end,_N}=I|Instructions], Env) ->
+  [I | trans_fun(Instructions,Env)];
+%%--- try --- ITS PROCESSING IS POSTPONED
+trans_fun([{'try',N,{_,EndLabel}}|Instructions], Env) ->
+  NewContLbl = mk_label(new),
+  [{'try',N,EndLabel},NewContLbl | trans_fun(Instructions,Env)];
+%%--- try_end ---
+trans_fun([{try_end,_N}|Instructions], Env) ->
+  [hipe_icode:mk_end_try() | trans_fun(Instructions,Env)];
+%%--- try_case --- ITS PROCESSING IS POSTPONED
+trans_fun([{try_case,_N}=I|Instructions], Env) ->
+  [I | trans_fun(Instructions,Env)];
+%%--- try_case_end ---
+trans_fun([{try_case_end,Arg}|Instructions], Env) ->
+  BadArg = trans_arg(Arg),
+  ErrVar = mk_var(new),
+  Vs = [mk_var(new)],
+  Atom = hipe_icode:mk_move(ErrVar,hipe_icode:mk_const(try_clause)),
+  Tuple = hipe_icode:mk_primop(Vs,mktuple,[ErrVar,BadArg]),
+  Fail = hipe_icode:mk_fail(Vs,error),
+  [Atom,Tuple,Fail | trans_fun(Instructions,Env)];
 %%--- raise ---
 trans_fun([{raise,{f,0},[Reg1,Reg2],{x,0}}|Instructions], Env) ->
   V1 = trans_arg(Reg1),
@@ -678,8 +689,8 @@ trans_fun([{call_fun,N}|Instructions], Env) ->
   Dst = [mk_var({r,0})],
   [hipe_icode:mk_comment('call_fun'),
    hipe_icode:mk_primop(Dst,call_fun,Args) | trans_fun(Instructions,Env)];
-%%--- patched_make_fun --- make_fun/make_fun2 after fixes
-trans_fun([{patched_make_fun,MFA,Magic,FreeVarNum,Index}|Instructions], Env) ->
+%%--- make_fun2 ---
+trans_fun([{make_fun2,MFA,Index,Magic,FreeVarNum}|Instructions], Env) ->
   Args = extract_fun_args(FreeVarNum),
   Dst = [mk_var({r,0})],
   Fun = hipe_icode:mk_primop(Dst,
@@ -1193,6 +1204,17 @@ trans_fun([{bs_get_position=Name,_,_,_}|_Instructions], _Env) ->
 trans_fun([{bs_set_position=Name,_,_}|_Instructions], _Env) ->
   nyi(Name);
 %%--------------------------------------------------------------------
+%% New instructions added in OTP 23.
+%%--------------------------------------------------------------------
+%%--- swap ---
+trans_fun([{swap,Reg1,Reg2}|Instructions], Env) ->
+  Var1 = mk_var(Reg1),
+  Var2 = mk_var(Reg2),
+  Temp = mk_var(new),
+  [hipe_icode:mk_move(Temp, Var1),
+   hipe_icode:mk_move(Var1, Var2),
+   hipe_icode:mk_move(Var2, Temp) | trans_fun(Instructions, Env)];
+%%--------------------------------------------------------------------
 %%--- ERROR HANDLING ---
 %%--------------------------------------------------------------------
 trans_fun([X|_], _) ->
@@ -1935,7 +1957,7 @@ mod_find_closure_info([FunCode|Fs], CI) ->
 mod_find_closure_info([], CI) ->
   CI.
 
-find_closure_info([{patched_make_fun,MFA={_M,_F,A},_Magic,FreeVarNum,_Index}|BeamCode],
+find_closure_info([{make_fun2,{_M,_F,A}=MFA,_Index,_Magic,FreeVarNum}|BeamCode],
 		  ClosureInfo) ->
   NewClosure = %% A-FreeVarNum+1 (The real arity + 1 for the closure)
     #closure_info{mfa=MFA, arity=A-FreeVarNum+1, fv_arity=FreeVarNum},
@@ -2013,41 +2035,8 @@ split_params(N, [ArgN|OrgArgs], Args) ->
 %%-----------------------------------------------------------------------
 
 preprocess_code(ModuleCode) ->
-  PatchedCode = patch_R7_funs(ModuleCode),
-  ClosureInfo = find_closure_info(PatchedCode),
-  {PatchedCode, ClosureInfo}.
-
-%%-----------------------------------------------------------------------
-%% Patches the "make_fun" BEAM instructions of R7 so that they also
-%% contain the index that the BEAM loader generates for funs.
-%% 
-%% The index starts from 0 and is incremented by 1 for each make_fun
-%% instruction encountered.
-%%
-%% Retained only for compatibility with BEAM code prior to R8.
-%%
-%% Temporarily, it also rewrites R8-PRE-RELEASE "make_fun2"
-%% instructions, since their embedded indices don't work.
-%%-----------------------------------------------------------------------
-
-patch_R7_funs(ModuleCode) ->
-  patch_make_funs(ModuleCode, 0).
-
-patch_make_funs([FunCode0|Fs], FunIndex0) ->
-  {PatchedFunCode,FunIndex} = patch_make_funs(FunCode0, FunIndex0, []),
-  [PatchedFunCode|patch_make_funs(Fs, FunIndex)];
-patch_make_funs([], _) -> [].
-
-patch_make_funs([{make_fun,MFA,Magic,FreeVarNum}|Is], FunIndex, Acc) ->
-  Patched = {patched_make_fun,MFA,Magic,FreeVarNum,FunIndex},
-  patch_make_funs(Is, FunIndex+1, [Patched|Acc]);
-patch_make_funs([{make_fun2,MFA,_BogusIndex,Magic,FreeVarNum}|Is], FunIndex, Acc) ->
-  Patched = {patched_make_fun,MFA,Magic,FreeVarNum,FunIndex},
-  patch_make_funs(Is, FunIndex+1, [Patched|Acc]);
-patch_make_funs([I|Is], FunIndex, Acc) ->
-  patch_make_funs(Is, FunIndex, [I|Acc]);
-patch_make_funs([], FunIndex, Acc) ->
-  {lists:reverse(Acc),FunIndex}.
+  ClosureInfo = find_closure_info(ModuleCode),
+  {ModuleCode, ClosureInfo}.
 
 %%-----------------------------------------------------------------------
 
@@ -2292,9 +2281,7 @@ fix_catch(Type, Lbl, ContLbl, Code, HandledCatchLbls, Instr) ->
   TLbl = {Type, Lbl},
   case gb_trees:lookup(TLbl, HandledCatchLbls) of
     {value, Catch} when is_integer(Catch) ->
-      NewCode = fix_catches(Code, HandledCatchLbls),
-      Cont = hipe_icode:label_name(ContLbl),
-      [hipe_icode:mk_begin_try(Catch,Cont),ContLbl | NewCode];
+      nyi(unsafe_catch);
     none ->
       OldCatch = map_label(Lbl),
       OldCatchLbl = hipe_icode:mk_label(OldCatch),
@@ -2349,9 +2336,8 @@ catch_handler('catch', [TagVar,ValueVar,TraceVar], OldCatchLbl) ->
 				       ValueVar]),
    hipe_icode:mk_goto(Cont),
    ErrorLbl,
-   %% We use the trace variable to hold the symbolic trace. Its previous
-   %% value is just that in p->ftrace, so get_stacktrace() works fine.
-   hipe_icode:mk_call([TraceVar],erlang,get_stacktrace,[],remote),
+   %% We use the trace variable to hold the symbolic trace.
+   hipe_icode:mk_primop([TraceVar],build_stacktrace,[TraceVar]),
    hipe_icode:mk_primop([ValueVar],mktuple, [ValueVar, TraceVar]),
    hipe_icode:mk_goto(hipe_icode:label_name(ExitLbl)),
    OldCatchLbl,  % normal execution paths must go through end_try
