@@ -61,6 +61,14 @@
 	 scheme_defaults/0, 
 	 encode/1, decode/1]).
 
+
+-deprecated({parse, 1, "use uri_string functions instead"}).
+-deprecated({parse, 2, "use uri_string functions instead"}).
+-deprecated({encode, 1, "use uri_string functions instead"}).
+-deprecated({decode, 1, "use uri_string functions instead"}).
+-deprecated({scheme_defaults, 0, "use uri_string functions instead"}).
+ 
+
 -export_type([uri/0,
               user_info/0,
               scheme/0, default_scheme_port_number/0,
