@@ -27,7 +27,7 @@
 -export([deliver/2]).
 
 %% Callback API
--export([do/1, load/2, store/2]).
+-export([do/1, store/2]).
 
 -include("httpd.hrl").
 -include("httpd_internal.hrl").
@@ -82,61 +82,6 @@ do(ModData) ->
     end.
 
 
-%%--------------------------------------------------------------------------
-%% load(Line, Context) ->  eof | ok | {ok, NewContext} | 
-%%                     {ok, NewContext, Directive} | 
-%%                     {ok, NewContext, DirectiveList} | {error, Reason}
-%% Line = string()
-%% Context = NewContext = DirectiveList = [Directive]
-%% Directive = {DirectiveKey , DirectiveValue}
-%% DirectiveKey = DirectiveValue = term()
-%% Reason = term() 
-%%
-%% Description: See httpd(3) ESWAPI CALLBACK FUNCTIONS
-%%-------------------------------------------------------------------------
-load("ErlScriptAlias " ++ ErlScriptAlias, []) ->
-    try re:split(ErlScriptAlias," ", [{return, list}]) of
-	[ErlName | StrModules] ->
-	    Modules = lists:map(fun(Str) -> 
-					list_to_atom(string:strip(Str)) 
-				end, StrModules),
-	    {ok, [], {erl_script_alias, {ErlName, Modules}}}
-    catch _:_ ->
-	    {error, ?NICE(string:strip(ErlScriptAlias) ++
-			      " is an invalid ErlScriptAlias")}
-    end;
-load("EvalScriptAlias " ++ EvalScriptAlias, []) ->
-    try re:split(EvalScriptAlias, " ",  [{return, list}]) of
-	[EvalName | StrModules] ->
-	    Modules = lists:map(fun(Str) -> 
-					list_to_atom(string:strip(Str)) 
-				end, StrModules),
-	    {ok, [], {eval_script_alias, {EvalName, Modules}}}
-    catch 
-	_:_ ->
-	    {error, ?NICE(string:strip(EvalScriptAlias) ++
-			      " is an invalid EvalScriptAlias")}
-    end;
-load("ErlScriptTimeout " ++ Timeout, [])->
-    case catch list_to_integer(string:strip(Timeout)) of
-	TimeoutSec when is_integer(TimeoutSec)  ->
-	   {ok, [], {erl_script_timeout, TimeoutSec}};
-	_ ->
-	   {error, ?NICE(string:strip(Timeout) ++
-			 " is an invalid ErlScriptTimeout")}
-    end;
-load("ErlScriptNoCache " ++ CacheArg, [])->
-    case catch list_to_atom(string:strip(CacheArg)) of
-        true ->
-	    {ok, [], {erl_script_nocache, true}};
-	false ->
-	   {ok, [], {erl_script_nocache, false}};
-	_ ->
-	   {error, ?NICE(string:strip(CacheArg)++
-			 " is an invalid ErlScriptNoCache directive")}
-    end.
-
-
 %%--------------------------------------------------------------------------
 %% store(Directive, DirectiveList) -> {ok, NewDirective} | 
 %%                                    {ok, [NewDirective]} |
@@ -161,16 +106,6 @@ store({erl_script_alias, {Name, Modules}} = Conf, _)
    	    {error, {wrong_type, {erl_script_alias, Error}}}
     end;
 
-store({eval_script_alias, {Name, Modules}} = Conf, _)  
-  when is_list(Name)->
-    try httpd_util:modules_validate(Modules) of
-  	ok ->
-   	    {ok, Conf}
-    catch
-   	throw:Error ->
-   	    {error, {wrong_type, {eval_script_alias, Error}}}
-    end;
-
 store({erl_script_alias, Value}, _) ->
     {error, {wrong_type, {erl_script_alias, Value}}};
 store({erl_script_timeout, TimeoutSec}, _) 
@@ -190,8 +125,6 @@ store({erl_script_nocache, Value}, _) ->
 %%%========================================================================   
 generate_response(ModData) ->
     case scheme(ModData#mod.request_uri, ModData#mod.config_db) of
-	{eval, ESIBody, Modules} ->
-	    eval(ModData, ESIBody, Modules);
 	{erl, ESIBody, Modules} ->
 	    erl(ModData, ESIBody, Modules);
 	no_scheme ->
@@ -201,12 +134,7 @@ generate_response(ModData) ->
 scheme(RequestURI, ConfigDB) ->
     case match_script(RequestURI, ConfigDB, erl_script_alias) of
 	no_match ->
-	    case match_script(RequestURI, ConfigDB, eval_script_alias) of
-		no_match ->
-		    no_scheme;
-		{EsiBody, ScriptModules} ->
-		    {eval, EsiBody, ScriptModules}
-	    end;
+            no_scheme;
 	{EsiBody, ScriptModules} ->
 	    {erl, EsiBody, ScriptModules}
     end.
@@ -231,10 +159,7 @@ match_esi_script(RequestURI, [{Alias,Modules} | Rest], AliasType) ->
     end.
 
 alias_match_str(Alias, erl_script_alias) ->
-    "^" ++ Alias ++ "/";
-alias_match_str(Alias, eval_script_alias) ->
-    "^" ++ Alias ++ "\\?".
-
+    "^" ++ Alias ++ "/".
 
 %%------------------------ Erl mechanism --------------------------------
 
@@ -261,8 +186,9 @@ erl(#mod{method = Method} = ModData, ESIBody, Modules)
 	    {proceed, [{status,{400, none, BadRequest}} | ModData#mod.data]}
     end;
 
-erl(#mod{method = "PUT", entity_body = Body} = ModData,
-    ESIBody, Modules) ->
+erl(#mod{method = Method, entity_body = Body} = ModData,
+    ESIBody, Modules) when Method =:= "PUT" orelse
+                           Method =:= "PATCH" ->
     case httpd_util:split(ESIBody,":|%3A|/",2) of
 	{ok, [ModuleName, FuncAndInput]} ->                
 	    case httpd_util:split(FuncAndInput,"[\?/]",2) of
@@ -292,15 +218,7 @@ erl(#mod{method = "POST", entity_body = Body} = ModData, ESIBody, Modules) ->
 			     Function, Body, []);
 	{ok, BadRequest} ->
 	    {proceed,[{status, {400, none, BadRequest}} | ModData#mod.data]}
-    end;
-
-erl(#mod{request_uri  = ReqUri, 
-	 method       = "PATCH",
-         http_version = Version, 
-	 data         = Data}, _ESIBody, _Modules) ->
-    {proceed, [{status,{501,{"PATCH", ReqUri, Version},
-			?NICE("Erl mechanism doesn't support method PATCH")}}|
-	       Data]}.
+    end.
 
 generate_webpage(ModData, ESIBody, [all], Module, FunctionName,
 		 Input, ScriptElements) ->
@@ -315,8 +233,8 @@ generate_webpage(ModData, ESIBody, Modules, Module, FunctionName,
 	    case erl_scheme_webpage_chunk(Module, Function, 
 					  Env, Input, ModData) of
 		{error, erl_scheme_webpage_chunk_undefined} ->
-		    erl_scheme_webpage_whole(Module, Function, Env, Input,
-					     ModData);
+                    {proceed, [{status, {404, ModData#mod.request_uri, "Not found"}}
+                               | ModData#mod.data]};
 		ResponseResult ->
 		    ResponseResult
 	    end;
@@ -326,38 +244,7 @@ generate_webpage(ModData, ESIBody, Modules, Module, FunctionName,
 				       ++  ESIBody)}} | ModData#mod.data]}
     end.
 
-%% Old API that waits for the dymnamic webpage to be totally generated
-%% before anythig is sent back to the client.
-erl_scheme_webpage_whole(Mod, Func, Env, Input, ModData) ->
-    case (catch Mod:Func(Env, Input)) of
-	{'EXIT',{undef, _}} ->
-	    {proceed, [{status, {404, ModData#mod.request_uri, "Not found"}}
-		       | ModData#mod.data]};
-	{'EXIT',Reason} ->
-	    {proceed, [{status, {500, none, Reason}} |
-		       ModData#mod.data]};
-	Response ->
-	    {Headers, Body} = 
-		httpd_esi:parse_headers(lists:flatten(Response)),
-	    Length =  httpd_util:flatlength(Body),
-            {ok, NewHeaders, StatusCode} = httpd_esi:handle_headers(Headers), 
-            send_headers(ModData, StatusCode, 
-                         [{"content-length", 
-                           integer_to_list(Length)}| NewHeaders]),
-            case ModData#mod.method of
-                "HEAD" ->
-                    {proceed, [{response, {already_sent, StatusCode, 0}} | 
-                               ModData#mod.data]};
-                _ ->
-                    httpd_response:send_body(ModData, 
-                                             StatusCode, Body),
-                    {proceed, [{response, {already_sent, StatusCode, 
-                                           Length}} | 
-                               ModData#mod.data]}
-            end
-    end.
-
-%% New API that allows the dynamic wepage to be sent back to the client 
+%% API that allows the dynamic wepage to be sent back to the client 
 %% in small chunks at the time during generation.
 erl_scheme_webpage_chunk(Mod, Func, Env, Input, ModData) -> 
     process_flag(trap_exit, true),
@@ -369,7 +256,6 @@ erl_scheme_webpage_chunk(Mod, Func, Env, Input, ModData) ->
 	    fun() ->
 		    case catch Mod:Func(Self, Env, Input) of
 			{'EXIT', {undef,_}} ->
-			    %% Will force fallback on the old API
 			    exit(erl_scheme_webpage_chunk_undefined);
 			{continue, _} = Continue ->
                             exit(Continue);
@@ -523,64 +409,3 @@ input_type([$?|_Rest]) ->
 input_type([_First|Rest]) ->
     input_type(Rest).
 
-%%------------------------ Eval mechanism --------------------------------
-
-eval(#mod{request_uri  = ReqUri, 
-	  method       = "PUT",
-	  http_version = Version, 
-	  data         = Data}, _ESIBody, _Modules) ->
-    {proceed,[{status,{501,{"PUT", ReqUri, Version},
-		       ?NICE("Eval mechanism doesn't support method PUT")}}|
-	      Data]};
-
-eval(#mod{request_uri  = ReqUri, 
-	  method       = "DELETE",
-	  http_version = Version, 
-	  data         = Data}, _ESIBody, _Modules) ->
-    {proceed,[{status,{501,{"DELETE", ReqUri, Version},
-		       ?NICE("Eval mechanism doesn't support method DELETE")}}|
-	      Data]};
-
-eval(#mod{request_uri  = ReqUri, 
-	  method       = "POST",
-	  http_version = Version, 
-	  data         = Data}, _ESIBody, _Modules) ->
-    {proceed,[{status,{501,{"POST", ReqUri, Version},
-		       ?NICE("Eval mechanism doesn't support method POST")}}|
-	      Data]};
-
-eval(#mod{method = Method} = ModData, ESIBody, Modules) 
-  when (Method =:= "GET") orelse (Method =:= "HEAD") ->
-    case is_authorized(ESIBody, Modules) of
-	true ->
-	    case generate_webpage(ESIBody) of
-		{error, Reason} ->
-		    {proceed, [{status, {500, none, Reason}} | 
-			       ModData#mod.data]};
-		{ok, Response} ->
-		    {Headers, _} = 
-			httpd_esi:parse_headers(lists:flatten(Response)),
-                    {ok, _, StatusCode} =httpd_esi:handle_headers(Headers), 
-                    {proceed,[{response, {StatusCode, Response}} | 
-                              ModData#mod.data]}
-            end;
-	false ->
-	    {proceed,[{status,
-		       {403, ModData#mod.request_uri,
-			?NICE("Client not authorized to evaluate: "
-			      ++ ESIBody)}} | ModData#mod.data]}
-    end.
-
-generate_webpage(ESIBody) ->
-    (catch erl_eval:eval_str(string:concat(ESIBody,". "))).
-
-is_authorized(_ESIBody, [all]) ->
-    true;
-is_authorized(ESIBody, Modules) ->
-    case re:run(ESIBody, "^[^\:(%3A)]*", [{capture, first}]) of
-	{match, [{Start, Length}]} ->
-	    lists:member(list_to_atom(string:substr(ESIBody, Start+1, Length)),
-			 Modules);
-	nomatch ->
-	    false
-    end.
