@@ -106,9 +106,9 @@ public class OtpOutputStream extends ByteArrayOutputStream {
     }
 
     /**
-     * Trims the capacity of this <tt>OtpOutputStream</tt> instance to be the
+     * Trims the capacity of this <code>OtpOutputStream</code> instance to be the
      * buffer's current size. An application can use this operation to minimize
-     * the storage of an <tt>OtpOutputStream</tt> instance.
+     * the storage of an <code>OtpOutputStream</code> instance.
      */
     public void trimToSize() {
         resize(super.count);
@@ -125,7 +125,7 @@ public class OtpOutputStream extends ByteArrayOutputStream {
     }
 
     /**
-     * Increases the capacity of this <tt>OtpOutputStream</tt> instance, if
+     * Increases the capacity of this <code>OtpOutputStream</code> instance, if
      * necessary, to ensure that it can hold at least the number of elements
      * specified by the minimum capacity argument.
      *
@@ -713,7 +713,7 @@ public class OtpOutputStream extends ByteArrayOutputStream {
      */
     public void write_pid(final String node, final int id, final int serial,
             final int creation) {
-	write1(OtpExternal.pidTag);
+	write1(OtpExternal.newPidTag);
 	write_atom(node);
 	write4BE(id & 0x7fff); // 15 bits
 	write4BE(serial & 0x1fff); // 13 bits
@@ -727,20 +727,11 @@ public class OtpOutputStream extends ByteArrayOutputStream {
      *            the pid
      */
     public void write_pid(OtpErlangPid pid) {
-	write1(pid.tag());
+	write1(OtpExternal.newPidTag);
 	write_atom(pid.node());
 	write4BE(pid.id());
 	write4BE(pid.serial());
-	switch (pid.tag()) {
-	case OtpExternal.pidTag:
-	    write1(pid.creation());
-	    break;
-	case OtpExternal.newPidTag:
-	    write4BE(pid.creation());
-	    break;
-	default:
-	    throw new AssertionError("Invalid pid tag " + pid.tag());
-	}
+        write4BE(pid.creation());
     }
 
 
@@ -758,7 +749,7 @@ public class OtpOutputStream extends ByteArrayOutputStream {
      *            be used.
      */
     public void write_port(final String node, final int id, final int creation) {
-	write1(OtpExternal.portTag);
+	write1(OtpExternal.newPortTag);
 	write_atom(node);
 	write4BE(id & 0xfffffff); // 28 bits
 	write1(creation & 0x3); // 2 bits
@@ -771,19 +762,10 @@ public class OtpOutputStream extends ByteArrayOutputStream {
      *            the port.
      */
     public void write_port(OtpErlangPort port) {
-	write1(port.tag());
+	write1(OtpExternal.newPortTag);
 	write_atom(port.node());
 	write4BE(port.id());
-	switch (port.tag()) {
-	case OtpExternal.portTag:	    
-	    write1(port.creation());
-	    break;
-	case OtpExternal.newPortTag:
-	    write4BE(port.creation());
-	    break;
-	default:
-	    throw new AssertionError("Invalid port tag " + port.tag());
-	}
+        write4BE(port.creation());
     }
 
     /**
@@ -829,7 +811,7 @@ public class OtpOutputStream extends ByteArrayOutputStream {
             arity = 3; // max 3 words in ref
         }
 
-	write1(OtpExternal.newRefTag);
+	write1(OtpExternal.newerRefTag);
 
 	// how many id values
 	write2BE(arity);
@@ -857,24 +839,12 @@ public class OtpOutputStream extends ByteArrayOutputStream {
 	int[] ids = ref.ids();
         int arity = ids.length;
 
-	write1(ref.tag());
+	write1(OtpExternal.newerRefTag);
 	write2BE(arity);
 	write_atom(ref.node());
+        write4BE(ref.creation());
 
-	switch (ref.tag()) {
-	case OtpExternal.newRefTag:
-	    write1(ref.creation());
-	    write4BE(ids[0] & 0x3ffff); // first word gets truncated to 18 bits
-	    break;
-	case OtpExternal.newerRefTag:
-	    write4BE(ref.creation());
-	    write4BE(ids[0]); // full first word
-	    break;
-	default:
-	    throw new AssertionError("Invalid ref tag " + ref.tag());
-	}
-
-	for (int i = 1; i < arity; i++) {
+	for (int i = 0; i < arity; i++) {
 	    write4BE(ids[i]);
 	}
     }
@@ -939,7 +909,7 @@ public class OtpOutputStream extends ByteArrayOutputStream {
      * @param o
      *            the Erlang term to write.
      * @param level
-     *            the compression level (<tt>0..9</tt>)
+     *            the compression level (<code>0..9</code>)
      */
     public void write_compressed(final OtpErlangObject o, final int level) {
         @SuppressWarnings("resource")
