@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1997-2017. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2018. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -133,7 +133,7 @@ log(Log, Term) ->
 blog(Log, Bytes) ->
     req(Log, {log, external, [ensure_binary(Bytes)]}).
 
--spec log_terms(Log, TermList) -> ok | {error, Resaon :: log_error_rsn()} when
+-spec log_terms(Log, TermList) -> ok | {error, Reason :: log_error_rsn()} when
       Log :: log(),
       TermList :: [term()].
 log_terms(Log, Terms) ->
@@ -266,7 +266,7 @@ inc_wrap_file(Log) ->
       Size :: dlog_size(),
       Reason :: no_such_log | nonode | {read_only_mode, Log}
               | {blocked_log, Log}
-              | {new_size_too_small, CurrentSize :: pos_integer()}
+              | {new_size_too_small, Log, CurrentSize :: pos_integer()}
               | {badarg, size}
               | {file_error, file:filename(), file_error()}.
 change_size(Log, NewSize) -> 
@@ -763,7 +763,7 @@ handle({From, {truncate, Head, F, A}}=Message, S) ->
 		ok ->
 		    erase(is_full),
 		    notify_owners({truncated, S#state.cnt}),
-		    N = if Head =:= none -> 0; true -> 1 end,
+		    N = if H =:= none -> 0; true -> 1 end,
 		    reply(From, ok, (state_ok(S))#state{cnt = N});
 		Error ->
 		    do_exit(S, From, Error, ?failure(Error, F, A))
