@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1998-2018. All Rights Reserved.
+%% Copyright Ericsson AB 1998-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -29,19 +29,22 @@
 -define(port_please_failure2(Term), noop).
 -endif.
 
+-include("dist.hrl").
+
 -ifndef(erlang_daemon_port).
 -define(erlang_daemon_port, 4369).
 -endif.
 -ifndef(epmd_dist_high).
--define(epmd_dist_high, 4370).
+-define(epmd_dist_high, ?ERL_DIST_VER_HIGH).
 -endif.
 -ifndef(epmd_dist_low).
--define(epmd_dist_low, 4370).
+-define(epmd_dist_low, ?ERL_DIST_VER_LOW).
 -endif.
 
 %% External exports
--export([start/0, start_link/0, stop/0, port_please/2, 
-	 port_please/3, names/0, names/1,
+-export([start/0, start_link/0, stop/0,
+         port_please/2, port_please/3, listen_port_please/2,
+         names/0, names/1,
 	 register_node/2, register_node/3, address_please/3, open/0, open/1, open/2]).
 
 %% gen_server callbacks
@@ -55,7 +58,7 @@
 
 -include("inet_int.hrl").
 -include("erl_epmd.hrl").
-
+-include_lib("kernel/include/inet.hrl").
 
 %%%----------------------------------------------------------------------
 %%% API
@@ -76,7 +79,7 @@ stop() ->
 %% return {port, P, Version} | noport
 %%
 
--spec port_please(Name, Host) -> {ok, Port, Version} | noport when
+-spec port_please(Name, Host) -> {ok, Port, Version} | noport | closed | {error, term()} when
 	  Name :: atom() | string(),
 	  Host :: atom() | string() | inet:ip_address(),
 	  Port :: non_neg_integer(),
@@ -85,36 +88,61 @@ stop() ->
 port_please(Node, Host) ->
   port_please(Node, Host, infinity).
 
--spec port_please(Name, Host, Timeout) -> {ok, Port, Version} | noport when
+-spec port_please(Name, Host, Timeout) -> {port, Port, Version} | noport | closed | {error, term()} when
 	  Name :: atom() | string(),
 	  Host :: atom() | string() | inet:ip_address(),
 	  Timeout :: non_neg_integer() | infinity,
 	  Port :: non_neg_integer(),
 	  Version :: non_neg_integer().
 
-port_please(Node,HostName, Timeout) when is_atom(HostName) ->
-  port_please1(Node,atom_to_list(HostName), Timeout);
-port_please(Node,HostName, Timeout) when is_list(HostName) ->
-  port_please1(Node,HostName, Timeout);
-port_please(Node, EpmdAddr, Timeout) ->
-  get_port(Node, EpmdAddr, Timeout).
+port_please(Node, HostName, Timeout) ->
+    case listen_port_please(Node, HostName) of
+        {ok, 0} ->
+            case getepmdbyname(HostName, Timeout) of
+                {ok, EpmdAddr} ->
+                    get_port(Node, EpmdAddr, Timeout);
+                _Error ->
+                    ?port_please_failure2(_Error),
+                    noport
+            end;
+        {ok, Prt} ->
+            %% We don't know which dist version the other node is running
+            %% so return the low version so that we can talk to older nodes
+            {port, Prt, ?epmd_dist_low}
+    end.
 
+getepmdbyname(HostName, Timeout) when is_atom(HostName) ->
+    getepmdbyname(atom_to_list(HostName), Timeout);
+getepmdbyname(HostName, Timeout) when is_list(HostName) ->
+    Family = case inet_db:res_option(inet6) of
+                 true ->
+                     inet6;
+                 false ->
+                     inet
+             end,
+    case inet:gethostbyname(HostName, Family, Timeout) of
+        {ok,#hostent{ h_addr_list = [EpmdAddr | _]}} ->
+            {ok, EpmdAddr};
+        Else ->
+            Else
+    end;
+getepmdbyname(HostName, _Timeout) ->
+    {ok, HostName}.
 
-
-port_please1(Node,HostName, Timeout) ->
-  Family = case inet_db:res_option(inet6) of
-             true ->
-               inet6;
-             false ->
-               inet
-           end,
-  case inet:gethostbyname(HostName, Family, Timeout) of
-    {ok,{hostent, _Name, _ , _Af, _Size, [EpmdAddr | _]}} ->
-      get_port(Node, EpmdAddr, Timeout);
-    _Else ->
-      ?port_please_failure2(_Else),
-      noport
-  end.
+-spec listen_port_please(Name, Host) -> {ok, Port} when
+      Name :: atom(),
+      Host :: string() | inet:ip_address(),
+      Port :: non_neg_integer().
+listen_port_please(_Name, _Host) ->
+    try
+        %% Should come up with a new name for this as ERL_EPMD_PORT describes what
+        %% port epmd runs on which could easily be confused with this.
+        {ok, [[StringPort]]} = init:get_argument(erl_epmd_port),
+        Port = list_to_integer(StringPort),
+        {ok, Port}
+    catch error:_ ->
+            {ok, 0}
+    end.
 
 -spec names() -> {ok, [{Name, Port}]} | {error, Reason} when
 	  Name :: string(),
@@ -131,15 +159,13 @@ names() ->
       Port :: non_neg_integer(),
       Reason :: address | file:posix().
 
-names(HostName) when is_atom(HostName); is_list(HostName) ->
-  case inet:gethostbyname(HostName) of
-    {ok,{hostent, _Name, _ , _Af, _Size, [EpmdAddr | _]}} ->
-      get_names(EpmdAddr);
-    Else ->
-      Else
-  end;
-names(EpmdAddr) ->
-  get_names(EpmdAddr).
+names(HostName) ->
+    case getepmdbyname(HostName, infinity) of
+        {ok,EpmdAddr} ->
+            get_names(EpmdAddr);
+        Else ->
+            Else
+    end.
 
 -spec register_node(Name, Port) -> Result when
 	  Name :: string(),
@@ -154,7 +180,7 @@ register_node(Name, PortNo) ->
 	  Name :: string(),
 	  Port :: non_neg_integer(),
 	  Driver :: inet_tcp | inet6_tcp | inet | inet6,
-	  Creation :: non_neg_integer(),
+	  Creation :: non_neg_integer() | -1,
 	  Result :: {ok, Creation} | {error, already_registered} | term().
 
 register_node(Name, PortNo, inet_tcp) ->
@@ -204,8 +230,15 @@ handle_call({register, Name, PortNo, Family}, _From, State) ->
 				    port_no = PortNo,
 				    name = Name},
 		    {reply, {ok, Creation}, S};
-		Error ->
-		    {reply, Error, State}
+                Error ->
+                    case init:get_argument(erl_epmd_port) of
+                        {ok, _} ->
+                            {reply, {ok, -1}, State#state{ socket = -1,
+                                                           port_no = PortNo,
+                                                           name = Name} };
+                        error ->
+                            {reply, Error, State}
+                    end
 	    end;
 	_ ->
 	    {reply, {error, already_registered}, State}
@@ -310,7 +343,7 @@ do_register_node(NodeName, TcpPort, Family) ->
                     Error
             end;
 	Error ->
-	    Error
+            Error
     end.
 
 epmd_dist_high() ->
@@ -347,6 +380,13 @@ wait_for_reg_reply(Socket, SoFar) ->
     receive
 	{tcp, Socket, Data0} ->
 	    case SoFar ++ Data0 of
+		[$v, Result, A, B, C, D] ->
+		    case Result of
+			0 ->
+			    {alive, Socket, ?u32(A, B, C, D)};
+			_ ->
+			    {error, duplicate_name}
+		    end;
 		[$y, Result, A, B] ->
 		    case Result of
 			0 ->
@@ -383,9 +423,8 @@ get_port(Node, EpmdAddress, Timeout) ->
 		    ?port_please_failure2(_Error),
 		    noport
 	    end;
-	_Error -> 
-	    ?port_please_failure2(_Error),
-	    noport
+	_Error ->
+            noport
     end.
 
 
