@@ -45,7 +45,8 @@
 	 monitor_nodes_many/1,
          monitor_nodes_down_up/1,
          dist_ctrl_proc_smoke/1,
-         dist_ctrl_proc_reject/1]).
+         dist_ctrl_proc_reject/1,
+         erl_1424/1]).
 
 %% Performs the test at another node.
 -export([get_socket_priorities/0,
@@ -83,7 +84,8 @@ all() ->
      dyn_node_name,
      hidden_node, setopts,
      table_waste, net_setuptime, inet_dist_options_options,
-     {group, monitor_nodes}].
+     {group, monitor_nodes},
+     erl_1424].
 
 groups() -> 
     [{monitor_nodes, [],
@@ -216,7 +218,7 @@ nodenames(Config) when is_list(Config) ->
     legal("a_1@b"),
 
     %% Test that giving two -sname works as it should
-    test_node("a_1@b", false, long_or_short() ++ "a_0@b"),
+    started = test_node("a_1@b", false, long_or_short() ++ "a_0@b"),
 
     illegal("cdé@a"),
     illegal("te欢st@a").
@@ -274,8 +276,8 @@ test_node(Name, Illigal) ->
     test_node(Name, Illigal, "").
 test_node(Name, Illigal, ExtraArgs) ->
     ProgName = ct:get_progname(),
-    Command = ProgName ++ " -noinput " ++ ExtraArgs ++
-        long_or_short() ++ Name ++
+    Command = ProgName ++ " -noinput " ++
+        long_or_short() ++ Name ++ ExtraArgs ++
         " -eval \"net_adm:ping('" ++ atom_to_list(node()) ++ "')\"" ++
         case Illigal of
             true ->
@@ -1703,6 +1705,11 @@ smoke_communicate(Node, OLoopMod, OLoopFun) ->
     exit(Pid, kill),
     ok.
 
+
+erl_1424(Config) when is_list(Config) ->
+    {error, Reason} = erl_epmd:names("."),
+    {comment, lists:flatten(io_lib:format("Reason: ~p", [Reason]))}.
+
 %% Misc. functions
 
 run_dist_configs(Func, Config) ->
