@@ -23,7 +23,9 @@
 %% because udp is not deterministic.
 %%
 -module(gen_udp_SUITE).
+
 -include_lib("common_test/include/ct.hrl").
+-include("kernel_test_lib.hrl").
 
 
 %% XXX - we should pick a port that we _know_ is closed. That's pretty hard.
@@ -43,6 +45,9 @@
 	 local_fdopen/1, local_fdopen_unbound/1, local_abstract/1,
          recv_close/1]).
 
+
+-define(TRY_TC(F), try_tc(F)).
+               
 suite() ->
     [{ct_hooks,[ts_install_cth]},
      {timetrap,{minutes,1}}].
@@ -80,11 +85,36 @@ local_cases() ->
      local_abstract
     ].
 
-init_per_suite(Config) ->
-    Config.
+init_per_suite(Config0) ->
 
-end_per_suite(_Config) ->
-    ok.
+    ?P("init_per_suite -> entry with"
+       "~n      Config: ~p"
+       "~n      Nodes:  ~p", [Config0, erlang:nodes()]),
+
+    case ?LIB:init_per_suite(Config0) of
+        {skip, _} = SKIP ->
+            SKIP;
+
+        Config1 when is_list(Config1) ->
+            
+            ?P("init_per_suite -> end when "
+               "~n      Config: ~p", [Config1]),
+            
+            Config1
+    end.
+
+end_per_suite(Config0) ->
+
+    ?P("end_per_suite -> entry with"
+       "~n      Config: ~p"
+       "~n      Nodes:  ~p", [Config0, erlang:nodes()]),
+
+    Config1 = ?LIB:end_per_suite(Config0),
+
+    ?P("end_per_suite -> "
+            "~n      Nodes: ~p", [erlang:nodes()]),
+
+    Config1.
 
 init_per_group(local, Config) ->
     case gen_udp:open(0, [local]) of
@@ -103,6 +133,9 @@ end_per_group(_GroupName, Config) ->
     Config.
 
 
+init_per_testcase(read_packets, Config) ->
+    ct:timetrap({minutes, 2}),
+    Config;
 init_per_testcase(_Case, Config) ->
     Config.
 
@@ -330,24 +363,33 @@ bad_address(Config) when is_list(Config) ->
 
 %% OTP-6249 UDP option for number of packet reads.
 read_packets(Config) when is_list(Config) ->
-    N1 = 5,
-    N2 = 1,
+    N1   = 5,
+    N2   = 1,
     Msgs = 30000,
-    {ok,R} = gen_udp:open(0, [{read_packets,N1}]),
-    {ok,RP} = inet:port(R),
+    ?P("open socket (with read-packets: ~p)", [N1]),
+    {ok, R}   = gen_udp:open(0, [{read_packets,N1}]),
+    {ok, RP}  = inet:port(R),
+    ?P("create slave node"),
     {ok,Node} = start_node(gen_udp_SUITE_read_packets),
     %%
+    ?P("perform read-packets test"),
     {V1, Trace1} = read_packets_test(R, RP, Msgs, Node),
+    ?P("verify read-packets (to ~w)", [N1]),
     {ok,[{read_packets,N1}]} = inet:getopts(R, [read_packets]),
     %%
-    ok = inet:setopts(R, [{read_packets,N2}]),
+    ?P("set new read-packets: ~p", [N2]),
+    ok = inet:setopts(R, [{read_packets, N2}]),
+    ?P("perform read-packets test"),
     {V2, Trace2} = read_packets_test(R, RP, Msgs, Node),
+    ?P("verify read-packets (to ~w)", [N2]),
     {ok,[{read_packets,N2}]} = inet:getopts(R, [read_packets]),
     %%
+    ?P("stop slave node"),
     stop_node(Node),
-    ct:log("N1=~p, V1=~p vs N2=~p, V2=~p",[N1,V1,N2,V2]),
 
+    ?P("dump trace 1"),
     dump_terms(Config, "trace1.terms", Trace1),
+    ?P("dump trace 2"),
     dump_terms(Config, "trace2.terms", Trace2),
 
     %% Because of the inherit racy-ness of the feature it is
@@ -357,6 +399,12 @@ read_packets(Config) when is_list(Config) ->
     %% the max number of executions a port is allowed to
     %% do before being re-scheduled is N * 20
 
+    ?P("read-packets test verification when: "
+       "~n      N1: ~p"
+       "~n      V1: ~p"
+       "~n   vs"
+       "~n      N2: ~p"
+       "~n      V2: ~p", [N1, V1, N2, V2]),
     if
         V1 > (N1 * 20) ->
             ct:fail("Got ~p msgs, max was ~p", [V1, N1]);
@@ -364,7 +412,9 @@ read_packets(Config) when is_list(Config) ->
             ct:fail("Got ~p msgs, max was ~p", [V2, N2]);
         true ->
             ok
-    end.
+    end,
+    ?P("done"),
+    ok.
 
 dump_terms(Config, Name, Terms) ->
     FName = filename:join(proplists:get_value(priv_dir, Config),Name),
@@ -427,7 +477,7 @@ read_packets_recv(N) ->
 
 read_packets_verify(R, SP, Trace) ->
     [Max | _] = Pkts = lists:reverse(lists:sort(read_packets_verify(R, SP, Trace, 0))),
-    ct:pal("~p",[lists:sublist(Pkts,10)]),
+    ?P("read-packets verify: ~p", [lists:sublist(Pkts,10)]),
     Max.
 
 read_packets_verify(R, SP, [{trace,R,OutIn,_}|Trace], M) 
@@ -644,11 +694,17 @@ recvtclass(_Config) ->
 
 
 sendtos(_Config) ->
+    ?TC_TRY(sendtos, fun() -> do_sendtos() end).
+
+do_sendtos() ->
     test_recv_opts(
       inet, [{recvtos,tos,96}], true,
       fun sendtos_ok/2).
 
 sendtosttl(_Config) ->
+    ?TC_TRY(sendtosttl, fun() -> do_sendtosttl() end).
+
+do_sendtosttl() ->
     test_recv_opts(
       inet, [{recvtos,tos,96},{recvttl,ttl,33}], true,
       fun (OSType, OSVer) ->
@@ -656,11 +712,17 @@ sendtosttl(_Config) ->
       end).
 
 sendttl(_Config) ->
+    ?TC_TRY(sendttl, fun() -> do_sendttl() end).
+
+do_sendttl() ->
     test_recv_opts(
       inet, [{recvttl,ttl,33}], true,
       fun sendttl_ok/2).
 
 sendtclass(_Config) ->
+    ?TC_TRY(sendtclass, fun() -> do_sendtclass() end).
+
+do_sendtclass() ->
     {ok,IFs} = inet:getifaddrs(),
     case
         [Name ||
@@ -672,9 +734,10 @@ sendtclass(_Config) ->
               inet6, [{recvtclass,tclass,224}], true,
               fun sendtclass_ok/2);
         [] ->
-            {skip,ipv6_not_supported,IFs}
+            {skip, {ipv6_not_supported, IFs}}
     end.
 
+
 %% These version numbers are just above the highest noted in daily tests
 %% where the test fails for a plausible reason, that is the lowest
 %% where we can expect that the test might succeed, so
@@ -689,7 +752,7 @@ sendtclass(_Config) ->
 %% Using the option returns einval, so it is not implemented.
 recvtos_ok({unix,darwin}, OSVer) -> not semver_lt(OSVer, {17,6,0});
 %% Using the option returns einval, so it is not implemented.
-recvtos_ok({unix,openbsd}, OSVer) -> not semver_lt(OSVer, {6,6,0});
+recvtos_ok({unix,openbsd}, OSVer) -> not semver_lt(OSVer, {6,8,0});
 %% Using the option returns einval, so it is not implemented.
 recvtos_ok({unix,sunos}, OSVer) -> not semver_lt(OSVer, {5,12,0});
 %%
@@ -718,22 +781,22 @@ recvtclass_ok(_, _) -> false.
 
 %% Using the option returns einval, so it is not implemented.
 sendtos_ok({unix,darwin}, OSVer) -> not semver_lt(OSVer, {19,0,0});
-sendtos_ok({unix,openbsd}, OSVer) -> not semver_lt(OSVer, {6,6,0});
+sendtos_ok({unix,openbsd}, OSVer) -> not semver_lt(OSVer, {6,8,0});
 sendtos_ok({unix,sunos}, OSVer) -> not semver_lt(OSVer, {5,12,0});
 sendtos_ok({unix,linux}, OSVer) -> not semver_lt(OSVer, {4,0,0});
-sendtos_ok({unix,freebsd}, OSVer) -> not semver_lt(OSVer, {12,1,0});
+sendtos_ok({unix,freebsd}, OSVer) -> not semver_lt(OSVer, {12,2,0});
 %%
 sendtos_ok({unix,_}, _) -> true;
 sendtos_ok(_, _) -> false.
 
 %% Using the option returns einval, so it is not implemented.
-sendttl_ok({unix,darwin}, OSVer) -> not semver_lt(OSVer, {19,0,0});
+sendttl_ok({unix,darwin}, OSVer) -> not semver_lt(OSVer, {19,4,0});
 sendttl_ok({unix,linux}, OSVer) -> not semver_lt(OSVer, {4,0,0});
 %% Using the option returns enoprotoopt, so it is not implemented.
-sendttl_ok({unix,freebsd}, OSVer) -> not semver_lt(OSVer, {12,1,0});
+sendttl_ok({unix,freebsd}, OSVer) -> not semver_lt(OSVer, {12,2,0});
 %% Option has no effect
 sendttl_ok({unix,sunos}, OSVer) -> not semver_lt(OSVer, {5,12,0});
-sendttl_ok({unix,openbsd}, OSVer) -> not semver_lt(OSVer, {6,6,0});
+sendttl_ok({unix,openbsd}, OSVer) -> not semver_lt(OSVer, {6,8,0});
 %%
 sendttl_ok({unix,_}, _) -> true;
 sendttl_ok(_, _) -> false.
@@ -748,24 +811,31 @@ sendtclass_ok({unix,_}, _) -> true;
 sendtclass_ok(_, _) -> false.
 
 
-semver_lt({X1,Y1,Z1}, {X2,Y2,Z2}) ->
+semver_lt({X1,Y1,Z1} = V1, {X2,Y2,Z2} = V2) ->
+    ?P("semver_lt -> OS version check:"
+       "~n   Version 1: ~p"
+       "~n   Version 2: ~p", [V1, V2]),
     if
-        X1 > X2 -> false;
-        X1 < X2 -> true;
-        Y1 > Y2 -> false;
-        Y1 < Y2 -> true;
-        Z1 > Z2 -> false;
-        Z1 < Z2 -> true;
-        true -> false
+        X1 > X2 -> ?P("semver_lt -> X1 > X2: ~p > ~p", [X1, X2]), false;
+        X1 < X2 -> ?P("semver_lt -> X1 < X2: ~p < ~p", [X1, X2]), true;
+        Y1 > Y2 -> ?P("semver_lt -> Y1 > Y2: ~p > ~p", [Y1, Y2]), false;
+        Y1 < Y2 -> ?P("semver_lt -> Y1 < Y2: ~p < ~p", [Y1, Y2]), true;
+        Z1 > Z2 -> ?P("semver_lt -> Z1 > Z2: ~p > ~p", [Z1, Z2]), false;
+        Z1 < Z2 -> ?P("semver_lt -> Z1 < Z2: ~p < ~p", [Z1, Z2]), true;
+        true    -> ?P("semver_lt -> default"), false
     end;
-semver_lt(_, {_,_,_}) -> false.
+semver_lt(V1, {_,_,_} = V2) ->
+    ?P("semver_lt -> fallback OS version check when: "
+       "~n   Version 1: ~p"
+       "~n   Version 2: ~p", [V1, V2]),
+    false.
 
 test_recv_opts(Family, Spec, TestSend, OSFilter) ->
     OSType = os:type(),
-    OSVer = os:version(),
+    OSVer  = os:version(),
     case OSFilter(OSType, OSVer) of
         true ->
-            io:format("Os: ~p, ~p~n", [OSType,OSVer]),
+            ?P("OS: ~p, ~p", [OSType, OSVer]),
             test_recv_opts(Family, Spec, TestSend, OSType, OSVer);
         false ->
             {skip,{not_supported_for_os_version,{OSType,OSVer}}}
@@ -811,8 +881,18 @@ test_recv_opts(Family, Spec, TestSend, _OSType, _OSVer) ->
     ok = gen_udp:send(S1, Addr, P2, <<"fghij">>),
     TestSend andalso
         begin
-            ok = gen_udp:send(S2, Addr, P1, OptsVals, <<"ABCDE">>),
-            ok = gen_udp:send(S2, {Addr,P1}, OptsVals, <<"12345">>)
+            case gen_udp:send(S2, Addr, P1, OptsVals, <<"ABCDE">>) of
+                ok ->
+                    ok;
+                {error, enoprotoopt = Reason1} ->
+                    ?SKIPT(?F("send (1) failed: ~p", [Reason1]))
+            end,
+            case gen_udp:send(S2, {Addr,P1}, OptsVals, <<"12345">>) of
+                ok ->
+                    ok;
+                {error, enoprotoopt = Reason2} ->
+                    ?SKIPT(?F("send (2) failed: ~p", [Reason2]))
+            end
         end,
     {ok,{_,P2,OptsVals3,<<"abcde">>}} = gen_udp:recv(S1, 0, Timeout),
     verify_sets_eq(OptsVals3, OptsVals2),
@@ -995,14 +1075,15 @@ local_handshake(S, SAddr, C, CAddr) ->
 %% Open a passive socket. Create a socket that reads from it.
 %% Then close the socket.
 recv_close(Config) when is_list(Config) ->
+    ?P("begin"),
     {ok, Sock} = gen_udp:open(0, [{active, false}]),
     RECV = fun() ->
-                   io:format("~p try recv~n", [self()]),
+                   ?P("try recv"),
                    Res = gen_udp:recv(Sock, 0),
-                   io:format("~p recv res: ~p~n", [self(), Res]),
+                   ?P("recv res: ~p", [Res]),
                    exit(Res)
            end,
-    io:format("~p spawn reader", [self()]),
+    ?P("spawn reader"),
     {Pid, MRef} = spawn_monitor(RECV),
     receive
         {'DOWN', MRef, process, Pid, PreReason} ->
@@ -1012,68 +1093,69 @@ recv_close(Config) when is_list(Config) ->
     after 5000 -> % Just in case...
             ok
     end,
-    io:format("~p close socket", [self()]),
+    ?P("close socket"),
     ok = gen_udp:close(Sock),
-    io:format("~p await reader termination", [self()]),
+    ?P("await reader termination"),
     receive
         {'DOWN', MRef, process, Pid, {error, closed}} ->
-            io:format("~p expected reader termination result", [self()]),
+            ?P("expected reader termination result"),
             ok;
         {'DOWN', MRef, process, Pid, PostReason} ->
-            io:format("~p unexpected reader termination: ~p",
-                      [self(), PostReason]),
+            ?P("unexpected reader termination: ~p", [PostReason]),
             ?line ct:fail("Unexpected post close from reader (~p): ~p",
                           [Pid, PostReason])
     after 5000 ->
-            io:format("~p unexpected reader termination timeout", [self()]),
+            ?P("unexpected reader termination timeout"),
             demonitor(MRef, [flush]),
             exit(Pid, kill),
             ?line ct:fail("Reader (~p) termination timeout", [Pid])
     end,
+    ?P("done"),
     ok.
 
 
 
 
-%%
-%% Utils
-%%
-
-start_node(Name) ->
-    Pa = filename:dirname(code:which(?MODULE)),
-    test_server:start_node(Name, slave, [{args, "-pa " ++ Pa}]).
-
-stop_node(Node) ->
-    test_server:stop_node(Node).
-
-
 %% Test that connect/3 has effect.
 connect(Config) when is_list(Config) ->
     Addr = {127,0,0,1},
+    ?P("try create first socket"),
     {ok,S1} = gen_udp:open(0),
     {ok,P1} = inet:port(S1),
+    ?P("try create second socket"),
     {ok,S2} = gen_udp:open(0),
+    ?P("try set second socket active: false"),
     ok = inet:setopts(S2, [{active,false}]),
+    ?P("try close first socket"),
     ok = gen_udp:close(S1),
+    ?P("try connect second socket to: ~p, ~p", [Addr, P1]),
     ok = gen_udp:connect(S2, Addr, P1),
+    ?P("try send on second socket"),
     ok = gen_udp:send(S2, <<16#deadbeef:32>>),
+    ?P("try recv on second socket - expect failure"),
     ok = case gen_udp:recv(S2, 0, 500) of
-	     {error,econnrefused} -> ok;
-	     {error,econnreset} -> ok;
-	     Other -> Other
+	     {error, econnrefused} -> ok;
+	     {error, econnreset}   -> ok;
+	     Other -> 
+                 ?P("UNEXPECTED failure: ~p", [Other]),
+                 Other
 	 end,
+    ?P("done"),
     ok.
 
 implicit_inet6(Config) when is_list(Config) ->
+    ?TC_TRY(implicit_inet6, fun() -> do_implicit_inet6(Config) end).
+
+do_implicit_inet6(_Config) ->
     Host = ok(inet:gethostname()),
     case inet:getaddr(Host, inet6) of
-	{ok,{16#fe80,0,0,0,_,_,_,_} = Addr} ->
+	{ok, {16#fe80,0,0,0,_,_,_,_} = Addr} ->
 	    {skip,
 	     "Got link local IPv6 address: "
 	     ++inet:ntoa(Addr)};
-	{ok,Addr} ->
+	{ok, Addr} ->
 	    implicit_inet6(Host, Addr);
-	{error,Reason} ->
+	{error, Reason} ->
 	    {skip,
 	     "Can not look up IPv6 address: "
 	     ++atom_to_list(Reason)}
@@ -1082,40 +1164,75 @@ implicit_inet6(Config) when is_list(Config) ->
 implicit_inet6(Host, Addr) ->
     Active = {active,false},
     Loopback = {0,0,0,0,0,0,0,1},
-    case gen_udp:open(0, [inet6,Active,{ip, Loopback}]) of
-	{ok,S1} ->
-	    io:format("~s ~p~n", ["::1",Loopback]),
-	    implicit_inet6(S1, Active, Loopback),
-	    ok = gen_udp:close(S1),
-	    %%
-	    Localaddr = ok(get_localaddr()),
-	    S2 = ok(gen_udp:open(0, [{ip,Localaddr},Active])),
-	    implicit_inet6(S2, Active, Localaddr),
-	    ok = gen_udp:close(S2),
-	    %%
-	    io:format("~s ~p~n", [Host,Addr]),
-	    S3 = ok(gen_udp:open(0, [{ifaddr,Addr},Active])),
-	    implicit_inet6(S3, Active, Addr),
-	    ok = gen_udp:close(S3);
-	_ ->
-	    {skip,"IPv6 not supported"}
-    end.
+    ?P("try 1 with explit inet6 on loopback"),
+    S1 = case gen_udp:open(0, [inet6, Active, {ip, Loopback}]) of
+             {ok, Sock1} ->
+                 Sock1;
+             {error, eaddrnotavail = Reason1} ->
+                 ?SKIPT(open_failed_str(Reason1));
+             _ ->
+                 ?SKIPT("IPv6 not supported")
+         end,
+    implicit_inet6(S1, Active, Loopback),
+    ok = gen_udp:close(S1),
+    %%
+    Localaddr = ok(get_localaddr()),
+    ?P("try 2 on local addr (~p)", [Localaddr]),
+    S2 = case gen_udp:open(0, [{ip, Localaddr}, Active]) of
+             {ok, Sock2} ->
+                 Sock2;
+             {error, eaddrnotavail = Reason2} ->
+                 ?SKIPT(open_failed_str(Reason2))
+         end,
+    implicit_inet6(S2, Active, Localaddr),
+    ok = gen_udp:close(S2),
+    %%
+    ?P("try 3 on addr ~p (~p)", [Addr, Host]),
+    S3 = case gen_udp:open(0, [{ifaddr, Addr}, Active]) of
+             {ok, Sock3} ->
+                 Sock3;
+             {error, eaddrnotavail = Reason3} ->
+                 ?SKIPT(open_failed_str(Reason3))
+         end,
+    implicit_inet6(S3, Active, Addr),
+    ok = gen_udp:close(S3),
+    ok.
 
 implicit_inet6(S1, Active, Addr) ->
+    ?P("get (\"local\") port number"),
     P1 = ok(inet:port(S1)),
-    S2 = ok(gen_udp:open(0, [inet6,Active])),
+    ?P("open \"remote\" socket"),
+    S2 = case gen_udp:open(0, [inet6, Active]) of
+             {ok, Sock2} ->
+                 Sock2;
+             {error, eaddrnotavail = Reason3} ->
+                 ?SKIPT(open_failed_str(Reason3))
+         end,
+    ?P("get (\"remote\") port number"),
     P2 = ok(inet:port(S2)),
+    ?P("connect (\"remote\") socket (to ~p:~p)", [Addr, P1]),
     ok = gen_udp:connect(S2, Addr, P1),
+    ?P("connect (\"local\") socket (to ~p:~p)", [Addr, P2]),
     ok = gen_udp:connect(S1, Addr, P2),
+    ?P("peername of \"local\" socket"),
     {Addr,P2} = ok(inet:peername(S1)),
+    ?P("peername of \"remote\" socket"),
     {Addr,P1} = ok(inet:peername(S2)),
+    ?P("sockname of \"local\" socket"),
     {Addr,P1} = ok(inet:sockname(S1)),
+    ?P("sockname of \"remote\" socket"),
     {Addr,P2} = ok(inet:sockname(S2)),
+    ?P("send ping on \"local\" socket (to ~p:~p)", [Addr, P2]),
     ok = gen_udp:send(S1, Addr, P2, "ping"),
+    ?P("recv ping on \"remote\" socket (from ~p:~p)", [Addr, P1]),
     {Addr,P1,"ping"} = ok(gen_udp:recv(S2, 1024, 1000)),
+    ?P("send pong on \"remote\" socket (to ~p:~p)", [Addr, P1]),
     ok = gen_udp:send(S2, Addr, P1, "pong"),
+    ?P("recv ping on \"local\" socket (from ~p:~p)", [Addr, P2]),
     {Addr,P2,"pong"} = ok(gen_udp:recv(S1, 1024)),
-    ok = gen_udp:close(S2).
+    ?P("close \"remote\" socket"),
+    ok = gen_udp:close(S2),
+    ok.
 
 ok({ok,V}) -> V;
 ok(NotOk) ->
@@ -1150,8 +1267,28 @@ get_localaddr([]) ->
 get_localaddr([Localhost|Ls]) ->
     case inet:getaddr(Localhost, inet6) of
        {ok, LocalAddr} ->
-           io:format("~s ~p~n", [Localhost, LocalAddr]),
+           ?P("found local address: ~s ~p", [Localhost, LocalAddr]),
            {ok, LocalAddr};
        _ ->
            get_localaddr(Ls)
     end.
+
+
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+%%
+%% Utils
+%%
+
+start_node(Name) ->
+    Pa = filename:dirname(code:which(?MODULE)),
+    test_server:start_node(Name, slave, [{args, "-pa " ++ Pa}]).
+
+stop_node(Node) ->
+    test_server:stop_node(Node).
+
+
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+open_failed_str(Reason) ->
+    ?F("Open failed: ~w", [Reason]).
