@@ -100,7 +100,7 @@ do_tracer(Clients,PI,Traci) ->
 				 {ok, H} = inet:gethostname(),
 				 H;
 			     _ ->
-				 [_,H] = string:tokens(atom_to_list(N),"@"),
+				 [_,H] = string:lexemes(atom_to_list(N),"@"),
 				 H
 			 end,
 		  case catch dbg:tracer(N,port,dbg:trace_port(ip,IpPortSpec)) of
@@ -976,7 +976,7 @@ decode_filename(3,Bin,latin1) ->
     File.
 
 host(Node) ->
-    [_name,Host] = string:tokens(atom_to_list(Node),"@"),
+    [_name,Host] = string:lexemes(atom_to_list(Node),"@"),
     Host.
 
 wait_for_fetch([]) ->
@@ -1064,7 +1064,7 @@ collect_files(Dirs) ->
     lists:map(fun(Dir) ->
                       MetaFiles = filelib:wildcard(filename:join(Dir,"*.ti")),
                       lists:map(fun(M) ->
-                                        Sub = string:left(M,length(M)-3),
+                                        Sub = filename:rootname(M,".ti"),
                                         case filelib:is_file(Sub) of
                                             true -> Sub;
                                             false -> Sub++".*.wrp"
