@@ -26,10 +26,164 @@
 
 -include_lib("public_key/include/public_key.hrl").
 
-%% Note: This directive should only be used in test suites.
--compile(export_all).
+-export([
+         %% CT callbaks:
+         suite/0,
+         all/0,
+         groups/0,
+         init_per_suite/1,
+         end_per_suite/1,
+         init_per_group/2,
+         end_per_group/2,
+         init_per_testcase/2,
+         end_per_testcase/2,
+
+         %% Test cases:
+         attrib_name_chain/1,
+         attrib_name_chain/0,
+         basic_invalid/1,
+         basic_invalid/0,
+         basic_valid/1,
+         basic_valid/0,
+         capitalization_name_chain/1,
+         capitalization_name_chain/0,
+         crl_signing_invalid/1,
+         crl_signing_invalid/0,
+         crl_signing_valid/1,
+         crl_signing_valid/0,
+         delta_without_crl/1,
+         delta_without_crl/0,
+         fresh_CRL/1,
+         fresh_CRL/0,
+         invalid_CRL/1,
+         invalid_CRL/0,
+         invalid_CRL_issuer/1,
+         invalid_CRL_issuer/0,
+         invalid_CRL_signature/1,
+         invalid_CRL_signature/0,
+         invalid_DN_and_rfc822_name_constraints/1,
+         invalid_DN_and_rfc822_name_constraints/0,
+         invalid_DN_name_constraints/1,
+         invalid_DN_name_constraints/0,
+         invalid_crl_issuer/1,
+         invalid_crl_issuer/0,
+         invalid_delta_crls/1,
+         invalid_delta_crls/0,
+         invalid_distribution_points/1,
+         invalid_distribution_points/0,
+         invalid_dns_name_constraints/1,
+         invalid_dns_name_constraints/0,
+         invalid_dsa_signature/1,
+         invalid_dsa_signature/0,
+         invalid_indirect_crl/1,
+         invalid_indirect_crl/0,
+         invalid_key_usage/1,
+         invalid_key_usage/0,
+         invalid_name_chain/1,
+         invalid_name_chain/0,
+         invalid_only_contains/1,
+         invalid_only_contains/0,
+         invalid_only_some_reasons/1,
+         invalid_only_some_reasons/0,
+         invalid_path_constraints/1,
+         invalid_path_constraints/0,
+         invalid_rfc822_name_constraints/1,
+         invalid_rfc822_name_constraints/0,
+         invalid_rsa_signature/1,
+         invalid_rsa_signature/0,
+         invalid_separate_keys/1,
+         invalid_separate_keys/0,
+         invalid_serial/1,
+         invalid_serial/0,
+         invalid_uri_name_constraints/1,
+         invalid_uri_name_constraints/0,
+         missing_CRL/1,
+         missing_CRL/0,
+         missing_basic_constraints/1,
+         missing_basic_constraints/0,
+         not_after_invalid/1,
+         not_after_invalid/0,
+         not_after_valid/1,
+         not_after_valid/0,
+         not_before_invalid/1,
+         not_before_invalid/0,
+         not_before_valid/1,
+         not_before_valid/0,
+         old_CRL/1,
+         old_CRL/0,
+         revoked_CA/1,
+         revoked_CA/0,
+         revoked_peer/1,
+         revoked_peer/0,
+         string_name_chain/1,
+         string_name_chain/0,
+         uid_name_chain/1,
+         uid_name_chain/0,
+         unknown_CRL_extension/1,
+         unknown_CRL_extension/0,
+         unknown_critical_extension/1,
+         unknown_critical_extension/0,
+         unknown_not_critical_extension/1,
+         unknown_not_critical_extension/0,
+         valid_CRL/1,
+         valid_CRL/0,
+         valid_DN_and_rfc822_name_constraints/1,
+         valid_DN_and_rfc822_name_constraints/0,
+         valid_DN_name_constraints/1,
+         valid_DN_name_constraints/0,
+         valid_basic_constraint/1,
+         valid_basic_constraint/0,
+         valid_crl_issuer/1,
+         valid_crl_issuer/0,
+         valid_delta_crls/1,
+         valid_delta_crls/0,
+         valid_distribution_points/1,
+         valid_distribution_points/0,
+         valid_distribution_points_no_issuing_distribution_point/1,
+         valid_distribution_points_no_issuing_distribution_point/0,
+         valid_dns_name_constraints/1,
+         valid_dns_name_constraints/0,
+         valid_dsa_signature/1,
+         valid_dsa_signature/0,
+         valid_indirect_crl/1,
+         valid_indirect_crl/0,
+         valid_key_usage/1,
+         valid_key_usage/0,
+         valid_only_contains/1,
+         valid_only_contains/0,
+         valid_only_some_reasons/1,
+         valid_only_some_reasons/0,
+         valid_path_constraints/1,
+         valid_path_constraints/0,
+         valid_rfc822_name_constraints/1,
+         valid_rfc822_name_constraints/0,
+         valid_rsa_signature/1,
+         valid_rsa_signature/0,
+         valid_seperate_keys/1,
+         valid_seperate_keys/0,
+         valid_serial/1,
+         valid_serial/0,
+         valid_uri_name_constraints/1,
+         valid_uri_name_constraints/0,
+         whitespace_name_chain/1,
+         whitespace_name_chain/0,
+
+         %% Marked as "Not supported yet":
+         certificate_policies/0,
+         certificate_policies/1,
+         require_explicit_policy/0,
+         require_explicit_policy/1,
+         policy_mappings/0,
+         policy_mappings/1,
+         inhibit_policy_mapping/0,
+         inhibit_policy_mapping/1,
+         inhibit_any_policy/0,
+         inhibit_any_policy/1
+        ]).
 
 -define(error(Format,Args), error(Format,Args,?FILE,?LINE)).
+
+-export([warning/4]).
 -define(warning(Format,Args), warning(Format,Args,?FILE,?LINE)).
 
 -define(CERTS, "pkits/certs").
