@@ -1,7 +1,7 @@
 %% 
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2005-2019. All Rights Reserved.
+%% Copyright Ericsson AB 2005-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -26,7 +26,7 @@
 	 start_v2_agent/1,        start_v2_agent/2, 
 	 start_v3_agent/1,        start_v3_agent/2, 
 	 start_bilingual_agent/1, start_bilingual_agent/2, 
-	 start_mt_agent/1,        start_mt_agent/2, 
+	 start_mt_agent/1,        start_mt_agent/2,         start_mt_agent/3, 
 	 stop_agent/1,
 
 	 %% start_sup/0,      stop_sup/2,
@@ -122,8 +122,8 @@
 
 init_all(Config) when is_list(Config) ->
 
-    ?LOG("init_all -> entry with"
-	 "~n   Config: ~p",[Config]),
+    ?IPRINT("init_all -> entry with"
+            "~n   Config: ~p",[Config]),
 
     %% -- 
     %% Start nodes
@@ -300,7 +300,7 @@ try_test(TcRunMod, TcRunFunc, TcRunArgs, TcRunOpts) ->
 %% process as well.
 
 tc_try(N, M, F, A) ->
-    ?PRINT2("tc_try -> entry with"
+    ?IPRINT("tc_try -> entry with"
             "~n      N:     ~p"
             "~n      M:     ~p"
             "~n      F:     ~p"
@@ -312,32 +312,32 @@ tc_try(N, M, F, A) ->
                    get()]),
     case net_adm:ping(N) of
         pong ->
-            ?PRINT2("tc_try -> ~p still running - start runner~n", [N]),
+            ?IPRINT("tc_try -> ~p still running - start runner~n", [N]),
             OldFlag = trap_exit(true), % Make sure we catch it
             Runner  = spawn_link(N, ?MODULE, tc_wait, [self(), get(), M, F, A]),
             await_tc_runner_started(Runner, OldFlag),
             await_tc_runner_done(Runner, OldFlag);
         pang ->
-            ?EPRINT2("tc_try -> ~p *not* running~n", [N]),
+            ?WPRINT("tc_try -> ~p *not* running~n", [N]),
             skip({node_not_running, N})
     end.
 
 await_tc_runner_started(Runner, OldFlag) ->
-    ?PRINT2("await tc-runner (~p) start ack~n", [Runner]),
+    ?IPRINT("await tc-runner (~p) start ack~n", [Runner]),
     receive
         {'EXIT', Runner, Reason} ->
-            ?EPRINT2("TC runner start failed: "
-                     "~n   ~p~n", [Reason]),
-            exit({tx_runner_start_failed, Reason});
+            ?EPRINT("TC runner start failed: "
+                    "~n   ~p~n", [Reason]),
+            exit({tc_runner_start_failed, Reason});
         {tc_runner_started, Runner} ->
-            ?PRINT2("TC runner start acknowledged~n"),
+            ?IPRINT("TC runner start acknowledged~n"),
             ok
     after 10000 -> %% We should *really* not have to wait this long, but...
             trap_exit(OldFlag),
             unlink_and_flush_exit(Runner),
-            RunnerInfo = process_info(Runner),
-            ?EPRINT2("TC runner start timeout: "
-                     "~n   ~p", [RunnerInfo]),
+            RunnerInfo = ?PINFO(Runner),
+            ?EPRINT("TC runner start timeout: "
+                    "~n   ~p", [RunnerInfo]),
             %% If we don't get a start ack within 10 seconds, we are f*ed
             exit(Runner, kill),
             exit({tc_runner_start, timeout, RunnerInfo})
@@ -345,25 +345,32 @@ await_tc_runner_started(Runner, OldFlag) ->
 
 await_tc_runner_done(Runner, OldFlag) ->
     receive
-        {'EXIT', Runner, Reason} ->
+        {'EXIT', Runner, {udp_error, _} = Reason} ->
+	    ?EPRINT("TC runner failed with an udp error: "
+		    "~n   Reason: ~p"
+		    "~n", [Reason]),
+	    skip([{reason, Reason}]);
+	    
+	{'EXIT', Runner, Reason} ->
             %% This is not a normal (tc) failure (that is the clause below).
             %% Instead the tc runner process crashed, for some reason. So
             %% check if have got any system events, and if so, skip.
             SysEvs = snmp_test_global_sys_monitor:events(),
             if
                 (SysEvs =:= []) ->
-                    ?EPRINT2("TC runner failed: "
-                             "~n   ~p~n", [Reason]),
-                    exit({tx_runner_failed, Reason});
+                    ?EPRINT("TC runner failed: "
+                            "~n   ~p"
+			    "~n", [Reason]),
+                    exit({tc_runner_failed, Reason});
                 true ->
-                    ?EPRINT2("TC runner failed when we got system events: "
-                             "~n   Reason:     ~p"
-                             "~n   Sys Events: ~p"
-                             "~n", [Reason, SysEvs]),
+                    ?WPRINT("TC runner failed when we got system events: "
+                            "~n   Reason:     ~p"
+                            "~n   Sys Events: ~p"
+                            "~n", [Reason, SysEvs]),
                     skip([{reason, Reason}, {system_events, SysEvs}])
             end;
 	{tc_runner_done, Runner, {'EXIT', {skip, Reason}}, Loc} ->
-	    ?PRINT2("call -> done with skip: "
+	    ?WPRINT("call -> done with skip: "
                     "~n   Reason: ~p"
                     "~n   Loc:    ~p"
                     "~n", [Reason, Loc]),
@@ -372,7 +379,7 @@ await_tc_runner_done(Runner, OldFlag) ->
 	    put(test_server_loc, Loc),
 	    skip(Reason);
 	{tc_runner_done, Runner, {'EXIT', Rn}, Loc} ->
-	    ?PRINT2("call -> done with exit: "
+	    ?EPRINT("call -> done with exit: "
                     "~n   Rn:  ~p"
                     "~n   Loc: ~p"
                     "~n", [Rn, Loc]),
@@ -380,10 +387,10 @@ await_tc_runner_done(Runner, OldFlag) ->
             unlink_and_flush_exit(Runner),
 	    put(test_server_loc, Loc),
 	    exit(Rn);
-	{tc_runner_done, Runner, Ret, _Zed} -> 
-	    ?DBG("call -> done:"
-		 "~n   Ret: ~p"
-		 "~n   Zed: ~p", [Ret, _Zed]),
+	{tc_runner_done, Runner, Ret, _Loc} -> 
+	    ?IPRINT("call -> done:"
+                    "~n   Ret: ~p"
+                    "~n   Loc: ~p", [Ret, _Loc]),
             trap_exit(OldFlag),
             unlink_and_flush_exit(Runner),
 	    case Ret of
@@ -409,7 +416,7 @@ unlink_and_flush_exit(Pid) ->
     end.
 
 tc_wait(From, Env, M, F, A) ->
-    ?PRINT2("tc_wait -> entry with"
+    ?IPRINT("tc_wait -> entry with"
             "~n   From: ~p"
             "~n   Env:  ~p"
             "~n   M:    ~p"
@@ -417,9 +424,9 @@ tc_wait(From, Env, M, F, A) ->
             "~n   A:    ~p", [From, Env, M, F, A]),
     From ! {tc_runner_started, self()},
     lists:foreach(fun({K,V}) -> put(K,V) end, Env),
-    ?PRINT2("tc_wait -> env set - now run tc~n"),
+    ?IPRINT("tc_wait -> env set - now run tc~n"),
     Res = (catch apply(M, F, A)),
-    ?PRINT2("tc_wait -> tc run done: "
+    ?IPRINT("tc_wait -> tc run done: "
             "~n   ~p"
             "~n", [Res]),
     From ! {tc_runner_done, self(), Res, get(test_server_loc)},
@@ -437,7 +444,7 @@ tc_wait(From, Env, M, F, A) ->
     end.
 
 tc_run(Mod, Func, Args, Opts) ->
-    ?PRINT2("tc_run -> entry with"
+    ?IPRINT("tc_run -> entry with"
             "~n   Mod:  ~p"
             "~n   Func: ~p"
             "~n   Args: ~p"
@@ -456,7 +463,7 @@ tc_run(Mod, Func, Args, Opts) ->
     ?DBG("tc_run -> Crypto: ~p", [_CryptoRes]),
     StdM        = join(code:priv_dir(snmp), "mibs") ++ "/",
     Vsn         = get(vsn), 
-    ?PRINT2("tc_run -> config:"
+    ?IPRINT("tc_run -> config:"
             "~n   M:           ~p"
             "~n   Vsn:         ~p"
             "~n   Dir:         ~p"
@@ -467,27 +474,33 @@ tc_run(Mod, Func, Args, Opts) ->
             "~n   Community:   ~p"
             "~n   StdM:        ~p"
             "~n", [M,Vsn,Dir,User,SecLevel,EngineID,CtxEngineID,Community,StdM]),
-    case snmp_test_mgr:start([%% {agent, snmp_test_lib:hostname()},
-			      {packet_server_debug, true},
-			      {debug,               true},
-			      {agent,               get(master_host)}, 
-			      {ipfamily,            get(ipfamily)},
-			      {agent_udp,           4000},
-			      {trap_udp,            5000},
-			      {recbuf,              65535},
-			      quiet,
-			      Vsn, 
-			      {community,           Community},
-			      {user,                User},
-			      {sec_level,           SecLevel},
-			      {engine_id,           EngineID},
-			      {context_engine_id,   CtxEngineID},
-			      {dir,                 Dir},
-			      {mibs,                mibs(StdM, M)}]) of
+    case snmp_test_mgr:start_link([%% {agent, snmp_test_lib:hostname()},
+                                   {packet_server_debug, true},
+                                   {debug,               false},
+                                   {agent,               get(master_host)}, 
+                                   {ipfamily,            get(ipfamily)},
+                                   {agent_udp,           4000},
+                                   %% <SEP-TRANSPORTS>
+                                   %% First port is used to request replies
+                                   %% Second port is used for traps sent
+                                   %% by the agent.
+                                   %% {agent_udp,           {4000, 4001}},
+                                   %% </SEP-TRANSPORTS>
+                                   {trap_udp,            5000},
+                                   {recbuf,              65535},
+                                   quiet,
+                                   Vsn, 
+                                   {community,           Community},
+                                   {user,                User},
+                                   {sec_level,           SecLevel},
+                                   {engine_id,           EngineID},
+                                   {context_engine_id,   CtxEngineID},
+                                   {dir,                 Dir},
+                                   {mibs,                mibs(StdM, M)}]) of
 	{ok, _Pid} ->
 	    case (catch apply(Mod, Func, Args)) of
 		{'EXIT', {skip, Reason}} ->
-                    ?EPRINT2("apply skip detected: "
+                    ?WPRINT("apply skip detected: "
                              "~n   ~p", [Reason]),
 		    (catch snmp_test_mgr:stop()),
 		    ?SKIP(Reason);
@@ -500,11 +513,11 @@ tc_run(Mod, Func, Args, Opts) ->
 		    (catch snmp_test_mgr:stop()),
                     if
                         (SysEvs =:= []) ->
-                            ?EPRINT2("TC runner failed: "
-                                     "~n   ~p~n", [Reason]),
+                            ?EPRINT("TC runner failed: "
+                                    "~n   ~p~n", [Reason]),
                             ?FAIL({apply_failed, {Mod, Func, Args}, Reason});
                         true ->
-                            ?EPRINT2("apply exit catched when we got system events: "
+                            ?WPRINT("apply exit catched when we got system events: "
                                      "~n   Reason:     ~p"
                                      "~n   Sys Events: ~p"
                                      "~n", [Reason, SysEvs]),
@@ -516,14 +529,14 @@ tc_run(Mod, Func, Args, Opts) ->
 	    end;
 
 	{error, Reason} ->
-	    ?EPRINT2("Failed starting (test) manager: "
-                     "~n   ~p", [Reason]),
+	    ?EPRINT("Failed starting (test) manager: "
+                    "~n   ~p", [Reason]),
 	    (catch snmp_test_mgr:stop()),
 	    ?line ?FAIL({mgr_start_error, Reason});
 
 	Err ->
-	    ?EPRINT2("Failed starting (test) manager: "
-                     "~n   ~p", [Err]),
+	    ?EPRINT("Failed starting (test) manager: "
+                    "~n   ~p", [Err]),
 	    (catch snmp_test_mgr:stop()),
 	    ?line ?FAIL({mgr_start_failure, Err})
     end.
@@ -560,20 +573,26 @@ start_bilingual_agent(Config, Opts)
   when is_list(Config) andalso is_list(Opts) ->
     start_agent(Config, [v1,v2], Opts).
  
-start_mt_agent(Config) when is_list(Config) ->
-    start_agent(Config, [v2], [{multi_threaded, true}]).
- 
-start_mt_agent(Config, Opts) when is_list(Config) andalso is_list(Opts) ->
-    start_agent(Config, [v2], [{multi_threaded, true}|Opts]).
+start_mt_agent(Config) ->
+    start_mt_agent(Config, true, []).
  
+start_mt_agent(Config, MT) ->
+    start_mt_agent(Config, MT, []).
+
+start_mt_agent(Config, MT, Opts)
+  when is_list(Config) andalso 
+       ((MT =:= true) orelse (MT =:= extended)) andalso 
+       is_list(Opts) ->
+    start_agent(Config, [v2], [{multi_threaded, MT} | Opts]).
+
 start_agent(Config, Vsns) ->
     start_agent(Config, Vsns, []).
 start_agent(Config, Vsns, Opts) -> 
 
-    ?LOG("start_agent -> entry (~p) with"
-	"~n   Config: ~p"
-	"~n   Vsns:   ~p"
-	"~n   Opts:   ~p", [node(), Config, Vsns, Opts]),
+    ?IPRINT("start_agent -> entry (~p) with"
+            "~n   Config: ~p"
+            "~n   Vsns:   ~p"
+            "~n   Opts:   ~p", [node(), Config, Vsns, Opts]),
     
     ?line AgentLogDir  = ?config(agent_log_dir,  Config),
     ?line AgentConfDir = ?config(agent_conf_dir, Config),
@@ -603,17 +622,17 @@ start_agent(Config, Vsns, Opts) ->
 
     process_flag(trap_exit,true),
 
-    ?PRINT2("start_agent -> try start snmp app supervisor", []),
+    ?IPRINT("start_agent -> try start snmp app supervisor", []),
     {ok, AppSup} = snmp_app_sup:start_link(),
     unlink(AppSup),
     ?DBG("start_agent -> snmp app supervisor: ~p", [AppSup]),
 
-    ?PRINT2("start_agent -> try start master agent",[]),
+    ?IPRINT("start_agent -> try start master agent",[]),
     ?line Sup = start_sup(Env), 
     ?line unlink(Sup),
     ?DBG("start_agent -> snmp supervisor: ~p", [Sup]),
 
-    ?PRINT2("start_agent -> try (rpc) start sub agent on ~p", [SaNode]),
+    ?IPRINT("start_agent -> try (rpc) start sub agent on ~p", [SaNode]),
     ?line SaDir = ?config(sa_dir, Config),
     ?line {ok, Sub} = start_sub_sup(SaNode, SaDir),
     ?DBG("start_agent -> done", []),
@@ -813,36 +832,33 @@ merge_agent_options([{Key, _Value} = Opt|Opts], Options) ->
 
 
 stop_agent(Config) when is_list(Config) ->
-    ?PRINT2("stop_agent -> entry with"
+    ?IPRINT("stop_agent -> entry with"
             "~n   Config: ~p",[Config]),
 
 
     %% Stop the sub-agent (the agent supervisor)
     {SubSup, SubPar} = ?config(snmp_sub, Config),
-    ?PRINT2("stop_agent -> attempt to stop sub agent (~p)"
+    ?IPRINT("stop_agent -> attempt to stop sub agent (~p)"
             "~n   Sub Sup info: "
             "~n      ~p"
             "~n   Sub Par info: "
             "~n      ~p",
-            [SubSup,
-             (catch process_info(SubSup)),
-             (catch process_info(SubPar))]),
+            [SubSup, ?PINFO(SubSup), ?PINFO(SubPar)]),
     stop_sup(SubSup, SubPar),
     Config2 = lists:keydelete(snmp_sub, 1, Config),
 
 
     %% Stop the master-agent (the top agent supervisor)
     {MasterSup, MasterPar} = ?config(snmp_sup, Config),
-    ?PRINT2("stop_agent -> attempt to stop master agent (~p)"
+    ?IPRINT("stop_agent -> attempt to stop master agent (~p)"
             "~n   Master Sup: "
             "~n      ~p"
             "~n   Master Par: "
             "~n      ~p"
             "~n   Agent Info: "
             "~n      ~p",
-            [MasterSup, 
-             (catch process_info(MasterSup)),
-             (catch process_info(MasterPar)),
+            [MasterSup,
+             ?PINFO(MasterSup), ?PINFO(MasterPar),
              agent_info(MasterSup)]),
     stop_sup(MasterSup, MasterPar),
     Config3 = lists:keydelete(snmp_sup, 1, Config2),
@@ -850,25 +866,24 @@ stop_agent(Config) when is_list(Config) ->
 
     %% Stop the top supervisor (of the snmp app)
     AppSup = ?config(snmp_app_sup, Config),
-    ?PRINT2("stop_agent -> attempt to app sup ~p"
+    ?IPRINT("stop_agent -> attempt to app sup ~p"
             "~n   App Sup: ~p",
-            [AppSup, 
-             (catch process_info(AppSup))]),
+            [AppSup, ?PINFO(AppSup)]),
     Config4 = lists:keydelete(snmp_app_sup, 1, Config3),
 
 
-    ?PRINT2("stop_agent -> done", []),
+    ?IPRINT("stop_agent -> done", []),
     Config4.
 
 
 start_sup(Env) ->
     case (catch snmp_app_sup:start_agent(normal, Env)) of
 	{ok, S} ->
-	    ?DBG("start_agent -> started, Sup: ~p",[S]),
+	    ?DBG("start_agent -> started, Sup: ~p", [S]),
 	    S;
 	
 	Else ->
-	    ?DBG("start_agent -> unknown result: ~n~p",[Else]),
+	    ?EPRINT("start_agent -> unknown result: ~n~p", [Else]),
 	    %% Get info about the apps we depend on
 	    ?FAIL({start_failed, Else, ?IS_MNESIA_RUNNING()})
     end.
@@ -876,18 +891,18 @@ start_sup(Env) ->
 stop_sup(Pid, _) when (node(Pid) =:= node()) ->
     case (catch process_info(Pid)) of
 	PI when is_list(PI) ->
-	    ?LOG("stop_sup -> attempt to stop ~p", [Pid]),
+	    ?IPRINT("stop_sup -> attempt to stop ~p", [Pid]),
 	    Ref = erlang:monitor(process, Pid),
 	    exit(Pid, kill),
 	    await_stopped(Pid, Ref);
 	{'EXIT', _Reason} ->
-	    ?LOG("stop_sup -> ~p not running", [Pid]),
+	    ?IPRINT("stop_sup -> ~p not running", [Pid]),
 	    ok
     end;
 stop_sup(Pid, _) ->
-    ?LOG("stop_sup -> attempt to stop ~p", [Pid]),
+    ?IPRINT("stop_sup -> attempt to stop ~p", [Pid]),
     Ref = erlang:monitor(process, Pid),
-    ?LOG("stop_sup -> Ref: ~p", [Ref]),
+    ?IPRINT("stop_sup -> Ref: ~p", [Ref]),
     exit(Pid, kill), 
     await_stopped(Pid, Ref).
 
@@ -897,7 +912,7 @@ await_stopped(Pid, Ref) ->
             ?DBG("received down message for ~p", [Pid]),
             ok
     after 10000 ->
-	    ?INF("await_stopped -> timeout for ~p",[Pid]),
+	    ?EPRINT("await_stopped -> timeout for ~p",[Pid]),
 	    erlang:demonitor(Ref),
 	    ?FAIL({failed_stop,Pid})
     end.
@@ -1076,7 +1091,7 @@ io_format_expect(F) ->
     io_format_expect(F, []).
 
 io_format_expect(F, A) ->
-    ?PRINT2("EXPECT " ++ F, A).
+    ?IPRINT("EXPECT " ++ F, A).
     
 
 do_expect(Expect) when is_atom(Expect) ->
@@ -1518,7 +1533,7 @@ get_req(Id, Vars) ->
 
 get_next_req(Vars) ->
     ?DBG("get_next_req -> entry with"
-	 "~n   Vars: ~p",[Vars]),
+	 "~n   Vars: ~p", [Vars]),
     snmp_test_mgr:gn(Vars),
     ?DBG("get_next_req -> await response",[]),
     Response = snmp_test_mgr:receive_response(),
@@ -1529,42 +1544,37 @@ get_next_req(Vars) ->
 %% --- start and stop nodes ---
 
 start_node(Name) ->
-    ?LOG("start_node -> entry with"
-	 "~n   Name: ~p"
-	 "~n when"
-	 "~n   hostname of this node: ~p",
-	 [Name, list_to_atom(?HOSTNAME(node()))]),
-    Pa = filename:dirname(code:which(?MODULE)),
-    ?DBG("start_node -> Pa: ~p",[Pa]),
+    ?IPRINT("start_node -> entry with"
+            "~n   Name: ~p"
+            "~n when"
+            "~n   hostname of this node: ~p",
+            [Name, list_to_atom(?HOSTNAME(node()))]),
 
-    Args = case init:get_argument('CC_TEST') of
-	       {ok, [[]]} ->
-		   " -pa /clearcase/otp/libraries/snmp/ebin ";
-	       {ok, [[Path]]} ->
-		   " -pa " ++ Path;
-	       error ->
-		      ""
-	      end,
-    %% Do not use start_link!!! (the proc that calls this one is tmp)
-    ?DBG("start_node -> Args: ~p~n", [Args]),
-    A = Args ++ " -pa " ++ Pa ++ 
+    Pa = filename:dirname(code:which(?MODULE)),
+    ?DBG("start_node -> Pa: ~p", [Pa]),
+
+    A = " -pa " ++ Pa ++ 
         " -s " ++ atom_to_list(snmp_test_sys_monitor) ++ " start" ++ 
         " -s global sync",
-    case (catch ?START_NODE(Name, A)) of
+    case ?START_NODE(Name, A) of
 	{ok, Node} ->
-	    %% Tell the test_server to not clean up things it never started.
-	    ?DBG("start_node -> Node: ~p",[Node]),
+	    ?DBG("start_node -> Node: ~p", [Node]),
             global:sync(),
 	    {ok, Node};
+	{error, Reason}  -> 
+	    ?WPRINT("start_node -> failed starting node ~p:"
+                    "~n      Reason: ~p", [Name, Reason]),
+	    ?line ?SKIP({failed_start_node, Reason});
 	Else  -> 
-	    ?ERR("start_node -> failed with(other): Else: ~p",[Else]),
+	    ?EPRINT("start_node -> failed starting node ~p:"
+                    "~n      ~p", [Name, Else]),
 	    ?line ?FAIL(Else)
     end.
 
 
 stop_node(Node) ->
-    ?LOG("stop_node -> Node: ~p",[Node]),
-    rpc:cast(Node, erlang, halt, []).
+    ?IPRINT("stop_node -> Node: ~p", [Node]),
+    ?STOP_NODE(Node).
 
 
 %%%-----------------------------------------------------------------
@@ -1575,25 +1585,39 @@ config(Vsns, MgrDir, AgentConfDir, MIp, AIp) ->
     config(Vsns, MgrDir, AgentConfDir, MIp, AIp, inet).
 
 config(Vsns, MgrDir, AgentConfDir, MIp, AIp, IpFamily) ->
-    ?LOG("config -> entry with"
-	 "~n   Vsns:         ~p"
-	 "~n   MgrDir:       ~p"
-	 "~n   AgentConfDir: ~p"
-	 "~n   MIp:          ~p"
-	 "~n   AIp:          ~p"
-	 "~n   IpFamily:     ~p",
-	 [Vsns, MgrDir, AgentConfDir, MIp, AIp, IpFamily]),
+    ?IPRINT("config -> entry with"
+            "~n   Vsns:         ~p"
+            "~n   MgrDir:       ~p"
+            "~n   AgentConfDir: ~p"
+            "~n   MIp:          ~p"
+            "~n   AIp:          ~p"
+            "~n   IpFamily:     ~p",
+            [Vsns, MgrDir, AgentConfDir, MIp, AIp, IpFamily]),
     ?line {Domain, ManagerAddr} =
 	case IpFamily of
 	    inet6 ->
-		Ipv6Domain = transportDomainUdpIpv6,
-		AgentIpv6Addr = {AIp, 4000},
-		ManagerIpv6Addr = {MIp, ?TRAP_UDP},
+		TransportDomain6 = transportDomainUdpIpv6,
+		AgentAddr6       = {AIp, 4000},
+		ManagerAddr6     = {MIp, ?TRAP_UDP},
 		?line ok =
 		    snmp_config:write_agent_snmp_files(
 		      AgentConfDir, Vsns,
-		      Ipv6Domain, ManagerIpv6Addr, AgentIpv6Addr, "test"),
-		{Ipv6Domain, ManagerIpv6Addr};
+		      TransportDomain6, ManagerAddr6, AgentAddr6, "test"),
+		{TransportDomain6, ManagerAddr6};
+	    inet ->
+		TransportDomain4 = transportDomainUdpIpv4,
+                AIp2 = maybe_fix_addr(AIp),
+		ManagerAddr4     = {MIp, ?TRAP_UDP},
+                %% AgentPreTransport  =
+                %%     [#{addr => {AIp2, 4000}, kind => req_responder},
+                %%      #{addr => {AIp2, 4001}, kind => trap_sender}],
+                AgentPreTransport  = [#{addr => {AIp2, 4000}}],
+		?line ok =
+		    snmp_config:write_agent_snmp_files(
+		      AgentConfDir, Vsns,
+		      TransportDomain4, ManagerAddr4, AgentPreTransport,
+                      "test"),
+		{TransportDomain4, ManagerAddr4};
 	    _ ->
 		?line ok =
 		    snmp_config:write_agent_snmp_files(
@@ -1616,6 +1640,12 @@ config(Vsns, MgrDir, AgentConfDir, MIp, AIp, IpFamily) ->
     ?line write_notify_conf(AgentConfDir),
     ok.
 
+maybe_fix_addr(Addr) when is_list(Addr) ->
+    list_to_tuple(Addr);
+maybe_fix_addr(Addr) when is_tuple(Addr) ->
+    Addr.
+
+    
 delete_files(Config) ->
     AgentDir = ?config(agent_dir, Config),
     delete_files(AgentDir, [db, conf]).
@@ -1753,8 +1783,8 @@ rewrite_target_addr_conf(Dir, NewPort) ->
 	{ok, _} -> 
 	    ok;
 	{error, _R} -> 
-	    ?ERR("failure reading file info of "
-		 "target address config file: ~p", [_R]),
+	    ?WPRINT("failure reading file info of "
+                    "target address config file: ~p", [_R]),
 	    ok  
     end,
 
@@ -1778,10 +1808,10 @@ rewrite_target_addr_conf_check(O) ->
 rewrite_target_addr_conf2(NewPort,
 			  {Name, Ip, _Port, Timeout, Retry,
 			   "std_trap", EngineId}) -> 
-    ?LOG("rewrite_target_addr_conf2 -> entry with std_trap",[]),
+    ?IPRINT("rewrite_target_addr_conf2 -> entry with std_trap",[]),
     {Name,Ip,NewPort,Timeout,Retry,"std_trap",EngineId};
 rewrite_target_addr_conf2(_NewPort,O) -> 
-    ?LOG("rewrite_target_addr_conf2 -> entry with "
+    ?IPRINT("rewrite_target_addr_conf2 -> entry with "
 	 "~n   O: ~p",[O]),
     O.
 
@@ -1835,12 +1865,12 @@ display_memory_usage() ->
     MibDbSize  = key1search([db_memory,mib],  Info),
     NodeDbSize = key1search([db_memory,node], Info),
     TreeDbSize = key1search([db_memory,tree], Info),
-    ?INF("Memory usage: "
-	"~n   Tree size:           ~p"
-	"~n   Process memory size: ~p"
-	"~n   Mib db size:         ~p"
-	"~n   Node db size:        ~p"
-	"~n   Tree db size:        ~p", 
+    ?IPRINT("Memory usage: "
+            "~n   Tree size:           ~p"
+            "~n   Process memory size: ~p"
+            "~n   Mib db size:         ~p"
+            "~n   Node db size:        ~p"
+            "~n   Tree db size:        ~p", 
     [TreeSize, ProcMem, MibDbSize, NodeDbSize, TreeDbSize]).
     
 key1search([], Res) ->
@@ -1876,51 +1906,3 @@ join(Dir, File) ->
 skip(R) ->
     exit({skip, R}).
 
-%% await_pdu(To) ->
-%%     await_response(To, pdu).
-%% 
-%% await_trap(To) ->
-%%     await_response(To, trap).
-%% 
-%% await_any(To) ->
-%%     await_response(To, any).
-%% 
-%% 
-%% await_response(To, What) ->
-%%     await_response(To, What, []).
-%% 
-%% await_response(To, What, Stuff) when is_integer(To) andalso (To >= 0) ->
-%%     T = t(),
-%%     receive
-%% 	{snmp_pdu, PDU} when is_record(Trap, pdu) andalso (What =:= pdu) ->
-%% 	    {ok, PDU};
-%% 	{snmp_pdu, Trap} is_when record(Trap, trappdu) andalso (What =:= trap) ->
-%% 	    {ok, Trap};
-%% 	Any when What =:= any ->
-%% 	    {ok, Any};
-%% 	Any ->
-%% 	    %% Recalc time
-%% 	    NewTo = To - (t() - T)
-%% 	    await_reponse(NewTo, What, [{NewTo, Any}|Stuff])
-%%     after To ->
-%% 	    {error, {timeout, Stuff}}
-%%     end;
-%% await_response(_, Stuff) ->
-%%     {error, {timeout, Stuff}}.
-%% 
-%% 
-%% t() ->
-%%     {A,B,C} = os:timestamp(),
-%%     A*1000000000+B*1000+(C div 1000).
-%% 
-%% 
-%% timeout() ->
-%%     timeout(os:type()).
-%% 
-%% timeout(_)       -> 3500.
-    
-
-%% Time in milli seconds
-%% t() ->
-%%     {A,B,C} = os:timestamp(),
-%%     A*1000000000+B*1000+(C div 1000).
