@@ -28,7 +28,6 @@
 	 stop/1, 
 	 send_discovery_pdu/2, 
 	 send_pdu/2, send_msg/4, send_bytes/2,
-	 error/2,
 	 get_pdu/1, set_pdu/2, format_hdr/1]).
 
 %% internal exports
@@ -62,9 +61,29 @@ start_link_packet(
 start_link_packet(
   InHandler, AgentIp, UdpPort, TrapUdp, VsnHdr, Version, Dir, BufSz,
   Dbg, IpFamily) when is_integer(UdpPort) ->
+    do_start_link_packet(InHandler,
+                         AgentIp, UdpPort, TrapUdp,
+                         VsnHdr, Version, Dir, BufSz,
+                         Dbg, IpFamily);
+start_link_packet(InHandler,
+                  AgentIp, {AReqPort, ATrapPort} = UdpPorts, TrapUdp,
+                  VsnHdr, Version, Dir, BufSz,
+                  Dbg, IpFamily) when is_integer(AReqPort) andalso
+                                      is_integer(ATrapPort) ->
+    do_start_link_packet(InHandler,
+                         AgentIp, UdpPorts, TrapUdp,
+                         VsnHdr, Version, Dir, BufSz,
+                         Dbg, IpFamily).
+
+do_start_link_packet(InHandler,
+                     AgentIp, UdpPorts, TrapUdp,
+                     VsnHdr, Version, Dir, BufSz,
+                     Dbg, IpFamily) ->
     Args =
 	[self(),
-	 InHandler, AgentIp, UdpPort, TrapUdp,  VsnHdr, Version, Dir, BufSz,
+	 InHandler,
+         AgentIp, UdpPorts, TrapUdp,
+         VsnHdr, Version, Dir, BufSz,
 	 Dbg, IpFamily],
     proc_lib:start_link(?MODULE, init_packet, Args).
 
@@ -100,7 +119,7 @@ send_bytes(Bytes, PacketPid) ->
 %%--------------------------------------------------
 init_packet(
   Parent,
-  SnmpMgr, AgentIp, UdpPort, TrapUdp, VsnHdr, Version, Dir, BufSz,
+  SnmpMgr, AgentIp, UdpPorts, TrapUdp, VsnHdr, Version, Dir, BufSz,
   DbgOptions, IpFamily) ->
     %% This causes "verbosity printouts" to print (from the
     %% specified level) in the modules we "borrow" from the
@@ -117,7 +136,7 @@ init_packet(
     init_usm(Version, Dir),
     proc_lib:init_ack(Parent, self()),
     ?IPRINT("started"),
-    packet_loop(SnmpMgr, UdpId, AgentIp, UdpPort, VsnHdr, Version, []).
+    packet_loop(SnmpMgr, UdpId, AgentIp, UdpPorts, VsnHdr, Version, []).
 
 init_debug(Dbg) when is_atom(Dbg) ->
     put(debug,Dbg),
@@ -139,7 +158,7 @@ init_debug(DbgOptions) when is_list(DbgOptions) ->
     ok.
 
 
-packet_loop(SnmpMgr, UdpId, AgentIp, UdpPort, VsnHdr, Version, MsgData) ->
+packet_loop(SnmpMgr, UdpId, AgentIp, UdpPorts, VsnHdr, Version, MsgData) ->
     receive
 	{send_discovery_pdu, From, Pdu} ->
 	    d("packet_loop -> received send_discovery_pdu with"
@@ -151,9 +170,10 @@ packet_loop(SnmpMgr, UdpId, AgentIp, UdpPort, VsnHdr, Version, MsgData) ->
 		{M, B} when is_list(B) -> 
 		    put(discovery, {M, From}),
 		    display_outgoing_message(M),
-		    udp_send(UdpId, AgentIp, UdpPort, B)
+                    Port = select_request_port(UdpPorts),
+		    udp_send(UdpId, AgentIp, Port, B)
 	    end,
-	    packet_loop(SnmpMgr, UdpId, AgentIp, UdpPort, VsnHdr, Version, []);
+	    packet_loop(SnmpMgr, UdpId, AgentIp, UdpPorts, VsnHdr, Version, []);
 
 	{send_pdu, Pdu} ->
 	    d("packet_loop -> received send_pdu with"
@@ -161,10 +181,11 @@ packet_loop(SnmpMgr, UdpId, AgentIp, UdpPort, VsnHdr, Version, MsgData) ->
 	    case mk_msg(Version, Pdu, VsnHdr, MsgData) of
 		error ->
 		    ok;
-		B when is_list(B) -> 
-		    udp_send(UdpId, AgentIp, UdpPort, B)
+		B when is_list(B) ->
+                    Port = select_request_port(UdpPorts),
+		    udp_send(UdpId, AgentIp, Port, B)
 	    end,
-	    packet_loop(SnmpMgr,UdpId,AgentIp,UdpPort,VsnHdr,Version,[]);
+	    packet_loop(SnmpMgr, UdpId, AgentIp, UdpPorts, VsnHdr, Version, []);
 
 	{send_msg, Msg, Ip, Udp} ->
 	    d("packet_loop -> received send_msg with"
@@ -173,44 +194,70 @@ packet_loop(SnmpMgr, UdpId, AgentIp, UdpPort, VsnHdr, Version, MsgData) ->
 	      "~n   Udp:  ~p", [Msg,Ip,Udp]),
 	    case catch snmp_pdus:enc_message(Msg) of
 		{'EXIT', Reason} ->
-		    error("Encoding error:"
-			  "~n   Msg:    ~w"
-			  "~n   Reason: ~w",[Msg, Reason]);
+		    ?EPRINT("Encoding error:"
+                            "~n   Msg:    ~w"
+                            "~n   Reason: ~w",[Msg, Reason]);
 		B when is_list(B) -> 
 		    udp_send(UdpId, Ip, Udp, B)
 	    end,
-	    packet_loop(SnmpMgr,UdpId,AgentIp,UdpPort,VsnHdr,Version,[]);
-	{udp, UdpId, Ip, UdpPort, Bytes} ->
+	    packet_loop(SnmpMgr, UdpId, AgentIp, UdpPorts, VsnHdr, Version, []);
+
+	{udp, UdpId, Ip, Port, Bytes} ->
 	    d("packet_loop -> received udp with"
 	      "~n   UdpId:     ~p"
 	      "~n   Ip:        ~p"
-	      "~n   UdpPort:   ~p"
-	      "~n   sz(Bytes): ~p", [UdpId, Ip, UdpPort, sz(Bytes)]),	    
+	      "~n   Port:      ~p (~p)"
+	      "~n   sz(Bytes): ~p", [UdpId, Ip, Port, UdpPorts, sz(Bytes)]),
+            case UdpPorts of
+                Port ->
+                    ok;
+                {Port, _} -> % Should be a (request) response
+                    ok;
+                {_, Port} -> % Should be a trap
+                    ok;
+                _ ->
+                    d("packet_loop -> received packet from unknown port"
+                      "~n   ~p", [Port]),
+                    exit({snmp_packet_from_unknown_port, Port, UdpPorts})
+            end,
 	    MsgData3 = handle_udp_packet(Version, erase(discovery),
-					 UdpId, Ip, UdpPort, Bytes,
+					 UdpId, Ip, Port, Bytes,
 					 SnmpMgr, AgentIp),
-	    packet_loop(SnmpMgr,UdpId,AgentIp,UdpPort,VsnHdr,Version,
+	    packet_loop(SnmpMgr, UdpId, AgentIp, UdpPorts, VsnHdr, Version,
 			MsgData3);
+
         {udp_error, UdpId, Reason} ->
 	    gen_udp:close(UdpId),
 	    exit({udp_error, Reason});
 
 	{send_bytes, B} ->
 	    d("packet_loop -> received send_bytes with"
-	      "~n   sz(B): ~p", [sz(B)]),	    
-	    udp_send(UdpId, AgentIp, UdpPort, B),
-	    packet_loop(SnmpMgr,UdpId,AgentIp,UdpPort,VsnHdr,Version,[]);
+	      "~n   sz(B): ~p", [sz(B)]),
+            Port = select_request_port(UdpPorts),
+	    udp_send(UdpId, AgentIp, Port, B),
+	    packet_loop(SnmpMgr, UdpId, AgentIp, UdpPorts, VsnHdr, Version, []);
+
 	{stop, Pid} ->
 	    d("packet_loop -> received stop from ~p", [Pid]),	    
 	    gen_udp:close(UdpId),
 	    Pid ! {self(), stopped},
 	    exit(normal);
+
 	Other ->
 	    d("packet_loop -> received unknown"
 	      "~n   ~p", [Other]),	    
 	    exit({snmp_packet_got_other, Other})
     end.
 
+select_request_port({Port, _}) when is_integer(Port) ->
+    Port;
+select_request_port(Port) when is_integer(Port) ->
+    Port.
+
+%% select_trap_port({_, Port}) when is_integer(Port) ->
+%%     Port;
+%% select_trap_port(Port) when is_integer(Port) ->
+%%     Port.
 
 handle_udp_packet(_V, undefined, 
 		  UdpId, Ip, UdpPort,
@@ -228,9 +275,13 @@ handle_udp_packet(_V, undefined,
 
     catch
         Class:Error:_ ->
-            error("Decoding error (~w). Bytes: ~w ~n Error: ~w "
-                  "(UDPport: ~w, Ip: ~w)",
-                  [Class, Bytes, Error, UdpPort, Ip]),
+            ?EPRINT("Decoding error: "
+                    "~n   Class: ~w"
+                    "~n   Error: ~p"
+                    "~n   Bytes: ~p"
+                    "~n   Port:  ~w"
+                    "~n   Ip:    ~p)",
+                    [Class, Error, Bytes, UdpPort, Ip]),
             []
     end;
 handle_udp_packet(V, {DiscoReqMsg, From}, _UdpId, _Ip, _UdpPort, 
@@ -271,26 +322,30 @@ handle_v3_message(Mgr, UdpId, Ip, UdpPort, AgentIp,
         catch
             throw:{error, Reason, B}:_ ->
                 udp_send(UdpId, AgentIp, UdpPort, B),
-                error("Decoding (v3) error. Auto-sending Report.\n"
-                      "~n   Reason: ~w "
-                      "(UDPport: ~w, Ip: ~w)",
-                      [Reason, UdpPort, Ip]),
+                ?EPRINT("Decoding (v3) error - Auto-sending Report:"
+                        "~n   Reason: ~p"
+                        "~n   Port:   ~p"
+                        "~n   Ip:     ~p",
+                        [Reason, UdpPort, Ip]),
                 [];
 
             throw:{error, Reason}:_ ->
-                error("Decoding (v3) error. "
-                      "~n   Bytes: ~w"
-                      "~n   Reason: ~w "
-                      "(UDPport: ~w, Ip: ~w)",
-                      [Bytes, Reason, UdpPort, Ip]),
+                ?EPRINT("Decoding (v3) error:"
+                        "~n   Reason: ~p"
+                        "~n   Bytes:  ~p"
+                        "~n   Port:   ~p"
+                        "~n   Ip:     ~p",
+                        [Reason, Bytes, UdpPort, Ip]),
                 [];
 
             Class:Error:_ ->
-                error("Decoding (v3) error (~w). "
-                      "~n   Bytes: ~w"
-                      "~n   Error: ~w "
-                      "(UDPport: ~w, Ip: ~w)",
-                      [Class, Bytes, Error, UdpPort, Ip]),
+                ?EPRINT("Decoding (v3) error:"
+                        "~n   Class: ~p"
+                        "~n   Error: ~p"
+                        "~n   Bytes: ~p"
+                        "~n   Port:  ~p"
+                        "~n   Ip:    ~p",
+                        [Class, Error, Bytes, UdpPort, Ip]),
                 []
 
     end.
@@ -325,11 +380,13 @@ handle_v1_or_v2_message(Mgr, _UdpId, Ip, UdpPort, _AgentIp,
 
     catch
         Class:Error:_ ->
-            error("Decoding (v1 or v2) error (~w): "
-                  "~n   Bytes: ~w"
-                  "~n   Error: ~w "
-                  "(UDPport: ~w, Ip: ~w)",
-                  [Class, Bytes, Error, UdpPort, Ip])
+            ?EPRINT("Decoding (v1 or v2) error: "
+                    "~n   Class: ~p"
+                    "~n   Error: ~p "
+                    "~n   Bytes: ~p"
+                    "~n   Port:  ~p"
+                    "~n   Ip:    ~p",
+                    [Class, Error, Bytes, UdpPort, Ip])
     end.
 
 
@@ -462,12 +519,6 @@ generate_v3_report_msg(_MsgID, _MsgSecurityModel, Data, ErrorInfo) ->
 	   undefined).
 
 
-error(Format, Data) ->
-    io:format("*** Error ***~n"),
-    ok = io:format(Format, Data),
-    io:format("~n").
-
-
 mk_discovery_msg('version-3', Pdu, _VsnHdr, UserName) ->
     ScopedPDU = #scopedPdu{contextEngineID = "",
 			   contextName     = "",
@@ -492,9 +543,9 @@ mk_discovery_msg('version-3', Pdu, _VsnHdr, UserName) ->
     Msg = #message{version = 'version-3', vsn_hdr = Hdr, data = Bytes},
     case (catch snmp_pdus:enc_message_only(Msg)) of
 	{'EXIT', Reason} ->
-	    error("Discovery encoding error: "
-		  "~n   Pdu:    ~w"
-		  "~n   Reason: ~w",[Pdu, Reason]),
+	    ?EPRINT("Discovery encoding error: "
+                    "~n   Pdu:    ~p"
+                    "~n   Reason: ~p", [Pdu, Reason]),
 	    error;
 	L when is_list(L) ->
 	    {Msg#message{data = ScopedPDU}, L}
@@ -503,9 +554,9 @@ mk_discovery_msg(Version, Pdu, {Com, _, _, _, _}, _UserName) ->
     Msg = #message{version = Version, vsn_hdr = Com, data = Pdu},
     case catch snmp_pdus:enc_message(Msg) of
 	{'EXIT', Reason} ->
-	    error("Discovery encoding error:"
-		  "~n   Pdu:    ~w"
-		  "~n   Reason: ~w",[Pdu, Reason]),
+	    ?EPRINT("Discovery encoding error:"
+                    "~n   Pdu:    ~p"
+                    "~n   Reason: ~p", [Pdu, Reason]),
 	    error;
 	L when is_list(L) -> 
 	    {Msg, L}
@@ -551,14 +602,14 @@ mk_msg('version-3', Pdu, {Context, User, EngineID, CtxEngineId, SecLevel},
     case catch snmpa_usm:generate_outgoing_msg(Message, SecEngineID,
 					       SecName, SecData, SecLevel) of
 	{'EXIT', Reason} ->
-	    error("version-3 message encoding exit"
-		  "~n   Pdu:    ~w"
-		  "~n   Reason: ~w",[Pdu, Reason]),
+	    ?EPRINT("version-3 message encoding exit"
+                    "~n   Pdu:    ~p"
+                    "~n   Reason: ~p", [Pdu, Reason]),
 	    error;
 	{error, Reason} ->
-	    error("version-3 message encoding error"
-		  "~n   Pdu:    ~w"
-		  "~n   Reason: ~w",[Pdu, Reason]),
+	    ?EPRINT("version-3 message encoding error"
+                    "~n   Pdu:    ~p"
+                    "~n   Reason: ~p", [Pdu, Reason]),
 	    error;
 	Packet ->
 	    Packet
@@ -567,9 +618,9 @@ mk_msg(Version, Pdu, {Com, _User, _EngineID, _Ctx, _SecLevel}, _SecData) ->
     Msg = #message{version = Version, vsn_hdr = Com, data = Pdu},
     case catch snmp_pdus:enc_message(Msg) of
 	{'EXIT', Reason} ->
-	    error("~w encoding error"
-		  "~n   Pdu:    ~w"
-		  "~n   Reason: ~w",[Version, Pdu, Reason]),
+	    ?EPRINT("~w encoding error"
+                    "~n   Pdu:    ~p"
+                    "~n   Reason: ~p", [Version, Pdu, Reason]),
 	    error;
 	B when is_list(B) -> 
 	    B
@@ -590,12 +641,14 @@ vsn('version-2') -> v2c.
 udp_send(UdpId, AgentIp, UdpPort, B) ->
     ?vlog("attempt send message (~w bytes) to ~p", [sz(B), {AgentIp, UdpPort}]),
     case (catch gen_udp:send(UdpId, AgentIp, UdpPort, B)) of
-	{error,ErrorReason} ->
-	    error("failed (error) sending message to ~p:~p: "
-		  "~n   ~p",[AgentIp, UdpPort, ErrorReason]);
-	{'EXIT',ExitReason} ->
-	    error("failed (exit) sending message to ~p:~p:"
-		  "~n   ~p",[AgentIp, UdpPort, ExitReason]);
+	{error, ErrorReason} ->
+	    ?EPRINT("failed (error) sending message to ~p:~p: "
+		  "~n   ~p",[AgentIp, UdpPort, ErrorReason]),
+            error;
+	{'EXIT', ExitReason} ->
+	    ?EPRINT("failed (exit) sending message to ~p:~p:"
+                    "~n   ~p",[AgentIp, UdpPort, ExitReason]),
+            error;
 	_ ->
 	    ok
     end.
