@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2008-2017. All Rights Reserved.
+%% Copyright Ericsson AB 2008-2018. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -26,7 +26,20 @@
 -include("ssh_test_lib.hrl").
 
 %% Note: This directive should only be used in test suites.
--compile(export_all).
+-export([
+         suite/0,
+         all/0,
+         groups/0,
+         init_per_suite/1,
+         end_per_suite/1,
+         init_per_group/2,
+         end_per_group/2
+        ]).
+
+-export([
+         simple_connect/1
+        ]).
+
 
 %%--------------------------------------------------------------------
 %% Common Test interface functions -----------------------------------
@@ -81,7 +94,7 @@ end_per_suite(Config) ->
 %%--------------------------------------------------------------------
 init_per_group(dsa_key, Config) ->
     case lists:member('ssh-dss',
-		      ssh_transport:default_algorithms(public_key)) of
+		      ssh_transport:supported_algorithms(public_key)) of
 	true ->
             start_daemon(Config, 'ssh-dss', "dsa_private_key.pem");
 	false ->
@@ -89,7 +102,7 @@ init_per_group(dsa_key, Config) ->
     end;
 init_per_group(rsa_key, Config) ->
     case lists:member('ssh-rsa',
-		      ssh_transport:default_algorithms(public_key)) of
+		      ssh_transport:supported_algorithms(public_key)) of
 	true ->
             start_daemon(Config, 'ssh-rsa', "rsa_private_key.pem");
 	false ->
@@ -102,9 +115,11 @@ start_daemon(Config, KeyType, KeyId) ->
     KeyCBOpts = [{engine, proplists:get_value(engine,Config)},
                  {KeyType, FullKeyId}
                 ],
-    Opts = [{key_cb, {ssh_key_cb_engine_keys, KeyCBOpts}}],
+    Opts = [{key_cb, {ssh_key_cb_engine_keys, KeyCBOpts}},
+            {modify_algorithms, [{append, [{public_key,[KeyType]}]}]}
+           ],
     {Pid, Host, Port} = ssh_test_lib:std_daemon(Config, Opts),
-    [{host_port,{Host,Port}}, {daemon_pid,Pid}| Config].
+    [{host_port,{Host,Port}}, {daemon_pid,Pid}, {key_type,KeyType}| Config].
 
 
 end_per_group(_, Config) ->
@@ -118,7 +133,11 @@ end_per_group(_, Config) ->
 %% A simple exec call
 simple_connect(Config) ->
     {Host,Port} = proplists:get_value(host_port, Config),
-    CRef = ssh_test_lib:std_connect(Config, Host, Port, []),
+    KeyType = proplists:get_value(key_type, Config),
+    Opts = [
+            {modify_algorithms, [{append, [{public_key,[KeyType]}]}]}
+           ],
+    CRef = ssh_test_lib:std_connect(Config, Host, Port, Opts),
     ssh:close(CRef).
 
 %%--------------------------------------------------------------------
@@ -126,10 +145,17 @@ simple_connect(Config) ->
 load_engine() ->
     case crypto:get_test_engine() of
         {ok, Engine} ->
-            try crypto:engine_load(<<"dynamic">>,
+            try
+                %% The test engine has it's own fake rsa sign/verify that
+                %% you don't want to use, so exclude it from methods to load:
+                Methods = 
+                    crypto:engine_get_all_methods() -- [engine_method_rsa],
+                crypto:engine_load(<<"dynamic">>,
                                    [{<<"SO_PATH">>, Engine},
                                     <<"LOAD">>],
-                                   [])
+                                   [],
+                                   Methods
+                                  )
             catch
                 error:notsup ->
                     {error, notsup}
@@ -139,8 +165,3 @@ load_engine() ->
             {error, Error}
     end.
 
-start_std_daemon(Opts, Config) ->
-    ct:log("starting std_daemon",[]),
-    {Pid, Host, Port} = ssh_test_lib:std_daemon(Config, Opts),
-    ct:log("started ~p:~p  ~p",[Host,Port,Opts]),
-    [{srvr_pid,Pid},{srvr_addr,{Host,Port}} | Config].
