@@ -21,8 +21,26 @@
 %%
 -module(ssh_sftpd_erlclient_SUITE).
 
-%% Note: This directive should only be used in test suites.
--compile(export_all).
+-export([
+         suite/0,
+         all/0,
+         groups/0,
+         init_per_suite/1,
+         end_per_suite/1,
+         init_per_group/2,
+         end_per_group/2,
+         init_per_testcase/2,
+         end_per_testcase/2
+        ]).
+
+-export([
+         close_file/1,
+         file_cb/1,
+         list_dir_limited/1,
+         quit/1,
+         root_dir/1,
+         ver6_basic/1
+        ]).
 
 -include_lib("common_test/include/ct.hrl").
 -include_lib("kernel/include/file.hrl").
@@ -58,24 +76,18 @@ init_per_suite(Config) ->
        begin
 	   catch ssh:stop(),
 	   DataDir = proplists:get_value(data_dir, Config),
-	   PrivDir = proplists:get_value(priv_dir, Config),
 	   FileAlt = filename:join(DataDir, "ssh_sftpd_file_alt.erl"),
 	   c:c(FileAlt),
 	   FileName = filename:join(DataDir, "test.txt"),
 	   {ok, FileInfo} = file:read_file_info(FileName),
 	   ok = file:write_file_info(FileName,
 				     FileInfo#file_info{mode = 8#400}),
-	   ssh_test_lib:setup_rsa(DataDir, PrivDir),
-	   ssh_test_lib:setup_dsa(DataDir, PrivDir),
+           ct:log("Pub keys setup for: ~p",
+                  [ssh_test_lib:setup_all_user_host_keys(Config)]),
 	   Config
        end).
 
-end_per_suite(Config) -> 
-    UserDir = filename:join(proplists:get_value(priv_dir, Config), nopubkey),
-    file:del_dir(UserDir),
-    SysDir = proplists:get_value(priv_dir, Config),
-    ssh_test_lib:clean_rsa(SysDir),
-    ssh_test_lib:clean_dsa(SysDir),
+end_per_suite(_Config) -> 
     ok.
 
 %%--------------------------------------------------------------------
@@ -86,52 +98,45 @@ init_per_group(_GroupName, Config) ->
 end_per_group(_GroupName, Config) ->
 	Config.
 %%--------------------------------------------------------------------
-
 init_per_testcase(TestCase, Config) ->
     ssh:start(),
-    PrivDir = proplists:get_value(priv_dir, Config),
-    SystemDir = filename:join(PrivDir, system),
+    UserDir = PrivDir = proplists:get_value(priv_dir, Config),
+    SysDir = filename:join(PrivDir,"system"),
 
     Options =
 	case atom_to_list(TestCase) of
 	    "file_cb" ++ _ ->
-		Spec =
-		    ssh_sftpd:subsystem_spec([{file_handler,
-					       ssh_sftpd_file_alt}]),
-		[{system_dir, SystemDir},
-		 {user_dir, PrivDir},
-		 {subsystems, [Spec]}];
+		Spec = ssh_sftpd:subsystem_spec([{file_handler,
+                                                  ssh_sftpd_file_alt}]),
+		[{subsystems, [Spec]}];
 	    "root_dir" ->
-		Privdir = proplists:get_value(priv_dir, Config),
-		Root = filename:join(Privdir, root),
+                PrivDir = proplists:get_value(priv_dir, Config),
+		Root = filename:join(PrivDir, root),
 		file:make_dir(Root),
 		Spec = ssh_sftpd:subsystem_spec([{root,Root}]),
-		[{system_dir, SystemDir},
-		 {user_dir, PrivDir},
-		 {subsystems, [Spec]}];
+		[{subsystems, [Spec]}];
 	    "list_dir_limited" ->
-		Spec =
-		    ssh_sftpd:subsystem_spec([{max_files,1}]),
-		[{system_dir, SystemDir},
-		 {user_dir, PrivDir},
-		 {subsystems, [Spec]}];
+		Spec = ssh_sftpd:subsystem_spec([{max_files,1}]),
+		[{subsystems, [Spec]}];
 	    "ver6_basic" ->
-		Spec =
-		    ssh_sftpd:subsystem_spec([{sftpd_vsn, 6}]),
-		[{system_dir, SystemDir},
-		 {user_dir, PrivDir},
-		 {subsystems, [Spec]}];
+		Spec = ssh_sftpd:subsystem_spec([{sftpd_vsn, 6}]),
+		[{subsystems, [Spec]}];
 	    _ ->
-		[{user_dir, PrivDir},
-		 {system_dir, SystemDir}]
+		[]
 	end,
 
-    {Sftpd, Host, Port} = ssh_test_lib:daemon(Options),
+    {Sftpd, Host, Port} = ssh_test_lib:daemon([{preferred_algorithms,
+                                                ssh_transport:supported_algorithms()},
+                                               {system_dir, SysDir},
+                                               {user_dir, UserDir},
+                                               {user_passwords, [{?USER,?PASSWD}]}
+                                               | Options]),
 
     {ok, ChannelPid, Connection} =
 	ssh_sftp:start_channel(Host, Port,
 			       [{silently_accept_hosts, true},
-				{user_dir, PrivDir},
+                                {preferred_algorithms, ssh_transport:supported_algorithms()},
+				{user_dir, UserDir},
 				{timeout, 30000}]),
     TmpConfig = lists:keydelete(sftp, 1, Config),
     NewConfig = lists:keydelete(sftpd, 1, TmpConfig),
@@ -147,10 +152,6 @@ end_per_testcase(_TestCase, Config) ->
 %%--------------------------------------------------------------------
 %% Test cases starts here. -------------------------------------------
 %%--------------------------------------------------------------------
-close_file() ->
-    [{doc, "Test that sftpd closes its fildescriptors after compleating the "
-     "transfer OTP-6350"}].
-
 close_file(Config) when is_list(Config) ->
     DataDir = proplists:get_value(data_dir, Config),
     FileName = filename:join(DataDir, "test.txt"),
@@ -166,12 +167,6 @@ close_file(Config) when is_list(Config) ->
     NumOfPorts = length(erlang:ports()).
 
 %%--------------------------------------------------------------------
-
-quit() ->
-    [{doc, " When the sftp client ends the session the "
-     "server will now behave correctly and not leave the "
-     "client hanging. OTP-6349"}].
-
 quit(Config) when is_list(Config) ->
     DataDir = proplists:get_value(data_dir, Config),
     FileName = filename:join(DataDir, "test.txt"),
@@ -189,6 +184,8 @@ quit(Config) when is_list(Config) ->
     timer:sleep(5000),
     {ok, NewSftp, _Conn} = ssh_sftp:start_channel(Host, Port,
 						 [{silently_accept_hosts, true},
+                                                  {preferred_algorithms,
+                                                   ssh_transport:supported_algorithms()},
 						  {user_dir, UserDir},
 						  {user, ?USER}, {password, ?PASSWD}]),
 
@@ -197,11 +194,6 @@ quit(Config) when is_list(Config) ->
     ok = ssh_sftp:stop_channel(NewSftp).
 
 %%--------------------------------------------------------------------
-
-file_cb() ->
-    [{"Test that it is possible to change the callback module for"
-      " the sftpds filehandling. OTP-6356"}].
-
 file_cb(Config) when is_list(Config) ->
     DataDir = proplists:get_value(data_dir, Config),
     PrivDir =  proplists:get_value(priv_dir, Config),
@@ -264,8 +256,6 @@ list_dir_limited(Config) when is_list(Config) ->
     ct:log("Listing: ~p~n", [Listing]).
 
 %%--------------------------------------------------------------------
-ver6_basic() ->
-    [{doc, "Test some version 6 features"}].
 ver6_basic(Config) when is_list(Config) ->
     PrivDir =  proplists:get_value(priv_dir, Config),
     NewDir = filename:join(PrivDir, "testdir2"),
@@ -273,10 +263,10 @@ ver6_basic(Config) when is_list(Config) ->
     ok =  ssh_sftp:make_dir(Sftp, NewDir),
     %%Test file_is_a_directory
     {error, file_is_a_directory} = ssh_sftp:delete(Sftp, NewDir).
+
 %%--------------------------------------------------------------------
 %% Internal functions ------------------------------------------------
 %%--------------------------------------------------------------------
- 
 alt_file_handler_check(Msg) ->
     receive
 	Msg ->
