@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2007-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -119,7 +119,7 @@ own_alert_txt(#alert{level = Level, description = Description, where = #{line :=
 
 alert_format(Alert) ->
     Txt = alert_txt(Alert),
-    {" ~s\n ", [Txt]}.
+    {" ~s\n", [Txt]}.
 
 alert_txt(#alert{level = Level, description = Description, role = Role}) ->
     "received " ++ string:uppercase(atom_to_list(Role)) ++ " ALERT: " ++
@@ -201,8 +201,8 @@ description_txt(?UNSUPPORTED_EXTENSION) ->
     "Unsupported Extension";
 description_txt(?CERTIFICATE_UNOBTAINABLE) ->
     "Certificate Unobtainable";
-description_txt(?UNRECOGNISED_NAME) ->
-    "Unrecognised Name";
+description_txt(?UNRECOGNIZED_NAME) ->
+    "Unrecognized Name";
 description_txt(?BAD_CERTIFICATE_STATUS_RESPONSE) ->
     "Bad Certificate Status Response";
 description_txt(?BAD_CERTIFICATE_HASH_VALUE) ->
@@ -272,8 +272,8 @@ description_atom(?UNSUPPORTED_EXTENSION) ->
     unsupported_extension;
 description_atom(?CERTIFICATE_UNOBTAINABLE) ->
     certificate_unobtainable;
-description_atom(?UNRECOGNISED_NAME) ->
-    unrecognised_name;
+description_atom(?UNRECOGNIZED_NAME) ->
+    unrecognized_name;
 description_atom(?BAD_CERTIFICATE_STATUS_RESPONSE) ->
     bad_certificate_status_response;
 description_atom(?BAD_CERTIFICATE_HASH_VALUE) ->
