@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2018-2019. All Rights Reserved.
+%% Copyright Ericsson AB 2018-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -26,27 +26,31 @@
 -include("ssl_alert.hrl").
 -include("ssl_handshake.hrl").
 -include("ssl_api.hrl").
+-include("ssl_record.hrl").
+-include("tls_handshake_1_3.hrl").
 
 %% API
--export([start/0, start/1, initialize/2, send_data/2, send_alert/2,
+-export([start/0, start/1, initialize/2, send_data/2,
+         send_post_handshake/2, send_alert/2,
          send_and_ack_alert/2, setopts/2, renegotiate/1, peer_renegotiate/1, downgrade/2,
-         update_connection_state/3, dist_tls_socket/1, dist_handshake_complete/3]).
+         update_connection_state/3,
+         dist_tls_socket/1, dist_handshake_complete/3]).
 
 %% gen_statem callbacks
 -export([callback_mode/0, init/1, terminate/3, code_change/4]).
 -export([init/3, connection/3, handshake/3, death_row/3]).
 
--define(SERVER, ?MODULE).
-
 -record(static,
         {connection_pid,
          role,
          socket,
          socket_options,
-         tracker,
+         trackers,
          transport_cb,
          negotiated_version,
          renegotiate_at,
+         key_update_at,  %% TLS 1.3
+         bytes_sent,     %% TLS 1.3
          connection_monitor,
          dist_handle,
          log_level
@@ -93,6 +97,13 @@ send_data(Pid, AppData) ->
     %% Needs error handling for external API
     call(Pid, {application_data, AppData}).
 
+%%--------------------------------------------------------------------
+-spec send_post_handshake(pid(), #key_update{}) -> ok | {error, term()}.
+%%  Description: Send post handshake data
+%%--------------------------------------------------------------------
+send_post_handshake(Pid, HandshakeData) ->
+    call(Pid, {post_handshake_data, HandshakeData}).
+
 %%--------------------------------------------------------------------
 -spec send_alert(pid(), #alert{}) -> _. 
 %% Description: TLS connection process wants to send an Alert
@@ -200,10 +211,11 @@ init({call, From}, {Pid, #{current_write := WriteState,
                            role := Role,
                            socket := Socket,
                            socket_options := SockOpts,
-                           tracker := Tracker,
+                           trackers := Trackers,
                            transport_cb := Transport,
                            negotiated_version := Version,
                            renegotiate_at := RenegotiateAt,
+                           key_update_at := KeyUpdateAt,
                            log_level := LogLevel}},
      #data{connection_states = ConnectionStates, static = Static0} = StateData0) ->
     Monitor = erlang:monitor(process, Pid),
@@ -214,10 +226,12 @@ init({call, From}, {Pid, #{current_write := WriteState,
                                                 role = Role,
                                                 socket = Socket,
                                                 socket_options = SockOpts,
-                                                tracker = Tracker,
+                                                trackers = Trackers,
                                                 transport_cb = Transport,
                                                 negotiated_version = Version,
                                                 renegotiate_at = RenegotiateAt,
+                                                key_update_at = KeyUpdateAt,
+                                                bytes_sent = 0,
                                                 log_level = LogLevel}},
     {next_state, handshake, StateData, [{reply, From, ok}]};
 init(_, _, _) ->
@@ -239,6 +253,8 @@ connection({call, From}, {application_data, AppData},
         Data ->
             send_application_data(Data, From, ?FUNCTION_NAME, StateData)
     end;
+connection({call, From}, {post_handshake_data, HSData}, StateData) ->
+    send_post_handshake_data(HSData, From, ?FUNCTION_NAME, StateData);
 connection({call, From}, {ack_alert, #alert{} = Alert}, StateData0) ->
     StateData = send_tls_alert(Alert, StateData0),
     {next_state, ?FUNCTION_NAME, StateData,
@@ -255,13 +271,14 @@ connection({call, From}, dist_get_tls_socket,
            #data{static = #static{transport_cb = Transport,
                                   socket = Socket,
                                   connection_pid = Pid,
-                                  tracker = Tracker}} = StateData) ->
-    TLSSocket = tls_connection:socket([Pid, self()], Transport, Socket, Tracker),
+                                  trackers = Trackers}} = StateData) ->
+    TLSSocket = tls_gen_connection:socket([Pid, self()], Transport, Socket, Trackers),
     {next_state, ?FUNCTION_NAME, StateData, [{reply, From, {ok, TLSSocket}}]};
 connection({call, From}, {dist_handshake_complete, _Node, DHandle},
            #data{static = #static{connection_pid = Pid} = Static} = StateData) ->
+    false = erlang:dist_ctrl_set_opt(DHandle, get_size, true),
     ok = erlang:dist_ctrl_input_handler(DHandle, Pid),
-    ok = ssl_connection:dist_handshake_complete(Pid, DHandle),
+    ok = ssl_gen_statem:dist_handshake_complete(Pid, DHandle),
     %% From now on we execute on normal priority
     process_flag(priority, normal),
     {keep_state, StateData#data{static = Static#static{dist_handle = DHandle}},
@@ -271,11 +288,12 @@ connection({call, From}, {dist_handshake_complete, _Node, DHandle},
               [];
           Data ->
               [{next_event, internal,
-               {application_packets,{self(),undefined},erlang:iolist_to_iovec(Data)}}]
+               {application_packets,{self(),undefined},Data}}]
       end]};
 connection(internal, {application_packets, From, Data}, StateData) ->
     send_application_data(Data, From, ?FUNCTION_NAME, StateData);
-%%
+connection(internal, {post_handshake_data, From, HSData}, StateData) ->
+    send_post_handshake_data(HSData, From, ?FUNCTION_NAME, StateData);
 connection(cast, #alert{} = Alert, StateData0) ->
     StateData = send_tls_alert(Alert, StateData0),
     {next_state, ?FUNCTION_NAME, StateData};
@@ -293,7 +311,7 @@ connection(info, dist_data, #data{static = #static{dist_handle = DHandle}}) ->
               [];
           Data ->
               [{next_event, internal,
-               {application_packets,{self(),undefined},erlang:iolist_to_iovec(Data)}}]
+               {application_packets,{self(),undefined},Data}}]
       end};
 connection(info, tick, StateData) ->  
     consume_ticks(),
@@ -324,11 +342,14 @@ handshake({call, _}, _, _) ->
     {keep_state_and_data, [postpone]};
 handshake(internal, {application_packets,_,_}, _) ->
     {keep_state_and_data, [postpone]};
-handshake(cast, {new_write, WritesState, Version}, 
-          #data{connection_states = ConnectionStates, static = Static} = StateData) ->
+handshake(cast, {new_write, WriteState, Version},
+          #data{connection_states = ConnectionStates,
+                static = #static{key_update_at = KeyUpdateAt0} = Static} = StateData) ->
+    KeyUpdateAt = key_update_at(Version, WriteState, KeyUpdateAt0),
     {next_state, connection, 
-     StateData#data{connection_states = ConnectionStates#{current_write => WritesState},
-                    static = Static#static{negotiated_version = Version}}};
+     StateData#data{connection_states = ConnectionStates#{current_write => WriteState},
+                    static = Static#static{negotiated_version = Version,
+                                           key_update_at = KeyUpdateAt}}};
 handshake(info, dist_data, _) ->
     {keep_state_and_data, [postpone]};
 handshake(info, tick, _) ->
@@ -394,16 +415,13 @@ handle_common(
   info, {'DOWN', Monitor, _, _, _},
   #data{static = #static{connection_monitor = Monitor}} = StateData) ->
     {stop, normal, StateData};
-handle_common(info, Msg, _) ->
-    Report =
-        io_lib:format("TLS sender: Got unexpected info: ~p ~n", [Msg]),
-    error_logger:info_report(Report),
+handle_common(info, Msg, #data{static = #static{log_level = Level}}) ->
+    ssl_logger:log(info, Level, #{event => "TLS sender recived unexpected info", 
+                                  reason => [{message, Msg}]}, ?LOCATION),
     keep_state_and_data;
-handle_common(Type, Msg, _) ->
-    Report =
-        io_lib:format(
-          "TLS sender: Got unexpected event: ~p ~n", [{Type,Msg}]),
-    error_logger:error_report(Report),
+handle_common(Type, Msg, #data{static = #static{log_level = Level}}) ->
+    ssl_logger:log(error, Level, #{event => "TLS sender recived unexpected event", 
+                                   reason => [{type, Type}, {message, Msg}]}, ?LOCATION),
     keep_state_and_data.
 
 send_tls_alert(#alert{} = Alert,
@@ -425,30 +443,113 @@ send_application_data(Data, From, StateName,
                                               negotiated_version = Version,
                                               transport_cb = Transport,
                                               renegotiate_at = RenegotiateAt,
+                                              key_update_at = KeyUpdateAt,
+                                              bytes_sent = BytesSent,
                                               log_level = LogLevel},
                              connection_states = ConnectionStates0} = StateData0) ->
-    case time_to_renegotiate(Data, ConnectionStates0, RenegotiateAt) of
-	true ->
-	    ssl_connection:internal_renegotiation(Pid, ConnectionStates0),
+    case time_to_rekey(Version, Data, ConnectionStates0, RenegotiateAt, KeyUpdateAt, BytesSent) of
+        key_update ->
+            KeyUpdate = tls_handshake_1_3:key_update(update_requested),
+            {keep_state_and_data, [{next_event, internal, {post_handshake_data, From, KeyUpdate}},
+                                   {next_event, internal, {key_update, From}},
+                                   {next_event, internal, {application_packets, From, Data}}]};
+	renegotiate ->
+	    tls_dtls_connection:internal_renegotiation(Pid, ConnectionStates0),
             {next_state, handshake, StateData0, 
              [{next_event, internal, {application_packets, From, Data}}]};
+        chunk_and_key_update ->
+            KeyUpdate = tls_handshake_1_3:key_update(update_requested),
+            %% Prevent infinite loop of key updates
+            {Chunk, Rest} = chunk_data(Data, KeyUpdateAt),
+            {keep_state_and_data, [{next_event, internal, {post_handshake_data, From, KeyUpdate}},
+                                   {next_event, internal, {application_packets, From, Chunk}},
+                                   {next_event, internal, {application_packets, From, Rest}}]};
 	false ->
 	    {Msgs, ConnectionStates} = tls_record:encode_data(Data, Version, ConnectionStates0),
             StateData = StateData0#data{connection_states = ConnectionStates},
 	    case tls_socket:send(Transport, Socket, Msgs) of
                 ok when DistHandle =/=  undefined ->
                     ssl_logger:debug(LogLevel, outbound, 'record', Msgs),
-                    {next_state, StateName, StateData, []};
+                    StateData1 = update_bytes_sent(Version, StateData, Data),
+                    {next_state, StateName, StateData1, []};
                 Reason when DistHandle =/= undefined ->
                     {next_state, death_row, StateData, [{state_timeout, 5000, Reason}]};
                 ok ->
                     ssl_logger:debug(LogLevel, outbound, 'record', Msgs),
-                    {next_state, StateName, StateData,  [{reply, From, ok}]};
+                    StateData1 = update_bytes_sent(Version, StateData, Data),
+                    {next_state, StateName, StateData1,  [{reply, From, ok}]};
                 Result ->
                     {next_state, StateName, StateData,  [{reply, From, Result}]}
             end
     end.
 
+%% TLS 1.3 Post Handshake Data
+send_post_handshake_data(Handshake, From, StateName,
+                         #data{static = #static{socket = Socket,
+                                                dist_handle = DistHandle,
+                                                negotiated_version = Version,
+                                                transport_cb = Transport,
+                                                log_level = LogLevel},
+                               connection_states = ConnectionStates0} = StateData0) ->
+    BinHandshake = tls_handshake:encode_handshake(Handshake, Version),
+    {Encoded, ConnectionStates} =
+        tls_record:encode_handshake(BinHandshake, Version, ConnectionStates0),
+    ssl_logger:debug(LogLevel, outbound, 'handshake', Handshake),
+    StateData1 = StateData0#data{connection_states = ConnectionStates},
+    case tls_socket:send(Transport, Socket, Encoded) of
+        ok when DistHandle =/=  undefined ->
+            ssl_logger:debug(LogLevel, outbound, 'record', Encoded),
+            StateData = maybe_update_cipher_key(StateData1, Handshake),
+            {next_state, StateName, StateData, []};
+        Reason when DistHandle =/= undefined ->
+            {next_state, death_row, StateData1, [{state_timeout, 5000, Reason}]};
+        ok ->
+            ssl_logger:debug(LogLevel, outbound, 'record', Encoded),
+            StateData = maybe_update_cipher_key(StateData1, Handshake),
+            {next_state, StateName, StateData,  [{reply, From, ok}]};
+        Result ->
+            {next_state, StateName, StateData1,  [{reply, From, Result}]}
+    end.
+
+maybe_update_cipher_key(#data{connection_states = ConnectionStates0,
+                              static = Static0} = StateData, #key_update{}) ->
+    ConnectionStates = tls_connection_1_3:update_cipher_key(current_write, ConnectionStates0),
+    Static = Static0#static{bytes_sent = 0},
+    StateData#data{connection_states = ConnectionStates,
+                   static = Static};
+maybe_update_cipher_key(StateData, _) ->
+    StateData.
+
+update_bytes_sent(Version, StateData, _) when Version < {3,4} ->
+    StateData;
+%% Count bytes sent in TLS 1.3 for AES-GCM
+update_bytes_sent(_, #data{static = #static{key_update_at = seq_num_wrap}} = StateData, _) ->
+    StateData;  %% Chacha20-Poly1305
+update_bytes_sent(_, #data{static = #static{bytes_sent = Sent} = Static} = StateData, Data) ->
+    StateData#data{static = Static#static{bytes_sent = Sent + iolist_size(Data)}}.  %% AES-GCM
+
+%% For AES-GCM, up to 2^24.5 full-size records (about 24 million) may be
+%% encrypted on a given connection while keeping a safety margin of
+%% approximately 2^-57 for Authenticated Encryption (AE) security.  For
+%% ChaCha20/Poly1305, the record sequence number would wrap before the
+%% safety limit is reached.
+key_update_at(Version, #{security_parameters :=
+                             #security_parameters{
+                                bulk_cipher_algorithm = CipherAlgo}}, KeyUpdateAt)
+  when Version >= {3,4} ->
+    case CipherAlgo of
+        ?AES_GCM ->
+            KeyUpdateAt;
+        ?CHACHA20_POLY1305 ->
+            seq_num_wrap;
+        ?AES_CCM ->
+            KeyUpdateAt;
+        ?AES_CCM_8 ->
+            KeyUpdateAt
+    end;
+key_update_at(_, _, KeyUpdateAt) ->
+    KeyUpdateAt.
+
 -compile({inline, encode_packet/2}).
 encode_packet(Packet, Data) ->
     Len = iolist_size(Data),
@@ -466,9 +567,29 @@ encode_packet(Packet, Data) ->
 set_opts(SocketOptions, [{packet, N}]) ->
     SocketOptions#socket_options{packet = N}.
 
-time_to_renegotiate(_Data, 
-		    #{current_write := #{sequence_number := Num}}, 
-		    RenegotiateAt) ->
+time_to_rekey(Version, _Data,
+              #{current_write := #{sequence_number := ?MAX_SEQUENCE_NUMBER}},
+              _, _, _) when Version >= {3,4} ->
+    key_update;
+time_to_rekey(Version, _Data, _, _, seq_num_wrap, _) when Version >= {3,4} ->
+    false;
+time_to_rekey(Version, Data, _, _, KeyUpdateAt, BytesSent) when Version >= {3,4} ->
+    DataSize = iolist_size(Data),
+    case (BytesSent + DataSize) > KeyUpdateAt of
+        true ->
+            %% Handle special case that causes an invite loop of key updates.
+            case DataSize > KeyUpdateAt of
+                true ->
+                    chunk_and_key_update;
+                false ->
+                    key_update
+                end;
+        false ->
+            false
+    end;
+time_to_rekey(_, _Data,
+              #{current_write := #{sequence_number := Num}},
+              RenegotiateAt, _, _) ->
     
     %% We could do test:
     %% is_time_to_renegotiate((erlang:byte_size(_Data) div
@@ -476,10 +597,14 @@ time_to_renegotiate(_Data,
     %% have a some what lower renegotiateAt and a much cheaper test
     is_time_to_renegotiate(Num, RenegotiateAt).
 
+chunk_data(Data, Size) ->
+    {Chunk, Rest} = split_binary(iolist_to_binary(Data), Size),
+    {[Chunk], [Rest]}.
+
 is_time_to_renegotiate(N, M) when N < M->
     false;
 is_time_to_renegotiate(_,_) ->
-    true.
+    renegotiate.
 
 call(FsmPid, Event) ->
     try gen_statem:call(FsmPid, Event)
@@ -494,7 +619,15 @@ call(FsmPid, Event) ->
 
 %%-------------- Erlang distribution helpers ------------------------------
 
+%% To avoid livelock, dist_data/2 will check for more bytes coming from
+%%  distribution channel, if amount of already collected bytes greater
+%%  or equal than the limit defined below.
+-define(TLS_BUNDLE_SOFT_LIMIT, 16 * 1024 * 1024).
+
 dist_data(DHandle) ->
+    dist_data(DHandle, 0).
+
+dist_data(DHandle, CurBytes) ->
     case erlang:dist_ctrl_get_data(DHandle) of
         none ->
             erlang:dist_ctrl_get_data_notification(DHandle),
@@ -503,15 +636,17 @@ dist_data(DHandle) ->
         %% since the emulator will always deliver a Data
         %% smaller than 4 GB, and the distribution will
         %% therefore always have to use {packet,4}
-        Data when is_binary(Data) ->
-            Len = byte_size(Data),
-            [[<<Len:32>>,Data]|dist_data(DHandle)];
-        [BA,BB] = Data ->
-            Len = byte_size(BA) + byte_size(BB),
-            [[<<Len:32>>|Data]|dist_data(DHandle)];
-        Data when is_list(Data) ->
-            Len = iolist_size(Data),
-            [[<<Len:32>>|Data]|dist_data(DHandle)]
+        {Len, Data} when Len + CurBytes >= ?TLS_BUNDLE_SOFT_LIMIT ->
+            %% Data is of type iovec(); lets keep it
+            %% as an iovec()...
+            erlang:dist_ctrl_get_data_notification(DHandle),
+            [<<Len:32>> | Data];
+        {Len, Data} ->
+            Packet = [<<Len:32>> | Data],
+            case dist_data(DHandle, CurBytes + Len) of
+                [] -> Packet;
+                More -> Packet ++ More
+            end
     end.
 
 
