@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1998-2018. All Rights Reserved.
+%% Copyright Ericsson AB 1998-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -35,7 +35,8 @@
          getstat/3, 
          peername/2, 
          sockname/2, 
-         port/2]).
+         port/2,
+         close/2]).
 
 -export([split_options/1, 
          get_socket_opts/3]).
@@ -48,6 +49,7 @@
          start_link/3, 
          terminate/2, 
          inherit_tracker/3, 
+         session_id_tracker/2,
 	 emulated_socket_options/2, 
          get_emulated_opts/1, 
 	 set_emulated_opts/2, 
@@ -61,7 +63,7 @@
 
 -record(state, {
 	  emulated_opts,
-	  port,
+          listen_monitor,      
 	  ssl_opts
 	 }).
 
@@ -77,9 +79,14 @@ listen(Transport, Port, #config{transport_info = {Transport, _, _, _, _},
     case Transport:listen(Port, Options ++ internal_inet_values()) of
 	{ok, ListenSocket} ->
 	    {ok, Tracker} = inherit_tracker(ListenSocket, EmOpts, SslOpts),
-            %% TODO not hard code
-            {ok, SessionHandler} = session_tickets_tracker(7200, SslOpts),
-            Trackers =  [{option_tracker, Tracker}, {session_tickets_tracker, SessionHandler}],
+            LifeTime = get_ticket_lifetime(),
+            TicketStoreSize = get_ticket_store_size(),
+            %% TLS-1.3 session handling
+            {ok, SessionHandler} = session_tickets_tracker(LifeTime, TicketStoreSize, SslOpts),
+            %% PRE TLS-1.3 session handling
+            {ok, SessionIdHandle} = session_id_tracker(ListenSocket, SslOpts),
+            Trackers =  [{option_tracker, Tracker}, {session_tickets_tracker, SessionHandler},
+                         {session_id_tracker, SessionIdHandle}],
             Socket = #sslsocket{pid = {ListenSocket, Config#config{trackers = Trackers}}},
             check_active_n(EmOpts, Socket),
 	    {ok, Socket};
@@ -101,7 +108,7 @@ accept(ListenSocket, #config{transport_info = {Transport,_,_,_,_} = CbInfo,
 			{SslOpts, emulated_socket_options(EmOpts, #socket_options{}), Trackers}, self(), CbInfo],
 	    case tls_connection_sup:start_child(ConnArgs) of
 		{ok, Pid} ->
-		    ssl_connection:socket_control(ConnectionCb, Socket, [Pid, Sender], Transport, Trackers);
+		    ssl_gen_statem:socket_control(ConnectionCb, Socket, [Pid, Sender], Transport, Trackers);
 		{error, Reason} ->
 		    {error, Reason}
 	    end;
@@ -115,7 +122,7 @@ upgrade(Socket, #config{transport_info = {Transport,_,_,_,_}= CbInfo,
     ok = setopts(Transport, Socket, tls_socket:internal_inet_values()),
     case peername(Transport, Socket) of
 	{ok, {Address, Port}} ->
-	    ssl_connection:connect(ConnectionCb, Address, Port, Socket,
+	    ssl_gen_statem:connect(ConnectionCb, Address, Port, Socket,
 				   {SslOptions, 
 				    emulated_socket_options(EmOpts, #socket_options{}), undefined},
 				   self(), CbInfo, Timeout);
@@ -130,7 +137,7 @@ connect(Address, Port,
     {Transport, _, _, _, _} = CbInfo,
     try Transport:connect(Address, Port,  SocketOpts, Timeout) of
 	{ok, Socket} ->
-	    ssl_connection:connect(ConnetionCb, Address, Port, Socket, 
+	    ssl_gen_statem:connect(ConnetionCb, Address, Port, Socket,
 				   {SslOpts, 
 				    emulated_socket_options(EmOpts, #socket_options{}), undefined},
 				   self(), CbInfo, Timeout);
@@ -232,6 +239,11 @@ port(gen_tcp, Socket) ->
 port(Transport, Socket) ->
     Transport:port(Socket).
 
+close(gen_tcp, Socket) ->
+    inet:close(Socket);
+close(Transport, Socket) ->
+    Transport:close(Socket).
+
 emulated_options() ->
     [mode, packet, active, header, packet_size].
 
@@ -249,17 +261,40 @@ inherit_tracker(ListenSocket, EmOpts, #{erl_dist := false} = SslOpts) ->
 inherit_tracker(ListenSocket, EmOpts, #{erl_dist := true} = SslOpts) ->
     ssl_listen_tracker_sup:start_child_dist([ListenSocket, EmOpts, SslOpts]).
 
-session_tickets_tracker(_, #{erl_dist := false,
-                             session_tickets := disabled}) ->
+session_tickets_tracker(_, _, #{erl_dist := false,
+                                session_tickets := disabled}) ->
     {ok, disabled};
-session_tickets_tracker(Lifetime, #{erl_dist := false,
-                                    session_tickets := Mode,
-                                    anti_replay := AntiReplay}) ->
-    tls_server_session_ticket_sup:start_child([Mode, Lifetime, AntiReplay]);
-session_tickets_tracker(Lifetime, #{erl_dist := true, session_tickets := Mode}) ->
-    tls_server_session_ticket_sup:start_child_dist([Mode, Lifetime]).
-
-
+session_tickets_tracker(Lifetime, TicketStoreSize, 
+                        #{erl_dist := false,
+                          session_tickets := Mode,
+                          anti_replay := AntiReplay}) ->
+    tls_server_session_ticket_sup:start_child([Mode, Lifetime, TicketStoreSize, AntiReplay]);
+session_tickets_tracker(Lifetime, TicketStoreSize,
+                        #{erl_dist := true,
+                          session_tickets := Mode,
+                          anti_replay := AntiReplay}) ->
+    SupName = tls_server_session_ticket_sup:sup_name(dist),
+    Children = supervisor:count_children(SupName),
+    Workers = proplists:get_value(workers, Children),
+    case Workers of
+        0 ->
+            tls_server_session_ticket_sup:start_child([Mode, Lifetime, TicketStoreSize, AntiReplay]);
+        1 ->
+            [{_,Child,_, _}] = supervisor:which_children(SupName),
+            {ok, Child}
+    end.
+session_id_tracker(_, #{versions := [{3,4}]}) ->
+    {ok, not_relevant};
+%% Regardless of the option reuse_sessions we need the session_id_tracker
+%% to generate session ids, but no sessions will be stored unless
+%% reuse_sessions = true.
+session_id_tracker(ssl_unknown_listener, #{erl_dist := false}) ->
+    ssl_upgrade_server_session_cache_sup:start_child(normal);
+session_id_tracker(ListenSocket, #{erl_dist := false}) ->
+    ssl_server_session_cache_sup:start_child(ListenSocket);
+session_id_tracker(_, #{erl_dist := true}) ->
+    ssl_upgrade_server_session_cache_sup:start_child(dist).
+       
 get_emulated_opts(TrackerPid) -> 
     call(TrackerPid, get_emulated_opts).
 set_emulated_opts(TrackerPid, InetValues) -> 
@@ -281,10 +316,12 @@ start_link(Port, SockOpts, SslOpts) ->
 %%
 %% Description: Initiates the server
 %%--------------------------------------------------------------------
-init([Port, Opts, SslOpts]) ->
+init([Listen, Opts, SslOpts]) ->
     process_flag(trap_exit, true),
-    true = link(Port),
-    {ok, #state{emulated_opts = do_set_emulated_opts(Opts, []), port = Port, ssl_opts = SslOpts}}.
+    Monitor = monitor_listen(Listen),
+    {ok, #state{emulated_opts = do_set_emulated_opts(Opts, []), 
+                listen_monitor = Monitor,
+                ssl_opts = SslOpts}}.
 
 %%--------------------------------------------------------------------
 -spec handle_call(msg(), from(), #state{}) -> {reply, reply(), #state{}}. 
@@ -329,7 +366,7 @@ handle_cast(_, State)->
 %%
 %% Description: Handling all non call/cast messages
 %%-------------------------------------------------------------------
-handle_info({'EXIT', Port, _}, #state{port = Port} = State) ->
+handle_info({'DOWN', Monitor, _, _, _}, #state{listen_monitor = Monitor} = State) ->
     {stop, normal, State}.
 
 
@@ -358,6 +395,9 @@ code_change(_OldVsn, State, _Extra) ->
 call(Pid, Msg) ->
     gen_server:call(Pid, Msg, infinity).
 
+monitor_listen(Listen) when is_port(Listen) ->
+    erlang:monitor(port, Listen).
+
 split_options(Opts) ->
     split_options(Opts, emulated_options(), [], []).
 split_options([], _, SocketOpts, EmuOpts) ->
@@ -463,3 +503,20 @@ validate_inet_option(active, Value)
     throw({error, {options, {active,Value}}});
 validate_inet_option(_, _) ->
     ok.
+
+get_ticket_lifetime() ->
+    case application:get_env(ssl, server_session_ticket_lifetime) of
+	{ok, Seconds} when is_integer(Seconds) andalso
+                           Seconds =< 604800 ->  %% MUST be less than 7 days
+	    Seconds;
+	_  ->
+	    7200 %% Default 2 hours
+    end.
+
+get_ticket_store_size() ->
+    case application:get_env(ssl, server_session_ticket_store_size) of
+	{ok, Size} when is_integer(Size) ->
+	    Size;
+	_  ->
+	    1000
+    end.
