@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2007-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -29,22 +29,52 @@
 -include("ssl_internal.hrl").
 -include("ssl_record.hrl").
 
--export([master_secret/4, finished/5, certificate_verify/3, mac_hash/7, hmac_hash/3,
-	 setup_keys/8, suites/1, prf/5,
-	 ecc_curves/1, ecc_curves/2, oid_to_enum/1, enum_to_oid/1, 
-	 default_signature_algs/1, signature_algs/2,
-         default_signature_schemes/1, signature_schemes/2,
-         groups/1, groups/2, group_to_enum/1, enum_to_group/1, default_groups/1]).
+-export([master_secret/4, 
+         finished/5, 
+         certificate_verify/3, 
+         mac_hash/7, 
+         hmac_hash/3,
+	 setup_keys/8, 
+         suites/1, 
+         exclusive_suites/1,
+         prf/5,
+	 ecc_curves/1, 
+         ecc_curves/2, 
+         oid_to_enum/1, 
+         enum_to_oid/1, 
+	 default_signature_algs/1, 
+         signature_algs/2,
+         default_signature_schemes/1, 
+         signature_schemes/2,
+         groups/1, 
+         groups/2, 
+         group_to_enum/1, 
+         enum_to_group/1, 
+         default_groups/1]).
 
--export([derive_secret/4, hkdf_expand_label/5, hkdf_extract/3, hkdf_expand/4,
-         key_schedule/3, key_schedule/4, create_info/3,
-         external_binder_key/2, resumption_binder_key/2,
-         client_early_traffic_secret/3, early_exporter_master_secret/3,
-         client_handshake_traffic_secret/3, server_handshake_traffic_secret/3,
-         client_application_traffic_secret_0/3, server_application_traffic_secret_0/3,
-         exporter_master_secret/3, resumption_master_secret/3,
-         update_traffic_secret/2, calculate_traffic_keys/3,
-         transcript_hash/2, finished_key/2, finished_verify_data/3]).
+-export([derive_secret/4, 
+         hkdf_expand_label/5, 
+         hkdf_extract/3, 
+         hkdf_expand/4,
+         key_schedule/3, 
+         key_schedule/4, 
+         create_info/3,
+         external_binder_key/2, 
+         resumption_binder_key/2,
+         client_early_traffic_secret/3, 
+         early_exporter_master_secret/3,
+         client_handshake_traffic_secret/3, 
+         server_handshake_traffic_secret/3,
+         client_application_traffic_secret_0/3, 
+         server_application_traffic_secret_0/3,
+         exporter_master_secret/3, 
+         resumption_master_secret/3,
+         update_traffic_secret/2, 
+         calculate_traffic_keys/3,
+         transcript_hash/2, 
+         finished_key/2, 
+         finished_verify_data/3, 
+         pre_shared_key/3]).
 
 -type named_curve() :: sect571r1 | sect571k1 | secp521r1 | brainpoolP512r1 |
                        sect409k1 | sect409r1 | brainpoolP384r1 | secp384r1 |
@@ -393,6 +423,15 @@ finished_verify_data(FinishedKey, HKDFAlgo, Messages) ->
     THash = tls_v1:transcript_hash(Context, HKDFAlgo),
     tls_v1:hmac_hash(HKDFAlgo, FinishedKey, THash).
 
+-spec pre_shared_key(binary(), binary(), atom()) -> binary().
+pre_shared_key(RMS, Nonce, Algo) ->
+    %% The PSK associated with the ticket is computed as:
+    %%
+    %%     HKDF-Expand-Label(resumption_master_secret,
+    %%                      "resumption", ticket_nonce, Hash.length)
+    ssl_cipher:hash_size(Algo),
+    hkdf_expand_label(RMS, <<"resumption">>, Nonce, ssl_cipher:hash_size(Algo), Algo).
+
 %% The next-generation application_traffic_secret is computed as:
 %%
 %%        application_traffic_secret_N+1 =
@@ -444,50 +483,68 @@ mac_hash(Method, Mac_write_secret, Seq_num, Type, {Major, Minor},
 
 %% TODO 1.3 same as above?
 
--spec suites(1|2|3|4|'TLS_v1.3') -> [ssl_cipher_format:cipher_suite()].
+-spec suites(1|2|3|4) -> [ssl_cipher_format:cipher_suite()].
 
 suites(Minor) when Minor == 1; Minor == 2 ->
-    [
-      ?TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA,
-      ?TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA,
-      ?TLS_DHE_RSA_WITH_AES_256_CBC_SHA,
-      ?TLS_DHE_DSS_WITH_AES_256_CBC_SHA,
-      ?TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA,
-      ?TLS_ECDH_RSA_WITH_AES_256_CBC_SHA,
-
-      ?TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA,
-      ?TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA,
-      ?TLS_DHE_RSA_WITH_AES_128_CBC_SHA,
-      ?TLS_DHE_DSS_WITH_AES_128_CBC_SHA,
-      ?TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA,
-      ?TLS_ECDH_RSA_WITH_AES_128_CBC_SHA
-    ];
+    exclusive_suites(2);
 suites(3) ->
+    exclusive_suites(3) ++ suites(2);
+
+suites(4) ->
+    exclusive_suites(4) ++ suites(3).
+
+exclusive_suites(4) ->
+    [?TLS_AES_256_GCM_SHA384,
+     ?TLS_AES_128_GCM_SHA256,
+     ?TLS_CHACHA20_POLY1305_SHA256,
+     ?TLS_AES_128_CCM_SHA256,
+     ?TLS_AES_128_CCM_8_SHA256
+    ];
+exclusive_suites(3) ->
     [?TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384,
      ?TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384,
+
+     ?TLS_ECDHE_ECDSA_WITH_AES_256_CCM,
+     ?TLS_ECDHE_ECDSA_WITH_AES_256_CCM_8,
+
      ?TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384,
      ?TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384,
-     ?TLS_ECDH_ECDSA_WITH_AES_256_GCM_SHA384,
-     ?TLS_ECDH_RSA_WITH_AES_256_GCM_SHA384,
-     ?TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA384,
-     ?TLS_ECDH_RSA_WITH_AES_256_CBC_SHA384,
 
-     ?TLS_DHE_RSA_WITH_AES_256_GCM_SHA384,
-     ?TLS_DHE_DSS_WITH_AES_256_GCM_SHA384,
-     ?TLS_DHE_RSA_WITH_AES_256_CBC_SHA256,
-     ?TLS_DHE_DSS_WITH_AES_256_CBC_SHA256,
+     ?TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256,
+     ?TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256,
 
      ?TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256,
      ?TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
-     ?TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256,
-     ?TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256,
+
+     ?TLS_ECDHE_ECDSA_WITH_AES_128_CCM,
+     ?TLS_ECDHE_ECDSA_WITH_AES_128_CCM_8,
+
+     ?TLS_ECDH_ECDSA_WITH_AES_256_GCM_SHA384,
+     ?TLS_ECDH_RSA_WITH_AES_256_GCM_SHA384,
+
+     ?TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA384,
+     ?TLS_ECDH_RSA_WITH_AES_256_CBC_SHA384,
+
      ?TLS_ECDH_ECDSA_WITH_AES_128_GCM_SHA256,
      ?TLS_ECDH_RSA_WITH_AES_128_GCM_SHA256,
+
+     ?TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256,
+     ?TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256,
+
      ?TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA256,
      ?TLS_ECDH_RSA_WITH_AES_128_CBC_SHA256,
 
+     ?TLS_DHE_RSA_WITH_AES_256_GCM_SHA384,
+     ?TLS_DHE_DSS_WITH_AES_256_GCM_SHA384,
+
+     ?TLS_DHE_RSA_WITH_AES_256_CBC_SHA256,
+     ?TLS_DHE_DSS_WITH_AES_256_CBC_SHA256,
+    
      ?TLS_DHE_RSA_WITH_AES_128_GCM_SHA256,
      ?TLS_DHE_DSS_WITH_AES_128_GCM_SHA256,
+     
+     ?TLS_DHE_RSA_WITH_CHACHA20_POLY1305_SHA256,
+     
      ?TLS_DHE_RSA_WITH_AES_128_CBC_SHA256,
      ?TLS_DHE_DSS_WITH_AES_128_CBC_SHA256
 
@@ -496,27 +553,24 @@ suites(3) ->
      %% ?TLS_DH_DSS_WITH_AES_256_GCM_SHA384,
      %% ?TLS_DH_RSA_WITH_AES_128_GCM_SHA256,
      %% ?TLS_DH_DSS_WITH_AES_128_GCM_SHA256
-    ] ++ suites(2);
-
-suites(4) ->
-    [?TLS_AES_256_GCM_SHA384,
-     ?TLS_AES_128_GCM_SHA256,
-     ?TLS_CHACHA20_POLY1305_SHA256,
-     ?TLS_AES_128_CCM_SHA256
-     %% Not supported
-     %% ?TLS_AES_128_CCM_8_SHA256
-    ] ++ suites(3);
-
-suites('TLS_v1.3') ->
-    [?TLS_AES_256_GCM_SHA384,
-     ?TLS_AES_128_GCM_SHA256,
-     ?TLS_CHACHA20_POLY1305_SHA256,
-     ?TLS_AES_128_CCM_SHA256
-     %% Not supported
-     %% ?TLS_AES_128_CCM_8_SHA256
+    ];
+exclusive_suites(Minor) when Minor == 1; Minor == 2 ->
+    [
+     ?TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA,
+     ?TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA,
+     ?TLS_DHE_RSA_WITH_AES_256_CBC_SHA,
+     ?TLS_DHE_DSS_WITH_AES_256_CBC_SHA,
+     ?TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA,
+     ?TLS_ECDH_RSA_WITH_AES_256_CBC_SHA,
+     
+     ?TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA,
+     ?TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA,
+     ?TLS_DHE_RSA_WITH_AES_128_CBC_SHA,
+     ?TLS_DHE_DSS_WITH_AES_128_CBC_SHA,
+     ?TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA,
+     ?TLS_ECDH_RSA_WITH_AES_128_CBC_SHA
     ].
 
-
 signature_algs({3, 4}, HashSigns) ->
     signature_algs({3, 3}, HashSigns);
 signature_algs({3, 3}, HashSigns) ->
@@ -675,7 +729,7 @@ hkdf_expand(Algo, PseudoRandKey, ContextInfo, Length, M, N, Prev, Acc) ->
 hmac_hash(?NULL, _, _) ->
     <<>>;
 hmac_hash(Alg, Key, Value) ->
-    crypto:hmac(mac_algo(Alg), Key, Value).
+    crypto:mac(hmac, mac_algo(Alg), Key, Value).
 
 mac_algo(Alg) when is_atom(Alg) -> 
     Alg;
@@ -875,7 +929,9 @@ oid_to_enum(?secp384r1) -> 24;
 oid_to_enum(?secp521r1) -> 25;
 oid_to_enum(?brainpoolP256r1) -> 26;
 oid_to_enum(?brainpoolP384r1) -> 27;
-oid_to_enum(?brainpoolP512r1) -> 28.
+oid_to_enum(?brainpoolP512r1) -> 28;
+oid_to_enum(?'id-X25519') -> 29;
+oid_to_enum(?'id-X448') -> 30.
 
 enum_to_oid(1) -> ?sect163k1;
 enum_to_oid(2) -> ?sect163r1;
@@ -905,5 +961,7 @@ enum_to_oid(25) -> ?secp521r1;
 enum_to_oid(26) -> ?brainpoolP256r1;
 enum_to_oid(27) -> ?brainpoolP384r1;
 enum_to_oid(28) -> ?brainpoolP512r1;
+enum_to_oid(29) -> ?'id-X25519';
+enum_to_oid(30) -> ?'id-X448';
 enum_to_oid(_) ->
     undefined.
