@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2007-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -22,12 +22,42 @@
 
 -module(ssl_ECC_SUITE).
 
-%% Note: This directive should only be used in test suites.
--compile(export_all).
+-behaviour(ct_suite).
 
 -include_lib("common_test/include/ct.hrl").
 -include_lib("public_key/include/public_key.hrl").
 
+
+%% Common test
+-export([all/0,
+         groups/0,
+         init_per_suite/1,
+         init_per_group/2,
+         init_per_testcase/2,
+         end_per_suite/1,
+         end_per_group/2,
+         end_per_testcase/2
+        ]).
+
+%% Test cases
+-export([ecc_default_order/1,
+         ecc_default_order_custom_curves/1,
+         ecc_client_order/1,
+         ecc_client_order_custom_curves/1,
+         ecc_unknown_curve/1,
+         client_ecdh_rsa_server_ecdhe_ecdsa_server_custom/1,
+         client_ecdh_rsa_server_ecdhe_rsa_server_custom/1,
+         client_ecdhe_rsa_server_ecdhe_ecdsa_server_custom/1,
+         client_ecdhe_rsa_server_ecdhe_rsa_server_custom/1,
+         client_ecdhe_rsa_server_ecdh_rsa_server_custom/1,
+         client_ecdhe_ecdsa_server_ecdhe_ecdsa_server_custom/1,
+         client_ecdhe_ecdsa_server_ecdhe_rsa_server_custom/1,
+         client_ecdhe_ecdsa_server_ecdhe_ecdsa_client_custom/1,
+         client_ecdhe_rsa_server_ecdhe_ecdsa_client_custom/1,
+         client_ecdsa_server_ecdsa_with_raw_key/1,
+         mix_sign/1
+        ]).
+
 %%--------------------------------------------------------------------
 %% Common Test interface functions -----------------------------------
 %%--------------------------------------------------------------------
@@ -94,29 +124,24 @@ end_per_suite(_Config) ->
 
 %%--------------------------------------------------------------------
 init_per_group(GroupName, Config) ->
-    case ssl_test_lib:is_tls_version(GroupName) of
-	true ->
-            [{tls_version, GroupName},
-             {server_type, erlang},
-             {client_type, erlang} | ssl_test_lib:init_tls_version(GroupName, Config)];
-        _ ->
+    case ssl_test_lib:is_protocol_version(GroupName) of
+	true ->   
+            ct:log("Ciphers: ~p~n ", [ssl:cipher_suites(default, GroupName)]),
+            ssl_test_lib:init_per_group(GroupName, 
+                                        [{client_type, erlang},
+                                         {server_type, erlang},
+                                         {version, GroupName} | Config]);
+        false ->
             Config
     end.
 
-end_per_group(GroupName, Config0) ->
-  case ssl_test_lib:is_tls_version(GroupName) of
-      true ->
-          Config = ssl_test_lib:clean_tls_version(Config0),
-          proplists:delete(tls_version, Config);
-      false ->
-          Config0
-  end.
+end_per_group(GroupName, Config) ->
+    ssl_test_lib:end_per_group(GroupName, Config).
 
 %%--------------------------------------------------------------------
 
 init_per_testcase(TestCase, Config) ->
     ssl_test_lib:ct_log_supported_protocol_versions(Config),
-    ct:log("Ciphers: ~p~n ", [ ssl:cipher_suites()]),
     end_per_testcase(TestCase, Config),
     ssl:start(),
     ct:timetrap({seconds, 15}),
@@ -130,7 +155,7 @@ end_per_testcase(_TestCase, Config) ->
 %% Test Cases --------------------------------------------------------
 %%--------------------------------------------------------------------
 %% Test diffrent certificate chain types, note that it is the servers
-%% chain that affect what cipher suit that will be choosen
+%% chain that affect what cipher suite that will be choosen
 
 client_ecdsa_server_ecdsa_with_raw_key(Config)  when is_list(Config) ->
      Default = ssl_test_lib:default_cert_chain_conf(),
