@@ -22,12 +22,116 @@
 
 -module(ssl_cipher_suite_SUITE).
 
-%% Note: This directive should only be used in test suites.
--compile(export_all).
+-behaviour(ct_suite).
 
 -include_lib("common_test/include/ct.hrl").
+%% Callback functions
+-export([all/0,
+         groups/0,
+         init_per_suite/1,
+         end_per_suite/1,
+         init_per_group/2,
+         end_per_group/2,
+         init_per_testcase/2,
+         end_per_testcase/2]).
 
--define(TIMEOUT, {seconds, 5}).
+%% Testcases
+-export([dhe_rsa_3des_ede_cbc/1,
+         dhe_rsa_aes_128_cbc/1,
+         dhe_rsa_aes_128_gcm/1,
+         dhe_rsa_aes_256_cbc/1,
+         dhe_rsa_aes_256_gcm/1,
+         dhe_rsa_chacha20_poly1305/1,
+         ecdhe_rsa_3des_ede_cbc/1,
+         ecdhe_rsa_rc4_128/1,
+         ecdhe_rsa_aes_128_cbc/1,
+         ecdhe_rsa_aes_128_gcm/1,
+         ecdhe_rsa_aes_256_cbc/1,
+         ecdhe_rsa_aes_256_gcm/1,
+         ecdhe_rsa_chacha20_poly1305/1,
+         ecdhe_ecdsa_rc4_128/1,
+         ecdhe_ecdsa_3des_ede_cbc/1,
+         ecdhe_ecdsa_aes_128_cbc/1,
+         ecdhe_ecdsa_aes_128_gcm/1,
+         ecdhe_ecdsa_aes_256_cbc/1,
+         ecdhe_ecdsa_aes_256_gcm/1,
+         ecdhe_ecdsa_chacha20_poly1305/1,
+         rsa_des_cbc/1,
+         rsa_3des_ede_cbc/1,
+         rsa_aes_128_cbc/1,
+         rsa_aes_256_cbc/1,
+         rsa_aes_128_gcm/1,
+         rsa_aes_256_gcm/1,
+         rsa_rc4_128/1,
+         dhe_dss_des_cbc/1,
+         dhe_dss_3des_ede_cbc/1,
+         dhe_dss_aes_128_cbc/1,
+         dhe_dss_aes_256_cbc/1,
+         dhe_dss_aes_128_gcm/1,
+         dhe_dss_aes_256_gcm/1,
+         srp_rsa_3des_ede_cbc/1,
+         srp_rsa_aes_128_cbc/1,
+         srp_rsa_aes_256_cbc/1,
+         srp_dss_3des_ede_cbc/1,
+         srp_dss_aes_128_cbc/1,
+         srp_dss_aes_256_cbc/1,
+         rsa_psk_3des_ede_cbc/1,
+         rsa_psk_rc4_128/1,
+         rsa_psk_aes_128_cbc/1,
+         rsa_psk_aes_256_cbc/1,
+         dhe_psk_des_cbc/1,
+         dhe_psk_3des_ede_cbc/1,
+         dhe_psk_rc4_128/1,
+         dhe_psk_aes_128_cbc/1,
+         dhe_psk_aes_128_gcm/1,
+         dhe_psk_aes_128_ccm/1,
+         dhe_psk_aes_128_ccm_8/1,
+         dhe_psk_aes_256_cbc/1,
+         dhe_psk_aes_256_gcm/1,
+         dhe_psk_aes_256_ccm/1,
+         dhe_psk_aes_256_ccm_8/1,
+         ecdhe_psk_3des_ede_cbc/1,
+         ecdhe_psk_rc4_128/1,
+         ecdhe_psk_aes_128_cbc/1,
+         ecdhe_psk_aes_128_gcm/1,
+         ecdhe_psk_aes_128_ccm/1,
+         ecdhe_psk_aes_128_ccm_8/1,
+         ecdhe_psk_aes_256_cbc/1,
+         ecdhe_psk_aes_256_gcm/1,
+         srp_anon_3des_ede_cbc/1,
+         srp_anon_aes_128_cbc/1,
+         srp_anon_aes_256_cbc/1,
+         psk_3des_ede_cbc/1,
+         psk_rc4_128/1,
+         psk_aes_128_cbc/1,
+         psk_aes_128_gcm/1,
+         psk_aes_128_ccm/1,
+         psk_aes_128_ccm_8/1,
+         psk_aes_256_cbc/1,
+         psk_aes_256_gcm/1,
+         psk_aes_256_ccm/1,
+         psk_aes_256_ccm_8/1,
+         dh_anon_rc4_128/1,
+         dh_anon_3des_ede_cbc/1,
+         dh_anon_aes_128_cbc/1,
+         dh_anon_aes_128_gcm/1,
+         dh_anon_aes_256_cbc/1,
+         dh_anon_aes_256_gcm/1,
+         ecdh_anon_3des_ede_cbc/1,
+         ecdh_anon_aes_128_cbc/1,
+         ecdh_anon_aes_256_cbc/1,
+         aes_256_gcm_sha384/1,
+         aes_128_gcm_sha256/1,
+         chacha20_poly1305_sha256/1,
+         aes_128_ccm_sha256/1,
+         aes_128_ccm_8_sha256/1,
+         ecdhe_ecdsa_with_aes_128_ccm/1,
+         ecdhe_ecdsa_with_aes_256_ccm/1,
+         ecdhe_ecdsa_with_aes_128_ccm_8/1,
+         ecdhe_ecdsa_with_aes_256_ccm_8/1
+        ]).
+
+-define(TIMEOUT, {seconds, 10}).
 
 %%--------------------------------------------------------------------
 %% Common Test interface functions -----------------------------------
@@ -52,7 +156,9 @@ groups() ->
      {'dtlsv1', [], kex()},
      {dhe_rsa, [],[dhe_rsa_3des_ede_cbc, 
                    dhe_rsa_aes_128_cbc,
+                   dhe_rsa_aes_128_gcm,
                    dhe_rsa_aes_256_cbc,
+                   dhe_rsa_aes_256_gcm,
                    dhe_rsa_chacha20_poly1305
                   ]},
      {ecdhe_rsa, [], [ecdhe_rsa_3des_ede_cbc, 
@@ -69,7 +175,11 @@ groups() ->
                        ecdhe_ecdsa_aes_128_gcm,
                        ecdhe_ecdsa_aes_256_cbc,
                        ecdhe_ecdsa_aes_256_gcm,
-                       ecdhe_ecdsa_chacha20_poly1305
+                       ecdhe_ecdsa_chacha20_poly1305,
+                       ecdhe_ecdsa_with_aes_128_ccm,
+                       ecdhe_ecdsa_with_aes_256_ccm,
+                       ecdhe_ecdsa_with_aes_128_ccm_8,
+                       ecdhe_ecdsa_with_aes_256_ccm_8
                       ]},
      {rsa, [], [rsa_3des_ede_cbc, 
                 rsa_aes_128_cbc,
@@ -112,21 +222,26 @@ groups() ->
                 psk_aes_256_ccm,
                 psk_aes_256_ccm_8
                ]},
-     {dhe_psk, [], [dhe_psk_3des_ede_cbc,                    
+     {dhe_psk, [], [dhe_psk_des_cbc,
+                    dhe_psk_3des_ede_cbc,
                     dhe_psk_rc4_128,
                     dhe_psk_aes_128_cbc,
+                    dhe_psk_aes_128_gcm,
                     dhe_psk_aes_128_ccm,
                     dhe_psk_aes_128_ccm_8,
                     dhe_psk_aes_256_cbc,
+                    dhe_psk_aes_256_gcm,
                     dhe_psk_aes_256_ccm,
                     dhe_psk_aes_256_ccm_8
                ]},
      {ecdhe_psk, [], [ecdhe_psk_3des_ede_cbc,                    
-                     ecdhe_psk_rc4_128,
-                     ecdhe_psk_aes_128_cbc,
-                     ecdhe_psk_aes_128_ccm,
-                     ecdhe_psk_aes_128_ccm_8,
-                     ecdhe_psk_aes_256_cbc
+                      ecdhe_psk_rc4_128,
+                      ecdhe_psk_aes_128_cbc,
+                      ecdhe_psk_aes_128_gcm,
+                      ecdhe_psk_aes_128_ccm,
+                      ecdhe_psk_aes_128_ccm_8,
+                      ecdhe_psk_aes_256_cbc,
+                      ecdhe_psk_aes_256_gcm
                ]}
     ].
 
@@ -182,15 +297,24 @@ init_per_suite(Config) ->
 end_per_suite(_Config) ->
     ssl:stop(),
     application:stop(crypto).
+init_per_group(GroupName, Config) ->
+    case ssl_test_lib:is_protocol_version(GroupName) of
+        true  ->
+            ssl_test_lib:init_per_group(GroupName, [{client_type, erlang},
+                                                    {server_type, erlang},
+                                                    {version, GroupName} | Config]);
+        false -> 
+            do_init_per_group(GroupName, Config)
+    end.
 
-init_per_group(GroupName, Config) when GroupName == ecdhe_1_3_rsa_cert ->    
+do_init_per_group(GroupName, Config) when GroupName == ecdhe_1_3_rsa_cert ->    
     case proplists:get_bool(ecdh, proplists:get_value(public_keys, crypto:supports())) of
         true ->
             init_certs(GroupName, Config);
         false ->
             {skip, "Missing EC crypto support"}
     end;
-init_per_group(GroupName, Config) when GroupName == ecdh_anon;
+do_init_per_group(GroupName, Config) when GroupName == ecdh_anon;
                                        GroupName == ecdhe_rsa;
                                        GroupName == ecdhe_psk ->
     case proplists:get_bool(ecdh, proplists:get_value(public_keys, crypto:supports())) of
@@ -199,7 +323,7 @@ init_per_group(GroupName, Config) when GroupName == ecdh_anon;
         false ->
             {skip, "Missing EC crypto support"}
     end;
-init_per_group(ecdhe_ecdsa = GroupName, Config) ->
+do_init_per_group(ecdhe_ecdsa = GroupName, Config) ->
     PKAlg = proplists:get_value(public_keys, crypto:supports()),
     case lists:member(ecdh, PKAlg) andalso lists:member(ecdsa, PKAlg) of
         true ->
@@ -207,7 +331,7 @@ init_per_group(ecdhe_ecdsa = GroupName, Config) ->
         false ->
             {skip, "Missing EC crypto support"}
     end;
-init_per_group(dhe_dss = GroupName, Config) ->
+do_init_per_group(dhe_dss = GroupName, Config) ->
     PKAlg = proplists:get_value(public_keys, crypto:supports()),
     case lists:member(dss, PKAlg) andalso lists:member(dh, PKAlg) of
         true ->
@@ -215,7 +339,7 @@ init_per_group(dhe_dss = GroupName, Config) ->
         false ->
             {skip, "Missing DSS crypto support"}
     end;
-init_per_group(srp_dss = GroupName, Config) ->
+do_init_per_group(srp_dss = GroupName, Config) ->
     PKAlg = proplists:get_value(public_keys, crypto:supports()),
     case lists:member(dss, PKAlg) andalso lists:member(srp, PKAlg) of
         true ->
@@ -223,8 +347,8 @@ init_per_group(srp_dss = GroupName, Config) ->
         false ->
             {skip, "Missing DSS_SRP crypto support"}
     end;
-init_per_group(GroupName, Config) when GroupName == srp_anon;
-                                       GroupName == srp_rsa ->
+do_init_per_group(GroupName, Config) when GroupName == srp_anon;
+                                          GroupName == srp_rsa ->
     PKAlg = proplists:get_value(public_keys, crypto:supports()),
     case lists:member(srp, PKAlg) of
         true ->
@@ -232,7 +356,7 @@ init_per_group(GroupName, Config) when GroupName == srp_anon;
         false ->
             {skip, "Missing SRP crypto support"}
     end;
-init_per_group(dhe_psk = GroupName, Config) ->
+do_init_per_group(dhe_psk = GroupName, Config) ->
     PKAlg = proplists:get_value(public_keys, crypto:supports()),
     case lists:member(dh, PKAlg) of
         true ->
@@ -240,13 +364,8 @@ init_per_group(dhe_psk = GroupName, Config) ->
         false ->
             {skip, "Missing SRP crypto support"}
     end;
-init_per_group(GroupName, Config0) ->
-    case ssl_test_lib:init_per_group(GroupName, Config0) of
-        {skip, _} = Skip ->
-            Skip;
-        Config ->
-            init_certs(GroupName, Config)
-    end.
+do_init_per_group(GroupName, Config) ->
+   init_certs(GroupName, Config).
   
 end_per_group(GroupName, Config) ->
     ssl_test_lib:end_per_group(GroupName, Config).
@@ -290,11 +409,9 @@ init_per_testcase(TestCase, Config) when TestCase == psk_rc4_128;
         _ ->
             {skip, "Missing RC4 crypto support"}
     end;
-init_per_testcase(TestCase, Config) when  TestCase == psk_aes_128_ccm_8;
-                                          TestCase == rsa_psk_aes_128_ccm_8;
-                                          TestCase == psk_aes_128_ccm_8;
-                                          TestCase == dhe_psk_aes_128_ccm_8;
-                                          TestCase == ecdhe_psk_aes_128_ccm_8 ->
+init_per_testcase(TestCase, Config) when TestCase == psk_aes_128_ccm_8;
+                                         TestCase == dhe_psk_aes_128_ccm_8;
+                                         TestCase == ecdhe_psk_aes_128_ccm_8 ->
     SupCiphers = proplists:get_value(ciphers, crypto:supports()),
     case lists:member(aes_128_ccm, SupCiphers) of
         true ->
@@ -304,7 +421,6 @@ init_per_testcase(TestCase, Config) when  TestCase == psk_aes_128_ccm_8;
             {skip, "Missing AES_128_CCM crypto support"}
     end;
 init_per_testcase(TestCase, Config) when TestCase == psk_aes_256_ccm_8;
-                                         TestCase == rsa_psk_aes_256_ccm_8;
                                          TestCase == psk_aes_256_ccm_8;
                                          TestCase == dhe_psk_aes_256_ccm_8;
                                          TestCase == ecdhe_psk_aes_256_ccm_8 ->
@@ -374,6 +490,26 @@ init_per_testcase(aes_128_ccm_8_sha256, Config) ->
         _ ->
             {skip, "Missing AES_128_CCM_8_SHA256 crypto support"}
     end;
+init_per_testcase(TestCase, Config) when TestCase == ecdhe_ecdsa_with_aes_128_ccm;
+                                         TestCase == ecdhe_ecdsa_with_aes_128_ccm_8->
+    SupCiphers = proplists:get_value(ciphers, crypto:supports()),
+    case lists:member(aes_128_ccm, SupCiphers) of
+        true ->
+            ct:timetrap(?TIMEOUT),
+            Config;
+        _ ->
+            {skip, "Missing AES_128_CCM crypto support"}
+    end;
+init_per_testcase(TestCase, Config) when TestCase == ecdhe_ecdsa_with_aes_256_ccm;
+                                         TestCase == ecdhe_ecdsa_with_aes_256_ccm_8 ->
+    SupCiphers = proplists:get_value(ciphers, crypto:supports()),
+    case lists:member(aes_256_ccm, SupCiphers) of
+        true ->
+            ct:timetrap(?TIMEOUT),
+            Config;
+        _ ->
+            {skip, "Missing AES_256_CCM crypto support"}
+    end;
 init_per_testcase(TestCase, Config) ->
     Cipher = ssl_test_lib:test_cipher(TestCase, Config),
     SupCiphers = proplists:get_value(ciphers, crypto:supports()),
@@ -540,20 +676,8 @@ rsa_psk_3des_ede_cbc(Config) when is_list(Config) ->
 rsa_psk_aes_128_cbc(Config) when is_list(Config) ->
     run_ciphers_test(rsa_psk, 'aes_128_cbc', Config).             
 
-rsa_psk_aes_128_ccm(Config) when is_list(Config) ->
-    run_ciphers_test(rsa_psk, 'aes_128_ccm', Config).             
-
-rsa_psk_aes_128_ccm_8(Config) when is_list(Config) ->
-    run_ciphers_test(rsa_psk, 'aes_128_ccm_8', Config).             
-
 rsa_psk_aes_256_cbc(Config) when is_list(Config) ->
     run_ciphers_test(rsa_psk, 'aes_256_cbc', Config). 
-
-rsa_psk_aes_256_ccm(Config) when is_list(Config) ->
-    run_ciphers_test(rsa_psk, 'aes_256_ccm', Config).             
-
-rsa_psk_aes_256_ccm_8(Config) when is_list(Config) ->
-    run_ciphers_test(rsa_psk, 'aes_256_ccm_8', Config).             
      
 rsa_psk_rc4_128(Config) when is_list(Config) ->
     run_ciphers_test(rsa_psk, 'rc4_128', Config).    
@@ -648,6 +772,18 @@ ecdhe_ecdsa_aes_256_gcm(Config) when is_list(Config) ->
 
 ecdhe_ecdsa_chacha20_poly1305(Config) when is_list(Config) ->
     run_ciphers_test(ecdhe_ecdsa, 'chacha20_poly1305', Config).
+
+ecdhe_ecdsa_with_aes_128_ccm(Config) when is_list(Config) ->
+    run_ciphers_test(ecdhe_ecdsa, 'aes_128_ccm', Config). 
+
+ecdhe_ecdsa_with_aes_256_ccm(Config) when is_list(Config) ->
+    run_ciphers_test(ecdhe_ecdsa, 'aes_256_ccm', Config). 
+
+ecdhe_ecdsa_with_aes_128_ccm_8(Config) when is_list(Config) ->
+    run_ciphers_test(ecdhe_ecdsa, 'aes_128_ccm_8', Config). 
+
+ecdhe_ecdsa_with_aes_256_ccm_8(Config) when is_list(Config) ->
+    run_ciphers_test(ecdhe_ecdsa, 'aes_256_ccm_8', Config). 
 %%--------------------------------------------------------------------
 %% DHE_DSS --------------------------------------------------------
 %%--------------------------------------------------------------------
@@ -741,9 +877,6 @@ dhe_psk_aes_128_ccm_8(Config) when is_list(Config) ->
 dhe_psk_aes_256_ccm_8(Config) when is_list(Config) ->
     run_ciphers_test(dhe_psk, 'aes_256_ccm_8', Config).
 
-ecdhe_psk_des_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_psk, 'des_cbc', Config).            
-
 ecdhe_psk_rc4_128(Config) when is_list(Config) ->
     run_ciphers_test(ecdhe_psk, 'rc4_128', Config).            
 
@@ -768,9 +901,6 @@ ecdhe_psk_aes_128_ccm(Config) when is_list(Config) ->
 ecdhe_psk_aes_128_ccm_8(Config) when is_list(Config) ->
     run_ciphers_test(ecdhe_psk, 'aes_128_ccm_8', Config).
 
-psk_des_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(psk, 'des_cbc', Config).            
-
 psk_rc4_128(Config) when is_list(Config) ->
     run_ciphers_test(psk, 'rc4_128', Config).            
 
