@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2008-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2008-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -21,12 +21,37 @@
 
 -module(ssl_crl_SUITE).
 
-%% Note: This directive should only be used in test suites.
--compile(export_all).
+-behaviour(ct_suite).
 
 -include_lib("common_test/include/ct.hrl").
 -include_lib("public_key/include/public_key.hrl").
 
+%% Common test
+-export([all/0,
+         groups/0,
+         init_per_suite/1,
+         init_per_group/2,
+         init_per_testcase/2,
+         end_per_suite/1,
+         end_per_group/2,
+         end_per_testcase/2
+        ]).
+
+
+%% Test cases
+-export([crl_verify_valid/0,
+         crl_verify_valid/1,
+         crl_verify_revoked/0,
+         crl_verify_revoked/1,
+         crl_verify_no_crl/0,
+         crl_verify_no_crl/1,
+         crl_hash_dir_collision/0,
+         crl_hash_dir_collision/1,
+         crl_hash_dir_expired/0,
+         crl_hash_dir_expired/1]).
+
+-define(TIMEOUT, {seconds, 30}).
+
 %%--------------------------------------------------------------------
 %% Common Test interface functions -----------------------------------
 %%--------------------------------------------------------------------
@@ -159,7 +184,7 @@ init_per_testcase(Case, Config0) ->
 	    {CertOpts, Config} = init_certs(CertDir, idp_crl, Config),
 	    case make_certs:all(DataDir, CertDir, CertOpts) of
                 {ok, _} ->
-                    ct:timetrap({seconds, 6}),
+                    ct:timetrap(?TIMEOUT),
                     [{cert_dir, CertDir} | Config];
                 _ ->
                     end_per_testcase(Case, Config0),
