@@ -22,7 +22,7 @@
 
 -module(ssl_handshake_SUITE).
 
--compile(export_all).
+-behaviour(ct_suite).
 
 -include_lib("common_test/include/ct.hrl").
 -include("ssl_alert.hrl").
@@ -32,6 +32,29 @@
 -include("tls_handshake.hrl").
 -include_lib("public_key/include/public_key.hrl").
 
+%% Common test
+-export([all/0,
+         init_per_suite/1,
+         init_per_group/2,
+         init_per_testcase/2,
+         end_per_suite/1,
+         end_per_group/2,
+         end_per_testcase/2
+        ]).
+
+%% Test cases
+-export([decode_hello_handshake/1,
+         decode_single_hello_extension_correctly/1,
+         decode_supported_elliptic_curves_hello_extension_correctly/1,
+         decode_unknown_hello_extension_correctly/1,
+         encode_single_hello_sni_extension_correctly/1,
+         decode_single_hello_sni_extension_correctly/1,
+         decode_empty_server_sni_correctly/1,
+         select_proper_tls_1_2_rsa_default_hashsign/1,
+         ignore_hassign_extension_pre_tls_1_2/1,
+         signature_algorithms/1,
+         encode_decode_srp/1]).
+
 %%--------------------------------------------------------------------
 %% Common Test interface functions -----------------------------------
 %%--------------------------------------------------------------------
@@ -44,7 +67,7 @@ all() -> [decode_hello_handshake,
 	  decode_empty_server_sni_correctly,
 	  select_proper_tls_1_2_rsa_default_hashsign,
 	  ignore_hassign_extension_pre_tls_1_2,
-	  unorded_chain, signature_algorithms,
+	  signature_algorithms,
 	  encode_decode_srp].
 
 %%--------------------------------------------------------------------
@@ -169,30 +192,12 @@ ignore_hassign_extension_pre_tls_1_2(Config) ->
     Opts = proplists:get_value(server_opts, Config),
     CertFile = proplists:get_value(certfile, Opts),
     [{_, Cert, _}] = ssl_test_lib:pem_to_der(CertFile),
-    HashSigns = #hash_sign_algos{hash_sign_algos = [{sha512, rsa}, {sha, dsa}, {sha, rsa}]},
+    HashSigns = #hash_sign_algos{hash_sign_algos = [{sha512, rsa}, {sha, dsa}, {sha256, rsa}]},
     {sha512, rsa} = ssl_handshake:select_hashsign({HashSigns, undefined}, Cert, ecdhe_rsa, tls_v1:default_signature_algs({3,3}), {3,3}),
     %%% Ignore
     {md5sha, rsa} = ssl_handshake:select_hashsign({HashSigns, undefined}, Cert, ecdhe_rsa, tls_v1:default_signature_algs({3,2}), {3,2}),
     {md5sha, rsa} = ssl_handshake:select_hashsign({HashSigns, undefined}, Cert, ecdhe_rsa, tls_v1:default_signature_algs({3,0}), {3,0}).
 
-unorded_chain(Config) when is_list(Config) ->
-    DefConf = ssl_test_lib:default_cert_chain_conf(),
-    CertChainConf = ssl_test_lib:gen_conf(rsa, rsa, DefConf, DefConf),
-    #{server_config := ServerConf,
-      client_config := _ClientConf} = public_key:pkix_test_data(CertChainConf),
-    PeerCert = proplists:get_value(cert, ServerConf),
-    CaCerts = [_, C1, C2] = proplists:get_value(cacerts, ServerConf),
-    {ok,  ExtractedCerts} = ssl_pkix_db:extract_trusted_certs({der, CaCerts}),
-    UnordedChain = case public_key:pkix_is_self_signed(C1) of
-                       true ->
-                           [C1, C2];
-                       false ->
-                           [C2, C1]
-                   end,
-    OrderedChain = [PeerCert | lists:reverse(UnordedChain)],
-    {ok, _, OrderedChain} = 
-        ssl_certificate:certificate_chain(PeerCert, ets:new(foo, []), ExtractedCerts, UnordedChain).
-
 encode_decode_srp(_Config) ->
     Exts = #{srp => #srp{username = <<"foo">>},
              sni => #sni{hostname = "bar"},
@@ -226,9 +231,9 @@ signature_algorithms(Config) ->
     HashSigns0 = #hash_sign_algos{
                    hash_sign_algos = [{sha512, rsa},
                                       {sha, dsa},
-                                      {sha, rsa}]},
+                                      {sha256, rsa}]},
     Schemes0 = #signature_algorithms_cert{
-                 signature_scheme_list = [rsa_pkcs1_sha1,
+                 signature_scheme_list = [rsa_pkcs1_sha256,
                                           ecdsa_sha1]},
     {sha512, rsa} = ssl_handshake:select_hashsign(
                       {HashSigns0, Schemes0},
@@ -237,14 +242,14 @@ signature_algorithms(Config) ->
                       {3,3}),
     HashSigns1 = #hash_sign_algos{
                     hash_sign_algos = [{sha, dsa},
-                                      {sha, rsa}]},
-    {sha, rsa} = ssl_handshake:select_hashsign(
+                                       {sha256, rsa}]},
+    {sha256, rsa} = ssl_handshake:select_hashsign(
                       {HashSigns1, Schemes0},
                       Cert, ecdhe_rsa,
                       tls_v1:default_signature_algs({3,3}),
-                   {3,3}),
+                      {3,3}),
     Schemes1 = #signature_algorithms_cert{
-                  signature_scheme_list = [rsa_pkcs1_sha256,
+                  signature_scheme_list = [rsa_pkcs1_sha1,
                                            ecdsa_sha1]},
     %% Signature not supported
     #alert{} = ssl_handshake:select_hashsign(
@@ -253,11 +258,11 @@ signature_algorithms(Config) ->
                  tls_v1:default_signature_algs({3,3}),
                  {3,3}),
     %% No scheme, hashsign is used
-    {sha, rsa} = ssl_handshake:select_hashsign(
-                   {HashSigns1, undefined},
-                   Cert, ecdhe_rsa,
-                   tls_v1:default_signature_algs({3,3}),
-                   {3,3}),
+    {sha256, rsa} = ssl_handshake:select_hashsign(
+                      {HashSigns1, undefined},
+                      Cert, ecdhe_rsa,
+                      tls_v1:default_signature_algs({3,3}),
+                      {3,3}),
     HashSigns2 = #hash_sign_algos{
                     hash_sign_algos = [{sha, dsa}]},
     %% Signature not supported
