@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2008-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2008-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -20,30 +20,107 @@
 
 -module(ssl_payload_SUITE).
 
-%% Note: This directive should only be used in test suites.
--compile(export_all).
+-behaviour(ct_suite).
 
 -include_lib("common_test/include/ct.hrl").
+%% Common test
+-export([all/0,
+         groups/0,
+         init_per_suite/1,
+         init_per_group/2,
+         init_per_testcase/2,
+         end_per_suite/1,
+         end_per_group/2,
+         end_per_testcase/2
+        ]).
 
--define(TIMEOUT, 600000).
+%% Test cases
+-export([server_echos_passive_small/0,
+         server_echos_passive_small/1,
+         server_echos_passive_chunk_small/0,
+         server_echos_passive_chunk_small/1,
+         server_echos_active_once_small/0,
+         server_echos_active_once_small/1,
+         server_echos_active_small/0,
+         server_echos_active_small/1,
+         client_echos_passive_small/0,
+         client_echos_passive_small/1,
+         client_echos_passive_chunk_small/0,
+         client_echos_passive_chunk_small/1,
+         client_echos_active_once_small/0,
+         client_echos_active_once_small/1,
+         client_echos_active_small/0,
+         client_echos_active_small/1,
+         server_echos_passive_big/0,
+         server_echos_passive_big/1,
+         server_echos_passive_chunk_big/0,
+         server_echos_passive_chunk_big/1,
+         server_echos_active_once_big/0,
+         server_echos_active_once_big/1,
+         server_echos_active_big/0,
+         server_echos_active_big/1,
+         client_echos_passive_big/0,
+         client_echos_passive_big/1,
+         client_echos_passive_chunk_big/0,
+         client_echos_passive_chunk_big/1,
+         client_echos_active_once_big/0,
+         client_echos_active_once_big/1,
+         client_echos_active_big/0,
+         client_echos_active_big/1,
+         server_echos_passive_huge/0,
+         server_echos_passive_huge/1,
+         server_echos_passive_chunk_huge/0,
+         server_echos_passive_chunk_huge/1,
+         server_echos_active_once_huge/0,
+         server_echos_active_once_huge/1,
+         server_echos_active_huge/0,
+         server_echos_active_huge/1,
+         client_echos_passive_huge/0,
+         client_echos_passive_huge/1,
+         client_echos_passive_chunk_huge/0,
+         client_echos_passive_chunk_huge/1,
+         client_echos_active_once_huge/0,
+         client_echos_active_once_huge/1,
+         client_echos_active_huge/0,
+         client_echos_active_huge/1,
+         client_active_once_server_close/0,
+         client_active_once_server_close/1]).
+
+%% Apply export
+-export([send/4,
+         send_close/2,
+         sender/2,
+         sender_active_once/2,
+         sender_active/2,
+         echoer/2,
+         echoer_chunk/2,
+         echoer_active_once/2,
+         echoer_active/2,
+         echo_recv/2,
+         echo_recv_chunk/3,
+         echo_active_once/2,
+         echo_active/2]).
+
+-define(TIMEOUT, {seconds, 20}).
+-define(TIMEOUT_LONG, {seconds, 80}).
 
 %%--------------------------------------------------------------------
 %% Common Test interface functions -----------------------------------
 %%--------------------------------------------------------------------
 all() -> 
     [
+     {group, 'tlsv1.3'},
      {group, 'tlsv1.2'},
      {group, 'tlsv1.1'},
-     {group, 'tlsv1'},
-     {group, 'sslv3'}
+     {group, 'tlsv1'}    
     ].
 
 groups() ->
     [
+     {'tlsv1.3', [], payload_tests()},
      {'tlsv1.2', [], payload_tests()},
      {'tlsv1.1', [], payload_tests()},
-     {'tlsv1', [], payload_tests()},
-     {'sslv3', [], payload_tests()}
+     {'tlsv1', [], payload_tests()}
     ].
 
 payload_tests() ->
@@ -92,26 +169,11 @@ end_per_suite(_Config) ->
     application:stop(crypto).
 
 init_per_group(GroupName, Config) ->
-     case ssl_test_lib:is_tls_version(GroupName) of
-	true ->
-	    case ssl_test_lib:sufficient_crypto_support(GroupName) of
-		true ->
-		    ssl_test_lib:init_tls_version(GroupName, Config);
-		false ->
-		    {skip, "Missing crypto support"}
-	    end;
-	_ ->
-	    ssl:start(),
-	    Config
-    end.
+    ssl_test_lib:init_per_group(GroupName, Config). 
 
 end_per_group(GroupName, Config) ->
-    case ssl_test_lib:is_tls_version(GroupName) of
-        true ->
-            ssl_test_lib:clean_tls_version(Config);
-        false ->
-            Config
-    end.
+  ssl_test_lib:end_per_group(GroupName, Config).
+
 
 init_per_testcase(TestCase, Config)
   when TestCase == server_echos_passive_huge;
@@ -126,7 +188,7 @@ init_per_testcase(TestCase, Config)
 	"sparc-sun-solaris2.10" ->
 	    {skip,"Will take to long time on an old Sparc"};
 	_ ->
-	    ct:timetrap({seconds, 90}),
+	    ct:timetrap(?TIMEOUT_LONG),
 	    Config
     end;
 
@@ -139,11 +201,11 @@ init_per_testcase(TestCase, Config)
        TestCase == client_echos_passive_chunk_big;
        TestCase == client_echos_active_once_big;
        TestCase == client_echos_active_big ->
-    ct:timetrap({seconds, 60}),
+    ct:timetrap(?TIMEOUT_LONG),
     Config;
 
 init_per_testcase(_TestCase, Config) ->
-    ct:timetrap({seconds, 15}),
+    ct:timetrap(?TIMEOUT),
     Config.
 
 end_per_testcase(_TestCase, Config) ->
@@ -226,7 +288,7 @@ client_echos_passive_small(Config) when is_list(Config) ->
       Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname).
 
 %%--------------------------------------------------------------------
-client_echos_passive_chunk__small() ->
+client_echos_passive_chunk_small() ->
     [{doc, "Server sends 1000 bytes in passive mode to client, that receives them in chunks, "
       "sends them back, and closes."}].
 
@@ -523,7 +585,7 @@ server_echos_passive(
           [{node, ServerNode}, {port, 0},
            {from, self()},
            {mfa, {?MODULE, echoer, [Length]}},
-           {options, [{active, false}, {mode, binary} | ServerOpts]}]),
+           {options, [{active, false}, {mode, binary} | ServerOpts] ++ ssl_test_lib:bigger_buffers()}]),
     Port = ssl_test_lib:inet_port(Server),
     Client =
         ssl_test_lib:start_client(
@@ -531,7 +593,7 @@ server_echos_passive(
            {host, Hostname},
            {from, self()},
            {mfa, {?MODULE, sender, [Data]}},
-           {options, [{active, false}, {mode, binary} | ClientOpts]}]),
+           {options, [{active, false}, {mode, binary} | ClientOpts] ++ ssl_test_lib:bigger_buffers() }]),
     %%
     ssl_test_lib:check_result(Server, ok, Client, ok),
     %%
@@ -569,7 +631,7 @@ server_echos_active_once(
           [{node, ServerNode}, {port, 0},
            {from, self()},
            {mfa, {?MODULE, echoer_active_once, [Length]}},
-           {options, [{active, once}, {mode, binary} | ServerOpts]}]),
+           {options, [{active, once}, {mode, binary} | ServerOpts] ++ ssl_test_lib:bigger_buffers()}]),
     Port = ssl_test_lib:inet_port(Server),
     Client =
         ssl_test_lib:start_client(
@@ -577,7 +639,7 @@ server_echos_active_once(
            {host, Hostname},
            {from, self()},
            {mfa, {?MODULE, sender_active_once, [Data]}},
-           {options, [{active, once}, {mode, binary} | ClientOpts]}]),
+           {options, [{active, once}, {mode, binary} | ClientOpts] ++ ssl_test_lib:bigger_buffers() }]),
     %%
     ssl_test_lib:check_result(Server, ok, Client, ok),
     %%
@@ -593,7 +655,7 @@ server_echos_active(
           [{node, ServerNode}, {port, 0},
            {from, self()},
            {mfa, {?MODULE, echoer_active, [Length]}},
-           {options, [{active, true}, {mode, binary} | ServerOpts]}]),
+           {options, [{active, true}, {mode, binary} | ServerOpts] ++ ssl_test_lib:bigger_buffers()}]),
     Port = ssl_test_lib:inet_port(Server),
     Client =
         ssl_test_lib:start_client(
@@ -601,7 +663,7 @@ server_echos_active(
            {host, Hostname},
            {from, self()},
            {mfa, {?MODULE, sender_active, [Data]}},
-           {options, [{active, true}, {mode, binary} | ClientOpts]}]),
+           {options, [{active, true}, {mode, binary} | ClientOpts] ++ ssl_test_lib:bigger_buffers()}]),
     %%
     ssl_test_lib:check_result(Server, ok, Client, ok),
     %%
@@ -616,7 +678,7 @@ client_echos_passive(
           [{node, ServerNode}, {port, 0},
            {from, self()},
            {mfa, {?MODULE, sender, [Data]}},
-           {options, [{active, false}, {mode, binary} | ServerOpts]}]),
+           {options, [{active, false}, {mode, binary} | ServerOpts] ++ ssl_test_lib:bigger_buffers()}]),
     Port = ssl_test_lib:inet_port(Server),
     Client =
         ssl_test_lib:start_client(
@@ -624,7 +686,7 @@ client_echos_passive(
            {host, Hostname},
            {from, self()},
            {mfa, {?MODULE, echoer, [Length]}},
-           {options, [{active, false}, {mode, binary} | ClientOpts]}]),
+           {options, [{active, false}, {mode, binary} | ClientOpts] ++ ssl_test_lib:bigger_buffers()}]),
     %%
     ssl_test_lib:check_result(Server, ok, Client, ok),
     %%
@@ -640,7 +702,7 @@ client_echos_passive_chunk(
           [{node, ServerNode}, {port, 0},
            {from, self()},
            {mfa, {?MODULE, sender, [Data]}},
-           {options, [{active, false}, {mode, binary} | ServerOpts]}]),
+           {options, [{active, false}, {mode, binary} | ServerOpts] ++ ssl_test_lib:bigger_buffers()}]),
     Port = ssl_test_lib:inet_port(Server),
     Client =
         ssl_test_lib:start_client(
@@ -648,7 +710,7 @@ client_echos_passive_chunk(
            {host, Hostname},
            {from, self()},
            {mfa, {?MODULE, echoer_chunk, [Length]}},
-           {options, [{active, false}, {mode, binary} | ClientOpts]}]),
+           {options, [{active, false}, {mode, binary} | ClientOpts] ++ ssl_test_lib:bigger_buffers()}]),
     %%
     ssl_test_lib:check_result(Server, ok, Client, ok),
     %%
@@ -664,7 +726,7 @@ client_echos_active_once(
           [{node, ServerNode}, {port, 0},
            {from, self()},
            {mfa, {?MODULE, sender_active_once, [Data]}},
-           {options, [{active, once}, {mode, binary} | ServerOpts]}]),
+           {options, [{active, once}, {mode, binary} | ServerOpts] ++ ssl_test_lib:bigger_buffers()}]),
     Port = ssl_test_lib:inet_port(Server),
     Client =
         ssl_test_lib:start_client(
@@ -672,7 +734,7 @@ client_echos_active_once(
            {host, Hostname},
            {from, self()},
            {mfa, {?MODULE, echoer_active_once, [Length]}},
-           {options,[{active, once}, {mode, binary} | ClientOpts]}]),
+           {options,[{active, once}, {mode, binary} | ClientOpts] ++ ssl_test_lib:bigger_buffers()}]),
     %%
     ssl_test_lib:check_result(Server, ok, Client, ok),
     %%
@@ -687,7 +749,7 @@ client_echos_active(
           [{node, ServerNode}, {port, 0},
            {from, self()},
            {mfa, {?MODULE, sender_active, [Data]}},
-           {options, [{active, true}, {mode, binary} | ServerOpts]}]),
+           {options, [{active, true}, {mode, binary} | ServerOpts] ++ ssl_test_lib:bigger_buffers()}]),
     Port = ssl_test_lib:inet_port(Server),
     Client =
         ssl_test_lib:start_client(
@@ -695,7 +757,7 @@ client_echos_active(
            {host, Hostname},
            {from, self()},
            {mfa, {?MODULE, echoer_active, [Length]}},
-           {options, [{active, true}, {mode, binary} | ClientOpts]}]),
+           {options, [{active, true}, {mode, binary} | ClientOpts] ++ ssl_test_lib:bigger_buffers()}]),
     %
     ssl_test_lib:check_result(Server, ok, Client, ok),
     %%
@@ -725,7 +787,7 @@ client_active_once_server_close(
 send(_Socket, _Data, 0, _) ->
     ok;
 send(Socket, Data, Count, RecvEcho) ->
-    ok = ssl:send(Socket, Data),
+    spawn(fun() -> ssl:send(Socket, Data) end),
     RecvEcho(),
     send(Socket, Data, Count - 1, RecvEcho).
 
@@ -811,3 +873,4 @@ echo_active(Socket, Size) ->
             echo_active(Socket, Size - byte_size(Data))
     end.    
         
+
