@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2019-2019. All Rights Reserved.
+%% Copyright Ericsson AB 2019-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -22,12 +22,54 @@
 
 -module(ssl_renegotiate_SUITE).
 
-%% Note: This directive should only be used in test suites.
--compile(export_all).
+-behaviour(ct_suite).
 
 -include_lib("common_test/include/ct.hrl").
 -include_lib("public_key/include/public_key.hrl").
 
+%% Common test
+-export([all/0,
+         groups/0,
+         init_per_suite/1,
+         init_per_group/2,
+         init_per_testcase/2,
+         end_per_suite/1,
+         end_per_group/2,
+         end_per_testcase/2
+        ]).
+
+%% Test cases
+-export([client_renegotiate/0,
+         client_renegotiate/1,
+         server_renegotiate/0,
+         server_renegotiate/1,
+         client_secure_renegotiate/0,
+         client_secure_renegotiate/1,
+         client_secure_renegotiate_fallback/0,
+         client_secure_renegotiate_fallback/1,
+         client_renegotiate_reused_session/0,
+         client_renegotiate_reused_session/1,
+         server_renegotiate_reused_session/0,
+         server_renegotiate_reused_session/1,
+         client_no_wrap_sequence_number/0,
+         client_no_wrap_sequence_number/1,
+         server_no_wrap_sequence_number/0,
+         server_no_wrap_sequence_number/1,
+         renegotiate_dos_mitigate_active/0,
+         renegotiate_dos_mitigate_active/1,
+         renegotiate_dos_mitigate_passive/0,
+         renegotiate_dos_mitigate_passive/1,
+         renegotiate_dos_mitigate_absolute/0,
+         renegotiate_dos_mitigate_absolute/1
+        ]).
+
+%% Apply export
+-export([renegotiate/2,
+         renegotiate_reuse_session/2,
+         renegotiate_immediately/1,
+         renegotiate_rejected/1,
+         erlang_ssl_receive/2]).
+
 -define(SLEEP, 500).
 -define(RENEGOTIATION_DISABLE_TIME, 12000).
 
@@ -39,7 +81,6 @@ all() ->
      {group, 'tlsv1.2'},
      {group, 'tlsv1.1'},
      {group, 'tlsv1'},
-     {group, 'sslv3'},
      {group, 'dtlsv1.2'},
      {group, 'dtlsv1'}
     ].
@@ -50,8 +91,7 @@ groups() ->
      {'tlsv1.3', [], renegotiate_tests()},
      {'tlsv1.2', [], renegotiate_tests()},
      {'tlsv1.1', [], renegotiate_tests()},
-     {'tlsv1', [], renegotiate_tests()},
-     {'sslv3', [], ssl3_renegotiate_tests()}
+     {'tlsv1', [], renegotiate_tests()}
     ].
 
 renegotiate_tests() ->
@@ -93,27 +133,22 @@ end_per_suite(_Config) ->
     application:stop(crypto).
 
 init_per_group(GroupName, Config) ->
-    ssl_test_lib:clean_tls_version(Config),                          
-    case ssl_test_lib:is_tls_version(GroupName) andalso ssl_test_lib:sufficient_crypto_support(GroupName) of
-	true ->
-	    ssl_test_lib:init_tls_version(GroupName, Config);
-	_ ->
-	    case ssl_test_lib:sufficient_crypto_support(GroupName) of
-		true ->
-		    ssl:start(),
-		    Config;
-		false ->
-		    {skip, "Missing crypto support"}
-	    end
-    end.
+    ssl_test_lib:init_per_group(GroupName, Config). 
 
 end_per_group(GroupName, Config) ->
-  case ssl_test_lib:is_tls_version(GroupName) of
-      true ->
-          ssl_test_lib:clean_tls_version(Config);
-      false ->
-          Config
-  end.
+  ssl_test_lib:end_per_group(GroupName, Config).
+
+init_per_testcase(TestCase, Config)  when TestCase == renegotiate_dos_mitigate_active;
+                                          TestCase == renegotiate_dos_mitigate_passive;
+                                          TestCase == renegotiate_dos_mitigate_absolute ->
+    ct:timetrap({seconds, 160}),
+    Config;
+init_per_testcase(_, Config) ->
+    ct:timetrap({seconds, 15}),
+    Config.
+
+end_per_testcase(_, Config) ->
+    Config.
 
 %%--------------------------------------------------------------------
 %% Test Cases --------------------------------------------------------
