@@ -21,8 +21,7 @@
 %%
 -module(ssl_session_SUITE).
 
-%% Note: This directive should only be used in test suites.
--compile(export_all).
+-behaviour(ct_suite).
 
 -include("tls_handshake.hrl").
 -include("ssl_record.hrl").
@@ -30,6 +29,34 @@
 -include_lib("common_test/include/ct.hrl").
 -include_lib("public_key/include/public_key.hrl").
 
+%% Common test
+-export([all/0,
+         groups/0,
+         init_per_suite/1,
+         init_per_group/2,
+         init_per_testcase/2,
+         end_per_suite/1,
+         end_per_group/2,
+         end_per_testcase/2
+        ]).
+
+%% Test cases
+-export([reuse_session/0,
+         reuse_session/1,
+         reuse_session_expired/0,
+         reuse_session_expired/1,
+         server_does_not_want_to_reuse_session/0,
+         server_does_not_want_to_reuse_session/1,
+         explicit_session_reuse/0,
+         explicit_session_reuse/1,
+         no_reuses_session_server_restart_new_cert/0,
+         no_reuses_session_server_restart_new_cert/1,
+         no_reuses_session_server_restart_new_cert_file/0,
+         no_reuses_session_server_restart_new_cert_file/1,
+         session_table_stable_size_on_tcp_close/0,
+         session_table_stable_size_on_tcp_close/1
+        ]).
+
 -define(SLEEP, 500).
 -define(EXPIRE, 10).
 
@@ -58,6 +85,7 @@ session_tests() ->
     [reuse_session,
      reuse_session_expired,
      server_does_not_want_to_reuse_session,
+     explicit_session_reuse,
      no_reuses_session_server_restart_new_cert,
      no_reuses_session_server_restart_new_cert_file].
 
@@ -70,7 +98,7 @@ init_per_suite(Config0) ->
 	ok ->
 	    ssl_test_lib:clean_start(),
             Config = ssl_test_lib:make_rsa_cert(Config0),
-            ssl_test_lib:make_dsa_cert(Config)
+            ssl_test_lib:make_rsa_1024_cert(Config)
     catch _:_ ->
 	    {skip, "Crypto did not start"}
     end.
@@ -80,27 +108,10 @@ end_per_suite(_Config) ->
     application:stop(crypto).
 
 init_per_group(GroupName, Config) ->
-    ssl_test_lib:clean_tls_version(Config),                          
-    case ssl_test_lib:is_tls_version(GroupName) andalso ssl_test_lib:sufficient_crypto_support(GroupName) of
-	true ->
-	    ssl_test_lib:init_tls_version(GroupName, Config);
-	_ ->
-	    case ssl_test_lib:sufficient_crypto_support(GroupName) of
-		true ->
-		    ssl:start(),
-		    Config;
-		false ->
-		    {skip, "Missing crypto support"}
-	    end
-    end.
+    ssl_test_lib:init_per_group(GroupName, Config). 
 
 end_per_group(GroupName, Config) ->
-  case ssl_test_lib:is_tls_version(GroupName) of
-      true ->
-          ssl_test_lib:clean_tls_version(Config);
-      false ->
-          Config
-  end.
+  ssl_test_lib:end_per_group(GroupName, Config).
 
 init_per_testcase(reuse_session_expired, Config)  ->
     Versions = ssl_test_lib:protocol_version(Config),
@@ -132,8 +143,9 @@ reuse_session() ->
 reuse_session(Config) when is_list(Config) -> 
     ClientOpts = ssl_test_lib:ssl_options(client_rsa_verify_opts, Config),
     ServerOpts = ssl_test_lib:ssl_options(server_rsa_verify_opts, Config),
-    
-    ssl_test_lib:reuse_session(ClientOpts, ServerOpts, Config).
+    Version = ssl_test_lib:protocol_version(Config),
+    ssl_test_lib:reuse_session([{versions,[Version]} | ClientOpts], 
+                               [{versions,[Version]} | ServerOpts], Config).
 %%--------------------------------------------------------------------
 reuse_session_expired() ->
     [{doc,"Test sessions is not reused when it has expired"}].
@@ -262,14 +274,54 @@ server_does_not_want_to_reuse_session(Config) when is_list(Config) ->
     ssl_test_lib:close(Server),
     ssl_test_lib:close(Client1).
 
+%%--------------------------------------------------------------------
+explicit_session_reuse() ->
+    [{doc,"Test {reuse_session, {ID, Data}}} option for explicit reuse of sessions not"
+     " saved in the clients automated session reuse"}].
+explicit_session_reuse(Config) when is_list(Config) ->
+    ClientOpts = ssl_test_lib:ssl_options(client_rsa_verify_opts, Config),
+    ServerOpts = ssl_test_lib:ssl_options(server_rsa_verify_opts, Config),
+    {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
+
+    Server =
+	ssl_test_lib:start_server([{node, ServerNode}, {port, 0},
+                                   {from, self()},
+                                   {mfa, {ssl_test_lib, no_result, []}},
+                                   {options, ServerOpts}]),
+    Port = ssl_test_lib:inet_port(Server),
+    {Client0, Client0Sock} =
+	ssl_test_lib:start_client([{node, ClientNode},
+                                   {port, Port}, {host, Hostname},
+                                   {mfa, {ssl_test_lib, no_result, []}},
+                                   {from, self()}, {options, [{reuse_sessions, false} | ClientOpts]},
+                                   return_socket
+                                  ]),
+
+    {ok, [{session_id, ID}, {session_data, SessData}]} = ssl:connection_information(Client0Sock, [session_id, session_data]),
+
+    ssl_test_lib:close(Client0),
+
+    Server ! listen,
+
+    {_, Client1Sock} =
+	ssl_test_lib:start_client([{node, ClientNode},
+                                   {port, Port}, {host, Hostname},
+                                   {mfa, {ssl_test_lib, no_result, []}},
+                                   {from, self()}, {options, [{reuse_session, {ID, SessData}} | ClientOpts]},
+                                   return_socket]),
+
+    {ok, [{session_id, ID}]} = ssl:connection_information(Client1Sock, [session_id]).
+
+
+%%--------------------------------------------------------------------
 no_reuses_session_server_restart_new_cert() ->
     [{doc,"Check that a session is not reused if the server is restarted with a new cert."}].
 no_reuses_session_server_restart_new_cert(Config) when is_list(Config) ->
 
     ClientOpts = ssl_test_lib:ssl_options(client_rsa_opts, Config),
     ServerOpts = ssl_test_lib:ssl_options(server_rsa_verify_opts, Config),
-    DsaServerOpts = ssl_test_lib:ssl_options(server_dsa_verify_opts, Config),
-    DsaClientOpts = ssl_test_lib:ssl_options(client_dsa_opts, Config),
+    RSA1024ServerOpts = ssl_test_lib:ssl_options(server_rsa_1024_opts, Config),
+    RSA1024ClientOpts = ssl_test_lib:ssl_options(client_rsa_1024_opts, Config),
     {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
 
     Server =
@@ -282,34 +334,27 @@ no_reuses_session_server_restart_new_cert(Config) when is_list(Config) ->
 	ssl_test_lib:start_client([{node, ClientNode},
 		      {port, Port}, {host, Hostname},
 			    {mfa, {ssl_test_lib, no_result, []}},
-		      {from, self()},  {options, ClientOpts}]),
+		      {from, self()},  {options, [{reuse_sessions, save} | ClientOpts]}]),
     SessionInfo =
 	receive
 	    {Server, Info} ->
 		Info
 	end,
 
-    %% Make sure session is registered
-    ct:sleep(?SLEEP),
-    Monitor = erlang:monitor(process, Server),
     ssl_test_lib:close(Server),
     ssl_test_lib:close(Client0),
-    receive
-	{'DOWN', Monitor, _, _, _} ->
-	    ok
-    end,
-    
+
     Server1 =
 	ssl_test_lib:start_server([{node, ServerNode}, {port, Port},
 				   {from, self()},
 		      {mfa, {ssl_test_lib, no_result, []}},
-				   {options, [{reuseaddr, true} | DsaServerOpts]}]),
+				   {options, [{reuseaddr, true} | RSA1024ServerOpts]}]),
 
     Client1 =
 	ssl_test_lib:start_client([{node, ClientNode},
 		      {port, Port}, {host, Hostname},
 		      {mfa, {ssl_test_lib, session_info_result, []}},
-		      {from, self()},  {options, DsaClientOpts}]),
+		      {from, self()},  {options, RSA1024ClientOpts}]),
     receive
 	{Client1, SessionInfo} ->
 	    ct:fail(session_reused_when_server_has_new_cert);
@@ -327,7 +372,7 @@ no_reuses_session_server_restart_new_cert_file() ->
 no_reuses_session_server_restart_new_cert_file(Config) when is_list(Config) ->
     ClientOpts = ssl_test_lib:ssl_options(client_rsa_opts, Config),
     ServerOpts = ssl_test_lib:ssl_options(server_rsa_verify_opts, Config),
-    DsaServerOpts = ssl_test_lib:ssl_options(server_dsa_verify_opts, Config),
+    RSA1024ServerOpts = ssl_test_lib:ssl_options(server_rsa_1024_verify_opts, Config),
     PrivDir =  proplists:get_value(priv_dir, Config),
 
     NewServerOpts0 = ssl_test_lib:new_config(PrivDir, ServerOpts),
@@ -358,7 +403,7 @@ no_reuses_session_server_restart_new_cert_file(Config) when is_list(Config) ->
 
     ssl:clear_pem_cache(),
 
-    NewServerOpts1 = ssl_test_lib:new_config(PrivDir, DsaServerOpts),
+    NewServerOpts1 = ssl_test_lib:new_config(PrivDir, RSA1024ServerOpts),
 
     Server1 =
 	ssl_test_lib:start_server([{node, ServerNode}, {port, Port},
@@ -386,37 +431,44 @@ session_table_stable_size_on_tcp_close(Config) when is_list(Config)->
     ServerOpts = ssl_test_lib:ssl_options(server_rsa_verify_opts, Config),
     {_, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
 
-    {status, _, _, StatusInfo} = sys:get_status(whereis(ssl_manager)),
-    [_, _,_, _, Prop] = StatusInfo,
-    State = ssl_test_lib:state(Prop),
-    ServerCache = element(3, State),
-
-    N = ets:info(ServerCache, size),
-
-    Server = ssl_test_lib:start_server_error([{node, ServerNode}, {port, 0},
-                                              {from, self()},
-                                              {options,  [{reuseaddr, true} | ServerOpts]}]),
+    Server = ssl_test_lib:start_server([{node, ServerNode}, {port, 0},
+                                        {from, self()},
+                                        {mfa, {ssl_test_lib, no_result, []}},
+                                        {options,  [{reuseaddr, true} | ServerOpts]}]),
     Port = ssl_test_lib:inet_port(Server),
+
+    Sup = whereis(ssl_server_session_cache_sup),
+
+    %% Will only be one process, that is one server, in our test senario
+    [{_, SessionCachePid, worker,[ssl_server_session_cache]}] = supervisor:which_children(Sup),
+
+
+    {SessionCacheCb, SessionCacheDb} = session_cachce_info(SessionCachePid),
+
+    N = SessionCacheCb:size(SessionCacheDb),
+
     faulty_client(Hostname, Port),
-    check_table_did_not_grow(ServerCache, N).
+    check_table_did_not_grow(SessionCachePid, N).
 
 
 %%--------------------------------------------------------------------
 %% Internal functions ------------------------------------------------
 %%--------------------------------------------------------------------
-check_table_did_not_grow(ServerCache, N) ->
-    ct:sleep(500),
-    check_table_did_not_grow(ServerCache, N, 10).
+session_cachce_info(SessionCache) ->
+    State = sys:get_state(SessionCache),
+    ServerCacheDb = element(4, State),
+    ServerCacheCb = element(2, State),
+    {ServerCacheCb, ServerCacheDb}.
 
-check_table_did_not_grow(_, _, 0) ->
-    ct:fail(table_grew);
-check_table_did_not_grow(ServerCache, N, Tries) ->
-    case ets:info(ServerCache, size) of
+
+check_table_did_not_grow(SessionCachePid, N) ->
+    {SessionCacheCb, SessionCacheDb} = session_cachce_info(SessionCachePid),
+    ct:pal("Run ~p ~p", [SessionCacheCb, SessionCacheDb]),
+    case catch SessionCacheCb:size(SessionCacheDb) of
         N ->
             ok;
-        _ ->
-            ct:sleep(500),
-            check_table_did_not_grow(ServerCache, N, Tries -1)
+        Size ->
+            ct:fail({table_grew, [{expected, N}, {got, Size}]})
     end.
 
 faulty_client(Host, Port) ->
@@ -429,18 +481,6 @@ faulty_client(Host, Port) ->
     gen_tcp:close(Sock).
 
 
-server(LOpts, Port) ->
-    {ok, LSock} = ssl:listen(Port, LOpts),
-    Pid = spawn_link(?MODULE, accept_loop, [LSock]),
-    ssl:controlling_process(LSock, Pid),
-    Pid.
-
-accept_loop(Sock) ->
-    {ok, CSock} = ssl:transport_accept(Sock),
-    _ = ssl:handshake(CSock),
-    accept_loop(Sock).
-
-
 encode_client_hello(CH, Random) ->
     HSBin = tls_handshake:encode_handshake(CH, {3,3}),
     CS = connection_states(Random),
