@@ -22,20 +22,47 @@
 
 -module(ssl_session_cache_SUITE).
 
-%% Note: This directive should only be used in test suites.
--compile(export_all).
+-behaviour(ct_suite).
 
 -include_lib("common_test/include/ct.hrl").
 
--define(SLEEP, 1000).
--define(TIMEOUT, {seconds, 20}).
--define(MAX_TABLE_SIZE, 5).
+%% Callback functions
+-export([all/0,
+         groups/0,
+         init_per_suite/1,
+         end_per_suite/1,
+         init_per_group/2,
+         end_per_group/2,
+         init_per_testcase/2,
+         end_per_testcase/2]).
+
+%% Testcases
+-export([session_cleanup/0,
+         session_cleanup/1,
+         session_cache_process_list/0,
+         session_cache_process_list/1,
+         session_cache_process_mnesia/0,
+         session_cache_process_mnesia/1,
+         client_unique_session/0,
+         client_unique_session/1,
+         max_table_size/0,
+         max_table_size/1,
+         save_specific_session/0,
+         save_specific_session/1
+        ]).
+
+%% Apply export
+-export([connection_info_result/1]).
 
 -behaviour(ssl_session_cache_api).
 
 %% For the session cache tests
 -export([init/1, terminate/1, lookup/2, update/3,
-	 delete/2, foldl/3, select_session/2]).
+         size/1, delete/2, foldl/3, select_session/2]).
+
+-define(SLEEP, 1000).
+-define(TIMEOUT, {seconds, 20}).
+-define(MAX_TABLE_SIZE, 5).
 
 %%--------------------------------------------------------------------
 %% Common Test interface functions -----------------------------------
@@ -85,27 +112,10 @@ end_per_suite(_Config) ->
     application:stop(crypto).
 
 init_per_group(GroupName, Config) ->
-    ssl_test_lib:clean_tls_version(Config),                          
-    case ssl_test_lib:is_tls_version(GroupName) andalso ssl_test_lib:sufficient_crypto_support(GroupName) of
-	true ->
-	    ssl_test_lib:init_tls_version(GroupName, Config);
-	_ ->
-	    case ssl_test_lib:sufficient_crypto_support(GroupName) of
-		true ->
-		    ssl:start(),
-		    Config;
-		false ->
-		    {skip, "Missing crypto support"}
-	    end
-    end.
+    ssl_test_lib:init_per_group(GroupName, Config). 
 
 end_per_group(GroupName, Config) ->
-  case ssl_test_lib:is_tls_version(GroupName) of
-      true ->
-          ssl_test_lib:clean_tls_version(Config);
-      false ->
-          Config
-  end.
+  ssl_test_lib:end_per_group(GroupName, Config).
 
 init_per_testcase(session_cache_process_list, Config) ->
     init_customized_session_cache(list, Config);
@@ -250,15 +260,12 @@ session_cleanup(Config) when is_list(Config) ->
     [_, _,_, _, Prop] = StatusInfo,
     State = ssl_test_lib:state(Prop),
     ClientCache = element(2, State),
-    ServerCache = element(3, State),
-    SessionTimer = element(7, State),
+    SessionTimer = element(6, State),
 
     Id = proplists:get_value(session_id, SessionInfo),
     CSession = ssl_session_cache:lookup(ClientCache, {{Hostname, Port}, Id}),
-    SSession = ssl_session_cache:lookup(ServerCache, {Port, Id}),
 
     true = CSession =/= undefined,
-    true = SSession =/= undefined,
 
     %% Make sure session has expired and been cleaned up
     check_timer(SessionTimer),
@@ -266,7 +273,6 @@ session_cleanup(Config) when is_list(Config) ->
     ct:sleep(?SLEEP),  %% Make sure clean has had time to run
     
     undefined = ssl_session_cache:lookup(ClientCache, {{Hostname, Port}, Id}),
-    undefined = ssl_session_cache:lookup(ServerCache, {Port, Id}),
 
     process_flag(trap_exit, false),
     ssl_test_lib:close(Server),
@@ -373,13 +379,10 @@ max_table_size(Config) when is_list(Config) ->
     [_, _,_, _, Prop] = StatusInfo,
     State = ssl_test_lib:state(Prop),
     ClientCache = element(2, State),	
-    ServerCache = element(3, State),
-    N = ssl_session_cache:size(ServerCache),
     M = ssl_session_cache:size(ClientCache),
-    ct:pal("~p",[{N, M}]),			
+    ct:pal("~p",[M]),
     ssl_test_lib:close(Server, 500),
     ssl_test_lib:close(LastClient),
-    true = N =< ?MAX_TABLE_SIZE,
     true = M =< ?MAX_TABLE_SIZE.
 
 %%--------------------------------------------------------------------
@@ -393,7 +396,7 @@ init(Opts) ->
 	mnesia ->
 	    mnesia:start(),
 	    Name = atom_to_list(proplists:get_value(role, Opts)),
-	    TabName = list_to_atom(Name ++ "sess_cache"),
+	    TabName = list_to_atom(Name ++ "sess_cache" ++ erlang:pid_to_list(self())),
 	    {atomic,ok} = mnesia:create_table(TabName, []),
 	    TabName
     end.
@@ -411,6 +414,16 @@ terminate(Cache) ->
 		mnesia:delete_table(Cache)
     end.
 
+size(Cache) ->
+    case session_cb() of
+	list ->
+            Cache ! {self(), size},
+            receive {Cache, Res} -> Res end;
+        mnesia ->
+            mnesia:table_info(Cache, size)
+    end.
+
+
 lookup(Cache, Key) ->
     case session_cb() of
 	list ->
@@ -447,7 +460,7 @@ delete(Cache, Key) ->
 	mnesia ->
 	    {atomic, ok} =
 		mnesia:transaction(fun() ->
-					   mnesia:delete(Cache, Key)
+					   mnesia:delete(Cache, Key, write)
 				   end)
     end.
 
@@ -460,22 +473,26 @@ foldl(Fun, Acc, Cache) ->
 	    Foldl = fun() ->
 			    mnesia:foldl(Fun, Acc, Cache)
 		    end,
-	    {atomic, Res} = mnesia:transaction(Foldl),
-	    Res
+	    case mnesia:transaction(Foldl) of
+                {atomic, {_,Key, Value}} ->
+                    {Key, Value};
+                Error ->
+                    Error
+            end
     end.
 
 select_session(Cache, PartialKey) ->
     case session_cb() of
 	list ->
-	    Cache ! {self(),select_session, PartialKey},
+            Cache ! {self(),select_session, PartialKey},
 	    receive
-		{Cache, Res} ->
+		{_Cache, Res} ->
 		    Res
 	    end;
 	mnesia ->
 	    Sel = fun() ->
 			  mnesia:select(Cache,
-					[{{Cache,{PartialKey,'_'}, '$1'},
+					[{{Cache, {PartialKey,'_'}, '$1'},
 					  [],['$1']}])
 		  end,
 	    {atomic, Res} = mnesia:transaction(Sel),
@@ -486,6 +503,9 @@ session_loop(Sess) ->
     receive
 	terminate ->
 	    ok;
+        {Pid, size} ->
+            Pid ! {self(), length(Sess)},
+            session_loop(Sess);
 	{Pid, lookup, Key} ->
 	    case lists:keysearch(Key,1,Sess) of
 		{value, {Key,Value}} ->
@@ -512,7 +532,13 @@ session_loop(Sess) ->
 	    Sessions = lists:foldl(Sel, [], Sess),
 	    Pid ! {self(), Sessions},
 	    session_loop(Sess)
-    end.
+     end.
+%%--------------------------------------------------------------------
+%%% callback functions
+%%--------------------------------------------------------------------
+
+connection_info_result(Socket) ->
+    ssl:connection_information(Socket, [protocol, cipher_suite]).
 
 %%--------------------------------------------------------------------
 %%% Internal functions
@@ -541,8 +567,6 @@ clients_start(Server, ClientNode, Hostname, Port, ClientOpts, N) ->
     wait_for_server(),
     clients_start(Server, ClientNode, Hostname, Port, ClientOpts, N-1).
 	
-connection_info_result(Socket) ->
-    ssl:connection_information(Socket, [protocol, cipher_suite]).
 
 check_timer(Timer) ->
     case erlang:read_timer(Timer) of
