@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2015-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2015-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -21,19 +21,55 @@
 
 -module(ssl_sni_SUITE).
 
--compile(export_all).
+-behaviour(ct_suite).
 
 -include_lib("common_test/include/ct.hrl").
 -include_lib("public_key/include/public_key.hrl").
 -include_lib("kernel/include/inet.hrl").
 
+%% Callback functions
+-export([all/0,
+         groups/0,
+         init_per_suite/1,
+         end_per_suite/1,
+         init_per_group/2,
+         end_per_group/2,
+         init_per_testcase/2,
+         end_per_testcase/2]).
+
+%% Testcases
+-export([no_sni_header/1,
+         sni_match/1,
+         sni_no_match/1,
+         no_sni_header_fun/1,
+         sni_match_fun/1,
+         sni_no_match_fun/1,
+         dns_name/1,
+         ip_fallback/1,
+         no_ip_fallback/1,
+         dns_name_reuse/1,
+         customize_hostname_check/0,
+         customize_hostname_check/1,
+         sni_no_trailing_dot/0,
+         sni_no_trailing_dot/1,
+         hostname_trailing_dot/0,
+         hostname_trailing_dot/1
+        ]).
+
+%% Apply export
+-export([send_and_hostname/1,
+         recv_and_certificate/1
+        ]).
+
+-define(TIMEOUT, {seconds, 6}).
 
 %%--------------------------------------------------------------------
 %% Common Test interface functions -----------------------------------
 %%--------------------------------------------------------------------
 
 all() ->
-    [{group, 'tlsv1.2'},
+    [{group, 'tlsv1.3'},
+     {group, 'tlsv1.2'},
      {group, 'tlsv1.1'},
      {group, 'tlsv1'},
      {group, 'dtlsv1.2'},
@@ -42,6 +78,7 @@ all() ->
 
 groups() ->
     [
+     {'tlsv1.3', [], sni_tests()},
      {'tlsv1.2', [], sni_tests()},
      {'tlsv1.1', [], sni_tests()},
      {'tlsv1', [], sni_tests()},
@@ -60,7 +97,9 @@ sni_tests() ->
      ip_fallback,
      no_ip_fallback,
      dns_name_reuse,
-     customize_hostname_check].
+     customize_hostname_check,
+     sni_no_trailing_dot,
+     hostname_trailing_dot].
 
 init_per_suite(Config0) ->
     catch crypto:stop(),
@@ -82,6 +121,11 @@ init_per_suite(Config0) ->
     catch _:_  ->
             {skip, "Crypto did not start"}
     end.
+init_per_group(GroupName, Config) ->
+    ssl_test_lib:init_per_group(GroupName, Config).
+
+end_per_group(GroupName, Config) ->
+   ssl_test_lib:end_per_group(GroupName, Config).
 
 end_per_suite(_) ->
     ssl:stop(),
@@ -89,13 +133,13 @@ end_per_suite(_) ->
 
 init_per_testcase(customize_hostname_check, Config) ->
     ssl_test_lib:ct_log_supported_protocol_versions(Config),
-    ssl_test_lib:clean_start(),
-    ct:timetrap({seconds, 5}),
+    ssl_test_lib:clean_start(keep_version),
+    ct:timetrap(?TIMEOUT),
     Config;
 init_per_testcase(_TestCase, Config) ->
     ssl_test_lib:ct_log_supported_protocol_versions(Config),
     ct:log("Ciphers: ~p~n ", [ ssl:cipher_suites()]),
-    ct:timetrap({seconds, 5}),
+    ct:timetrap(?TIMEOUT),
     Config.
 
 end_per_testcase(_TestCase, Config) ->     
@@ -125,41 +169,46 @@ sni_no_match_fun(Config) ->
 dns_name(Config) ->
     Hostname = "OTP.test.server",
     #{server_config := ServerConf,
-      client_config := ClientConf} = public_key:pkix_test_data(#{server_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
-                                                                       peer => [{extensions,  [#'Extension'{extnID = 
-                                                                                                                ?'id-ce-subjectAltName',
-                                                                                                            extnValue = [{dNSName, Hostname}],
-                                                                                                            critical = false}]},
-                                                                                {key, ssl_test_lib:hardcode_rsa_key(3)}]},
-                                                                 client_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
-                                                                       peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
+      client_config := ClientConf} =
+        public_key:pkix_test_data(#{server_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
+                                          peer => [{extensions,  [#'Extension'{extnID =
+                                                                                   ?'id-ce-subjectAltName',
+                                                                               extnValue = [{dNSName, Hostname}],
+                                                                               critical = false}]},
+                                                   {key, ssl_test_lib:hardcode_rsa_key(3)}]},
+                                    client_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
+                                          peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
     unsuccessfull_connect(ServerConf, [{verify, verify_peer} | ClientConf], undefined, Config),
-    successfull_connect(ServerConf, [{verify, verify_peer}, {server_name_indication, Hostname} | ClientConf], undefined, Config),
-    unsuccessfull_connect(ServerConf, [{verify, verify_peer}, {server_name_indication, "foo"} | ClientConf], undefined, Config),
-    successfull_connect(ServerConf, [{verify, verify_peer}, {server_name_indication, disable} | ClientConf], undefined, Config).
+    successfull_connect(ServerConf, [{verify, verify_peer},
+                                     {server_name_indication, Hostname} | ClientConf], undefined, Config),
+    unsuccessfull_connect(ServerConf, [{verify, verify_peer}, {server_name_indication, "foo"} | ClientConf],
+                          undefined, Config),
+    successfull_connect(ServerConf, [{verify, verify_peer}, {server_name_indication, disable} | ClientConf],
+                        undefined, Config).
 
 ip_fallback(Config) ->
     Hostname = net_adm:localhost(),
     {ok, #hostent{h_addr_list = [IP |_]}} = inet:gethostbyname(net_adm:localhost()),
     IPStr = tuple_to_list(IP),
     #{server_config := ServerConf,
-      client_config := ClientConf} = public_key:pkix_test_data(#{server_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
-                                                                       peer => [{extensions, [#'Extension'{extnID = 
-                                                                                                               ?'id-ce-subjectAltName',
-                                                                                                           extnValue = [{dNSName, Hostname},
-                                                                                                                        {iPAddress, IPStr}],
-                                                                                                           critical = false}]},
-                                                                                {key, ssl_test_lib:hardcode_rsa_key(3)}]},
-                                                                 client_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
-                                                                       peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
+      client_config := ClientConf} =
+        public_key:pkix_test_data(#{server_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
+                                          peer => [{extensions, [#'Extension'{extnID =
+                                                                                  ?'id-ce-subjectAltName',
+                                                                              extnValue = [{dNSName, Hostname},
+                                                                                           {iPAddress, IPStr}],
+                                                                              critical = false}]},
+                                                   {key, ssl_test_lib:hardcode_rsa_key(3)}]},
+                                    client_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
+                                          peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
     successfull_connect(ServerConf, [{verify, verify_peer} | ClientConf], Hostname, Config),
     successfull_connect(ServerConf, [{verify, verify_peer} | ClientConf], IP, Config).
 
@@ -167,42 +216,44 @@ no_ip_fallback(Config) ->
     Hostname = net_adm:localhost(),
     {ok, #hostent{h_addr_list = [IP |_]}} = inet:gethostbyname(net_adm:localhost()),
     #{server_config := ServerConf,
-      client_config := ClientConf} = public_key:pkix_test_data(#{server_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
-                                                                       peer => [{extensions, [#'Extension'{extnID = 
-                                                                                                               ?'id-ce-subjectAltName',
-                                                                                                           extnValue = [{dNSName, Hostname}],
-                                                                                                           critical = false}]},
-                                                                                {key, ssl_test_lib:hardcode_rsa_key(3)}
-                                                                               ]},
-                                                                 client_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
-                                                                       peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
+      client_config := ClientConf} =
+        public_key:pkix_test_data(#{server_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
+                                          peer => [{extensions, [#'Extension'{extnID =
+                                                                                  ?'id-ce-subjectAltName',
+                                                                              extnValue = [{dNSName, Hostname}],
+                                                                              critical = false}]},
+                                                   {key, ssl_test_lib:hardcode_rsa_key(3)}
+                                                  ]},
+                                    client_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
+                                          peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
     successfull_connect(ServerConf, [{verify, verify_peer} | ClientConf], Hostname, Config),
     unsuccessfull_connect(ServerConf, [{verify, verify_peer} | ClientConf], IP, Config).
 
 dns_name_reuse(Config) ->
     SNIHostname = "OTP.test.server",
     #{server_config := ServerConf,
-      client_config := ClientConf} = public_key:pkix_test_data(#{server_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
-                                                                       peer => [{extensions,  [#'Extension'{extnID = 
-                                                                                                                ?'id-ce-subjectAltName',
-                                                                                                            extnValue = [{dNSName, SNIHostname}],
-                                                                                                            critical = false}
-                                                                                              ]},
-                                                                                {key, ssl_test_lib:hardcode_rsa_key(3)}
-                                                                               ]},
-                                                                 client_chain => 
-                                                                     #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
-                                                                       intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
-                                                                       peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
+      client_config := ClientConf} =
+        public_key:pkix_test_data(#{server_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
+                                          peer => [{extensions,  [#'Extension'{extnID =
+                                                                                   ?'id-ce-subjectAltName',
+                                                                               extnValue = [{dNSName, SNIHostname}],
+                                                                               critical = false}
+                                                                 ]},
+                                                   {key, ssl_test_lib:hardcode_rsa_key(3)}
+                                                  ]},
+                                    client_chain =>
+                                        #{root => [{key, ssl_test_lib:hardcode_rsa_key(4)}],
+                                          intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(5)}]],
+                                          peer => [{key, ssl_test_lib:hardcode_rsa_key(6)}]}}),
     
     {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
-  
+
     unsuccessfull_connect(ServerConf, [{verify, verify_peer} | ClientConf], undefined, Config),
     
     Server = 
@@ -288,28 +339,61 @@ customize_hostname_check(Config) when is_list(Config) ->
                                               ]),    
     ssl_test_lib:check_client_alert(Server, Client1, handshake_failure).
 
-%%--------------------------------------------------------------------
-%% Internal Functions ------------------------------------------------
-%%--------------------------------------------------------------------
-ssl_recv(SSLSocket, Expect) ->
-    ssl_recv(SSLSocket, "", Expect).
+sni_no_trailing_dot() ->
+      [{doc,"Test that sni may not include a triling dot"}].
+sni_no_trailing_dot(Config) when is_list(Config) ->
+    ClientOpts = ssl_test_lib:ssl_options(client_cert_opts, Config),
+    ServerOpts = ssl_test_lib:ssl_options(server_cert_opts, Config),
 
-ssl_recv(SSLSocket, CurrentData, ExpectedData) ->
-    receive
-        {ssl, SSLSocket, Data} ->
-            NeweData = CurrentData ++ Data,
-            case NeweData of
-                ExpectedData ->
-                    ok;
-                _  ->
-                    ssl_recv(SSLSocket, NeweData, ExpectedData)
-            end;
-        Other ->
-            ct:fail({unexpected_message, Other})
-    after 4000 ->
-            ct:fail({timeout, CurrentData, ExpectedData})
+    {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
+
+    Server = ssl_test_lib:start_server_error([{node, ServerNode}, {port, 0},
+                                 {from, self()},
+                                 {mfa, {ssl_test_lib, no_result, []}},
+                                 {options, [{log_level, debug} | ServerOpts]}]),
+
+    Port  = ssl_test_lib:inet_port(Server),
+
+    Client = ssl_test_lib:start_client_error([{node, ClientNode}, {port, Port},
+					      {host, Hostname},
+					      {from, self()},
+                                              {mfa, {ssl_test_lib, no_result, []}},
+					      {options, [{log_level, debug},
+                                                         {server_name_indication, Hostname ++ "."} |ClientOpts]}]),
+    ssl_test_lib:check_server_alert(Server, Client, unrecognized_name).
+
+hostname_trailing_dot() ->
+    [{doc,"Test that fallback sni removes trailing dot of hostname"}].
+
+hostname_trailing_dot(Config) when is_list(Config) ->
+    ClientOpts = ssl_test_lib:ssl_options(client_rsa_opts, Config),
+    ServerOpts = ssl_test_lib:ssl_options(server_rsa_opts, Config),
+    {ClientNode, ServerNode, Hostname0} = ssl_test_lib:run_where(Config),
+
+    case trailing_dot_hostname(Hostname0) of
+        {ok, Hostname} ->
+            Server = ssl_test_lib:start_server([{node, ClientNode}, {port, 0},
+                                                {from, self()},
+                                        {mfa, {ssl_test_lib, send_recv_result_active, []}},
+                                                {options, ServerOpts}]),
+            Port = ssl_test_lib:inet_port(Server),
+            Client = ssl_test_lib:start_client([{node, ServerNode}, {port, Port},
+                                                {host, Hostname},
+                                                {from, self()},
+                                                {mfa, {ssl_test_lib, send_recv_result_active, []}},
+                                        {options, ClientOpts}]),
+
+            ssl_test_lib:check_result(Server, ok, Client, ok),
+
+            ssl_test_lib:close(Server),
+            ssl_test_lib:close(Client);
+        {skip, _ } = Skip ->
+            Skip
     end.
 
+%%--------------------------------------------------------------------
+%% Callback Functions ------------------------------------------------
+%%--------------------------------------------------------------------
 send_and_hostname(SSLSocket) ->
     ssl:send(SSLSocket, "OK"),
     case  ssl:connection_information(SSLSocket, [sni_hostname]) of
@@ -319,18 +403,6 @@ send_and_hostname(SSLSocket) ->
 	    undefined
     end.
 
-rdnPart([[#'AttributeTypeAndValue'{type=Type, value=Value} | _] | _], Type) -> 
-    Value;
-rdnPart([_ | Tail], Type) -> 
-    rdnPart(Tail, Type);
-rdnPart([], _) -> 
-    unknown.
-
-rdn_to_string({utf8String, Binary}) ->
-    erlang:binary_to_list(Binary);
-rdn_to_string({printableString, String}) ->
-    String.
-
 recv_and_certificate(SSLSocket) ->
     ssl_recv(SSLSocket, "OK"),
     {ok, PeerCert} = ssl:peercert(SSLSocket),
@@ -339,6 +411,9 @@ recv_and_certificate(SSLSocket) ->
     ct:log("Subject of certificate received from server: ~p", [Subject]),
     rdn_to_string(rdnPart(Subject, ?'id-at-commonName')).
 
+%%--------------------------------------------------------------------
+%% Internal Functions ------------------------------------------------
+%%--------------------------------------------------------------------
 run_sni_fun_handshake(Config, SNIHostname, ExpectedSNIHostname, ExpectedCN) ->
     ct:log("Start running handshake for sni_fun, Config: ~p, SNIHostname: ~p, "
 	   "ExpectedSNIHostname: ~p, ExpectedCN: ~p", 
@@ -426,3 +501,47 @@ host_name(undefined, Hostname) ->
     Hostname;
 host_name(Hostname, _) ->
     Hostname.
+
+trailing_dot_hostname(HostName) ->
+    case lists:member($., HostName) of
+        true ->
+            case lists:last(HostName) =/= $. of
+                true ->
+                    {ok, HostName ++ "."};
+                false ->
+                    {ok, HostName}
+            end;
+        _ ->
+            {skip, "Trailing dot conf not possible"}
+     end.
+
+ssl_recv(SSLSocket, Expect) ->
+    ssl_recv(SSLSocket, "", Expect).
+
+ssl_recv(SSLSocket, CurrentData, ExpectedData) ->
+    receive
+        {ssl, SSLSocket, Data} ->
+            NeweData = CurrentData ++ Data,
+            case NeweData of
+                ExpectedData ->
+                    ok;
+                _  ->
+                    ssl_recv(SSLSocket, NeweData, ExpectedData)
+            end;
+        Other ->
+            ct:fail({unexpected_message, Other})
+    after 4000 ->
+            ct:fail({timeout, CurrentData, ExpectedData})
+    end.
+
+rdnPart([[#'AttributeTypeAndValue'{type=Type, value=Value} | _] | _], Type) ->
+    Value;
+rdnPart([_ | Tail], Type) ->
+    rdnPart(Tail, Type);
+rdnPart([], _) ->
+    unknown.
+
+rdn_to_string({utf8String, Binary}) ->
+    erlang:binary_to_list(Binary);
+rdn_to_string({printableString, String}) ->
+    String.
