@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2008-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2008-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -20,9 +20,6 @@
 
 -module(tls_1_3_record_SUITE).
 
-%% Note: This directive should only be used in test suites.
--compile(export_all).
-
 -include_lib("common_test/include/ct.hrl").
 -include_lib("ssl/src/tls_record.hrl").
 -include_lib("ssl/src/tls_handshake.hrl").
@@ -30,6 +27,26 @@
 -include_lib("ssl/src/ssl_cipher.hrl").
 -include_lib("ssl/src/ssl_internal.hrl").
 
+%% Callback functions
+-export([all/0,
+         init_per_suite/1,
+         end_per_suite/1]).
+
+%% Testcases
+-export([encode_decode/0,
+         encode_decode/1,
+         finished_verify_data/0,
+          finished_verify_data/1,
+         '1_RTT_handshake'/0,
+         '1_RTT_handshake'/1,
+         '0_RTT_handshake'/0,
+         '0_RTT_handshake'/1
+        ]).
+
+%%--------------------------------------------------------------------
+%% Common Test interface functions -----------------------------------
+%%--------------------------------------------------------------------
+
 all() ->
     [encode_decode,
      finished_verify_data,
@@ -81,7 +98,7 @@ encode_decode(_Config) ->
                       <<128,229,186,211,62,127,182,20,62,166,233,23,135,64,121,
                         3,104,251,214,161,253,31,3,2,232,37,8,221,189,72,64,218,
                         121,41,112,148,254,34,68,164,228,60,161,201,132,55,56,
-                        157>>}, undefined,
+                        157>>}, undefined, undefined,
                      undefined,
                      <<92,24,205,75,244,60,136,212,250,32,214,20,37,3,213,87,61,207,
                        147,61,168,145,177,118,160,153,33,53,48,108,191,174>>,
@@ -107,12 +124,12 @@ encode_decode(_Config) ->
                       <<128,229,186,211,62,127,182,20,62,166,233,23,135,64,121,
                         3,104,251,214,161,253,31,3,2,232,37,8,221,189,72,64,218,
                         121,41,112,148,254,34,68,164,228,60,161,201,132,55,56,
-                        157>>}, undefined,
+                        157>>}, undefined, undefined,
                      undefined,
                      <<92,24,205,75,244,60,136,212,250,32,214,20,37,3,213,87,61,207,
                        147,61,168,145,177,118,160,153,33,53,48,108,191,174>>,
                      undefined},
-                sequence_number => 0,server_verify_data => undefined}},
+                sequence_number => 0,server_verify_data => undefined},max_fragment_length => undefined},
 
     PlainText = [11,
                  <<0,2,175>>,
@@ -168,7 +185,8 @@ encode_decode(_Config) ->
        #{current_write =>
              #{security_parameters =>
                    #security_parameters{cipher_suite = ?TLS_NULL_WITH_NULL_NULL},
-               sequence_number => 0
+               sequence_number => 0,
+               max_fragment_length => undefined
               }
         },
 
@@ -249,7 +267,7 @@ encode_decode(_Config) ->
     %%       e2 10 ad f3 00 aa 1f 26 60 e1 b2 2e 10 f1 70 f9 2a
     HKDFAlgo = sha256,
     Salt = binary:copy(<<?BYTE(0)>>, 32),
-    IKM = binary:copy(<<?BYTE(0)>>, 32),
+    _IKM = binary:copy(<<?BYTE(0)>>, 32),
     EarlySecret =
         hexstr2bin("33 ad 0a 1c 60 7e c0 3b 09 e6 cd 98 93 68 0c
           e2 10 ad f3 00 aa 1f 26 60 e1 b2 2e 10 f1 70 f9 2a"),
@@ -278,7 +296,7 @@ encode_decode(_Config) ->
         hexstr2bin("b1 58 0e ea df 6d d5 89 b8 ef 4f 2d 56
          52 57 8c c8 10 e9 98 01 91 ec 8d 05 83 08 ce a2 16 a2 1e"),
 
-    SPublicKey =
+    _SPublicKey =
         hexstr2bin("c9 82 88 76 11 20 95 fe 66 76 2b db f7 c6
          72 e1 56 d6 cc 25 3b 83 3d f1 dd 69 b1 b0 4e 75 1f 0f"),
 
@@ -744,14 +762,12 @@ encode_decode(_Config) ->
     %%    PRK (32 octets):  7d f2 35 f2 03 1d 2a 05 12 87 d0 2b 02 41 b0 bf
     %%       da f8 6c c8 56 23 1f 2d 5a ba 46 c4 34 ec 19 6c
     %%
-    RMS = hexstr2bin("7d f2 35 f2 03 1d 2a 05 12 87 d0 2b 02 41 b0 bf
+    _RMS = hexstr2bin("7d f2 35 f2 03 1d 2a 05 12 87 d0 2b 02 41 b0 bf
                       da f8 6c c8 56 23 1f 2d 5a ba 46 c4 34 ec 19 6c"),
 
     %% Verify calculation of resumption master secret that is used to create
     %% the pre shared key in '0-RTT'.
-    Temp = tls_v1:resumption_master_secret(HKDFAlgo, {master_secret, MasterSecret}, CHSF),
-    erlang:display({rms, RMS}),
-    erlang:display({new_rms, Temp}),
+    _Temp = tls_v1:resumption_master_secret(HKDFAlgo, {master_secret, MasterSecret}, CHSF),
 
     %% {server}  derive secret "tls13 exp master":
     %%
