@@ -2,7 +2,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1999-2018. All Rights Reserved.
+%% Copyright Ericsson AB 1999-2020. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -81,8 +81,15 @@ eval_field({bin_element, Line, {string, _, S}, Size0, Options0}, Bs0, Fun) ->
         make_bit_type(Line, Size0, Options0),
     {value,Size,Bs1} = Fun(Size1, Bs0),
     Res = << <<(eval_exp_field1(C, Size, Unit,
-				Type, Endian, Sign))/binary>> ||
+				Type, Endian, Sign))/bitstring>> ||
 	      C <- S >>,
+    case S of
+        "" -> % find errors also when the string is empty
+            _ = eval_exp_field1(0, Size, Unit, Type, Endian, Sign),
+            ok;
+        _ ->
+            ok
+    end,
     {Res,Bs1};
 eval_field({bin_element,Line,E,Size0,Options0}, Bs0, Fun) ->
     {value,V,Bs1} = Fun(E, Bs0),
@@ -119,10 +126,14 @@ eval_exp_field(Val, _Size, _Unit, utf16, big, _) ->
     <<Val/big-utf16>>;
 eval_exp_field(Val, _Size, _Unit, utf16, little, _) ->
     <<Val/little-utf16>>;
+eval_exp_field(Val, _Size, _Unit, utf16, native, _) ->
+    <<Val/native-utf16>>;
 eval_exp_field(Val, _Size, _Unit, utf32, big, _) ->
     <<Val/big-utf32>>;
 eval_exp_field(Val, _Size, _Unit, utf32, little, _) ->
     <<Val/little-utf32>>;
+eval_exp_field(Val, _Size, _Unit, utf32, native, _) ->
+    <<Val/native-utf32>>;
 eval_exp_field(Val, Size, Unit, float, little, _) ->
     <<Val:(Size*Unit)/float-little>>;
 eval_exp_field(Val, Size, Unit, float, native, _) ->
@@ -187,7 +198,6 @@ bin_gen_field({bin_element,Line,{string,SLine,S},Size0,Options0},
               Bin0, Bs0, BBs0, Mfun, Efun) ->
     {Size1, [Type,{unit,Unit},Sign,Endian]} =
         make_bit_type(Line, Size0, Options0),
-    match_check_size(Mfun, Size1, BBs0),
     {value, Size, _BBs} = Efun(Size1, BBs0),
     F = fun(C, Bin, Bs, BBs) ->
                 bin_gen_field1(Bin, Type, Size, Unit, Sign, Endian,
@@ -200,7 +210,6 @@ bin_gen_field({bin_element,Line,VE,Size0,Options0},
         make_bit_type(Line, Size0, Options0),
     V = erl_eval:partial_eval(VE),
     NewV = coerce_to_float(V, Type),
-    match_check_size(Mfun, Size1, BBs0, false),
     {value, Size, _BBs} = Efun(Size1, BBs0),
     bin_gen_field1(Bin, Type, Size, Unit, Sign, Endian, NewV, Bs0, BBs0, Mfun).
 
@@ -269,7 +278,6 @@ match_field_1({bin_element,Line,{string,SLine,S},Size0,Options0},
     {Size1, [Type,{unit,Unit},Sign,Endian]} =
         make_bit_type(Line, Size0, Options0),
     Size2 = erl_eval:partial_eval(Size1),
-    match_check_size(Mfun, Size2, BBs0),
     {value, Size, _BBs} = Efun(Size2, BBs0),
     F = fun(C, Bin, Bs, BBs) ->
                 match_field(Bin, Type, Size, Unit, Sign, Endian,
@@ -283,7 +291,6 @@ match_field_1({bin_element,Line,VE,Size0,Options0},
     V = erl_eval:partial_eval(VE),
     NewV = coerce_to_float(V, Type),
     Size2 = erl_eval:partial_eval(Size1),
-    match_check_size(Mfun, Size2, BBs0),
     {value, Size, _BBs} = Efun(Size2, BBs0),
     match_field(Bin, Type, Size, Unit, Sign, Endian, NewV, Bs0, BBs0, Mfun).
 
@@ -331,12 +338,18 @@ get_value(Bin, utf16, undefined, _Unit, _Sign, big) ->
 get_value(Bin, utf16, undefined, _Unit, _Sign, little) ->
     <<I/little-utf16,Rest/bits>> = Bin,
     {I,Rest};
+get_value(Bin, utf16, undefined, _Unit, _Sign, native) ->
+    <<I/native-utf16,Rest/bits>> = Bin,
+    {I,Rest};
 get_value(Bin, utf32, undefined, _Unit, _Sign, big) ->
     <<Val/big-utf32,Rest/bits>> = Bin,
     {Val,Rest};
 get_value(Bin, utf32, undefined, _Unit, _Sign, little) ->
     <<Val/little-utf32,Rest/bits>> = Bin,
     {Val,Rest};
+get_value(Bin, utf32, undefined, _Unit, _Sign, native) ->
+    <<Val/native-utf32,Rest/bits>> = Bin,
+    {Val,Rest};
 get_value(Bin, binary, all, Unit, _Sign, _Endian) ->
     0 = (bit_size(Bin) rem Unit),
     {Bin,<<>>};
@@ -387,24 +400,3 @@ make_bit_type(_Line, Size, Type0) -> %Size evaluates to an integer or 'all'
         {ok,Size,Bt} -> {Size,erl_bits:as_list(Bt)};
         {error,Reason} -> erlang:raise(error, Reason, ?STACKTRACE)
     end.
-
-match_check_size(Mfun, Size, Bs) ->
-    match_check_size(Mfun, Size, Bs, true).
-
-match_check_size(Mfun, {var,_,V}, Bs, _AllowAll) ->
-    case Mfun(binding, {V,Bs}) of
-        {value,_} -> ok;
-	unbound -> throw(invalid) % or, rather, error({unbound,V})
-    end;
-match_check_size(_, {atom,_,all}, _Bs, true) ->
-    ok;
-match_check_size(_, {atom,_,all}, _Bs, false) ->
-    throw(invalid);
-match_check_size(_, {atom,_,undefined}, _Bs, _AllowAll) ->
-    ok;
-match_check_size(_, {integer,_,_}, _Bs, _AllowAll) ->
-    ok;
-match_check_size(_, {value,_,_}, _Bs, _AllowAll) ->
-    ok;	%From the debugger.
-match_check_size(_, _, _Bs, _AllowAll) ->
-    throw(invalid).
