@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2006-2019. All Rights Reserved.
+%% Copyright Ericsson AB 2006-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -47,12 +47,14 @@
 	  hook/1,
 	  neg_indent/1,
 	  maps_syntax/1,
+	  format_options/1,
           quoted_atom_types/1,
 
 	  otp_6321/1, otp_6911/1, otp_6914/1, otp_8150/1, otp_8238/1,
 	  otp_8473/1, otp_8522/1, otp_8567/1, otp_8664/1, otp_9147/1,
           otp_10302/1, otp_10820/1, otp_11100/1, otp_11861/1, pr_1014/1,
-          otp_13662/1, otp_14285/1, otp_15592/1, otp_15751/1, otp_15755/1]).
+          otp_13662/1, otp_14285/1, otp_15592/1, otp_15751/1, otp_15755/1,
+          otp_16435/1]).
 
 %% Internal export.
 -export([ehook/6]).
@@ -75,14 +77,15 @@ groups() ->
     [{expr, [],
       [func, call, recs, try_catch, if_then, receive_after,
        bits, head_tail, cond1, block, case1, ops,
-       messages, maps_syntax, quoted_atom_types
+       messages, maps_syntax, quoted_atom_types,
+       format_options
     ]},
      {attributes, [], [misc_attrs, import_export, dialyzer_attrs]},
      {tickets, [],
       [otp_6321, otp_6911, otp_6914, otp_8150, otp_8238,
        otp_8473, otp_8522, otp_8567, otp_8664, otp_9147,
        otp_10302, otp_10820, otp_11100, otp_11861, pr_1014, otp_13662,
-       otp_14285, otp_15592, otp_15751, otp_15755]}].
+       otp_14285, otp_15592, otp_15751, otp_15755, otp_16435]}].
 
 init_per_suite(Config) ->
     Config.
@@ -544,6 +547,36 @@ import_export(Config) when is_list(Config) ->
     compile(Config, Ts),
     ok.
 
+format_options(Config) when is_list(Config) ->
+    "case 1 of\n"
+    "  2 ->\n"
+    "    3;\n"
+    "  4 ->\n"
+    "    5\n"
+    "end" = flat_parse_and_pp_expr("case 1 of 2 -> 3; 4 -> 5 end", 0, [{indent, 2}]),
+
+    "-spec foo(bar(),\n"
+    "          qux()) ->\n"
+    "           T |\n"
+    "           baz(T)\n"
+    "           when\n"
+    "             T ::\n"
+    "               tuple().\n" =
+	lists:flatten(
+	    parse_and_pp_forms(
+		"-spec foo(bar(), qux()) -> T | baz(T) when T :: tuple().",
+		[{indent, 2}, {linewidth, 20}]
+	    )
+	),
+
+    "-spec foo(bar(), qux()) -> T | baz(T) when T :: tuple().\n" =
+	lists:flatten(
+	    parse_and_pp_forms(
+		"-spec foo(bar(), qux()) -> T | baz(T) when T :: tuple().",
+		[{indent, 2}, {linewidth, 1000}]
+	    )
+	).
+
 misc_attrs(Config) when is_list(Config) ->
     ok = pp_forms(<<"-module(m). ">>),
     ok = pp_forms(<<"-module(m, [Aafjlksfjdlsjflsdfjlsdjflkdsfjlk,"
@@ -1189,36 +1222,34 @@ otp_15592(_Config) ->
     ok.
 
 otp_15751(_Config) ->
-    ok = pp_expr(<<"try foo:bar()
-                        catch
-                            Reason : Stacktrace ->
-                                {Reason, Stacktrace}
-                    end">>),
-    ok = pp_expr(<<"try foo:bar()
-                        catch
-                            throw: Reason : Stacktrace ->
-                                {Reason, Stacktrace}
-                    end">>),
-    ok = pp_expr(<<"try foo:bar()
-                        catch
-                            Reason : _ ->
-                                Reason
-                    end">>),
-    ok = pp_expr(<<"try foo:bar()
-                        catch
-                            throw: Reason : _ ->
-                                Reason
-                    end">>),
-    ok = pp_expr(<<"try foo:bar()
-                        catch
-                            Reason ->
-                                Reason
-                    end">>),
-    ok = pp_expr(<<"try foo:bar()
-                        catch
-                            throw: Reason ->
-                                Reason
-                    end">>),
+    Check = fun(L) ->
+                    ok = pp_expr(L),
+                    remove_indentation(flat_parse_and_pp_expr(L, 0, []))
+            end,
+    "try foo:bar() catch Reason:Stacktrace -> {Reason, Stacktrace} end" =
+        Check("try foo:bar()
+           catch Reason:Stacktrace -> {Reason, Stacktrace}  end"),
+
+    "try foo:bar() catch throw:Reason:Stacktrace -> {Reason, Stacktrace} end" =
+        Check("try foo:bar()
+           catch throw:Reason:Stacktrace -> {Reason, Stacktrace} end"),
+
+    "try foo:bar() catch Reason:_ -> Reason end" =
+        Check("try foo:bar()
+           catch Reason:_ -> Reason end"),
+
+    "try foo:bar() catch throw:Reason -> Reason end" = % ":_" removed
+        Check("try foo:bar()
+           catch throw:Reason:_-> Reason end"),
+
+    "try foo:bar() catch throw:Reason -> Reason end" = % "throw:" added
+        Check("try foo:bar()
+           catch Reason -> Reason end"),
+
+    "try foo:bar() catch throw:Reason -> Reason end" =
+        Check("try foo:bar()
+           catch throw:Reason -> Reason end"),
+
     ok.
 
 otp_15755(_Config) ->
@@ -1261,6 +1292,45 @@ otp_15755(_Config) ->
              " 'sf s sdf', [], {}, {[]}}.", [])),
     ok.
 
+otp_16435(_Config) ->
+    CheckF = fun(S) -> S = lists:flatten(parse_and_pp_forms(S, [])) end,
+    CheckF("-type t() :: A :: integer().\n"),
+    CheckF("-type t() :: A :: (B :: integer()).\n"),
+    CheckF("-type t() :: {A :: (B :: integer())}.\n"),
+    CheckF("-record(r,{f :: {A :: (B :: integer())}}).\n"),
+    CheckF("-record(r,{f = 3 :: {A :: (B :: integer())}}).\n"),
+    CheckF("-type t() :: #r{f :: A :: (B :: integer())}.\n"),
+    CheckF("-spec t(X) -> X when X :: Y :: (Z :: #r{}).\n"),
+
+    CheckF("f() ->\n    << \n      (catch <<1:4>>) ||\n"
+           "          A <- []\n    >>.\n"),
+    CheckF("f() ->\n    [ \n     (catch foo) ||\n         A <- []\n    ].\n"),
+    CheckF("f() when erlang:float(3.0) ->\n    true.\n"),
+
+    Check = fun(S) -> S = flat_parse_and_pp_expr(S, 0, []) end,
+    Check("5 #r4.f1"),
+    Check("17 #{[] => true}"),
+    Check("0 #r1{f2 = foo}"),
+    Check("fun foo:bar/17 #{}"),
+    Check("fun a/2 #{}"),
+
+    Check("try foo:bar() of\n"
+          "    a ->\n"
+          "        b\n"
+          "after\n"
+          "    d\n"
+          "end"),
+
+    Check("try foo:bar() of\n"
+          "    a ->\n"
+          "        b\n"
+          "catch\n"
+          "    _:_ ->\n"
+          "        c\n"
+          "end"),
+
+    ok.
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 compile(Config, Tests) ->
