@@ -886,7 +886,7 @@ t_nativename_bin(Config) when is_list(Config) ->
 
 safe_relative_path(Config) ->
     PrivDir = proplists:get_value(priv_dir, Config),
-    Root = filename:join(PrivDir, ?FUNCTION_NAME),
+    Root = filename:join(PrivDir, "filename_SUITE_safe_relative_path"),
     ok = file:make_dir(Root),
     ok = file:set_cwd(Root),
 
@@ -1081,7 +1081,10 @@ check_basedir_xdg([Type|Types]) ->
     Opt  = #{os=>linux},
     Key  = basedir_xdg_env(Type),
     io:format("type: ~p~n", [Type]),
-    Home = os:getenv("HOME"),
+    Home = case os:getenv("WSLENV") of
+               false -> os:getenv("HOME");
+               _ -> os:getenv("USERPROFILE")
+           end,
     NDir = "/some/absolute/path",
     DefPath = basedir_xdg_def(Type,Home,Name),
     EnvPath = case Type of
