@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2016-2019. All Rights Reserved.
+%% Copyright Ericsson AB 2016-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -42,7 +42,7 @@ all() ->
      event_types, generic_timers, code_change,
      {group, sys},
      hibernate, auto_hibernate, enter_loop, {group, undef_callbacks},
-     undef_in_terminate].
+     undef_in_terminate, {group, format_log}].
 
 groups() ->
     [{start, [], tcs(start)},
@@ -53,7 +53,8 @@ groups() ->
      {abnormal_handle_event, [], tcs(abnormal)},
      {sys, [], tcs(sys)},
      {sys_handle_event, [], tcs(sys)},
-     {undef_callbacks, [], tcs(undef_callbacks)}].
+     {undef_callbacks, [], tcs(undef_callbacks)},
+     {format_log, [], tcs(format_log)}].
 
 tcs(start) ->
     [start1, start2, start3, start4, start5, start6, start7,
@@ -68,7 +69,10 @@ tcs(sys) ->
      error_format_status, terminate_crash_format,
      get_state, replace_state];
 tcs(undef_callbacks) ->
-    [undef_code_change, undef_terminate1, undef_terminate2].
+    [undef_code_change, undef_terminate1, undef_terminate2,
+     pop_too_many];
+tcs(format_log) ->
+    [format_log_1, format_log_2].
 
 init_per_suite(Config) ->
     Config.
@@ -83,9 +87,7 @@ init_per_group(GroupName, Config)
        GroupName =:= sys_handle_event ->
     [{callback_mode,handle_event_function}|Config];
 init_per_group(undef_callbacks, Config) ->
-    DataDir = ?config(data_dir, Config),
-    StatemPath = filename:join(DataDir, "oc_statem.erl"),
-    {ok, oc_statem} = compile:file(StatemPath),
+    compile_oc_statem(Config),
     Config;
 init_per_group(_GroupName, Config) ->
     Config.
@@ -98,6 +100,9 @@ init_per_testcase(_CaseName, Config) ->
 %%%    dbg:tracer(),
 %%%    dbg:p(all, c),
 %%%    dbg:tpl(gen_statem, cx),
+%%%    dbg:tpl(gen_statem, loop_receive, cx),
+%%%    dbg:tpl(gen_statem, loop_state_callback, cx),
+%%%    dbg:tpl(gen_statem, loop_callback_mode_result, cx),
 %%%    dbg:tpl(proc_lib, cx),
 %%%    dbg:tpl(gen, cx),
 %%%    dbg:tpl(sys, cx),
@@ -107,6 +112,12 @@ end_per_testcase(_CaseName, Config) ->
 %%%    dbg:stop(),
     Config.
 
+compile_oc_statem(Config) ->
+    DataDir = ?config(data_dir, Config),
+    StatemPath = filename:join(DataDir, "oc_statem.erl"),
+    {ok, oc_statem} = compile:file(StatemPath),
+    ok.
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 -define(EXPECT_FAILURE(Code, Reason),
 	try begin Code end of
@@ -132,8 +143,18 @@ start1(Config) ->
 %%	?EXPECT_FAILURE(gen_statem:call(Pid0, hej), Reason),
 
     %%process_flag(trap_exit, OldFl),
-    ok = verify_empty_msgq().
+    ok = verify_empty_msgq(),
 
+    {ok,{Pid1,Mon1}} = gen_statem:start_monitor(?MODULE, start_arg(Config, []), []),
+    ok = do_func_test(Pid1),
+    ok = do_sync_func_test(Pid1),
+    stop_it(Pid1),
+    receive
+        {'DOWN', Mon1, process, Pid1, _Reason} ->
+            ok
+    end,
+    ok = verify_empty_msgq().
+    
 %% anonymous w. shutdown
 start2(Config) ->
     %% Dont link when shutdown
@@ -189,7 +210,7 @@ start6(Config) ->
 
     ok = verify_empty_msgq().
 
-%% global register linked
+%% global register linked & monitored
 start7(Config) ->
     STM = {global,my_stm},
 
@@ -199,6 +220,8 @@ start7(Config) ->
 	gen_statem:start_link(STM, ?MODULE, start_arg(Config, []), []),
     {error,{already_started,Pid}} =
 	gen_statem:start(STM, ?MODULE, start_arg(Config, []), []),
+    {error,{already_started,Pid}} =
+	gen_statem:start_monitor(STM, ?MODULE, start_arg(Config, []), []),
 
     ok = do_func_test(Pid),
     ok = do_sync_func_test(Pid),
@@ -206,6 +229,28 @@ start7(Config) ->
     ok = do_sync_func_test(STM),
     stop_it(STM),
 
+    ok = verify_empty_msgq(),
+
+    {ok,{Pid1,Mon1}} =
+	gen_statem:start_monitor(STM, ?MODULE, start_arg(Config, []), []),
+    {error,{already_started,Pid1}} =
+	gen_statem:start_link(STM, ?MODULE, start_arg(Config, []), []),
+    {error,{already_started,Pid1}} =
+	gen_statem:start(STM, ?MODULE, start_arg(Config, []), []),
+    {error,{already_started,Pid1}} =
+	gen_statem:start_monitor(STM, ?MODULE, start_arg(Config, []), []),
+
+    ok = do_func_test(Pid1),
+    ok = do_sync_func_test(Pid1),
+    ok = do_func_test(STM),
+    ok = do_sync_func_test(STM),
+    stop_it(STM),
+
+    receive
+        {'DOWN', Mon1, process, Pid1, _Reason} ->
+            ok
+    end,
+
     ok = verify_empty_msgq().
 
 
@@ -229,7 +274,7 @@ start8(Config) ->
     %%process_flag(trap_exit, OldFl),
     ok = verify_empty_msgq().
 
-%% local register linked
+%% local register linked & monitored
 start9(Config) ->
     %%OldFl = process_flag(trap_exit, true),
     Name = my_stm,
@@ -247,6 +292,24 @@ start9(Config) ->
     stop_it(Pid),
 
     %%process_flag(trap_exit, OldFl),
+    ok = verify_empty_msgq(),
+
+    {ok,{Pid1,Mon1}} =
+	gen_statem:start_monitor(STM, ?MODULE, start_arg(Config, []), []),
+    {error,{already_started,Pid1}} =
+	gen_statem:start_monitor(STM, ?MODULE, start_arg(Config, []), []),
+
+    ok = do_func_test(Pid1),
+    ok = do_sync_func_test(Pid1),
+    ok = do_func_test(Name),
+    ok = do_sync_func_test(Name),
+    stop_it(Pid1),
+
+    receive
+        {'DOWN', Mon1, process, Pid1, _Reason} ->
+            ok
+    end,
+
     ok = verify_empty_msgq().
 
 %% global register
@@ -390,19 +453,24 @@ stop7(Config) ->
 stop8(Config) ->
     Node = gen_statem_stop8,
     {ok,NodeName} = ct_slave:start(Node),
-    Dir = filename:dirname(code:which(?MODULE)),
-    rpc:call(NodeName, code, add_path, [Dir]),
-    {ok,Pid} =
-	rpc:call(
-	  NodeName, gen_statem,start,
-	  [?MODULE,start_arg(Config, []),[]]),
-    ok = gen_statem:stop(Pid),
-    false = rpc:call(NodeName, erlang, is_process_alive, [Pid]),
-    noproc =
-	?EXPECT_FAILURE(gen_statem:stop(Pid), Reason1),
-    {ok,NodeName} = ct_slave:stop(Node),
+    Statem =
+        try
+            Dir = filename:dirname(code:which(?MODULE)),
+            rpc:block_call(NodeName, code, add_path, [Dir]),
+            {ok,Pid} =
+                rpc:block_call(
+                  NodeName, gen_statem,start,
+                  [?MODULE,start_arg(Config, []),[]]),
+            ok = gen_statem:stop(Pid),
+            false = rpc:block_call(NodeName, erlang, is_process_alive, [Pid]),
+            noproc =
+                ?EXPECT_FAILURE(gen_statem:stop(Pid), Reason1),
+            Pid
+        after
+            {ok,NodeName} = ct_slave:stop(Node)
+        end,
     {{nodedown,NodeName},{sys,terminate,_}} =
-	?EXPECT_FAILURE(gen_statem:stop(Pid), Reason2),
+	?EXPECT_FAILURE(gen_statem:stop(Statem), Reason2),
     ok.
 
 %% Registered name on remote node
@@ -411,21 +479,26 @@ stop9(Config) ->
     LocalSTM = {local,Name},
     Node = gen_statem__stop9,
     {ok,NodeName} = ct_slave:start(Node),
-    STM = {Name,NodeName},
-    Dir = filename:dirname(code:which(?MODULE)),
-    rpc:call(NodeName, code, add_path, [Dir]),
-    {ok,Pid} =
-	rpc:call(
-	  NodeName, gen_statem, start,
-	  [LocalSTM,?MODULE,start_arg(Config, []),[]]),
-    ok = gen_statem:stop(STM),
-    undefined = rpc:call(NodeName,erlang,whereis,[Name]),
-    false = rpc:call(NodeName,erlang,is_process_alive,[Pid]),
-    noproc =
-	?EXPECT_FAILURE(gen_statem:stop(STM), Reason1),
-    {ok,NodeName} = ct_slave:stop(Node),
+    Statem =
+        try
+            STM = {Name,NodeName},
+            Dir = filename:dirname(code:which(?MODULE)),
+            rpc:block_call(NodeName, code, add_path, [Dir]),
+            {ok,Pid} =
+                rpc:block_call(
+                  NodeName, gen_statem, start,
+                  [LocalSTM,?MODULE,start_arg(Config, []),[]]),
+            ok = gen_statem:stop(STM),
+            undefined = rpc:block_call(NodeName,erlang,whereis,[Name]),
+            false = rpc:block_call(NodeName,erlang,is_process_alive,[Pid]),
+            noproc =
+                ?EXPECT_FAILURE(gen_statem:stop(STM), Reason1),
+            STM
+        after
+            {ok,NodeName} = ct_slave:stop(Node)
+        end,
     {{nodedown,NodeName},{sys,terminate,_}} =
-	?EXPECT_FAILURE(gen_statem:stop(STM), Reason2),
+	?EXPECT_FAILURE(gen_statem:stop(Statem), Reason2),
     ok.
 
 %% Globally registered name on remote node
@@ -433,18 +506,21 @@ stop10(Config) ->
     Node = gen_statem_stop10,
     STM = {global,to_stop},
     {ok,NodeName} = ct_slave:start(Node),
-    Dir = filename:dirname(code:which(?MODULE)),
-    rpc:call(NodeName,code,add_path,[Dir]),
-    {ok,Pid} =
-	rpc:call(
-	  NodeName, gen_statem, start,
-	  [STM,?MODULE,start_arg(Config, []),[]]),
-    global:sync(),
-    ok = gen_statem:stop(STM),
-    false = rpc:call(NodeName, erlang, is_process_alive, [Pid]),
-    noproc =
-	?EXPECT_FAILURE(gen_statem:stop(STM), Reason1),
-    {ok,NodeName} = ct_slave:stop(Node),
+    try
+        Dir = filename:dirname(code:which(?MODULE)),
+        rpc:block_call(NodeName,code,add_path,[Dir]),
+        {ok,Pid} =
+            rpc:block_call(
+              NodeName, gen_statem, start,
+              [STM,?MODULE,start_arg(Config, []),[]]),
+        global:sync(),
+        ok = gen_statem:stop(STM),
+        false = rpc:block_call(NodeName, erlang, is_process_alive, [Pid]),
+        noproc =
+            ?EXPECT_FAILURE(gen_statem:stop(STM), Reason1)
+    after
+        {ok,NodeName} = ct_slave:stop(Node)
+    end,
     noproc =
 	?EXPECT_FAILURE(gen_statem:stop(STM), Reason2),
     ok.
@@ -458,10 +534,10 @@ abnormal1(Config) ->
 	gen_statem:start(LocalSTM, ?MODULE, start_arg(Config, []), []),
 
     %% timeout call.
-    delayed = gen_statem:call(Name, {delayed_answer,1}, 100),
+    delayed = gen_statem:call(Name, {delayed_answer,100}, 2000),
     {timeout,_} =
 	?EXPECT_FAILURE(
-	   gen_statem:call(Name, {delayed_answer,1000}, 10),
+	   gen_statem:call(Name, {delayed_answer,2000}, 100),
 	   Reason),
     ok = gen_statem:stop(Name),
     ?t:sleep(1100),
@@ -1361,7 +1437,7 @@ hibernate(Config) ->
     {ok,Pid0} =
 	gen_statem:start_link(
 	  ?MODULE, start_arg(Config, hiber_now), []),
-    is_in_erlang_hibernate(Pid0),
+    wait_erlang_hibernate(Pid0),
     stop_it(Pid0),
     receive
 	{'EXIT',Pid0,normal} -> ok
@@ -1374,23 +1450,23 @@ hibernate(Config) ->
     true = ({current_function,{erlang,hibernate,3}} =/=
 		erlang:process_info(Pid,current_function)),
     hibernating = gen_statem:call(Pid, hibernate_sync),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     good_morning = gen_statem:call(Pid, wakeup_sync),
     is_not_in_erlang_hibernate(Pid),
     hibernating = gen_statem:call(Pid, hibernate_sync),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     please_just_five_more = gen_statem:call(Pid, snooze_sync),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     good_morning = gen_statem:call(Pid, wakeup_sync),
     is_not_in_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, hibernate_async),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, wakeup_async),
     is_not_in_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, hibernate_async),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, snooze_async),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, wakeup_async),
     is_not_in_erlang_hibernate(Pid),
 
@@ -1398,14 +1474,14 @@ hibernate(Config) ->
     true =
 	({current_function,{erlang,hibernate,3}} =/=
 	     erlang:process_info(Pid, current_function)),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
 
     'alive!' = gen_statem:call(Pid, 'alive?'),
     true =
 	({current_function,{erlang,hibernate,3}} =/=
 	     erlang:process_info(Pid, current_function)),
     Pid ! hibernate_now,
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
 
     'alive!' = gen_statem:call(Pid, 'alive?'),
     true =
@@ -1413,34 +1489,34 @@ hibernate(Config) ->
 	     erlang:process_info(Pid, current_function)),
 
     hibernating = gen_statem:call(Pid, hibernate_sync),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     good_morning = gen_statem:call(Pid, wakeup_sync),
     is_not_in_erlang_hibernate(Pid),
     hibernating = gen_statem:call(Pid, hibernate_sync),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     please_just_five_more = gen_statem:call(Pid, snooze_sync),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     good_morning = gen_statem:call(Pid, wakeup_sync),
     is_not_in_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, hibernate_async),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     ok  = gen_statem:cast(Pid, wakeup_async),
     is_not_in_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, hibernate_async),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, snooze_async),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     ok = gen_statem:cast(Pid, wakeup_async),
     is_not_in_erlang_hibernate(Pid),
 
     hibernating = gen_statem:call(Pid, hibernate_sync),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     sys:suspend(Pid),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     sys:resume(Pid),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     receive after 1000 -> ok end,
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
 
     good_morning  = gen_statem:call(Pid, wakeup_sync),
     is_not_in_erlang_hibernate(Pid),
@@ -1456,15 +1532,16 @@ hibernate(Config) ->
 %% Auto-hibernation timeout
 auto_hibernate(Config) ->
     OldFl = process_flag(trap_exit, true),
-    HibernateAfterTimeout = 100,
+    HibernateAfterTimeout = 1000,
 
     {ok,Pid} =
         gen_statem:start_link(
-            ?MODULE, start_arg(Config, []), [{hibernate_after, HibernateAfterTimeout}]),
+            ?MODULE, start_arg(Config, []),
+          [{hibernate_after, HibernateAfterTimeout}]),
     %% After init test
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(HibernateAfterTimeout),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     %% After info test
     Pid ! {hping, self()},
     receive
@@ -1475,7 +1552,7 @@ auto_hibernate(Config) ->
     end,
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(HibernateAfterTimeout),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     %% After cast test
     ok = gen_statem:cast(Pid, {hping, self()}),
     receive
@@ -1486,42 +1563,42 @@ auto_hibernate(Config) ->
     end,
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(HibernateAfterTimeout),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     %% After call test
     hpong = gen_statem:call(Pid, hping),
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(HibernateAfterTimeout),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     %% Timer test 1
-    TimerTimeout1 = 50,
-    ok = gen_statem:call(Pid, {arm_htimer, self(), TimerTimeout1}),
+    TimerTimeout1 = HibernateAfterTimeout div 2,
+    ok = gen_statem:call(Pid, {start_htimer, self(), TimerTimeout1}),
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(TimerTimeout1),
     is_not_in_erlang_hibernate(Pid),
     receive
-        {Pid, htimer_armed} ->
+        {Pid, htimer_timeout} ->
             ok
     after 1000 ->
         ct:fail(timer1)
     end,
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(HibernateAfterTimeout),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     %% Timer test 2
-    TimerTimeout2 = 150,
-    ok = gen_statem:call(Pid, {arm_htimer, self(), TimerTimeout2}),
+    TimerTimeout2 = HibernateAfterTimeout * 2,
+    ok = gen_statem:call(Pid, {start_htimer, self(), TimerTimeout2}),
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(HibernateAfterTimeout),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     receive
-        {Pid, htimer_armed} ->
+        {Pid, htimer_timeout} ->
             ok
-    after 1000 ->
+    after TimerTimeout2 ->
         ct:fail(timer2)
     end,
     is_not_in_erlang_hibernate(Pid),
     timer:sleep(HibernateAfterTimeout),
-    is_in_erlang_hibernate(Pid),
+    wait_erlang_hibernate(Pid),
     stop_it(Pid),
     process_flag(trap_exit, OldFl),
     receive
@@ -1531,38 +1608,38 @@ auto_hibernate(Config) ->
     end,
     ok = verify_empty_msgq().
 
-is_in_erlang_hibernate(Pid) ->
-    receive after 1 -> ok end,
-    is_in_erlang_hibernate_1(200, Pid).
 
-is_in_erlang_hibernate_1(0, Pid) ->
+wait_erlang_hibernate(Pid) ->
+    receive after 1 -> ok end,
+    wait_erlang_hibernate_1(200, Pid).
+
+wait_erlang_hibernate_1(0, Pid) ->
     ct:log("~p\n", [erlang:process_info(Pid, current_function)]),
-    ct:fail(not_in_erlang_hibernate_3);
-is_in_erlang_hibernate_1(N, Pid) ->
+    ct:fail(should_be_in_erlang_hibernate_3);
+wait_erlang_hibernate_1(N, Pid) ->
     {current_function,MFA} = erlang:process_info(Pid, current_function),
     case MFA of
 	{erlang,hibernate,3} ->
 	    ok;
 	_ ->
 	    receive after 10 -> ok end,
-	    is_in_erlang_hibernate_1(N-1, Pid)
+	    wait_erlang_hibernate_1(N-1, Pid)
     end.
 
 is_not_in_erlang_hibernate(Pid) ->
     receive after 1 -> ok end,
     is_not_in_erlang_hibernate_1(200, Pid).
 
-is_not_in_erlang_hibernate_1(0, Pid) ->
-    ct:log("~p\n", [erlang:process_info(Pid, current_function)]),
-    ct:fail(not_in_erlang_hibernate_3);
+is_not_in_erlang_hibernate_1(0, _Pid) ->
+    ct:fail(should_not_be_in_erlang_hibernate_3);
 is_not_in_erlang_hibernate_1(N, Pid) ->
     {current_function,MFA} = erlang:process_info(Pid, current_function),
     case MFA of
-	{erlang,hibernate,3} ->
+ 	{erlang,hibernate,3} ->
 	    receive after 10 -> ok end,
 	    is_not_in_erlang_hibernate_1(N-1, Pid);
-	_ ->
-	    ok
+ 	_ ->
+ 	    ok
     end.
 
 
@@ -1722,7 +1799,7 @@ enter_loop(Reg1, Reg2, Opts) ->
     end.
 
 undef_code_change(_Config) ->
-    {ok, Statem} = gen_statem:start(oc_statem, [], []),
+    {ok, Statem} = gen_statem:start(oc_statem, [], [{debug, [trace]}]),
     {error, {'EXIT',
              {undef, [{oc_statem, code_change, [_, _, _, _], _}|_]}}}
         = fake_upgrade(Statem, oc_statem).
@@ -1735,7 +1812,7 @@ fake_upgrade(Pid, Mod) ->
     Ret.
 
 undef_terminate1(_Config) ->
-    {ok, Statem} = gen_statem:start(oc_statem, [], []),
+    {ok, Statem} = gen_statem:start(oc_statem, [], [{debug,[trace]}]),
     MRef = monitor(process, Statem),
     ok = gen_statem:stop(Statem),
     verify_down(Statem, MRef, normal),
@@ -1743,7 +1820,7 @@ undef_terminate1(_Config) ->
 
 undef_terminate2(_Config) ->
     Reason = {error, test},
-    {ok, Statem} = oc_statem:start(),
+    {ok, Statem} = oc_statem:start([{debug,[trace]}]),
     MRef = monitor(process, Statem),
     ok = gen_statem:stop(Statem, Reason, infinity),
     verify_down(Statem, MRef, Reason).
@@ -1769,6 +1846,54 @@ verify_down(Statem, MRef, Reason) ->
         ct:fail(default_terminate_failed)
     end.
 
+
+pop_too_many(_Config) ->
+    _ = process_flag(trap_exit, true),
+
+    Machine =
+	#{init =>
+	      fun () ->
+		      {ok,start,undefined}
+	      end,
+	  start =>
+	      fun ({call, From}, {change_callback_module, _Module} = Action,
+                   undefined = _Data) ->
+		      {keep_state_and_data,
+                       [Action,
+                        {reply,From,ok}]};
+                  ({call, From}, {verify, ?MODULE},
+                   undefined = _Data) ->
+		      {keep_state_and_data,
+                       [{reply,From,ok}]};
+                  ({call, From}, pop_callback_module = Action,
+                   undefined = _Data) ->
+		      {keep_state_and_data,
+                       [Action,
+                        {reply,From,ok}]}
+	      end},
+    {ok, STM} =
+	gen_statem:start_link(
+          ?MODULE,
+          {map_statem, Machine, []},
+          [{debug, [trace]}]),
+
+    ok = gen_statem:call(STM, {change_callback_module, oc_statem}),
+    ok = gen_statem:call(STM, {push_callback_module, ?MODULE}),
+    ok = gen_statem:call(STM, {verify, ?MODULE}),
+    ok = gen_statem:call(STM, pop_callback_module),
+    BadAction = {bad_action_from_state_function, pop_callback_module},
+    {{BadAction, _},
+     {gen_statem,call,[STM,pop_callback_module,infinity]}} =
+        ?EXPECT_FAILURE(gen_statem:call(STM, pop_callback_module), Reason),
+
+    receive
+        {'EXIT', STM, {BadAction, _}} ->
+            ok;
+        Other ->
+            ct:fail({surprise, Other})
+    end.
+
+
 %% Test the order for multiple {next_event,T,C}
 next_events(Config) ->
     {ok,Pid} = gen_statem:start(?MODULE, start_arg(Config, []), []),
@@ -1780,6 +1905,315 @@ next_events(Config) ->
 	?EXPECT_FAILURE(gen_statem:stop(Pid), Reason).
 
 
+%% Test report callback for Logger handler error_logger
+format_log_1(_Config) ->
+    FD = application:get_env(kernel, error_logger_format_depth),
+    application:unset_env(kernel, error_logger_format_depth),
+    Term = lists:seq(1,15),
+    Name = self(),
+    Reason = {bad_reply_action_from_state_function,[]},
+    Report1 = simple_report(Name, Term, Reason),
+    Report2 = elaborate_report(Name, Term, Reason),
+
+    {F1,A1} = gen_statem:format_log(Report1),
+    ct:log("F1: ~ts~nA1: ~tp",[F1,A1]),
+    FExpected1 = "** State machine ~tp terminating~n"
+        "** When server state  = ~tp~n"
+        "** Reason for termination = ~tp:~tp~n"
+        "** Callback modules = ~tp~n"
+        "** Callback mode = ~tp~n",
+    FExpected1 = F1,
+    [Name,Term,error,Reason,[?MODULE],state_functions] = A1,
+
+    {F3,A3} = gen_statem:format_log(Report2),
+    ct:log("F3: ~ts~nA3: ~tp",[F3,A3]),
+    FExpected3 = "** State machine ~tp terminating~n"
+        "** Last event = ~tp~n"
+        "** When server state  = ~tp~n"
+        "** Reason for termination = ~tp:~tp~n"
+        "** Callback modules = ~tp~n"
+        "** Callback mode = ~tp~n"
+        "** Queued = ~tp~n"
+        "** Postponed = ~tp~n"
+        "** Stacktrace =~n**  ~tp~n"
+        "** Time-outs: ~tp~n"
+        "** Log =~n**  ~tp~n"
+        "** Client ~tp stacktrace~n"
+        "** ~tp~n",
+    FExpected3 = F3,
+    Stacktrace = stacktrace(),
+    [Name,Term,Term,error,Reason,[?MODULE],[state_functions,state_enter],[Term],
+     [{internal,Term}],Stacktrace,{1,[{timeout,message}]},[Term],Name,[]] = A3,
+
+    Depth = 10,
+    ok = application:set_env(kernel, error_logger_format_depth, Depth),
+    Limited = [1,2,3,4,5,6,7,8,9,'...'],
+    {F2,A2} = gen_statem:format_log(Report1),
+    ct:log("F2: ~ts~nA2: ~tp",[F2,A2]),
+    FExpected2 = "** State machine ~tP terminating~n"
+        "** When server state  = ~tP~n"
+        "** Reason for termination = ~tP:~tP~n"
+        "** Callback modules = ~tP~n"
+        "** Callback mode = ~tP~n",
+    FExpected2 = F2,
+    [Name,Depth,Limited,Depth,error,Depth,Reason,Depth,
+     [?MODULE],Depth,state_functions,Depth] = A2,
+
+    {F4,A4} = gen_statem:format_log(Report2),
+    ct:log("F4: ~ts~nA4: ~tp",[F4,A4]),
+    FExpected4 = "** State machine ~tP terminating~n"
+        "** Last event = ~tP~n"
+        "** When server state  = ~tP~n"
+        "** Reason for termination = ~tP:~tP~n"
+        "** Callback modules = ~tP~n"
+        "** Callback mode = ~tP~n"
+        "** Queued = ~tP~n"
+        "** Postponed = ~tP~n"
+        "** Stacktrace =~n**  ~tP~n"
+        "** Time-outs: ~tP~n"
+        "** Log =~n**  ~tP~n"
+        "** Client ~tP stacktrace~n"
+        "** ~tP~n",
+    FExpected4 = F4,
+    LimitedPostponed = [{internal,[1,2,3,4,5,6,'...']}],
+    LimitedStacktrace = io_lib:limit_term(Stacktrace, Depth),
+    LimitedQueue = io_lib:limit_term([Term], Depth),
+    [Name,Depth,Limited,Depth,Limited,Depth,error,Depth,Reason,Depth,
+     [?MODULE],Depth,[state_functions,state_enter],Depth,LimitedQueue,Depth,
+     LimitedPostponed,Depth,LimitedStacktrace,Depth,{1,[{timeout,message}]},
+     Depth,[Limited],Depth,Name,Depth,[],Depth] = A4,
+
+    case FD of
+        undefined ->
+            application:unset_env(kernel, error_logger_format_depth);
+        _ ->
+            application:set_env(kernel, error_logger_format_depth, FD)
+    end,
+    ok.
+
+%% Test report callback for any Logger handler
+format_log_2(_Config) ->
+    format_log_2_simple(),
+    format_log_2_elaborate(),
+    ok.
+
+format_log_2_simple() ->
+    FD = application:get_env(kernel, error_logger_format_depth),
+    application:unset_env(kernel, error_logger_format_depth),
+
+    Term = lists:seq(1,15),
+    Name = self(),
+    NameStr = pid_to_list(Name),
+    Reason = {bad_reply_action_from_state_function,[]},
+    Report = simple_report(Name, Term, Reason),
+
+    FormatOpts1 = #{},
+    Str1 = flatten_format_log(Report, FormatOpts1),
+    L1 = length(Str1),
+    Expected1 = "** State machine " ++ NameStr ++ " terminating\n"
+        "** When server state  = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15]\n"
+        "** Reason for termination = "
+           "error:{bad_reply_action_from_state_function,[]}\n"
+        "** Callback modules = ["?MODULE_STRING"]\n"
+        "** Callback mode = state_functions\n",
+    ct:log("Str1: ~ts", [Str1]),
+    ct:log("length(Str1): ~p", [L1]),
+    Expected1 = Str1,
+
+    Depth = 10,
+    FormatOpts2 = #{depth=>Depth},
+    Str2 = flatten_format_log(Report, FormatOpts2),
+    L2 = length(Str2),
+    Expected2 = "** State machine " ++ NameStr ++ " terminating\n"
+        "** When server state  = [1,2,3,4,5,6,7,8,9|...]\n"
+        "** Reason for termination = "
+           "error:{bad_reply_action_from_state_function,[]}\n"
+        "** Callback modules = ["?MODULE_STRING"]\n"
+        "** Callback mode = state_functions\n",
+    ct:log("Str2: ~ts", [Str2]),
+    ct:log("length(Str2): ~p", [L2]),
+    true = Expected2 =:= Str2,
+
+    FormatOpts3 = #{chars_limit=>200},
+    Str3 = flatten_format_log(Report, FormatOpts3),
+    L3 = length(Str3),
+    Expected3 = "** State machine " ++ NameStr ++ " terminating\n"
+        "** When server state  = [",
+    ct:log("Str3: ~ts", [Str3]),
+    ct:log("length(Str3): ~p", [L3]),
+    true = lists:prefix(Expected3, Str3),
+    true = L3 < L1,
+
+    FormatOpts4 = #{single_line=>true},
+    Str4 = flatten_format_log(Report, FormatOpts4),
+    L4 = length(Str4),
+    Expected4 = "State machine " ++ NameStr ++ " terminating. "
+        "Reason: {bad_reply_action_from_state_function,[]}. "
+        "State: [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15].",
+    ct:log("Str4: ~ts", [Str4]),
+    ct:log("length(Str4): ~p", [L4]),
+    Expected4 = Str4,
+
+    FormatOpts5 = #{single_line=>true, depth=>Depth},
+    Str5 = flatten_format_log(Report, FormatOpts5),
+    L5 = length(Str5),
+    Expected5 = "State machine " ++ NameStr ++ " terminating. "
+        "Reason: {bad_reply_action_from_state_function,[]}. "
+        "State: [1,2,3,4,5,6,7,8,9|...].",
+    ct:log("Str5: ~ts", [Str5]),
+    ct:log("length(Str5): ~p", [L5]),
+    Expected5 = Str5,
+
+    FormatOpts6 = #{single_line=>true, chars_limit=>100},
+    Str6 = flatten_format_log(Report, FormatOpts6),
+    L6 = length(Str6),
+    Expected6 = "State machine " ++ NameStr ++ " terminating. "
+        "Reason: ",
+    ct:log("Str6: ~ts", [Str6]),
+    ct:log("length(Str6): ~p", [L6]),
+    true = lists:prefix(Expected6, Str6),
+    true = L6 < L4,
+
+    case FD of
+        undefined ->
+            application:unset_env(kernel, error_logger_format_depth);
+        _ ->
+            application:set_env(kernel, error_logger_format_depth, FD)
+    end,
+    ok.
+
+format_log_2_elaborate() ->
+    FD = application:get_env(kernel, error_logger_format_depth),
+    application:unset_env(kernel, error_logger_format_depth),
+
+    Term = lists:seq(1,15),
+    Name = self(),
+    NameStr = pid_to_list(Name),
+    Reason = {bad_reply_action_from_state_function,[]},
+    Report = elaborate_report(Name, Term, Reason),
+    FormatOpts1 = #{},
+    Str1 = flatten_format_log(Report, FormatOpts1),
+    L1 = length(Str1),
+    Expected1 = "** State machine " ++ NameStr ++ " terminating\n"
+        "** Last event = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15]\n",
+    ct:log("Str1: ~ts", [Str1]),
+    ct:log("length(Str1): ~p", [L1]),
+    true = lists:prefix(Expected1, Str1),
+
+    Depth = 10,
+    FormatOpts2 = #{depth=>Depth},
+    Str2 = flatten_format_log(Report, FormatOpts2),
+    L2 = length(Str2),
+    Expected2 = "** State machine " ++ NameStr ++ " terminating\n"
+        "** Last event = [1,2,3,4,5,6,7,8,9|...]\n"
+        "** When server state  = [1,2,3,4,5,6,7,8,9|...]\n"
+        "** Reason for termination = "
+           "error:{bad_reply_action_from_state_function,[]}\n"
+        "** Callback modules = ["?MODULE_STRING"]\n"
+        "** Callback mode = [state_functions,state_enter]\n"
+        "** Queued = [[1,2,3,4,5,6,7,8|...]]\n"
+        "** Postponed = [{internal,[1,2,3,4,5,6|...]}]\n"
+        "** Stacktrace =\n"
+        "**  [{m,f,1,[1,2,3,4|...]}]\n"
+        "** Time-outs: {1,[{timeout,message}]}\n"
+        "** Log =\n"
+        "**  [[1,2,3,4,5,6,7,8|...]]\n"
+        "** Client "++NameStr ++ " stacktrace\n"
+        "** []\n",
+    ct:log("Str2: ~ts", [Str2]),
+    ct:log("length(Str2): ~p", [L2]),
+    Expected2 = Str2,
+
+    FormatOpts3 = #{chars_limit=>300},
+    Str3 = flatten_format_log(Report, FormatOpts3),
+    L3 = length(Str3),
+    Expected3 = "** State machine " ++ NameStr ++ " terminating\n"
+        "** Last event = ",
+    ct:log("Str3: ~ts", [Str3]),
+    ct:log("length(Str3): ~p", [L3]),
+    true = lists:prefix(Expected3, Str3),
+    true = L3 < L1,
+
+    FormatOpts4 = #{single_line=>true},
+    Str4 = flatten_format_log(Report, FormatOpts4),
+    L4 = length(Str4),
+    Expected4 = "State machine " ++ NameStr ++ " terminating. "
+        "Reason: {bad_reply_action_from_state_function,[]}. "
+        "Stack: [{m,f,1,[1,2,3,4,5,6,7,8,9,10,11,12,13,14,15]}]. "
+        "Last event: [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15]. "
+        "State: [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15]. "
+        "Log: [[1,2,3,4,5,6,7,8,9,10,11,12,13,14,15]]. "
+        "Client " ++ NameStr ++ " stacktrace: [].",
+    ct:log("Str4: ~ts", [Str4]),
+    ct:log("length(Str4): ~p", [L4]),
+    Expected4 = Str4,
+
+    FormatOpts5 = #{single_line=>true, depth=>Depth},
+    Str5 = flatten_format_log(Report, FormatOpts5),
+    L5 = length(Str5),
+    Expected5 = "State machine " ++ NameStr ++ " terminating. "
+        "Reason: {bad_reply_action_from_state_function,[]}. "
+        "Stack: [{m,f,1,[1,2,3,4|...]}]. "
+        "Last event: [1,2,3,4,5,6,7,8,9|...]. "
+        "State: [1,2,3,4,5,6,7,8,9|...]. "
+        "Log: [[1,2,3,4,5,6,7,8|...]]. "
+        "Client " ++ NameStr ++ " stacktrace: [].",
+    ct:log("Str5: ~ts", [Str5]),
+    ct:log("length(Str5): ~p", [L5]),
+    Expected5 = Str5,
+
+    FormatOpts6 = #{single_line=>true, chars_limit=>300},
+    Str6 = flatten_format_log(Report, FormatOpts6),
+    L6 = length(Str6),
+    Expected6 = "State machine " ++ NameStr ++ " terminating. "
+        "Reason:",
+    ct:log("Str6: ~ts", [Str6]),
+    ct:log("length(Str6): ~p", [L6]),
+    true = lists:prefix(Expected6, Str6),
+    true = L6 < L4,
+
+    case FD of
+        undefined ->
+            application:unset_env(kernel, error_logger_format_depth);
+        _ ->
+            application:set_env(kernel, error_logger_format_depth, FD)
+    end,
+    ok.
+
+simple_report(Name, Term, Reason) ->
+    #{label=>{gen_statem,terminate},
+      name=>Name,
+      queue=>[],
+      postponed=>[],
+      modules=>[?MODULE],
+      callback_mode=>state_functions,
+      state_enter=>false,
+      state=>Term,
+      timeouts=>{0,[]},
+      log=>[],
+      reason=>{error,Reason,[]},
+      client_info=>undefined}.
+
+elaborate_report(Name, Term, Reason) ->
+    #{label=>{gen_statem,terminate},
+      name=>Name,
+      queue=>[Term,Term],
+      postponed=>[{internal,Term}],
+      modules=>[?MODULE],
+      callback_mode=>state_functions,
+      state_enter=>true,
+      state=>Term,
+      timeouts=>{1,[{timeout,message}]},
+      log=>[Term],
+      reason=>{error,Reason,stacktrace()},
+      client_info=>{self(),{self(),[]}}}.
+
+stacktrace() ->
+    [{m,f,1,lists:seq(1, 15)}].
+
+flatten_format_log(Report, Format) ->
+    lists:flatten(gen_statem:format_log(Report, Format)).
+
 %%
 %% Functionality check
 %%
@@ -1815,7 +2249,17 @@ do_func_test(STM) ->
     wfor(yes),
     ok = do_disconnect(STM),
     ok = gen_statem:cast(STM, {'alive?',self()}),
+    P0 = gen_statem:send_request(STM, 'alive?'),
+    timeout = gen_statem:wait_response(P0, 0),
     wfor(yes),
+    {reply, yes} = gen_statem:wait_response(P0, infinity),
+    _ = flush(),
+    P1 = gen_statem:send_request(STM, 'alive?'),
+    receive Msg ->
+            no_reply = gen_statem:check_response(Msg, P0),
+            {reply, yes} = gen_statem:check_response(Msg, P1)
+    after 1000 -> exit(timeout)
+    end,
     ok.
 
 
@@ -1910,13 +2354,13 @@ init(stop_shutdown) ->
     {stop,shutdown};
 init(sleep) ->
     ?t:sleep(1000),
-    {ok,idle,data};
+    init_sup({ok,idle,data});
 init(hiber) ->
-    {ok,hiber_idle,[]};
+    init_sup({ok,hiber_idle,[]});
 init(hiber_now) ->
-    {ok,hiber_idle,[],[hibernate]};
+    init_sup({ok,hiber_idle,[],[hibernate]});
 init({data, Data}) ->
-    {ok,idle,Data};
+    init_sup({ok,idle,Data});
 init({callback_mode,CallbackMode,Arg}) ->
     ets:new(?MODULE, [named_table,private]),
     ets:insert(?MODULE, {callback_mode,CallbackMode}),
@@ -1926,14 +2370,35 @@ init({map_statem,#{init := Init}=Machine,Modes}) ->
     ets:insert(?MODULE, {callback_mode,[handle_event_function|Modes]}),
     case Init() of
 	{ok,State,Data,Ops} ->
-	    {ok,State,[Data|Machine],Ops};
+	    init_sup({ok,State,[Data|Machine],Ops});
 	{ok,State,Data} ->
-	    {ok,State,[Data|Machine]};
+	    init_sup({ok,State,[Data|Machine]});
 	Other ->
-	    Other
+	    init_sup(Other)
     end;
 init([]) ->
-    {ok,idle,data}.
+    init_sup({ok,idle,data}).
+
+%% Supervise state machine parent i.e the test case, and if it dies
+%% (fails due to some reason), kill the state machine,
+%% just to not leak resources (process, name, ETS table, etc...)
+%%
+init_sup(Result) ->
+    Parent = gen:get_parent(),
+    Statem = self(),
+    _Supervisor =
+        spawn(
+          fun () ->
+                  StatemRef = monitor(process, Statem),
+                  ParentRef = monitor(process, Parent),
+                  receive
+                      {'DOWN', StatemRef, _, _, Reason} ->
+                          exit(Reason);
+                      {'DOWN', ParentRef, _, _, _} ->
+                          exit(Statem, kill)
+                  end
+          end),
+    Result.
 
 callback_mode() ->
     try ets:lookup(?MODULE, callback_mode) of
@@ -1966,10 +2431,10 @@ idle(cast, {hping,Pid}, Data) ->
     {keep_state, Data};
 idle({call, From}, hping, _Data) ->
     {keep_state_and_data, [{reply, From, hpong}]};
-idle({call, From}, {arm_htimer, Pid, Timeout}, _Data) ->
-    {keep_state_and_data, [{reply, From, ok}, {timeout, Timeout, {arm_htimer, Pid}}]};
-idle(timeout, {arm_htimer, Pid}, _Data) ->
-    Pid ! {self(), htimer_armed},
+idle({call, From}, {start_htimer, Pid, Timeout}, _Data) ->
+    {keep_state_and_data, [{reply, From, ok}, {timeout, Timeout, {htimer, Pid}}]};
+idle(timeout, {htimer, Pid}, _Data) ->
+    Pid ! {self(), htimer_timeout},
     keep_state_and_data;
 idle(cast, {connect,Pid}, Data) ->
     Pid ! accept,
