@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1999-2018. All Rights Reserved.
+%% Copyright Ericsson AB 1999-2019. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -32,7 +32,7 @@
          io_with_huge_message_queue/1, format_string/1,
 	 maps/1, coverage/1, otp_14178_unicode_atoms/1, otp_14175/1,
          otp_14285/1, limit_term/1, otp_14983/1, otp_15103/1, otp_15076/1,
-         otp_15159/1]).
+         otp_15159/1, otp_15639/1, otp_15705/1, otp_15847/1, otp_15875/1]).
 
 -export([pretty/2, trf/3]).
 
@@ -64,7 +64,8 @@ all() ->
      io_lib_print_binary_depth_one, otp_10302, otp_10755, otp_10836,
      io_lib_width_too_small, io_with_huge_message_queue,
      format_string, maps, coverage, otp_14178_unicode_atoms, otp_14175,
-     otp_14285, limit_term, otp_14983, otp_15103, otp_15076, otp_15159].
+     otp_14285, limit_term, otp_14983, otp_15103, otp_15076, otp_15159,
+     otp_15639, otp_15705, otp_15847, otp_15875].
 
 %% Error cases for output.
 error_1(Config) when is_list(Config) ->
@@ -2503,9 +2504,11 @@ otp_14983(_Config) ->
 
 trunc_string() ->
     "str " = trf("str ", [], 10),
-    "str ..." = trf("str ~s", ["str"], 6),
+    "str str" = trf("str ~s", ["str"], 6),
+    "str ..." = trf("str ~s", ["str1"], 6),
     "str str" = trf("str ~s", ["str"], 7),
-    "str ..." = trf("str ~8s", ["str"], 6),
+    "str str" = trf("str ~8s", ["str"], 6),
+    "str ..." = trf("str ~8s", ["str1"], 6),
     Pa = filename:dirname(code:which(?MODULE)),
     {ok, UNode} = test_server:start_node(printable_range_unicode, slave,
 					 [{args, " +pc unicode -pa " ++ Pa}]),
@@ -2647,3 +2650,71 @@ otp_15076(_Config) ->
     {'EXIT', {badarg, _}} = (catch io_lib:build_text(L)),
     {'EXIT', {badarg, _}} = (catch io_lib:build_text(L, [])),
     ok.
+
+otp_15639(_Config) ->
+    L = lists:duplicate(10, "a"),
+    LOpts = [{encoding, latin1},  {chars_limit, 10}],
+    UOpts = [{encoding, unicode}, {chars_limit, 10}],
+    "[[...]|...]" = pretty(L, LOpts),
+    "[[...]|...]" = pretty(L, UOpts),
+    "[\"a\",[...]|...]" =
+        pretty(L, [{chars_limit, 12}, {encoding, latin1}]),
+    "[\"a\",[...]|...]" =
+        pretty(L, [{chars_limit, 12}, {encoding, unicode}]),
+
+    %% Latin-1
+    "\"12345678\"" = pretty("12345678", LOpts),
+    "\"12345678\"..." = pretty("123456789", LOpts),
+    "\"\\r\\n123456\"..." = pretty("\r\n1234567", LOpts),
+    "\"\\r1234567\"..." = pretty("\r12345678", LOpts),
+    "\"\\r\\n123456\"..." = pretty("\r\n12345678", LOpts),
+    "\"12345678\"..." = pretty("12345678"++[x], LOpts),
+    "[49,50|...]" = pretty("1234567"++[x], LOpts),
+    "[49,x]" = pretty("1"++[x], LOpts),
+    "[[...]|...]" = pretty(["1","2","3","4","5","6","7","8"], LOpts),
+    %% Unicode
+    "\"12345678\"" = pretty("12345678", UOpts),
+    "\"12345678\"..." = pretty("123456789", UOpts),
+    "\"\\r\\n1234567\"" = pretty("\r\n1234567", UOpts),
+    "\"\\r1234567\"..." = pretty("\r12345678", UOpts),
+    "\"\\r\\n1234567\"..." = pretty("\r\n12345678", UOpts),
+    "[49,50|...]" = pretty("12345678"++[x], UOpts),
+    "\"12345678\"..." = pretty("123456789"++[x], UOpts),
+    "[[...]|...]" = pretty(["1","2","3","4","5","6","7","8"], UOpts),
+    ok.
+
+otp_15705(_Config) ->
+    L = [<<"an">>,["at"],[["om"]]],
+    "..." = trf("~s", [L], 0),
+    "..." = trf("~s", [L], 1),
+    "..." = trf("~s", [L], 2),
+    "..." = trf("~s", [L], 3),
+    "a..." = trf("~s", [L], 4),
+    "an..." = trf("~s", [L], 5),
+    "anatom" = trf("~s", [L], 6),
+    L2 = ["a",[<<"na">>],[["tom"]]],
+    "..." = trf("~s", [L2], 3),
+    "a..." = trf("~s", [L2], 4),
+    "an..." = trf("~s", [L2], 5),
+    "anatom" = trf("~s", [L2], 6),
+
+    A = [[<<"äpple"/utf8>>, "plus", <<"äpple">>]],
+    "äp..." = trf("~ts", [A], 5),
+    "äppleplusäpple" = trf("~ts", [A], 14),
+    U = [["ки"],"рилл","и́ческий атом"],
+    "ки..." = trf("~ts", [U], 5),
+    "кирилли́ческий..." = trf("~ts", [U], 16),
+    "кирилли́ческий атом" = trf("~ts", [U], 20),
+
+    "|кирилли́чес|" = trf("|~10ts|", [U], -1),
+    ok.
+
+otp_15847(_Config) ->
+    T = {someRecord,<<"1234567890">>,some,more},
+    "{someRecord,<<...>>,...}" =
+        pretty(T, [{chars_limit,20}, {encoding,latin1}]),
+    ok.
+
+otp_15875(_Config) ->
+    S = io_lib:format("~tp", [[{0, [<<"00">>]}]], [{chars_limit, 18}]),
+    "[{0,[<<48,...>>]}]" = lists:flatten(S).
