@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2001-2016. All Rights Reserved.
+%% Copyright Ericsson AB 2001-2017. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -1783,7 +1783,7 @@ multiple_relative_product(Conf) when is_list(Conf) ->
     ok.
 
 digraph(Conf) when is_list(Conf) ->
-    T0 = ets:all(),
+    T0 = lists:sort(ets:all()),
     E = empty_set(),
     R = relation([{a,b},{b,c},{c,d},{d,a}]),
     F = relation_to_family(R),
@@ -1833,15 +1833,12 @@ digraph(Conf) when is_list(Conf) ->
         true -> ok
     end,
 
-    true = T0 == ets:all(),
+    true = T0 == lists:sort(ets:all()),
     ok.
 
 digraph_fail(ExitReason, Fail) ->
-    {'EXIT', {ExitReason, [{sofs,family_to_digraph,A,_}|_]}} = Fail,
-    case {test_server:is_native(sofs),A} of
-	{false,[_,_]} -> ok;
-	{true,2} -> ok
-    end.
+    {'EXIT', {ExitReason, [{sofs,family_to_digraph,2,_}|_]}} = Fail,
+    ok.
 
 constant_function(Conf) when is_list(Conf) ->
     E = empty_set(),
