@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1997-2016. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2020. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -27,6 +27,8 @@
          init_per_testcase/2, end_per_testcase/2,
          app_test/1, appup_test/1, assert_test/1]).
 
+-compile(r21).
+
 suite() -> [{ct_hooks,[ts_install_cth]}].
 
 all() -> 
@@ -95,7 +97,7 @@ appup_tests(App,{OkVsns0,NokVsns}) ->
 create_test_vsns(App) ->
     ThisMajor = erlang:system_info(otp_release),
     FirstMajor = previous_major(ThisMajor),
-    SecondMajor = previous_major(FirstMajor),
+    SecondMajor = previous_major(previous_major(FirstMajor)),
     Ok = app_vsn(App,[ThisMajor,FirstMajor]),
     Nok0 = app_vsn(App,[SecondMajor]),
     Nok = case Ok of
