@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2001-2017. All Rights Reserved.
+%% Copyright Ericsson AB 2001-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@
 %% Test suites
 -export([stack_seq/1, tail_seq/1, create_file_slow/1, spawn_simple/1,
          imm_tail_seq/1, imm_create_file_slow/1, imm_compile/1,
-         cpu_create_file_slow/1, unicode/1]).
+         cpu_create_file_slow/1, unicode/1, parsify_maps/1]).
 
 %% Other exports
 -export([create_file_slow/2]).
@@ -59,7 +59,7 @@ all() ->
         false ->
             [stack_seq, tail_seq, create_file_slow, spawn_simple,
              imm_tail_seq, imm_create_file_slow, imm_compile,
-             cpu_create_file_slow, unicode]
+             cpu_create_file_slow, unicode, parsify_maps]
     end.
 
 
@@ -501,7 +501,11 @@ cpu_create_file_slow(Config) when is_list(Config) ->
             %%
             {ok, [T, P]} = parse(AnalysisFile),
             io:format("~p~n~n~p~n", [P, ets:tab2list(T)]),
-            ok = verify(T, P),
+            try
+                ok = verify(T, P)
+            catch throw:{negative,_Weird} ->
+                    io:format("Aborted as counter is negative because of bad cpu_time")
+            end,
             Proc = pid_to_list(self()),
             case P of
                 [{analysis_options, _},
@@ -520,13 +524,7 @@ cpu_create_file_slow(Config) when is_list(Config) ->
             io:format("cpu_ts:~w, fprof:~w~n", [Acc, Acc1]),
             {comment, io_lib:format("~p% cpu utilization", [100*divide(Acc,Acc1)])};
         {'EXIT', not_supported} ->
-            case {os:type(), os:version()} of
-                {{unix, sunos}, {Major, Minor, _}}
-                  when Major >= 5, Minor >= 7 ->
-                    ct:fail(Result);
-                _ ->
-                    {skipped, "not_supported"}
-            end;
+            {skipped, "not_supported"};
         _ ->
             ct:fail(Result)
     end,
@@ -544,6 +542,35 @@ unicode(Config) when is_list(Config) ->
     ok = fprof:profile(dump, AnalysisFile),
     ok = fprof:analyse(dest, AnalysisFile).
 
+parsify_maps(Config) when is_list(Config) ->
+    Pid = self(),
+    Ref = make_ref(),
+    Port = hd(erlang:ports()),
+    Fun = fun () -> ok end,
+    M = #{pid => Pid, Pid => pid,
+          ref => Ref, Ref => ref,
+          port => Port, Port => port,
+          a_fun => Fun, Fun => a_fun},
+    io:format("M = ~p~n", [M]),
+    L = [{tuple, M}, M, #{my_map => M, M => my_map}],
+    PL = fprof:parsify(L),
+    [{tuple, PM}, PM, PMap] = PL,
+    #{my_map := PM, PM := my_map} = PMap,
+    io:format("PM = ~p~n", [PM]),
+    LPid = pid_to_list(Pid),
+    LRef = ref_to_list(Ref),
+    LPort = port_to_list(Port),
+    LFun = erlang:fun_to_list(Fun),
+    LPid = maps:get(pid, PM),
+    pid = maps:get(LPid, PM),
+    LRef = maps:get(ref, PM),
+    ref = maps:get(LRef, PM),
+    LPort = maps:get(port, PM),
+    port = maps:get(LPort, PM),
+    LFun = maps:get(a_fun, PM),
+    a_fun = maps:get(LFun, PM),
+    ok.
+
 %%%---------------------------------------------------------------------
 %%% Functions to test
 %%%---------------------------------------------------------------------
@@ -604,6 +631,8 @@ verify(Tab, [{analysis_options, _},
                   {Proc, Cnt_P, Acc_P, Own_P} = Clocks
                     when Acc_P >= Own_P ->
                       Clocks;
+                  {_, _, Acc_p, _} = Weird when Acc_p < 0 ->
+                      throw({negative, Weird});
                   Weird ->
                       throw({error, [?MODULE, ?LINE, Weird]})
               end
