@@ -28,7 +28,11 @@
 	 init_per_suite/1, end_per_suite/1, 
 	 init_per_testcase/2, end_per_testcase/2]).
 
--compile(export_all).
+-export([silent_start/1, create_window/1, several_apps/1, wx_api/1, wx_misc/1,
+         data_types/1, wx_object/1, undef_in_handle_info/1, undef_in_terminate/1,
+         undef_handle_event/1, undef_handle_call/1, undef_handle_cast/1, undef_handle_info/1,
+         undef_code_change/1, undef_terminate1/1, undef_terminate2/1
+        ]).
 
 -include("wx_test_lib.hrl").
 
@@ -390,10 +394,19 @@ wx_object(Config) ->
     {call, {Frame,Panel}, _} = wx_object:call(Frame, fun(US) -> US end),
     ?m(false, wxWindow:getParent(Panel) =:= Frame),
     ?m(true, wx:equal(wxWindow:getParent(Panel),Frame)),
+    flush(),
+    ReqId = wx_object:send_request(Frame, fun(_US) -> timer:sleep(10), yes end),
+    timeout = wx_object:wait_response(ReqId, 0),
+    {reply, {call, yes, {Me,{_,ReqId}}}} = wx_object:wait_response(ReqId, 1000),
+    ReqId2 = wx_object:send_request(Frame, yes),
+    [Msg] = flush(),
+    no_reply = wx_object:check_response(Msg, ReqId),
+    {reply, {call, yes, {Me,{_,ReqId2}}}} = wx_object:check_response(Msg, ReqId2),
+
     FramePid = wx_object:get_pid(Frame),
     io:format("wx_object pid ~p~n",[FramePid]),
     FramePid ! foo3,
-    ?m([{info, foo3}|_], flush()),
+    ?m([{info, foo3}], flush()),
 
     ?m(ok, wx_object:cast(Frame, fun(_) -> hehe end)),
     ?m([{cast, hehe}|_], flush()),
